@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# encoding: utf8
 
 import wmldata, os, glob, sys
 import re
@@ -7,6 +6,12 @@ import re
 """Module implementing a WML parser."""
 
 class Error(Exception):
+    """
+    This is a custom exception the parser throws on errors. It can display
+    the position of the error as a tree of all files and macros leading to
+    the error, with line numbers.
+    """
+
     def __init__(self, parser, text):
         self.text = "%s:%d: %s" % (parser.filename, parser.line, text)
         for i in range(len(parser.texts)):
@@ -17,10 +22,15 @@ class Error(Exception):
         return self.text
 
 class Parser:
+    """
+    The main parser class. An instance of this is needed for parsing.
+    """
+
     class Macro:
         """Class to hold one single macro."""
-        def __init__(self, name, params, text):
-            self.name, self.params, self.text = name, params, text
+        def __init__(self, name, params, text, textdomain):
+            self.name, self.params, self.text, self.textdomain =\
+                name, params, text, textdomain
 
     class TextState:
         def __init__(self, filename, text, textpos, line, current_path,
@@ -30,7 +40,7 @@ class Parser:
             self.current_path = current_path
             self.textdomain = textdomain
 
-    def __init__(self, data_dir, user_dir = None):
+    def __init__(self, data_dir, user_dir=None, no_macros_in_string=False):
         """
         Initialize a new WMLParser instance.
 
@@ -38,8 +48,10 @@ class Parser:
         user_dir is used for resolving {~filepath} and {@filepath}
         See http://www.wesnoth.org/wiki/PreprocessorRef
         """
+
         self.data_dir = data_dir
         self.user_dir = user_dir
+        self.no_macros_in_string = no_macros_in_string
 
         self.textpos = 0
         self.line = 1
@@ -53,8 +65,20 @@ class Parser:
 
         self.textdomain = ""
 
+        # Callback which is called for each macro. If it returns False, the
+        # macro is ignored.
+        self.macro_callback = None
         self.macro_not_found_callback = None
         self.no_macros = False
+        
+        # If set to a function, the function is called with the current
+        # textdomain and string as arguments for any translatable string, and
+        # is expected to return a translation.
+        self.gettext = None
+
+        # A list containing the stacked up #ifdefs.
+        self.preprocessor_nesting = []
+        self.stay_in_file = False
 
         # If set, included files are only parsed when under the given directory.
         self.only_expand_pathes = []
@@ -77,7 +101,9 @@ class Parser:
         except UnicodeDecodeError:
             u = text.decode("latin1")
         text = u
-        text = text.replace("\r\n", "\n").replace("\t", " ")
+        text = text.replace("\r\n", "\n").replace("\t", " ").replace("\r", "\n")
+        if text == "" or text[-1] != "\n":
+            text += "\n"
         return text
 
     def set_macro_not_found_callback(self, callback):
@@ -96,37 +122,39 @@ class Parser:
         text = self.read_encoded(filename)
         self.push_text(filename, text, cd = os.path.dirname(filename))
 
-    def parse_stream(self, stream):
+    def parse_stream(self, stream, binary = False):
         """
         Set the parser to parse from a file object.
         """
         text = stream.read()
-        text = text.replace("\r\n", "\n").replace("\t", " ")
+        if not binary:
+            text = text.replace("\r\n", "\n").replace("\t", " ").replace("\r", "\n")
         self.push_text("inline", text)
 
-    def parse_text(self, text):
+    def parse_text(self, text, binary = False):
         """
         Set the parser to directly parse from the given string.
         """
-        text = text.replace("\r\n", "\n").replace("\t", " ")
+        if not binary:
+            text = text.replace("\r\n", "\n").replace("\t", " ")
         self.push_text("inline", text)
 
-    def push_text(self, filename, text, params = None, cd = None):
+    def push_text(self, filename, text, params = None, cd = None,
+        initial_textdomain = ""):
         """
         Recursively parse a sub-document, e.g. when a file is included or a
         macro is executed.
         """
         if self.verbose:
             sys.stderr.write("%s:%d: Now parsing %s.\n" % (self.filename,
-                self.line, filename))
-        text = text.replace("\r\n", "\n").replace("\t", " ")
+                                                           self.line, filename))
         if not text: text = "\n"
         self.texts.append(self.TextState(self.filename, self.text, self.textpos,
             self.line, self.current_path, self.textdomain))
         self.filename, self.text, self.params = filename, text, params
         self.textpos = 0
         self.line = 1
-        self.textdomain = ""
+        self.textdomain = initial_textdomain
         if cd: self.current_path = cd
 
     def pop_text(self):
@@ -161,7 +189,7 @@ class Parser:
         if c == "\n":
             self.line += 1
         if self.textpos == len(self.text):
-            if len(self.texts): self.pop_text()
+            if len(self.texts) and not self.stay_in_file: self.pop_text()
         return c
 
     def at_end(self):
@@ -169,12 +197,17 @@ class Parser:
         Return True if the parser is at the very end of the input, that is the
         last character of the topmost input text has been read.
         """
-        return len(self.texts) == 0 and self.textpos == len(self.text)
+        return (len(self.texts) == 0
+            or self.stay_in_file) and self.textpos == len(self.text)
+
+    def check_end(self):
+        if self.textpos == len(self.text):
+            if len(self.texts): self.pop_text()
 
     def peek_next(self):
         """Like read_next, but does not consume."""
         if self.textpos >= len(self.text):
-            if len(self.texts):
+            if len(self.texts) and not self.stay_in_file:
                 ts = self.texts[-1]
                 if ts.textpos >= len(ts.text): return ""
                 return ts.text[ts.textpos]
@@ -187,14 +220,14 @@ class Parser:
             found = mob.group(0)
             self.line += found.count("\n")
             self.textpos = mob.end(0)
-            if self.textpos == len(self.text):
+            if self.textpos == len(self.text) and not self.stay_in_file:
                 if len(self.texts): self.pop_text()
             return found
         else:
             found = self.text[self.textpos:]
             self.line += found.count("\n")
             self.textpos = len(self.text)
-            if len(self.texts):
+            if len(self.texts) and not self.stay_in_file:
                 self.pop_text()
                 found += self.read_until(sep)
             return found
@@ -213,18 +246,63 @@ class Parser:
     def skip_whitespace_and_newlines(self):
         self.read_while(" \t\r\n")
 
+    preprocessor_commands = ["#define", "#undef", "#textdomain", "#ifdef",
+        "#ifndef", "#else", "#enddef", "#endif"]
+
+    def read_lines_until(self, string):
+        """
+        Read lines until one contains the given string, but throw away any
+        comments.
+        """
+        text = ""
+        in_string = False
+        while 1:
+            if self.at_end():
+                return None
+            line = self.read_until("\n")
+
+            line_start = 0
+            if in_string:
+                string_end = line.find('"')
+                if string_end < 0:
+                    text += line
+                    continue
+                in_string = False
+                line_start = string_end + 1
+
+            elif line.lstrip().startswith("#"):
+                possible_comment = line.lstrip()
+                for com in self.preprocessor_commands:
+                    if possible_comment.startswith(com):
+                        break
+                else:
+                    continue
+            
+            
+            quotation = line.find('"', line_start)
+            while quotation >= 0:
+                in_string = True
+                string_end = line.find('"', quotation + 1)
+                if string_end < 0: break
+                line_start = string_end + 1
+                in_string = False
+                quotation = line.find('"', line_start)
+            
+            if not in_string:
+                end = line.find(string, line_start)
+                if end >= 0:
+                    text += line[:end]
+                    break
+            text += line
+        return text
+
     def skip_whitespace_inside_statement(self):
         self.read_while(" \t\r\n")
         if not self.at_end():
             c = self.peek_next()
             if c == "#":
-                if self.check_for("#define"): return
-                if self.check_for("#undef"): return
-                if self.check_for("#textdomain"): return
-                if self.check_for("#ifdef"): return
-                if self.check_for("#ifndef"): return
-                if self.check_for("#else"): return
-                if self.check_for("#end"): return
+                for command in self.preprocessor_commands:
+                    if self.check_for(command): return
                 self.read_until("\n")
                 self.skip_whitespace_inside_statement()
 
@@ -235,18 +313,6 @@ class Parser:
         """Compare the following text with str."""
         return self.text[self.textpos:self.textpos + len(str)] == str
 
-    def read_upto_string(self, str):
-        """Read input up to and including the given string."""
-        pos = self.text.find(str, self.textpos)
-        if pos == -1:
-            return None
-        found = self.text[self.textpos:pos]
-        self.textpos = pos + len(str)
-        self.line += found.count("\n")
-        if self.textpos == len(self.text):
-            self.pop_text()
-        return found
-
     def parse_macro(self):
         """No recursive macro processing is done here. If a macro is passed as
         parameter to a macro, then whoever parses the macro replacement will do
@@ -256,13 +322,24 @@ class Parser:
         innermost macro?
         """
         params = []
-        macro = self.read_until("}")
-        if macro[-1] != "}":
-            raise Error(self, "Unclosed macro")
-            return
+        balance = 1
+        macro = ""
+        while balance > 0:
+            macro += self.read_until("{}")
+            if macro[-1] == "{":
+                balance += 1
+            elif macro[-1] == "}":
+                balance -= 1
+            else:
+                raise Error(self, "Unclosed macro")
+                return
 
         preserve = macro
         macro = macro[:-1] # Get rid of final }
+        
+        if self.macro_callback:
+            if not self.macro_callback(macro): return None
+        
         # If the macro starts with ~, assume a file in userdata.
         if macro[0] == "~":
             if self.user_dir:
@@ -318,7 +395,13 @@ class Parser:
                 self.push_text(path, self.read_encoded(path), cd = os.path.dirname(path))
             return None
 
-        # No file was found, try to do macro expansion.
+        # It's OK for user directories not to exist.
+        # Nothing prefixed with ~ can be a macro.
+        if macro.startswith("~") and not self.verbose:
+            return None
+
+        # No file was found, try to do macro expansion. First, push the
+        # macro call again. E.g. {blah 1 2 3}.
         self.push_text("macro", preserve)
 
         # Find all parameters.
@@ -357,14 +440,20 @@ class Parser:
                 params += [read]
 
             elif sep == "(":
+                in_string = False
                 balance = 1
                 param = read
                 while balance:
                     c = self.read_next()
-                    if c == "(":
-                        balance += 1
-                    elif c == ")":
-                        balance -= 1
+                    # Ignore () within strings
+                    if c == '"':
+                        in_string = not in_string
+                        
+                    if not in_string:
+                        if c == "(":
+                            balance += 1
+                        elif c == ")":
+                            balance -= 1
                     param += c
                 params += [param[:-1]]
 
@@ -385,31 +474,46 @@ class Parser:
                     raise Error(self, "Not enough parameters for macro %s. " % name +
                         "%d given but %d needed %s." % (len(params) - 1,
                             len(macro.params), macro.params))
-                if self.verbose:
-                    s = "Replacing {%s} with %s" % (macro.params[i], params[1 + i])
-                    print s.encode("utf8")
-                text = text.replace("{%s}" % macro.params[i],
-                    params[1 + i])
+                rep = params[1 + i]
+                # Handle gettext replacement here, since inside the macro
+                # the textdomain will be wrong.
+                if self.gettext and rep and rep[0] == "_":
+                    q = rep.find('"')
+                    qe = rep.find('"', q + 1)
+                    rep = self.gettext(self.textdomain, rep[q + 1:qe])
+                    rep = '"' + rep + '"'
+                if self.verbose:                        
+                    #s = "Replacing {%s} with %s\n" % (macro.params[i], rep)
+                    ##sys.stderr.write(s.encode("utf8"))
+                    pass
+                text = text.replace("{%s}" % macro.params[i], rep)
 
             if text:
-                self.push_text(name, text)
+                self.push_text(name, text, initial_textdomain = macro.textdomain)
             else:
                 pass # empty macro, nothing to do
         else:
             if self.macro_not_found_callback:
                 keep_macro = self.macro_not_found_callback(self, name, params)
                 if keep_macro: return keep_macro
-            sys.stderr.write("No macro %s.\n" % name)
-            sys.stderr.write(" (%s:%d)\n" % (self.filename, self.line))
+            sys.stderr.write("No macro %s.\n" % name.encode("utf8"))
+            if self.verbose:
+                sys.stderr.write(" (%s:%d)\n" % (self.filename, self.line))
             return name
         return None
 
     def parse_string(self):
         text = ""
+        match_read_end = '"'
+        if not self.no_macros_in_string:
+            match_read_end += '{'
         while not self.at_end():
-            text += self.read_until('"{')
+            text += self.read_until(match_read_end)
             if text[-1] == '"':
-                return text[:-1]
+                if self.peek_next() == '"':
+                    self.read_next()
+                else:
+                    return text[:-1]
             elif text[-1] == '{':
                 text = text[:-1]
                 not_found = self.parse_macro()
@@ -427,6 +531,8 @@ class Parser:
         value = ""
         got_assign = False
         spaces = ""
+        filename = "(None)"
+        line = -1
         while 1:
             if c == "{":
                 keep_macro = self.parse_macro()
@@ -448,6 +554,8 @@ class Parser:
                     variables += [variable.rstrip()]
                     got_assign = True
                     translatable = False
+                    filename = self.filename
+                    line = self.line
                     self.skip_whitespace()
                 else:
                     if c == ",":
@@ -457,22 +565,29 @@ class Parser:
                         variable += c  
             else:
                 if c == '"':
+                    # We want the textdomain at the beginning of the string,
+                    # the end of the string may be outside a macro and already
+                    # in another textdomain.
+                    textdomain = self.textdomain
                     # remove possible _
                     i = len(value)
                     while i > 0:
                         i -= 1
                         if value[i] != " ": break
+                    got_underscore = False
                     if value and value[i] == "_":
+                        got_underscore = True
                         translatable = True
-                        # This is not the assignement =, but from e.g. MENU_IMG_TXT
-                        if i == 0 or value[i - 1] in [" ", "="]:
-                            # remove whitespace before _
-                            while i > 1:
-                                if value[i - 1] != " ": break
-                                i -= 1
-                            value = value[:i]
+                        # remove whitespace before _
+                        while i > 1:
+                            if value[i - 1] != " ": break
+                            i -= 1
+                        value = value[:i]
 
                     string = self.parse_string()
+                    if got_underscore:
+                        if self.gettext:
+                            string = self.gettext(textdomain, string)
                     value += string
                     spaces = ""
                 else:
@@ -495,11 +610,14 @@ class Parser:
         values += [value]
 
         data = []
-        j = 0
         for i in range(len(variables)):
-            data += [wmldata.DataText(variables[i], values[j],
-                translatable = translatable, textdomain = self.textdomain)]
-            j += 1
+            try:
+                key = wmldata.DataText(variables[i], values[i], translatable)
+                key.set_meta(filename, line)
+                data.append(key)
+            except IndexError:
+                raise Error(self, "Assignement does not match: %s = %s" % (
+                    str(variables), str(values)))
         return data
 
     def parse_top(self, data, state = None):
@@ -519,70 +637,121 @@ class Parser:
                         if name: params += [name]
                         if sep == "\n": break
                         self.read_while(" ")
-                    text = self.read_upto_string("#enddef")
+                        
+                    text = self.read_lines_until("#enddef")
                     if text == None:
                         raise Error(self, "#define without #enddef")
-                        return
-                    self.macros[params[0]] = self.Macro(params[0], params[1:], text)
+
+                    self.macros[params[0]] = self.Macro(
+                        params[0], params[1:], text, self.textdomain)
+                    if self.verbose:
+                        sys.stderr.write("New macro: %s.\n" % params[0])
 
                 elif self.check_for("undef "):
                     self.read_until(" ")
                     name = self.read_until(" \n")
-                    self.macros[name] = None
-                elif self.check_for("ifdef "):
-                    self.read_until(" ")
+                    name = name.rstrip()
+                    if name in self.macros: del self.macros[name]
+                elif self.check_for("ifdef ") or self.check_for("ifndef"):
+
+                    what = "#" + self.read_until(" ").rstrip()
+
                     name = self.read_until(" \n")
                     if name[-1] == " ": self.read_while(" \n")
                     name = name[:-1]
-                    subdata = wmldata.DataIfDef(name, [], "then")
-                    self.parse_top(subdata, "#ifdef")
-                    data.insert(subdata)
-                elif self.check_for("ifndef "):
-                    self.read_until(" ")
-                    name = self.read_until(" \n")
-                    if name[-1] == " ": self.read_while(" \n")
-                    name = name[:-1]
-                    subdata = wmldata.DataIfDef(name, [], "then")
-                    self.parse_top(subdata, "#ifndef")
-                    data.insert(subdata)
+
+                    condition_failed = False
+                    if what == "#ifdef":
+                        if name in self.macros:
+                            pass
+                        else:
+                            condition_failed = True
+                    else: # what == "#ifndef"
+                        if not name in self.macros:
+                            pass
+                        else:
+                            condition_failed = True
+
+                    self.preprocessor_nesting.append((what, condition_failed))
+
+                    # If the condition is true, we simply continue parsing. At
+                    # some point we will either hit an #else or #endif, and
+                    # things continue there. If the condition failed, we skip
+                    # over everything until we find the matching #else or
+                    # endif.
+                    if condition_failed:
+                        self.stay_in_file = True
+                        balance = 1
+                        while balance > 0 and not self.at_end():
+                            line = self.read_until("\n")
+                            line = line.lstrip()
+                            if line.startswith("#ifdef"): balance += 1
+                            if line.startswith("#ifndef"): balance += 1
+                            if line.startswith("#endif"): balance -= 1
+                            if line.startswith("#else"):
+                                if balance == 1:
+                                    balance = -1
+                                    break
+
+                        self.stay_in_file = False
+                        if balance == 0:
+                            self.preprocessor_nesting.pop()
+                        if balance > 0:
+                            raise Error(self, "Missing  #endif for %s" % what)
+
+                        self.check_end()
 
                 elif self.check_for("else"):
-                    self.read_until("\n")
-                    if state != "#ifdef" and state != "#ifndef":
+                    if not self.preprocessor_nesting:
                         raise Error(self, "#else without #ifdef")
-                    subdata = wmldata.DataIfDef("else", [], "else")
-                    self.parse_top(subdata, "#else")
-                    data.insert(subdata)
-                    return
+
+                    self.read_until("\n")
+                    
+                    # We seen an #else - that means we are at the end of a
+                    # conditional preprocessor block which has executed. So
+                    # we should now ignore everything up to the #endif.
+                    balance = 1
+                    self.stay_in_file = True
+                    while balance > 0 and not self.at_end():
+                        line = self.read_until("\n")
+                        line = line.lstrip()
+                        if line.startswith("#ifdef"): balance += 1
+                        if line.startswith("#ifndef"): balance += 1
+                        if line.startswith("#endif"): balance -= 1
+                    self.stay_in_file = False
+                    if balance != 0:
+                        raise Error(self, "Missing #endif for #else")
+
+                    self.check_end()
 
                 elif self.check_for("endif"):
+                    if not self.preprocessor_nesting:
+                        raise Error(self, "#endif without #ifdef")
+                    self.preprocessor_nesting.pop()
                     self.read_until("\n")
-                    if state != "#ifdef" and state != "#else" and state !=\
-                        "#ifndef":
-                        self.read_until("\n")
-                        raise Error(self, "#endif without #ifdef or #else")
-                    return
 
                 elif self.check_for("textdomain"):
                     self.read_until(" ")
-                    name = self.read_until("\n").strip()
-                    if name == "wesnoth":
-                        self.textdomain = "wesnoth"
-                    else:
-                        self.textdomain = ""
+                    self.textdomain = self.read_until("\n").strip()
                 else: # comment
                     line = self.read_until("\n")
-                    comment = c + line
+                    #comment = c + line
                     if self.verbose:
-                        msg = "Comment removed: %s" % comment
-                        sys.stderr.write(msg.encode("utf8"))
+                        #msg = "Comment removed: %s" % comment
+                        ##sys.stderr.write(msg.encode("utf8"))
+                        pass
             elif c == '[':
                 name = self.read_until("]")[:-1]
                 if name[0] == '/':
-                    if state == name[1:] or state == "+" + name[1:]:
+                    if state == name[1:]:
                         return
                     raise Error(self, "Mismatched closing tag [%s], expected [/%s]" % (name, state))
+                #TODO: make a [+tag] properly append to most recent [tag]
+                # The below is an ugly hack, but better than keeping the '+' in the tag name.
+                elif name[0] == '+':
+                    name = name[1:]
                 subdata = wmldata.DataSub(name)
+                subdata.set_meta(self.filename, self.line)
                 self.parse_top(subdata, name)
                 data.insert(subdata)
             elif c == '{':
@@ -632,6 +801,12 @@ if __name__ == "__main__":
 
     if options.verbose:
         wmlparser.verbose = True
+        def gt(domain, x):
+            print "gettext: '%s' '%s'" % (domain, x)
+            return x
+        wmlparser.gettext = gt
+    
+    wmlparser.do_preprocessor_logic = True
 
     if options.execute:
         wmlparser.parse_text(options.execute)
