@@ -1,6 +1,6 @@
-/* $Id: attack_prediction.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: attack_prediction.cpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2006 - 2008 by Rusty Russell <rusty@rustcorp.com.au>
+   Copyright (C) 2006 - 2009 by Rusty Russell <rusty@rustcorp.com.au>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,28 +17,23 @@
    this code is kinda tricky.  Do not remove it.
 */
 
-//! @file attack_prediction.cpp
-//! Simulate combat to calculate attacks. Standalone program, benchmark.
+/**
+ * @file attack_prediction.cpp
+ * Simulate combat to calculate attacks. Standalone program, benchmark.
+ */
 
 #include "attack_prediction.hpp"
 
-#include <cassert>
-#include <cstring> // For memset
-#include <vector>
 
 
 // Compile with -O3 -DBENCHMARK for speed testing,
 // -DCHECK for testing correctness
 // (run tools/wesnoth-attack-sim.c --check on output)
-#if !defined(BENCHMARK) && !defined(CHECK)
-#include "util.hpp"
-#else
+#if defined(BENCHMARK) || defined(CHECK)
 #include <time.h>
 #include <sys/time.h>
 #include <stdio.h>
 #include <stdlib.h>
-#define maximum(a,b) ((a) > (b) ? (a) : (b))
-#define minimum(a,b) ((a) < (b) ? (a) : (b))
 #endif
 
 #ifdef ATTACK_PREDICTION_DEBUG
@@ -49,7 +44,7 @@
 
 namespace
 {
-//! A matrix of A's hitpoints vs B's hitpoints.
+/** A matrix of A's hitpoints vs B's hitpoints. */
 struct prob_matrix
 {
 	// Simple matrix, both known HP.
@@ -112,12 +107,12 @@ private:
 	void shift_rows(unsigned dst, unsigned src,
 					unsigned damage, double prob, bool drain);
 
-	//! @todo FIXME: rename using _ at end.
-	unsigned int rows, cols;
-	double *plane[4];
+	/** @todo FIXME: rename using _ at end. */
+	unsigned int rows_, cols_;
+	double *plane_[4];
 
 	// For optimization, we keep track of the lower row/col we need to consider
-	unsigned int min_row[4], min_col[4];
+	unsigned int min_row_[4], min_col_[4];
 };
 
 prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
@@ -125,7 +120,7 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 						 unsigned int a_hp, unsigned int b_hp,
 						 const std::vector<double> a_summary[2],
 						 const std::vector<double> b_summary[2])
-	: rows(a_max_hp+1), cols(b_max_hp+1)
+	: rows_(a_max_hp+1), cols_(b_max_hp+1)
 {
 	if (!a_summary[0].empty()) {
 		// A has fought before.  Do we need a slow plane for it?
@@ -140,31 +135,31 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 			a_slows = true;
 	}
 
-	plane[NEITHER_SLOWED] = new_arr(rows*cols);
+	plane_[NEITHER_SLOWED] = new_arr(rows_*cols_);
 	if (b_slows)
-		plane[A_SLOWED] = new_arr(rows*cols);
+		plane_[A_SLOWED] = new_arr(rows_*cols_);
 	else
-		plane[A_SLOWED] = NULL;
+		plane_[A_SLOWED] = NULL;
 	if (a_slows)
-		plane[B_SLOWED] = new_arr(rows*cols);
+		plane_[B_SLOWED] = new_arr(rows_*cols_);
 	else
-		plane[B_SLOWED] = NULL;
+		plane_[B_SLOWED] = NULL;
 	if (a_slows && b_slows)
-		plane[BOTH_SLOWED] = new_arr(rows*cols);
+		plane_[BOTH_SLOWED] = new_arr(rows_*cols_);
 	else
-		plane[BOTH_SLOWED] = NULL;
+		plane_[BOTH_SLOWED] = NULL;
 
-	min_row[NEITHER_SLOWED] = a_hp - 1;
-	min_col[NEITHER_SLOWED] = b_hp - 1;
-	min_row[A_SLOWED] = min_row[B_SLOWED] = min_row[BOTH_SLOWED] = rows;
-	min_col[A_SLOWED] = min_col[B_SLOWED] = min_col[BOTH_SLOWED] = cols;
+	min_row_[NEITHER_SLOWED] = a_hp - 1;
+	min_col_[NEITHER_SLOWED] = b_hp - 1;
+	min_row_[A_SLOWED] = min_row_[B_SLOWED] = min_row_[BOTH_SLOWED] = rows_;
+	min_col_[A_SLOWED] = min_col_[B_SLOWED] = min_col_[BOTH_SLOWED] = cols_;
 
 	// Transfer HP distribution from A?
 	if (!a_summary[0].empty()) {
 		// @todo FIXME: Can optimize here.
-		min_row[NEITHER_SLOWED] = 0;
-		min_row[A_SLOWED] = 0;
-		min_col[A_SLOWED] = b_hp - 1;
+		min_row_[NEITHER_SLOWED] = 0;
+		min_row_[A_SLOWED] = 0;
+		min_col_[A_SLOWED] = b_hp - 1;
 		for (unsigned int row = 0; row < a_summary[0].size(); row++)
 			val(NEITHER_SLOWED, row, b_hp) = a_summary[0][row];
 		if (!a_summary[1].empty()) {
@@ -174,9 +169,9 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 		debug(("A has fought before\n"));
 		dump();
 	} else if (!b_summary[0].empty()) {
-		min_col[NEITHER_SLOWED] = 0;
-		min_col[B_SLOWED] = 0;
-		min_row[B_SLOWED] = a_hp - 1;
+		min_col_[NEITHER_SLOWED] = 0;
+		min_col_[B_SLOWED] = 0;
+		min_row_[B_SLOWED] = a_hp - 1;
 		for (unsigned int col = 0; col < b_summary[0].size(); col++)
 			val(NEITHER_SLOWED, a_hp, col) = b_summary[0][col];
 		if (!b_summary[1].empty()) {
@@ -188,18 +183,18 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 	} else {
 		// If a unit has drain it might end with more HP than before.
 		// Make sure we don't access the matrix in invalid positions.
-		a_hp = minimum<unsigned int>(a_hp, rows - 1);
-		b_hp = minimum<unsigned int>(b_hp, cols - 1);
+		a_hp = std::min<unsigned int>(a_hp, rows_ - 1);
+		b_hp = std::min<unsigned int>(b_hp, cols_ - 1);
 		val(NEITHER_SLOWED, a_hp, b_hp) = 1.0;
 	}
 }
 
 prob_matrix::~prob_matrix()
 {
-	delete[] plane[NEITHER_SLOWED];
-	delete[] plane[A_SLOWED];
-	delete[] plane[B_SLOWED];
-	delete[] plane[BOTH_SLOWED];
+	delete[] plane_[NEITHER_SLOWED];
+	delete[] plane_[A_SLOWED];
+	delete[] plane_[B_SLOWED];
+	delete[] plane_[BOTH_SLOWED];
 }
 
 // Allocate a new probability array, initialized to 0.
@@ -212,16 +207,16 @@ double *prob_matrix::new_arr(unsigned int size)
 
 double &prob_matrix::val(unsigned p, unsigned row, unsigned col)
 {
-	assert(row < rows);
-	assert(col < cols);
-	return plane[p][row * cols + col];
+	assert(row < rows_);
+	assert(col < cols_);
+	return plane_[p][row * cols_ + col];
 }
 
 const double &prob_matrix::val(unsigned p, unsigned row, unsigned col) const
 {
-	assert(row < rows);
-	assert(col < cols);
-	return plane[p][row * cols + col];
+	assert(row < rows_);
+	assert(col < cols_);
+	return plane_[p][row * cols_ + col];
 }
 
 #ifdef CHECK
@@ -232,12 +227,12 @@ void prob_matrix::dump() const
 		= { "NEITHER_SLOWED", "A_SLOWED", "B_SLOWED", "BOTH_SLOWED" };
 
 	for (m = 0; m < 4; m++) {
-		if (!plane[m])
+		if (!plane_[m])
 			continue;
 		debug(("%s:\n", names[m]));
-		for (row = 0; row < rows; row++) {
+		for (row = 0; row < rows_; row++) {
 			debug(("  "));
-			for (col = 0; col < cols; col++)
+			for (col = 0; col < cols_; col++)
 				debug(("%4.3g ", val(m, row, col)*100));
 			debug(("\n"));
 		}
@@ -261,10 +256,10 @@ void prob_matrix::xfer(unsigned dst_plane, unsigned src_plane,
 		src -= diff;
 
 		// This is here for drain.
-		if (col_dst >= cols)
-			col_dst = cols - 1;
-		if (row_dst >= rows)
-			row_dst = rows - 1;
+		if (col_dst >= cols_)
+			col_dst = cols_ - 1;
+		if (row_dst >= rows_)
+			row_dst = rows_ - 1;
 
 		val(dst_plane, row_dst, col_dst) += diff;
 
@@ -288,15 +283,15 @@ void prob_matrix::shift_cols(unsigned dst, unsigned src,
 	unsigned int row, col;
 	unsigned int shift = drain ? 1 : 31; // Avoids a branch.
 
-	if (damage >= cols)
-		damage = cols - 1;
+	if (damage >= cols_)
+		damage = cols_ - 1;
 
 	// Loop backwards so we write drain behind us, for when src == dst.
-	for (row = rows - 1; row > min_row[src]; row--) {
+	for (row = rows_ - 1; row > min_row_[src]; row--) {
 		// These are all going to die (move to col 0).
 		for (col = 1; col <= damage; col++)
 			xfer(dst, src, row+(col>>shift), 0, row, col, prob);
-		for (col = damage+1; col < cols; col++)
+		for (col = damage+1; col < cols_; col++)
 			xfer(dst, src, row+(damage>>shift), col - damage, row, col, prob);
 	}
 }
@@ -307,15 +302,15 @@ void prob_matrix::shift_rows(unsigned dst, unsigned src,
 	unsigned int row, col;
 	unsigned int shift = drain ? 1 : 31; // Avoids a branch.
 
-	if (damage >= rows)
-		damage = rows - 1;
+	if (damage >= rows_)
+		damage = rows_ - 1;
 
 	// Loop downwards so if we drain, we write behind us.
-	for (col = cols - 1; col > min_col[src]; col--) {
+	for (col = cols_ - 1; col > min_col_[src]; col--) {
 		// These are all going to die (move to row 0).
 		for (row = 1; row <= damage; row++)
 			xfer(dst, src, 0, col+(row>>shift), row, col, prob);
-		for (row = damage+1; row < rows; row++)
+		for (row = damage+1; row < rows_; row++)
 			xfer(dst, src, row - damage, col+(damage>>shift), row, col, prob);
 	}
 }
@@ -331,7 +326,7 @@ void prob_matrix::receive_blow_b(unsigned damage, unsigned slow_damage, double h
 	for (src = 3; src >=0; src--) {
 		unsigned int actual_damage;
 
-		if (!plane[src])
+		if (!plane_[src])
 			continue;
 
 		// If A slows us, we go from 0=>2, 1=>3, 2=>2 3=>3.
@@ -347,12 +342,12 @@ void prob_matrix::receive_blow_b(unsigned damage, unsigned slow_damage, double h
 			actual_damage = damage;
 
 		shift_cols(dst, src, actual_damage, hit_chance, a_drains);
-		if (min_col[src] < damage)
-			min_col[dst] = 0;
-		else if (min_col[src] - damage < min_col[dst])
-			min_col[dst] = min_col[src] - damage;
-		if (min_row[src] < min_row[dst])
-			min_row[dst] = min_row[src];
+		if (min_col_[src] < damage)
+			min_col_[dst] = 0;
+		else if (min_col_[src] - damage < min_col_[dst])
+			min_col_[dst] = min_col_[src] - damage;
+		if (min_row_[src] < min_row_[dst])
+			min_row_[dst] = min_row_[src];
 	}
 }
 
@@ -361,17 +356,17 @@ void prob_matrix::remove_stone_distortion_a(unsigned damage, unsigned slow_damag
 											unsigned b_hp)
 {
 	for (int p = 0; p < 4; p++) {
-		if (!plane[p])
+		if (!plane_[p])
 			continue;
 
 		// A is slow in planes 1 and 3.
 		if (p & 1) {
 			if (b_hp > slow_damage)
-				for (unsigned int row = 0; row < rows; row++)
+				for (unsigned int row = 0; row < rows_; row++)
 					xfer(p, p, row, b_hp - slow_damage, row, 0, 1.0);
 		} else {
 			if (b_hp > damage)
-				for (unsigned int row = 0; row < rows; row++)
+				for (unsigned int row = 0; row < rows_; row++)
 					xfer(p, p, row, b_hp - damage, row, 0, 1.0);
 		}
 	}
@@ -381,17 +376,17 @@ void prob_matrix::remove_stone_distortion_b(unsigned damage, unsigned slow_damag
 											unsigned a_hp)
 {
 	for (int p = 0; p < 4; p++) {
-		if (!plane[p])
+		if (!plane_[p])
 			continue;
 
 		// B is slow in planes 2 and 3.
 		if (p & 2) {
 			if (a_hp > slow_damage)
-				for (unsigned int col = 0; col < cols; col++)
+				for (unsigned int col = 0; col < cols_; col++)
 					xfer(p, p, a_hp - slow_damage, col, 0, col, 1.0);
 		} else {
 			if (a_hp > damage)
-				for (unsigned int col = 0; col < cols; col++)
+				for (unsigned int col = 0; col < cols_; col++)
 					xfer(p, p, a_hp - damage, col, 0, col, 1.0);
 		}
 	}
@@ -402,25 +397,25 @@ void prob_matrix::extract_results(std::vector<double> summary_a[2],
 {
 	unsigned int p, row, col;
 
-	summary_a[0] = std::vector<double>(rows);
-	summary_b[0] = std::vector<double>(cols);
+	summary_a[0] = std::vector<double>(rows_);
+	summary_b[0] = std::vector<double>(cols_);
 
-	if (plane[A_SLOWED])
-		summary_a[1] = std::vector<double>(rows);
-	if (plane[B_SLOWED])
-		summary_b[1] = std::vector<double>(cols);
+	if (plane_[A_SLOWED])
+		summary_a[1] = std::vector<double>(rows_);
+	if (plane_[B_SLOWED])
+		summary_b[1] = std::vector<double>(cols_);
 
 	for (p = 0; p < 4; p++) {
 		int dst_a, dst_b;
-		if (!plane[p])
+		if (!plane_[p])
 			continue;
 
 		// A is slow in planes 1 and 3.
 		dst_a = (p & 1);
 		// B is slow in planes 2 and 3.
 		dst_b = !!(p & 2);
-		for (row = 0; row < rows; row++) {
-			for (col = 0; col < cols; col++) {
+		for (row = 0; row < rows_; row++) {
+			for (col = 0; col < cols_; col++) {
 				summary_a[dst_a][row] += val(p, row, col);
 				summary_b[dst_b][col] += val(p, row, col);
 			}
@@ -435,12 +430,12 @@ double prob_matrix::dead_prob() const
 	double prob = 0.0;
 
 	for (p = 0; p < 4; p++) {
-		if (!plane[p])
+		if (!plane_[p])
 			continue;
 		// We might count 0,0 twice, but that is always 0 anyway.
-		for (row = min_row[p]; row < rows; row++)
+		for (row = min_row_[p]; row < rows_; row++)
 			prob += val(p, row, 0);
-		for (col = min_col[p]; col < cols; col++)
+		for (col = min_col_[p]; col < cols_; col++)
 			prob += val(p, 0, col);
 	}
 	return prob;
@@ -457,7 +452,7 @@ void prob_matrix::receive_blow_a(unsigned damage, unsigned slow_damage, double h
 	for (src = 3; src >=0; src--) {
 		unsigned actual_damage;
 
-		if (!plane[src])
+		if (!plane_[src])
 			continue;
 
 		// If B slows us, we go from 0=>1, 1=>1, 2=>3 3=>3.
@@ -473,12 +468,12 @@ void prob_matrix::receive_blow_a(unsigned damage, unsigned slow_damage, double h
 			actual_damage = damage;
 
 		shift_rows(dst, src, actual_damage, hit_chance, b_drains);
-		if (min_row[src] < damage)
-			min_row[dst] = 0;
-		else if (min_row[src] - damage < min_row[dst])
-			min_row[dst] = min_row[src] - damage;
-		if (min_col[src] < min_col[dst])
-			min_col[dst] = min_col[src];
+		if (min_row_[src] < damage)
+			min_row_[dst] = 0;
+		else if (min_row_[src] - damage < min_row_[dst])
+			min_row_[dst] = min_row_[src] - damage;
+		if (min_col_[src] < min_col_[dst])
+			min_col_[dst] = min_col_[src];
 	}
 }
 
@@ -492,14 +487,15 @@ unsigned combatant::hp_dist_size(const battle_context::unit_stats &u, const comb
 	}
 
 	// If this unit drains, HP can increase, so alloc full array.
-	if (u.drains) {
-		return u.max_hp + 1;
-	}
-	return u.hp+1;
+	// Do this anyway in case we level up.
+	return u.max_hp + 1;
 }
 
 combatant::combatant(const battle_context::unit_stats &u, const combatant *prev)
 	: hp_dist(hp_dist_size(u, prev)),
+	  untouched(0.0),
+	  poisoned(0.0),
+	  slowed(0.0),
 	  u_(u),
 	  hit_chances_(u.num_blows, u.chance_to_hit / 100.0)
 {
@@ -573,7 +569,7 @@ unsigned combatant::min_hp() const
 	assert(summary[1].empty());
 
 	unsigned int i;
-	for (i = 0; summary[0][i] == 0; i++);
+	for (i = 0; summary[0][i] == 0; i++) {};
 	return i;
 }
 
@@ -582,7 +578,7 @@ void combatant::no_death_fight(combatant &opp)
 {
 	if (summary[0].empty()) {
 		// Starts with a known HP, so Pascal's triangle.
-		summary[0] = std::vector<double>(u_.hp+1);
+		summary[0] = std::vector<double>(u_.max_hp+1);
 		summary[0][u_.hp] = 1.0;
 		for (unsigned int i = 0; i < opp.hit_chances_.size(); i++) {
 			for (int j = i; j >= 0; j--) {
@@ -604,7 +600,7 @@ void combatant::no_death_fight(combatant &opp)
 
 	if (opp.summary[0].empty()) {
 		// Starts with a known HP, so Pascal's triangle.
-		opp.summary[0] = std::vector<double>(opp.u_.hp+1);
+		opp.summary[0] = std::vector<double>(opp.u_.max_hp+1);
 		opp.summary[0][opp.u_.hp] = 1.0;
 		for (unsigned int i = 0; i < hit_chances_.size(); i++) {
 			for (int j = i; j >= 0; j--) {
@@ -629,10 +625,10 @@ void combatant::no_death_fight(combatant &opp)
 void combatant::one_strike_fight(combatant &opp)
 {
 	if (opp.summary[0].empty()) {
-		opp.summary[0] = std::vector<double>(opp.u_.hp+1);
+		opp.summary[0] = std::vector<double>(opp.u_.max_hp+1);
 		if (hit_chances_.size() == 1) {
 			opp.summary[0][opp.u_.hp] = 1.0 - hit_chances_[0];
-			opp.summary[0][maximum<int>(opp.u_.hp - u_.damage, 0)] = hit_chances_[0];
+			opp.summary[0][std::max<int>(opp.u_.hp - u_.damage, 0)] = hit_chances_[0];
 		} else {
 			assert(hit_chances_.size() == 0);
 			opp.summary[0][opp.u_.hp] = 1.0;
@@ -642,7 +638,7 @@ void combatant::one_strike_fight(combatant &opp)
 			for (unsigned int i = 1; i < opp.summary[0].size(); i++) {
 				double move = opp.summary[0][i] * hit_chances_[0];
 				opp.summary[0][i] -= move;
-				opp.summary[0][maximum<int>(i - u_.damage, 0)] += move;
+				opp.summary[0][std::max<int>(i - u_.damage, 0)] += move;
 			}
 		}
 	}
@@ -650,10 +646,10 @@ void combatant::one_strike_fight(combatant &opp)
 	// If we killed opponent, it won't attack us.
 	double opp_alive_prob = 1.0 - opp.summary[0][0];
 	if (summary[0].empty()) {
-		summary[0] = std::vector<double>(u_.hp+1);
+		summary[0] = std::vector<double>(u_.max_hp+1);
 		if (opp.hit_chances_.size() == 1) {
 			summary[0][u_.hp] = 1.0 - opp.hit_chances_[0] * opp_alive_prob;
-			summary[0][maximum<int>(u_.hp - opp.u_.damage, 0)] = opp.hit_chances_[0] * opp_alive_prob;
+			summary[0][std::max<int>(u_.hp - opp.u_.damage, 0)] = opp.hit_chances_[0] * opp_alive_prob;
 		} else {
 			assert(opp.hit_chances_.size() == 0);
 			summary[0][u_.hp] = 1.0;
@@ -663,7 +659,7 @@ void combatant::one_strike_fight(combatant &opp)
 			for (unsigned int i = 1; i < summary[0].size(); i++) {
 				double move = summary[0][i] * opp.hit_chances_[0] * opp_alive_prob;
 				summary[0][i] -= move;
-				summary[0][maximum<int>(i - opp.u_.damage, 0)] += move;
+				summary[0][std::max<int>(i - opp.u_.damage, 0)] += move;
 			}
 		}
 	}
@@ -675,7 +671,7 @@ void combatant::complex_fight(combatant &opp, unsigned int rounds)
 				  u_.slows && !opp.u_.is_slowed, opp.u_.slows && !u_.is_slowed,
 				  u_.hp, opp.u_.hp, summary, opp.summary);
 
-	unsigned max_attacks = maximum(hit_chances_.size(), opp.hit_chances_.size());
+	unsigned max_attacks = std::max(hit_chances_.size(), opp.hit_chances_.size());
 
 	debug(("A gets %u attacks, B %u\n", hit_chances_.size(), opp.hit_chances_.size()));
 
@@ -683,7 +679,7 @@ void combatant::complex_fight(combatant &opp, unsigned int rounds)
 	unsigned int b_damage = opp.u_.damage, b_slow_damage = opp.u_.slow_damage;
 
 	// To simulate stoning, we set to amount which kills, and re-adjust after.
-	//! @todo FIXME: This doesn't work for rolling calculations, just first battle.
+	/** @todo FIXME: This doesn't work for rolling calculations, just first battle. */
 	if (u_.stones)
 		a_damage = a_slow_damage = opp.u_.max_hp;
 	if (opp.u_.stones)
@@ -718,6 +714,48 @@ void combatant::complex_fight(combatant &opp, unsigned int rounds)
 	m.extract_results(summary, opp.summary);
 }
 
+void combatant::consider_levelup(combatant &opp) {
+	// adjust the probabilities to take into consideration the
+	// probability of us levelling up, and hence healing.
+	// We do not currently estimate how many HP we will have.
+
+	if (u_.experience + opp.u_.level >= u_.max_experience) {
+		// if we survive the combat, we will level up. So the probability
+		// of death is unchanged, but all other cases get merged into
+		// the fully healed case.
+		std::vector<double>::iterator i;
+		i = hp_dist.begin();
+		i++; // skip to the second value
+		for ( ; i != hp_dist.end(); i++) {
+			*i = 0;
+		}
+		// fully healed unless dead
+		hp_dist.back() = 1 - hp_dist.front();
+
+
+
+	} else if (u_.experience + ((opp.u_.level == 0) ? 4 : opp.u_.level * 8)
+						 >= u_.max_experience) {
+		// if we kill, we will level up. So then the damage we had
+		// becomes less probable since it's now conditional on us not
+		// levelling up.  This doesn't apply to the probability of us
+		// dying, of course.
+		float scalefactor =
+			(1 - hp_dist.front() - opp.hp_dist.front()) / (1 - hp_dist.front());
+		std::vector<double>::iterator i;
+		i = hp_dist.begin();
+		i++; // skip to the second value
+		for( ; i != hp_dist.end(); i++) {
+			*i *= scalefactor;
+		}
+
+		// if we level up, we get fully healed. (FIXME: actually more so; we
+		// need to calculate how many HP we will gain)
+		hp_dist.back() += opp.hp_dist.front();
+	}
+
+}
+
 // Two man enter.  One man leave!
 // ... Or maybe two.  But definitely not three.
 // Of course, one could be a woman.  Or both.
@@ -725,7 +763,7 @@ void combatant::complex_fight(combatant &opp, unsigned int rounds)
 // Um, ok, it was a stupid thing to say.
 void combatant::fight(combatant &opp)
 {
-	unsigned int rounds = maximum<unsigned int>(u_.rounds, opp.u_.rounds);
+	unsigned int rounds = std::max<unsigned int>(u_.rounds, opp.u_.rounds);
 
 	// If defender has firststrike and we don't, reverse.
 	if (opp.u_.firststrike && !u_.firststrike) {
@@ -799,11 +837,14 @@ void combatant::fight(combatant &opp)
 			opp.hp_dist[i] = opp.summary[0][i] + opp.summary[1][i];
 	}
 
+	consider_levelup(opp);
+	opp.consider_levelup(*this);
+
 	// Make sure we don't try to access the vectors out of bounds,
 	// drain increases HPs so we determine the number of HP here
 	// and make sure it stays within bounds
-	const unsigned int hp = minimum<unsigned int>(u_.hp, hp_dist.size() - 1);
-	const unsigned int opp_hp = minimum<unsigned int>(opp.u_.hp, opp.hp_dist.size() - 1);
+	const unsigned int hp = std::min<unsigned int>(u_.hp, hp_dist.size() - 1);
+	const unsigned int opp_hp = std::min<unsigned int>(opp.u_.hp, opp.hp_dist.size() - 1);
 
 	// Chance that we / they were touched this time.
 	double touched = untouched - hp_dist[hp];
@@ -818,7 +859,7 @@ void combatant::fight(combatant &opp)
 	if (u_.slows)
 		opp.slowed += (1 - opp.slowed) * opp_touched;
 
-	//! @todo FIXME: This is approximate: we could drain, then get hit.
+	/** @todo FIXME: This is approximate: we could drain, then get hit. */
 	untouched = hp_dist[hp];
 	opp.untouched = opp.hp_dist[opp_hp];
 }
@@ -829,7 +870,7 @@ double combatant::average_hp(unsigned int healing) const
 
 	// Since sum of probabilities is 1.0, we can just tally weights.
 	for (unsigned int i = 1; i < hp_dist.size(); i++) {
-		total += hp_dist[i] * minimum<unsigned int>(i + healing, u_.max_hp);
+		total += hp_dist[i] * std::min<unsigned int>(i + healing, u_.max_hp);
 	}
 	return total;
 }
