@@ -1,6 +1,6 @@
-/* $Id: campaign_server.cpp 28352 2008-08-06 16:46:45Z suokko $ */
+/* $Id: campaign_server.cpp 33338 2009-03-04 12:25:00Z silene $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,38 +12,62 @@
    See the COPYING file for more details.
    */
 
-#include "config.hpp"
+/**
+ * @file campaign_server/campaign_server.cpp
+ * Wesnoth addon server.
+ * Expects a "server.cfg" config file in the current directory
+ * and saves addons under data/.
+ */
+
 #include "filesystem.hpp"
 #include "log.hpp"
-#include "network.hpp"
-#include "publish_campaign.hpp"
-#include "util.hpp"
-#include "scoped_resource.hpp"
-#include "serialization/binary_wml.hpp"
+#include "network_worker.hpp"
 #include "serialization/binary_or_text.hpp"
+#include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
 #include "game_config.hpp"
-
+#include "addon_checks.hpp"
+#include "version.hpp"
 #include "server/input_stream.hpp"
 
-#include "SDL.h"
+#include <csignal>
 
-#include <iostream>
-#include <map>
-#include <algorithm>	// Required for gcc 4.3.0
+#include <boost/iostreams/filter/gzip.hpp>
 
 // the fork execute is unix specific only tested on Linux quite sure it won't
 // work on Windows not sure which other platforms have a problem with it.
 #if !(defined(_WIN32))
 #include <errno.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <sys/wait.h>
-#include <sys/types.h>
 #endif
 
 #define LOG_CS lg::err(lg::network, false)
 
+//compatibility code for MS compilers
+#ifndef SIGHUP
+#define SIGHUP 20
+#endif
+/** @todo FIXME: should define SIGINT here too, but to what? */
+
+static void exit_sighup(int signal) {
+	assert(signal == SIGHUP);
+	LOG_CS << "SIGHUP caught, exiting without cleanup immediately.\n";
+	exit(-1);
+}
+
+static void exit_sigint(int signal) {
+	assert(signal == SIGINT);
+	LOG_CS << "SIGINT caught, exiting without cleanup immediately.\n";
+	exit(0);
+}
+
+static void exit_sigterm(int signal) {
+	assert(signal == SIGTERM);
+	LOG_CS << "SIGTERM caught, exiting without cleanup immediately.\n";
+	exit(-1);
+}
+
+#define DEFAULT_CAMPAIGND_PORT          15003
+
 namespace {
 
 	config construct_message(const std::string& msg)
@@ -56,7 +80,7 @@ namespace {
 	config construct_error(const std::string& msg)
 	{
 		config cfg;
-		cfg.add_child("error")["message"] = msg;
+		cfg.add_child("error")["message"] = "#Error: " + msg;
 		LOG_CS << "ERROR: "<<msg<<"\n";
 		return cfg;
 	}
@@ -101,26 +125,26 @@ namespace {
 		if(script == "") return;
 
 #if (defined(_WIN32))
-		LOG_CS << "ERROR: Tried to execute a script on a not supporting platform\n";
+		LOG_CS << "ERROR: Tried to execute a script on an unsupported platform\n";
 		return;
 #else
 		pid_t childpid;
 
 		if((childpid = fork()) == -1) {
-			LOG_CS << "ERROR fork failed while updating campaign " << addon << "\n";
+			LOG_CS << "ERROR: fork failed while updating campaign " << addon << "\n";
 			return;
 		}
 
 		if(childpid == 0) {
 			/*** We're the child process ***/
 
-			// execute the script, we run is a separate thread and share the 
+			// execute the script, we run is a separate thread and share the
 			// output which will make the logging look ugly.
 			execlp(script.c_str(), script.c_str(), addon.c_str(), (char *)NULL);
 
-			// exec() and family never return if they do we have a problem
-			std::cerr << "ERROR exec failed for addon " << addon 
-				<< " with errno = " << errno << "\n";
+			// exec() and family never return; if they do, we have a problem
+			std::cerr << "ERROR: exec failed with errno " << errno << " for addon " << addon
+			          << '\n';
 			exit(errno);
 
 		} else {
@@ -134,15 +158,31 @@ namespace {
 	{
 		scoped_istream stream = istream_file(file_);
 		read(cfg_, *stream);
-		/** Seems like compression level above 6 is waste of cpu cycle */
-		compress_level_ = lexical_cast_default<int>(cfg_["compress_level"],6);
-		cfg_["compress_level"] = lexical_cast<std::string>(compress_level_);
-		return lexical_cast_default<int>(cfg_["port"], 15005);
+		bool network_use_system_sendfile = utils::string_bool(cfg_["network_use_system_sendfile"],false);
+		network_worker_pool::set_use_system_sendfile(network_use_system_sendfile);
+ 		cfg_["network_use_system_sendfile"] = (network_use_system_sendfile?"yes":"no");
+ 		/** Seems like compression level above 6 is waste of cpu cycle */
+ 		compress_level_ = lexical_cast_default<int>(cfg_["compress_level"],6);
+ 		cfg_["compress_level"] = lexical_cast<std::string>(compress_level_);
+		return lexical_cast_default<int>(cfg_["port"], DEFAULT_CAMPAIGND_PORT);
 	}
 
-	campaign_server::campaign_server(const std::string& cfgfile,size_t min_thread,size_t max_thread)
-		: file_(cfgfile), net_manager_(min_thread,max_thread), server_manager_(load_config()), input_(0)
+	campaign_server::campaign_server(const std::string& cfgfile,
+			size_t min_thread, size_t max_thread) :
+		cfg_(),
+		file_(cfgfile),
+		net_manager_(min_thread,max_thread),
+		server_manager_(load_config()),
+		hooks_(),
+		input_(0),
+		compress_level_(compress_level_) // Already set by load_config()
 	{
+#ifndef _MSC_VER
+		signal(SIGHUP, exit_sighup);
+#endif
+		signal(SIGINT, exit_sigint);
+		signal(SIGTERM, exit_sigterm);
+
 		if(cfg_.child("campaigns") == NULL) {
 			cfg_.add_child("campaigns");
 		}
@@ -241,68 +281,108 @@ namespace {
 		if (dir->find_child("file", "name", "COPYING.TXT")) return;
 
 		// Copy over COPYING.txt
-		std::string contents = read_file("data/COPYING.txt");
+		std::string contents = read_file("COPYING.txt");
 		config &copying = dir->add_child("file");
 		copying["name"] = "COPYING.txt";
 		copying["contents"] = contents;
 
 		if (contents.empty()) {
-			std::cerr << "Could not find " << "data/COPYING.txt" <<
-				", path is \"" << game_config::path << "\"\n";
+			LOG_CS << "Could not find COPYING.txt, path is \""
+				<< game_config::path << "\"\n";
 		}
 	}
-	void campaign_server::convert_binary_to_gzip()
-	{
-		if (cfg_["converted_to_gzipped_data"] != "yes")
+ 	void campaign_server::convert_binary_to_gzip()
+ 	{
+ 		if (cfg_["converted_to_gzipped_data"] != "yes")
+ 		{
+			// Convert all addons to gzip
+ 			config::child_list camps = campaigns().get_children("campaign");
+ 			LOG_CS << "Converting all stored addons to gzip format. Number of addons: " << camps.size() <<"\n";
+
+ 			for (config::child_list::iterator itor = camps.begin();
+ 					itor != camps.end(); ++itor)
+ 			{
+ 				LOG_CS << "Converting " << (**itor)["name"] << "\n";
+ 				scoped_istream binary_stream = istream_file((**itor)["filename"]);
+ 				config data;
+ 				if (binary_stream->peek() == 31) //This is gzip file allready
+ 				{
+ 					LOG_CS << "Already converted\n";
+ 					continue;
+ 				}
+ 				read_compressed(data, *binary_stream);
+
+ 				scoped_ostream gzip_stream = ostream_file((**itor)["filename"]);
+				config_writer writer(*gzip_stream, true, compress_level_);
+ 				writer.write(data);
+ 			}
+
+ 			cfg_["converted_to_gzipped_data"] = "yes";
+ 		}
+		if (cfg_["encoded"] != "yes")
 		{
 			// Convert all addons to gzip
-			config::child_list camps = campaigns().get_children("campaign");
-			LOG_CS << "Converting all stored addons to gzip format. Number of addons: " << camps.size() <<"\n";
+ 			config::child_list camps = campaigns().get_children("campaign");
+ 			LOG_CS << "Encoding all stored campaigns. Number of addons: " << camps.size() <<"\n";
 
-			for (config::child_list::iterator itor = camps.begin();
-					itor != camps.end(); ++itor)
-			{
-				LOG_CS << "Converting " << (**itor)["name"] << "\n";
-				scoped_istream binary_stream = istream_file((**itor)["filename"]);
-				config data;
-				if (binary_stream->peek() == 31) //This is gzip file allready
+ 			for (config::child_list::iterator itor = camps.begin();
+ 					itor != camps.end(); ++itor)
+ 			{
+ 				LOG_CS << "Encoding " << (**itor)["name"] << "\n";
 				{
-					LOG_CS << "All ready converted\n";
-					continue;
-				}
-				read_compressed(data, *binary_stream);
+					scoped_istream in_file = istream_file((**itor)["filename"]);
+					boost::iostreams::filtering_stream<boost::iostreams::input> in_filter;
+					in_filter.push(boost::iostreams::gzip_decompressor());
+					in_filter.push(*in_file);
 
-				scoped_ostream gzip_stream = ostream_file((**itor)["filename"]);
-				config_writer writer(*gzip_stream, true, "",compress_level_);
-				writer.write(data);
+					scoped_ostream out_file = ostream_file((**itor)["filename"] + ".new");
+					boost::iostreams::filtering_stream<boost::iostreams::output> out_filter;
+					out_filter.push(boost::iostreams::gzip_compressor(boost::iostreams::gzip_params(compress_level_)));
+					out_filter.push(*out_file);
+
+					unsigned char c = in_filter.get();
+					while( in_filter.good())
+					{
+						if (needs_escaping(c) && c != '\x01')
+						{
+							out_filter.put('\x01');
+						   	out_filter.put(c+1);
+						} else {
+							out_filter.put(c);
+						}
+						c = in_filter.get();
+					}
+
+					std::remove((**itor)["filename"].c_str());
+					std::rename(((**itor)["filename"]+".new").c_str(),((**itor)["filename"]).c_str());
+
+				}
 			}
-			
-			cfg_["converted_to_gzipped_data"] = "yes";
+
+			cfg_["encoded"] = "yes";
 		}
-	}
+ 	}
 
 	void campaign_server::run()
 	{
-		
-		convert_binary_to_gzip();
-		
-		if (!cfg_["control_socket"].empty())
-			input_ = new input_stream(cfg_["control_socket"]); 
-		
+ 		convert_binary_to_gzip();
+
+ 		if (!cfg_["control_socket"].empty())
+ 			input_ = new input_stream(cfg_["control_socket"]);
 		bool gzipped;
 		network::connection sock = 0;
 		for(int increment = 0; ; ++increment) {
 			try {
-				std::string admin_cmd;
-				if (input_ && input_->read_line(admin_cmd))
-				{
-					// process command
-					if (admin_cmd == "shut_down")
-					{
-						break;
-					}
-				}
-				//write config to disk every ten minutes
+ 				std::string admin_cmd;
+ 				if (input_ && input_->read_line(admin_cmd))
+ 				{
+ 					// process command
+ 					if (admin_cmd == "shut_down")
+ 					{
+ 						break;
+ 					}
+ 				}
+			//write config to disk every ten minutes
 				if((increment%(60*10*50)) == 0) {
 					scoped_ostream cfgfile = ostream_file(file_);
 					write(*cfgfile, cfg_);
@@ -368,59 +448,77 @@ namespace {
 						for(config::child_list::iterator j = cmps.begin(); j != cmps.end(); ++j) {
 							(**j)["passphrase"] = "";
 							(**j)["upload_ip"] = "";
+							(**j)["email"] = "";
 						}
 
 						config response;
 						response.add_child("campaigns",campaign_list);
-
 						std::cerr << " size: " << (network::send_data(response, sock, gzipped)/1024) << "kb\n";
 					} else if(const config* req = data.child("request_campaign")) {
-						LOG_CS << "sending campaign '" << (*req)["name"] << "' to " << network::ip_address(sock) << (gzipped?" using gzip":"");
+						LOG_CS << "sending campaign '" << (*req)["name"] << "' to " << network::ip_address(sock)  << (gzipped?" using gzip":"");
 						config* const campaign = campaigns().find_child("campaign","name",(*req)["name"]);
 						if(campaign == NULL) {
-
 							network::send_data(construct_error("Add-on '" + (*req)["name"] + "'not found."), sock, gzipped);
-							std::cerr << "\n";
 						} else {
-							if (gzipped)
-							{
+ 							if (gzipped)
+ 							{
 								std::cerr << " size: " << (file_size((*campaign)["filename"])/1024) << "kb\n";
-								network::send_file((*campaign)["filename"], sock);
-							} else {
-								scoped_istream stream = istream_file((*campaign)["filename"]);
-								config cfg;
-								read_gz(cfg, *stream);
+ 								network::send_file((*campaign)["filename"], sock);
+ 							} else {
+ 								scoped_istream stream = istream_file((*campaign)["filename"]);
+ 								config cfg;
+ 								read_gz(cfg, *stream);
 								std::cerr << " size: " <<
 									(network::send_data(cfg, sock, false)/1024)
 									<< "kb\n";
 							}
-
 							const int downloads = lexical_cast_default<int>((*campaign)["downloads"],0)+1;
 							(*campaign)["downloads"] = lexical_cast<std::string>(downloads);
-
 						}
 
 					} else if(data.child("request_terms") != NULL) {
 						LOG_CS << "sending terms " << network::ip_address(sock) << "\n";
-
 						network::send_data(construct_message("All add-ons uploaded to this server must be licensed under the terms of the GNU General Public License (GPL). By uploading content to this server, you certify that you have the right to place the content under the conditions of the GPL, and choose to do so."), sock, gzipped);
 						LOG_CS << " Done\n";
 					} else if(config* upload = data.child("upload")) {
 						LOG_CS << "uploading campaign '" << (*upload)["name"] << "' from " << network::ip_address(sock) << ".\n";
 						config* data = upload->child("data");
-						config* campaign = campaigns().find_child("campaign","name",(*upload)["name"]);
+						const std::string& name = (*upload)["name"];
+						std::string lc_name(name.size(), ' ');
+						std::transform(name.begin(), name.end(), lc_name.begin(), tolower);
+						const config::child_list& campaign_list = campaigns().get_children("campaign");
+						config* campaign = NULL;
+						for (config::child_list::const_iterator i = campaign_list.begin(); i != campaign_list.end(); ++i) {
+							if (utils::lowercase((**i)["name"]) == lc_name) {
+								campaign = *i;
+								break;
+							}
+						}
 						if(data == NULL) {
-
-							LOG_CS << "Upload aborted no data.\n";
-							network::send_data(construct_error("No add-on data was supplied."), sock, gzipped);
-						} else if(campaign_name_legal((*upload)["name"]) == false) {
-
-							LOG_CS << "Upload aborted invalid name.\n";
-							network::send_data(construct_error("The name of the add-on is invalid"), sock, gzipped);
+							LOG_CS << "Upload aborted - no add-on data.\n";
+							network::send_data(construct_error("Add-on rejected: No add-on data was supplied."), sock, gzipped);
+						} else if(addon_name_legal((*upload)["name"]) == false) {
+							LOG_CS << "Upload aborted - invalid add-on name.\n";
+							network::send_data(construct_error("Add-on rejected: The name of the add-on is invalid."), sock, gzipped);
+						} else if(((*upload)["title"]).empty()) {
+							LOG_CS << "Upload aborted - no add-on title specified.\n";
+							network::send_data(construct_error("Add-on rejected: You did not specify the title of the add-on in the pbl file!"), sock, gzipped);
+						} else if(((*upload)["type"]).empty()) {
+							LOG_CS << "Upload aborted - no add-on type specified.\n";
+							network::send_data(construct_error("Add-on rejected: You did not specify the type of the add-on in the pbl file!"), sock, gzipped);
+						} else if(((*upload)["author"]).empty()) {
+							LOG_CS << "Upload aborted - no add-on author specified.\n";
+							network::send_data(construct_error("Add-on rejected: You did not specify the author(s) of the add-on in the pbl file!"), sock, gzipped);
+						} else if(((*upload)["version"]).empty()) {
+							LOG_CS << "Upload aborted - no add-on version specified.\n";
+							network::send_data(construct_error("Add-on rejected: You did not specify the version of the add-on in the pbl file!"), sock, gzipped);
+						} else if(((*upload)["description"]).empty()) {
+							LOG_CS << "Upload aborted - no add-on description specified.\n";
+							network::send_data(construct_error("Add-on rejected: You did not specify a description of the add-on in the pbl file!"), sock, gzipped);
 						} else if(check_names_legal(*data) == false) {
-
-							LOG_CS << "Upload aborted invalid file name.\n";
-							network::send_data(construct_error("The add-on contains an illegal file or directory name."), sock, gzipped);
+							LOG_CS << "Upload aborted - invalid file names in add-on data.\n";
+							network::send_data(construct_error("Add-on rejected: The add-on contains an illegal file or directory name."
+									" File or directory names may not contain any of the following characters: '/ \\ : ~'"), sock, gzipped);
 						} else if(campaign != NULL && (*campaign)["passphrase"] != (*upload)["passphrase"]) {
 							// the user password failed, now test for the master password, in master password
 							// mode the upload behaves different since it's only intended to update translations.
@@ -431,6 +529,10 @@ namespace {
 
 								std::string message = "Add-on accepted.";
 
+								if (!version_info((*upload)["version"]).good()) {
+									message += "\n#Note: The version you specified is invalid. This addon will be ignored for automatic update checks.";
+								}
+
 								std::string filename = (*campaign)["filename"];
 								(*data)["title"] = (*campaign)["title"];
 								(*data)["name"] = "";
@@ -441,13 +543,14 @@ namespace {
 								(*data)["timestamp"] = (*campaign)["timestamp"];
 								(*data)["icon"] = (*campaign)["icon"];
 								(*data)["translate"] = (*campaign)["translate"];
+								(*data)["type"] = (*campaign)["type"];
 								(*campaign).clear_children("translation");
 								find_translations(*data, *campaign);
 
 								add_license(*data);
 								{
 									scoped_ostream campaign_file = ostream_file(filename);
-									config_writer writer(*campaign_file, true, "",compress_level_);
+									config_writer writer(*campaign_file, true, compress_level_);
 									writer.write(*data);
 								}
 //								write_compressed(*campaign_file, *data);
@@ -456,24 +559,27 @@ namespace {
 										file_size(filename));
 								scoped_ostream cfgfile = ostream_file(file_);
 								write(*cfgfile, cfg_);
-
 								network::send_data(construct_message(message), sock, gzipped);
 
 							} else {
-
-								LOG_CS << "Upload aborted invalid passphrase.\n";
-								network::send_data(construct_error("The add-on already exists, and your passphrase was incorrect."), sock, gzipped);
+								LOG_CS << "Upload aborted - incorrect passphrase.\n";
+								network::send_data(construct_error("Add-on rejected: The add-on already exists, and your passphrase was incorrect."), sock, gzipped);
 							}
 						} else {
 							LOG_CS << "Upload is owner upload.\n";
 							std::string message = "Add-on accepted.";
+
+							if (!version_info((*upload)["version"]).good()) {
+								message += "\n<255,255,0>Note: The version you specified is invalid. This add-on will be ignored for automatic update checks.";
+							}
+
 							if(campaign == NULL) {
 								campaign = &campaigns().add_child("campaign");
 							}
 
 							(*campaign)["title"] = (*upload)["title"];
 							(*campaign)["name"] = (*upload)["name"];
-							(*campaign)["filename"] = (*upload)["name"];
+							(*campaign)["filename"] = "data/" + (*upload)["name"];
 							(*campaign)["passphrase"] = (*upload)["passphrase"];
 							(*campaign)["author"] = (*upload)["author"];
 							(*campaign)["description"] = (*upload)["description"];
@@ -482,6 +588,8 @@ namespace {
 							(*campaign)["translate"] = (*upload)["translate"];
 							(*campaign)["dependencies"] = (*upload)["dependencies"];
 							(*campaign)["upload_ip"] = network::ip_address(sock);
+							(*campaign)["type"] = (*upload)["type"];
+							(*campaign)["email"] = (*upload)["email"];
 
 							if((*campaign)["downloads"].empty()) {
 								(*campaign)["downloads"] = "0";
@@ -500,6 +608,7 @@ namespace {
 							(*data)["version"] = (*campaign)["version"];
 							(*data)["timestamp"] = (*campaign)["timestamp"];
 							(*data)["icon"] = (*campaign)["icon"];
+							(*data)["type"] = (*campaign)["type"];
 							(*campaign).clear_children("translation");
 							find_translations(*data, *campaign);
 
@@ -512,7 +621,7 @@ namespace {
 
 							{
 								scoped_ostream campaign_file = ostream_file(filename);
-								config_writer writer(*campaign_file, true, "",compress_level_);
+								config_writer writer(*campaign_file, true, compress_level_);
 								writer.write(*data);
 							}
 //							write_compressed(*campaign_file, *data);
@@ -521,7 +630,6 @@ namespace {
 									file_size(filename));
 							scoped_ostream cfgfile = ostream_file(file_);
 							write(*cfgfile, cfg_);
-
 							network::send_data(construct_message(message), sock, gzipped);
 
 							fire("hook_post_upload", (*upload)["name"]);
@@ -530,16 +638,14 @@ namespace {
 						LOG_CS << "deleting campaign '" << (*erase)["name"] << "' requested from " << network::ip_address(sock) << "\n";
 						config* const campaign = campaigns().find_child("campaign","name",(*erase)["name"]);
 						if(campaign == NULL) {
-
 							network::send_data(construct_error("The add-on does not exist."), sock, gzipped);
 							continue;
 						}
 
-						if((*campaign)["passphrase"] != (*erase)["passphrase"] 
+						if((*campaign)["passphrase"] != (*erase)["passphrase"]
 								&& (campaigns()["master_password"] == ""
 								|| campaigns()["master_password"] != (*erase)["passphrase"])) {
 
-
 							network::send_data(construct_error("The passphrase is incorrect."), sock, gzipped);
 							continue;
 						}
@@ -553,7 +659,6 @@ namespace {
 						campaigns().remove_child("campaign",index);
 						scoped_ostream cfgfile = ostream_file(file_);
 						write(*cfgfile, cfg_);
-
 						network::send_data(construct_message("Add-on deleted."), sock, gzipped);
 
 						fire("hook_post_erase", (*erase)["name"]);
@@ -561,33 +666,26 @@ namespace {
 					} else if(const config* cpass = data.child("change_passphrase")) {
 						config* campaign = campaigns().find_child("campaign","name",(*cpass)["name"]);
 						if(campaign == NULL) {
-
 							network::send_data(construct_error("No add-on with that name exists."), sock, gzipped);
 						} else if((*campaign)["passphrase"] != (*cpass)["passphrase"]) {
-
 							network::send_data(construct_error("Your old passphrase was incorrect."), sock, gzipped);
 						} else if((const t_string)(*cpass)["new_passphrase"] == "") {
-
 							network::send_data(construct_error("No new passphrase was supplied."), sock, gzipped);
 						} else {
 							(*campaign)["passphrase"] = (*cpass)["new_passphrase"];
 							scoped_ostream cfgfile = ostream_file(file_);
 							write(*cfgfile, cfg_);
-
 							network::send_data(construct_message("Passphrase changed."), sock, gzipped);
 						}
 					} else if(const config* cvalidate = data.child("validate_scripts")) {
 						config* campaign = campaigns().find_child("campaign","name",(*cvalidate)["name"]);
 						if(campaign == NULL) {
-
 							network::send_data(construct_error(
 										"No add-on with that name exists."), sock, gzipped);
 						} else if(campaigns()["master_password"] == "") {
-
 							network::send_data(construct_error(
 										"Sever does not allow scripts."), sock, gzipped);
 						} else if (campaigns()["master_password"] != (*cvalidate)["master_password"]) {
-
 							network::send_data(construct_error(
 										"Password was incorrect."), sock, gzipped);
 						} else {
@@ -600,18 +698,16 @@ namespace {
 								// Write the campaign with changed filenames back to disk
 								{
 									scoped_ostream ostream = ostream_file((*campaign)["filename"]);
-									config_writer writer(*ostream, true, "",compress_level_);
+									config_writer writer(*ostream, true, compress_level_);
 									writer.write(campaign_file);
 								}
 //								write_compressed(*ostream, campaign_file);
 								(*campaign)["size"] = lexical_cast<std::string>(
 										file_size((*campaign)["filename"]));
 
-
 								network::send_data(construct_message("The following scripts have been validated: " +
 											scripts), sock, gzipped);
 							} else {
-
 								network::send_data(construct_message("No unchecked scripts found!"), sock, gzipped);
 							}
 						}
@@ -619,10 +715,10 @@ namespace {
 				}
 			} catch(network::error& e) {
 				if(!e.socket) {
-					LOG_CS << "fatal network error: " << e.message <<"\n";
+					LOG_CS << "fatal network error: " << e.message << "\n";
 					throw;
 				} else {
-					LOG_CS <<"client disconnect : "<<e.message<<" " << network::ip_address(e.socket) << "\n";
+					LOG_CS << "client disconnect: " << e.message << " " << network::ip_address(e.socket) << "\n";
 					e.disconnect();
 				}
 			} catch(config::error& /*e*/) {
@@ -638,13 +734,15 @@ namespace {
 
 int main(int argc, char**argv)
 {
+	game_config::path = get_cwd();
 	lg::timestamps(true);
 	try {
 		printf("argc %d argv[0] %s 1 %s\n",argc,argv[0],argv[1]);
+		std::string cfg_path = normalize_path("server.cfg");
 		if(argc >= 2 && atoi(argv[1])){
-			campaign_server("server.cfg",atoi(argv[1])).run();
+			campaign_server(cfg_path, atoi(argv[1])).run();
 		}else {
-			campaign_server("server.cfg").run();
+			campaign_server(cfg_path).run();
 		}
 	} catch(config::error& /*e*/) {
 		std::cerr << "Could not parse config file\n";
