@@ -1,6 +1,6 @@
-/* $Id: cavegen.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: cavegen.cpp 32318 2009-01-26 17:50:26Z jhinrichs $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,26 +12,39 @@
    See the COPYING file for more details.
 */
 
-//! @file cavegen.cpp 
-//! Map-generator for caves.
+/**
+ * @file cavegen.cpp
+ * Map-generator for caves.
+ */
 
 #include "global.hpp"
 
 #include "cavegen.hpp"
 #include "log.hpp"
+#include "map.hpp"
 #include "pathfind.hpp"
-#include "util.hpp"
-#include "serialization/string_utils.hpp"
 
-#include <cassert>
 
 #define LOG_NG LOG_STREAM(info, engine)
 
-cave_map_generator::cave_map_generator(const config* cfg) : wall_(t_translation::CAVE_WALL),
-	clear_(t_translation::CAVE), village_(t_translation::UNDERGROUND_VILLAGE),
-    castle_(t_translation::DWARVEN_CASTLE), keep_(t_translation::DWARVEN_KEEP),
-	cfg_(cfg), width_(50), height_(50),
-	village_density_(0), flipx_(false), flipy_(false)
+cave_map_generator::cave_map_generator(const config* cfg) :
+	wall_(t_translation::CAVE_WALL),
+	clear_(t_translation::CAVE),
+	village_(t_translation::UNDERGROUND_VILLAGE),
+	castle_(t_translation::DWARVEN_CASTLE),
+	keep_(t_translation::DWARVEN_KEEP),
+	map_(),
+	starting_positions_(),
+	chamber_ids_(),
+	chambers_(),
+	passages_(),
+	res_(),
+	cfg_(cfg),
+	width_(50),
+	height_(50),
+	village_density_(0),
+	flipx_(false),
+	flipy_(false)
 {
 	if(cfg_ == NULL) {
 		static const config default_cfg;
@@ -55,6 +68,11 @@ cave_map_generator::cave_map_generator(const config* cfg) : wall_(t_translation:
 
 }
 
+std::string cave_map_generator::config_name() const
+{
+	return "";
+}
+
 size_t cave_map_generator::translate_x(size_t x) const
 {
 	if(flipx_) {
@@ -107,7 +125,7 @@ config cave_map_generator::create_scenario(const std::vector<std::string>& /*arg
 
 	LOG_NG << "outputting map....\n";
 
-	res_["map_data"] = gamemap::default_map_header + 
+	res_["map_data"] = gamemap::default_map_header +
 		t_translation::write_game_map(map_, starting_positions_);
 
 	LOG_NG << "returning result...\n";
@@ -115,14 +133,14 @@ config cave_map_generator::create_scenario(const std::vector<std::string>& /*arg
 	return res_;
 }
 
-void cave_map_generator::build_chamber(gamemap::location loc, std::set<gamemap::location>& locs, size_t size, size_t jagged)
+void cave_map_generator::build_chamber(map_location loc, std::set<map_location>& locs, size_t size, size_t jagged)
 {
 	if(size == 0 || locs.count(loc) != 0 || !on_board(loc))
 		return;
 
 	locs.insert(loc);
 
-	gamemap::location adj[6];
+	map_location adj[6];
 	get_adjacent_tiles(loc,adj);
 	for(size_t n = 0; n != 6; ++n) {
 		if((rand() % 100) < (100l - static_cast<long>(jagged))) {
@@ -178,7 +196,7 @@ void cave_map_generator::generate_chambers()
 		}
 
 		chamber new_chamber;
-		new_chamber.center = gamemap::location(x,y);
+		new_chamber.center = map_location(x,y);
 		build_chamber(new_chamber.center,new_chamber.locs,chamber_size,jagged_edges);
 
 		new_chamber.items = (**i).child("items");
@@ -208,7 +226,7 @@ void cave_map_generator::generate_chambers()
 
 void cave_map_generator::place_chamber(const chamber& c)
 {
-	for(std::set<gamemap::location>::const_iterator i = c.locs.begin(); i != c.locs.end(); ++i) {
+	for(std::set<map_location>::const_iterator i = c.locs.begin(); i != c.locs.end(); ++i) {
 		set_terrain(*i,clear_);
 	}
 
@@ -239,7 +257,7 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 		}
 		const std::string loc_var = cfg["store_location_as"];
 
-		std::set<gamemap::location>::const_iterator loc = c.locs.begin();
+		std::set<map_location>::const_iterator loc = c.locs.begin();
 		std::advance(loc,index);
 
 		char xbuf[50];
@@ -293,7 +311,7 @@ struct passage_path_calculator : cost_calculator
 		map_(mapdata), wall_(wall), laziness_(laziness), windiness_(windiness)
 	{}
 
-	virtual double cost(const gamemap::location& src,const gamemap::location& loc, const double so_far, const bool is_dest) const;
+	virtual double cost(const map_location& src,const map_location& loc, const double so_far) const;
 private:
 	const t_translation::t_map& map_;
 	t_translation::t_terrain wall_;
@@ -301,7 +319,7 @@ private:
 	size_t windiness_;
 };
 
-double passage_path_calculator::cost(const gamemap::location& /*src*/,const gamemap::location& loc, const double, const bool) const
+double passage_path_calculator::cost(const map_location& /*src*/,const map_location& loc, const double) const
 {
 	assert(loc.x >= 0 && loc.y >= 0 && size_t(loc.x) < map_.size() &&
 	        !map_.empty() && size_t(loc.y) < map_.front().size());
@@ -327,26 +345,26 @@ void cave_map_generator::place_passage(const passage& p)
 
 
 	const size_t windiness = atoi(p.cfg["windiness"].c_str());
-	const double laziness = maximum<double>(1.0,atof(p.cfg["laziness"].c_str()));
+	const double laziness = std::max<double>(1.0,atof(p.cfg["laziness"].c_str()));
 
 	passage_path_calculator calc(map_,wall_,laziness,windiness);
 
 	const paths::route rt = a_star_search(p.src, p.dst, 10000.0, &calc, width_, height_);
 
-	const size_t width = maximum<size_t>(1,atoi(p.cfg["width"].c_str()));
+	const size_t width = std::max<size_t>(1,atoi(p.cfg["width"].c_str()));
 
 	const size_t jagged = atoi(p.cfg["jagged"].c_str());
 
-	for(std::vector<gamemap::location>::const_iterator i = rt.steps.begin(); i != rt.steps.end(); ++i) {
-		std::set<gamemap::location> locs;
+	for(std::vector<map_location>::const_iterator i = rt.steps.begin(); i != rt.steps.end(); ++i) {
+		std::set<map_location> locs;
 		build_chamber(*i,locs,width,jagged);
-		for(std::set<gamemap::location>::const_iterator j = locs.begin(); j != locs.end(); ++j) {
+		for(std::set<map_location>::const_iterator j = locs.begin(); j != locs.end(); ++j) {
 			set_terrain(*j,clear_);
 		}
 	}
 }
 
-void cave_map_generator::set_terrain(gamemap::location loc, t_translation::t_terrain t)
+void cave_map_generator::set_terrain(map_location loc, t_translation::t_terrain t)
 {
 	if(on_board(loc)) {
 		if(t == clear_ &&
@@ -362,7 +380,7 @@ void cave_map_generator::set_terrain(gamemap::location loc, t_translation::t_ter
 	}
 }
 
-void cave_map_generator::place_castle(const std::string& side, gamemap::location loc)
+void cave_map_generator::place_castle(const std::string& side, map_location loc)
 {
 	const int starting_position = lexical_cast_default<int>(side, -1);
 	if(starting_position != -1) {
@@ -372,7 +390,7 @@ void cave_map_generator::place_castle(const std::string& side, gamemap::location
 		starting_positions_[starting_position] = coord;
 	}
 
-	gamemap::location adj[6];
+	map_location adj[6];
 	get_adjacent_tiles(loc,adj);
 	for(size_t n = 0; n != 6; ++n) {
 		set_terrain(adj[n],castle_);
