@@ -1,7 +1,7 @@
-/* $Id: config.cpp 26448 2008-05-07 17:35:33Z mordante $ */
+/* $Id: config.cpp 33687 2009-03-15 16:50:42Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,19 +13,18 @@
    See the COPYING file for more details.
 */
 
-//! @file config.cpp
-//! Routines related to configuration-files / WML.
+/**
+ * @file config.cpp
+ * Routines related to configuration-files / WML.
+ */
 
 #include "global.hpp"
 
-#include <algorithm>
-#include <sstream>
-#include <string.h>
-#include <ostream>
 #include "config.hpp"
-#include "gettext.hpp"
 #include "log.hpp"
-#include "util.hpp"
+
+#include <cstring>
+#include <deque>
 
 #define ERR_CF LOG_STREAM(err, config)
 
@@ -111,6 +110,15 @@ config::const_child_itors config::child_range(const std::string& key) const
 	}
 }
 
+size_t config::child_count(const std::string& key) const
+{
+	child_map::const_iterator i = children.find(key);
+	if(i != children.end()) {
+		return i->second.size();
+	}
+	return 0;
+}
+
 const config::child_list& config::get_children(const std::string& key) const
 {
 	const child_map::const_iterator i = children.find(key);
@@ -348,15 +356,69 @@ const config* config::find_child(const std::string& key,
 		return NULL;
 }
 
+namespace {
+	/**
+	 * Helper struct for iterative config clearing.
+	 */
+	struct config_clear_state
+	{
+		config_clear_state()
+			: c(NULL)
+			, mi()
+			, vi(0)
+		{
+		}
+
+		config* c; //the config being inspected
+		config::child_map::iterator mi; //current child map entry
+		size_t vi; //index into the child map item vector
+	};
+}
+
 void config::clear()
 {
-	for(std::map<std::string,std::vector<config*> >::iterator i = children.begin(); i != children.end(); ++i) {
-		std::vector<config*>& v = i->second;
-		for(std::vector<config*>::iterator j = v.begin(); j != v.end(); ++j)
-			delete *j;
+	if (!children.empty()) {
+		//start with this node, the first entry in the child map,
+		//zeroeth element in that entry
+		config_clear_state init;
+		init.c = this;
+		init.mi = children.begin();
+		init.vi = 0;
+		std::deque<config_clear_state> l;
+		l.push_back(init);
+
+		while (!l.empty()) {
+			config_clear_state& state = l.back();
+			if (state.mi != state.c->children.end()) {
+				std::vector<config*>& v = state.mi->second;
+				if (state.vi < v.size()) {
+					config* c = v[state.vi];
+					++state.vi;
+					if (c->children.empty()) {
+						delete c; //special case for a slight speed increase?
+					} else {
+						//descend to the next level
+						config_clear_state next;
+						next.c = c;
+						next.mi = c->children.begin();
+						next.vi = 0;
+						l.push_back(next);
+					}
+				} else {
+					state.vi = 0;
+					++state.mi;
+				}
+			} else {
+				//reached end of child map for this element - all child nodes
+				//have beed deleted, so it's safe to clear the map, delete the
+				//node and move up one level
+				state.c->children.clear();
+				if (state.c != this) delete state.c;
+				l.pop_back();
+			}
+		}
 	}
 
-	children.clear();
 	values.clear();
 	ordered_children.clear();
 }
@@ -441,7 +503,7 @@ void config::get_diff(const config& c, config& res) const
 	string_map::const_iterator i;
 	for(i = values.begin(); i != values.end(); ++i) {
 		const string_map::const_iterator j = c.values.find(i->first);
-		if(j == c.values.end() || i->second != j->second && i->second != "") {
+		if(j == c.values.end() || (i->second != j->second && i->second != "")) {
 			if(inserts == NULL) {
 				inserts = &res.add_child("insert");
 			}
@@ -627,6 +689,26 @@ void config::merge_with(const config& c)
 	}
 }
 
+void config::merge_and_keep(const config& c)
+{
+	// Merge not existing attributes first
+	string_map::const_iterator attrib_it, attrib_end = c.values.end();
+	for(attrib_it = c.values.begin(); attrib_it != attrib_end; ++attrib_it) {
+	    if (values[attrib_it->first] == "")
+            values[attrib_it->first] = attrib_it->second;
+	}
+
+	// Now merge unshared tags
+	for(child_map::const_iterator i = c.children.begin(); i != c.children.end(); ++i) {
+		const std::string& tag = i->first;
+		child_map::const_iterator j = children.find(tag);
+		if (j == children.end()) {
+		    for (size_t count=0; count < i->second.size(); count++)
+                add_child(tag, *i->second[count]);
+		}
+	}
+}
+
 bool config::matches(const config &filter) const
 {
 	// First match values. all values should match.
