@@ -1,6 +1,6 @@
-/* $Id: config_adapter.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: config_adapter.cpp 33602 2009-03-13 14:13:30Z silene $ */
 /*
-   Copyright (C) 2005 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2005 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -13,20 +13,20 @@
    See the COPYING file for more details.
 */
 
-//! @file config_adapter.cpp
-//! Construct objects like 'team' or 'unit' out of WML-based config-infos.
+/**
+ * @file config_adapter.cpp
+ * Construct objects like 'team' or 'unit' out of WML-based config-infos.
+ */
 
 #include "global.hpp"
-
-#include <sstream>
 #include "config_adapter.hpp"
-#include "game_errors.hpp"
+
 #include "game_preferences.hpp"
 #include "gamestatus.hpp"
-#include "gettext.hpp"
 #include "log.hpp"
-#include "team.hpp"
+#include "map.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
 
 #define LOG_NG LOG_STREAM(info, engine)
 #define ERR_NG LOG_STREAM(err, engine)
@@ -36,7 +36,7 @@ std::string get_unique_saveid(const config& cfg, std::set<std::string>& seen_sav
 	std::string save_id = cfg["save_id"];
 
 	if(save_id.empty()) {
-		save_id=cfg["description"];
+		save_id=cfg["id"];
 	}
 
 	if(save_id.empty()) {
@@ -53,15 +53,19 @@ std::string get_unique_saveid(const config& cfg, std::set<std::string>& seen_sav
 
 void get_player_info(const config& cfg, game_state& gamestate,
 					 std::string save_id, std::vector<team>& teams,
-					 const config& level, const game_data& gameinfo,
-					 gamemap& map, unit_map& units,
-					 gamestatus& game_status, bool snapshot, bool replay)
+					 const config& level, gamemap& map, unit_map& units,
+					 gamestatus& game_status, bool snapshot)
 {
 	player_info *player = NULL;
 
+	if(map.empty()) {
+		throw game::load_game_failed("Map not found");
+	}
+
 	if(cfg["controller"] == "human" ||
 	   cfg["controller"] == "network" ||
-	   cfg["persistent"] == "1") {
+	   cfg["controller"] == "network_ai" ||
+	   cfg["controller"] == "human_ai") {
 		player = gamestate.get_player(save_id);
 
 		if(player == NULL && !save_id.empty()) {
@@ -78,12 +82,12 @@ void get_player_info(const config& cfg, game_state& gamestate,
 	LOG_NG << "found gold: '" << gold << "'\n";
 
 	int ngold = lexical_cast_default<int>(gold);
-	
+
 	/* This is the gold carry-over mechanism for subsequent campaign
        scenarios. Snapshots and replays are loaded from savegames and
        got their own gold information, which must not be altered here
     */
-	if ( (player != NULL)  && (!snapshot) && (!replay) ) {
+	if ( (player != NULL)  && (!snapshot) ) {
 		if(player->gold_add) {
 			ngold +=  player->gold;
 		} else if(player->gold >= ngold) {
@@ -93,7 +97,7 @@ void get_player_info(const config& cfg, game_state& gamestate,
 
 	LOG_NG << "set gold to '" << ngold << "'\n";
 
-	team temp_team(cfg, ngold);
+	team temp_team(cfg, map, ngold);
 	teams.push_back(temp_team);
 
 	// Update/fix the recall list for this side,
@@ -114,7 +118,7 @@ void get_player_info(const config& cfg, game_state& gamestate,
 
 	// If this side tag describes the leader of the side
 	if(!utils::string_bool(cfg["no_leader"]) && cfg["controller"] != "null") {
-		unit new_unit(&gameinfo, &units, &map, &game_status, &teams, cfg, true);
+		unit new_unit(&units, &map, &game_status, &teams, cfg, true);
 
 		// Search the recall list for leader units, and if there is one,
 		// use it in place of the config-described unit
@@ -123,7 +127,7 @@ void get_player_info(const config& cfg, game_state& gamestate,
 				it != player->available_units.end(); ++it) {
 				if(it->can_recruit()) {
 					new_unit = *it;
-					new_unit.set_game_context(&gameinfo, &units, &map, &game_status, &teams);
+					new_unit.set_game_context(&units, &map, &game_status, &teams);
 					player->available_units.erase(it);
 					break;
 				}
@@ -132,11 +136,8 @@ void get_player_info(const config& cfg, game_state& gamestate,
 
 		// See if the side specifies its location.
 		// Otherwise start it at the map-given starting position.
-		gamemap::location start_pos(cfg, &gamestate);
+		map_location start_pos(cfg, &gamestate);
 
-		if(map.empty()) {
-			throw game::load_game_failed("Map not found");
-		}
 		if(cfg["x"].empty() && cfg["y"].empty()) {
 			start_pos = map.starting_position(side);
 		}
@@ -152,11 +153,10 @@ void get_player_info(const config& cfg, game_state& gamestate,
 
 		utils::string_map symbols;
 		symbols["side"] = lexical_cast<std::string>(side);
-		VALIDATE(units.count(start_pos) == 0, 
+		VALIDATE(units.count(start_pos) == 0,
 			 t_string(vgettext("Duplicate side definition for side '$side|' found.", symbols)));
 
-		new_unit.new_turn();
-		units.add(new std::pair<gamemap::location,unit>(map.starting_position(new_unit.side()), new_unit));
+		units.add(map.starting_position(new_unit.side()), new_unit);
 		LOG_NG << "initializing side '" << cfg["side"] << "' at "
 			<< start_pos << '\n';
 	}
@@ -164,8 +164,7 @@ void get_player_info(const config& cfg, game_state& gamestate,
 	// If the game state specifies units that
 	// can be recruited for the player, add them.
 	if(player != NULL && player->can_recruit.empty() == false) {
-		std::copy(player->can_recruit.begin(),player->can_recruit.end(),
-				std::inserter(teams.back().recruits(),teams.back().recruits().end()));
+		teams.back().add_recruits(player->can_recruit);
 	}
 
 	if(player != NULL) {
@@ -184,14 +183,14 @@ void get_player_info(const config& cfg, game_state& gamestate,
 		player->available_units.clear();
 	}
 	for(config::child_list::const_iterator su = starting_units.begin(); su != starting_units.end(); ++su) {
-		unit new_unit(&gameinfo, &units, &map, &game_status,&teams,**su,true);
+		unit new_unit(&units, &map, &game_status,&teams,**su,true);
 
 		new_unit.set_side(side);
 
 		const std::string& x = (**su)["x"];
 		const std::string& y = (**su)["y"];
 
-		gamemap::location loc(**su, &gamestate);
+		map_location loc(**su, &gamestate);
 		if(x.empty() && y.empty()) {
 			if(player) {
 				player->available_units.push_back(new_unit);
@@ -213,7 +212,7 @@ void get_player_info(const config& cfg, game_state& gamestate,
 			if (units.find(loc) != units.end()) {
 				ERR_NG << "[unit] trying to overwrite existing unit at " << loc << "\n";
 			} else {
-				units.add(new std::pair<gamemap::location,unit>(loc,new_unit));
+				units.add(loc, new_unit);
 				LOG_NG << "inserting unit for side " << new_unit.side() << "\n";
 			}
 		}
@@ -223,7 +222,7 @@ void get_player_info(const config& cfg, game_state& gamestate,
 int get_first_human_team(const config::child_list::const_iterator& cfg, const config::child_list& unit_cfg){
 	int result = -1;
 	const std::string& controller = (**cfg)["controller"];
-	if (controller == preferences::client_type() && (**cfg)["description"] == preferences::login()) {
+	if (controller == preferences::client_type() && (**cfg)["id"] == preferences::login()) {
 		result = cfg - unit_cfg.begin();
 	} else if((**cfg)["controller"] == "human") {
 		result = cfg - unit_cfg.begin();
@@ -231,7 +230,6 @@ int get_first_human_team(const config::child_list::const_iterator& cfg, const co
 	return result;
 }
 
-//! Return NULL if theme is not found.
 const config* get_theme(const config& game_config, std::string theme_name){
 	const config* theme_cfg = NULL;
 	if(theme_name != "") {
@@ -242,6 +240,10 @@ const config* get_theme(const config& game_config, std::string theme_name){
 		theme_cfg = game_config.find_child("theme","name",preferences::theme());
 	}
 
+	if(theme_cfg == NULL) {
+		theme_cfg = game_config.find_child("theme","name","default");
+	}
+
 	return theme_cfg;
 }
 
