@@ -1,6 +1,6 @@
-/* $Id: construct_dialog.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: construct_dialog.cpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2006 - 2008 by Patrick Parker <patrick_x99@hotmail.com>
+   Copyright (C) 2006 - 2009 by Patrick Parker <patrick_x99@hotmail.com>
    wesnoth widget Copyright (C) 2003-5 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -18,31 +18,13 @@
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "construct_dialog.hpp"
-#include "config.hpp"
-#include "cursor.hpp"
 #include "display.hpp"
-#include "events.hpp"
 #include "gettext.hpp"
-#include "image.hpp"
-#include "key.hpp"
 #include "sound.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
-#include "thread.hpp"
-#include "language.hpp"
-#include "sdl_utils.hpp"
-#include "tooltips.hpp"
-#include "util.hpp"
-#include "video.hpp"
-#include "widgets/button.hpp"
-#include "widgets/menu.hpp"
-#include "widgets/progressbar.hpp"
-#include "widgets/textbox.hpp"
 
-#include "sdl_ttf/SDL_ttf.h"
 
-#include <iostream>
-#include <numeric>
 
 #define ERR_DP LOG_STREAM(err, display)
 #define LOG_DP LOG_STREAM(info, display)
@@ -87,10 +69,10 @@ dialog_textbox::~dialog_textbox()
 }
 
 dialog::dimension_measurements::dimension_measurements() :
-	x(-1), 
-	y(-1), 
+	x(-1),
+	y(-1),
 	interior(empty_rect),
-	message(empty_rect), 
+	message(empty_rect),
 	textbox(empty_rect),
 	menu_width(0),
 	panes(),
@@ -111,8 +93,8 @@ dialog::dimension_measurements::dimension_measurements() :
 }
 
 dialog::dialog(display &disp, const std::string& title, const std::string& message,
-		const DIALOG_TYPE type, const style& dialog_style) : 
-	disp_(disp), 
+		const DIALOG_TYPE type, const style& dialog_style) :
+	disp_(disp),
 	image_(NULL),
 	title_(title),
 	style_(dialog_style),
@@ -191,7 +173,7 @@ dialog::~dialog()
 //	}
 }
 
-const bool dialog::option_checked(unsigned int option_index)
+bool dialog::option_checked(unsigned int option_index)
 {
 	unsigned int i = 0;
 	button_pool_iterator b;
@@ -443,8 +425,8 @@ dialog::dimension_measurements dialog::layout(int xloc, int yloc)
 	int text_widget_height = 0;
 	if(use_textbox) {
 		const SDL_Rect& area = font::text_area(text_widget_->text(),message_font_size);
-		dim.textbox.w = minimum<size_t>(screen.getx()/2,maximum<size_t>(area.w,text_widget_->width()));
-		dim.textbox.h = minimum<size_t>(screen.gety()/2,maximum<size_t>(area.h,text_widget_->height()));
+		dim.textbox.w = std::min<size_t>(screen.getx()/2,std::max<size_t>(area.w,text_widget_->width()));
+		dim.textbox.h = std::min<size_t>(screen.gety()/2,std::max<size_t>(area.h,text_widget_->height()));
 		text_widget_width = dim.textbox.w;
 		text_widget_width += (text_widget_->get_label() == NULL) ? 0 : text_widget_->get_label()->width();
 		text_widget_height = dim.textbox.h + message_font_size;
@@ -483,7 +465,7 @@ dialog::dimension_measurements dialog::layout(int xloc, int yloc)
 			break;
 		}
 	}
-	check_button_height = maximum<int>(check_button_height, left_check_button_height);
+	check_button_height = std::max<int>(check_button_height, left_check_button_height);
 
 	size_t above_preview_pane_height = 0, above_left_preview_pane_width = 0, above_right_preview_pane_width = 0;
 	size_t preview_pane_height = 0, left_preview_pane_width = 0, right_preview_pane_width = 0;
@@ -492,14 +474,14 @@ dialog::dimension_measurements dialog::layout(int xloc, int yloc)
 			preview_pane const *const pane = *i;
 			const SDL_Rect& rect = pane->location();
 			if(pane->show_above() == false) {
-				preview_pane_height = maximum<size_t>(rect.h,preview_pane_height);
+				preview_pane_height = std::max<size_t>(rect.h,preview_pane_height);
 				if(pane->left_side()) {
 					left_preview_pane_width += rect.w;
 				} else {
 					right_preview_pane_width += rect.w;
 				}
 			} else {
-				above_preview_pane_height = maximum<size_t>(rect.h,above_preview_pane_height);
+				above_preview_pane_height = std::max<size_t>(rect.h,above_preview_pane_height);
 				if(pane->left_side()) {
 					above_left_preview_pane_width += rect.w;
 				} else {
@@ -545,10 +527,10 @@ dialog::dimension_measurements dialog::layout(int xloc, int yloc)
 	const int total_height = text_and_image_height + padding_height + menu_->height() +
 		text_widget_height + check_button_height;
 
-	dim.interior.w = maximum<int>(total_width,above_left_preview_pane_width + above_right_preview_pane_width);
-	dim.interior.h = maximum<int>(total_height,static_cast<int>(preview_pane_height));
-	dim.interior.x = maximum<int>(0,dim.x >= 0 ? dim.x : scr->w/2 - (dim.interior.w + left_preview_pane_width + right_preview_pane_width)/2);
-	dim.interior.y = maximum<int>(0,dim.y >= 0 ? dim.y : scr->h/2 - (dim.interior.h + above_preview_pane_height)/2);
+	dim.interior.w = std::max<int>(total_width,above_left_preview_pane_width + above_right_preview_pane_width);
+	dim.interior.h = std::max<int>(total_height,static_cast<int>(preview_pane_height));
+	dim.interior.x = std::max<int>(0,dim.x >= 0 ? dim.x : scr->w/2 - (dim.interior.w + left_preview_pane_width + right_preview_pane_width)/2);
+	dim.interior.y = std::max<int>(0,dim.y >= 0 ? dim.y : scr->h/2 - (dim.interior.h + above_preview_pane_height)/2);
 
 	DBG_DP << "above_preview_pane_height: " << above_preview_pane_height << "; "
 		<< "dim.interior.y: " << scr->h/2 << " - " << (dim.interior.h + above_preview_pane_height)/2 << " = "
@@ -569,7 +551,7 @@ dialog::dimension_measurements dialog::layout(int xloc, int yloc)
 	const int frame_top_pad = get_frame().top_padding();
 	const int frame_bottom_pad = get_frame().bottom_padding();
 	if(dim.y + dim.interior.h + frame_bottom_pad > scr->h) {
-		dim.y = maximum<int>(frame_top_pad, scr->h - dim.interior.h - frame_bottom_pad);
+		dim.y = std::max<int>(frame_top_pad, scr->h - dim.interior.h - frame_bottom_pad);
 		if(dim.y < dim.interior.y) {
 			dim.interior.y = dim.y;
 		}
@@ -583,7 +565,7 @@ dialog::dimension_measurements dialog::layout(int xloc, int yloc)
 		//try to rein in the menu height a little bit
 		const int menu_height = menu_->height();
 		if(menu_height > 0) {
-			dim.menu_height = maximum<int>(1, max_height - dim.interior.h + menu_height);
+			dim.menu_height = std::max<int>(1, max_height - dim.interior.h + menu_height);
 			dim.interior.h -= menu_height - dim.menu_height;
 		}
 	}
@@ -706,8 +688,8 @@ int dialog::process(dialog_process_info &info)
 	const bool use_text_input = (text_widget_!=NULL);
 	const bool has_input = (use_menu||use_text_input);//input of any sort has to be made
 
-	if((!info.key_down && info.key[SDLK_RETURN] || info.key[SDLK_KP_ENTER] || info.double_clicked) &&
-	   (type_ == YES_NO || type_ == OK_CANCEL || type_ == OK_ONLY || type_ == CLOSE_ONLY)) {
+	if((((!info.key_down && (info.key[SDLK_RETURN] || info.key[SDLK_KP_ENTER])) || info.double_clicked) &&
+	   (type_ == YES_NO || type_ == OK_CANCEL || type_ == OK_ONLY || type_ == CLOSE_ONLY))) {
 
 		return (use_menu ? menu_->selection() : 0);
 	}
@@ -814,12 +796,14 @@ void dialog::action(dialog_process_info& info)
 		if(selection >= 0) {
 			menu_ref.erase_item(selection);
 		}
-		if(menu_ref.nitems() == 0) {
-			set_result(CLOSE_DIALOG);
-		} else {
-			set_result(CONTINUE_DIALOG);
-			info.first_time = true;
-		}
+		// was used before to auto close empty menu
+		//if(menu_ref.nitems() == 0) {
+		//	set_result(CLOSE_DIALOG);
+		//} else {
+
+		set_result(CONTINUE_DIALOG);
+				info.first_time = true;
+
 	}
 }
 
@@ -850,6 +834,79 @@ void dialog_image::draw_contents()
 	video().blit_surface(location().x, location().y, surf_);
 }
 
+int filter_textbox::get_index(int selection) const {
+	// don't translate special values
+	if(selection < 0) {
+		return selection;
+	}
+	//we must the header row value to the index to ignore this row and
+	//then subtract it from the result to return the index not including
+	//the possible header row.
+
+	if (size_t(selection+header_row_) >= index_map_.size()) {
+		return -1; // bad index, cancel
+	}
+
+	return index_map_[selection+header_row_]-header_row_;
+}
+
+void filter_textbox::delete_item(int selection) {
+	// use the real selection
+	size_t adjusted_selection = selection + header_row_;
+
+	if (adjusted_selection >= index_map_.size())
+		return;
+
+	filtered_items_.erase(filtered_items_.begin() + adjusted_selection);
+	items_to_filter_.erase(items_to_filter_.begin() + index_map_[adjusted_selection]);
+	items_.erase(items_.begin() + index_map_[adjusted_selection]);
+	index_map_.erase(index_map_.begin() + adjusted_selection);
+
+	// don't forget to also shift the next index values
+	// this assume that index_map_and items_ have the same order
+	for(size_t i = adjusted_selection; i < index_map_.size(); ++i) {
+		index_map_[i] = index_map_[i]-1;
+	}
+
+	//for now, assume the dialog menu item is deleted using DELETE_ITEM
+	/* dialog_.set_menu_items(filtered_items_); */
+}
+
+void filter_textbox::handle_text_changed(const wide_string& text) {
+	const std::vector<std::string> words = utils::split(utils::wstring_to_string(text),' ');
+	if (words == last_words)
+		return;
+	last_words = words;
+
+	filtered_items_.clear();
+	index_map_.clear();
+
+	if(header_row_ == 1) {
+			filtered_items_.push_back(items_[0]);
+			index_map_.push_back(0);
+	}
+
+	// we keep all items containing each word
+	for(size_t n = header_row_; n < items_to_filter_.size(); ++n) {
+		std::vector<std::string>::const_iterator w = words.begin();
+		for(; w != words.end(); ++w)
+		{
+			if (std::search(items_to_filter_[n].begin(), items_to_filter_[n].end(),
+						w->begin(), w->end(),
+						chars_equal_insensitive) == items_to_filter_[n].end())
+				break; // one word doesn't match, we don't reach words.end()
+		}
+		if (w == words.end()) {
+			// all words have matched, keep the item
+			filtered_items_.push_back(items_[n]);
+			index_map_.push_back(n);
+		}
+	}
+
+	dialog_.set_menu_items(filtered_items_);
+	dialog_.get_menu().reset_selection();
+}
+
 int message_dialog::show(msecs minimum_lifetime)
 {
 	prevent_misclick_until_ = SDL_GetTicks() + minimum_lifetime;
