@@ -1,6 +1,6 @@
-/* $Id: dialogs.cpp 26766 2008-05-22 09:56:53Z baufo $ */
+/* $Id: dialogs.cpp 33775 2009-03-17 20:26:18Z soliton $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,30 +12,38 @@
    See the COPYING file for more details.
 */
 
-//! @file dialogs.cpp 
-//! Various dialogs: advance_unit, show_objectives, save+load game, network::connection.
+/**
+ * @file dialogs.cpp
+ * Various dialogs: advance_unit, show_objectives, save+load game, network::connection.
+ */
 
 #include "global.hpp"
 
 #include "dialogs.hpp"
-#include "game_errors.hpp"
 #include "game_events.hpp"
 #include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "help.hpp"
 #include "language.hpp"
 #include "log.hpp"
+#include "map.hpp"
+#include "map_exception.hpp"
 #include "marked-up_text.hpp"
 #include "menu_events.hpp"
+#include "mouse_handler_base.hpp"
 #include "minimap.hpp"
 #include "replay.hpp"
 #include "thread.hpp"
-#include "unit.hpp"
 #include "wml_separators.hpp"
 #include "widgets/progressbar.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
+
+
+//#ifdef _WIN32
+//#include "locale.h"
+//#endif
 
-#include <assert.h>
 #include <clocale>
 
 #define LOG_NG LOG_STREAM(info, engine)
@@ -46,10 +54,9 @@
 namespace dialogs
 {
 
-void advance_unit(const game_data& info,
-		  const gamemap& map,
+void advance_unit(const gamemap& map,
                   unit_map& units,
-                  gamemap::location loc,
+                  map_location loc,
                   game_display& gui,
 		  bool random_choice,
 		  const bool add_replay_event)
@@ -58,7 +65,7 @@ void advance_unit(const game_data& info,
 	if(u == units.end() || u->second.advances() == false)
 		return;
 
-	LOG_DP << "advance_unit: " << u->second.id() << "\n";
+	LOG_DP << "advance_unit: " << u->second.type_id() << "\n";
 
 	const std::vector<std::string>& options = u->second.advances_to();
 
@@ -66,21 +73,22 @@ void advance_unit(const game_data& info,
 
 	std::vector<unit> sample_units;
 	for(std::vector<std::string>::const_iterator op = options.begin(); op != options.end(); ++op) {
-		sample_units.push_back(::get_advanced_unit(info,units,loc,*op));
+		sample_units.push_back(::get_advanced_unit(units,loc,*op));
 		const unit& type = sample_units.back();
 
 #ifdef LOW_MEM
-		lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + COLUMN_SEPARATOR + type.language_name());
+		lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + COLUMN_SEPARATOR + type.type_name());
 #else
-		lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + u->second.image_mods() + COLUMN_SEPARATOR + type.language_name());
+		lang_options.push_back(IMAGE_PREFIX + type.absolute_image() + u->second.image_mods() + COLUMN_SEPARATOR + type.type_name());
 #endif
 		preferences::encountered_units().insert(*op);
 	}
 
 	const config::child_list& mod_options = u->second.get_modification_advances();
-
+	bool always_display = false;
 	for(config::child_list::const_iterator mod = mod_options.begin(); mod != mod_options.end(); ++mod) {
-		sample_units.push_back(::get_advanced_unit(info,units,loc,u->second.id()));
+		if (utils::string_bool((**mod)["always_display"])) always_display = true;
+		sample_units.push_back(::get_advanced_unit(units,loc,u->second.type_id()));
 		sample_units.back().add_modification("advance",**mod);
 		const unit& type = sample_units.back();
 		if((**mod)["image"].str().size()){
@@ -102,7 +110,7 @@ void advance_unit(const game_data& info,
 		return;
 	} else if(random_choice) {
 		res = rand()%lang_options.size();
-	} else if(lang_options.size() > 1) {
+	} else if(lang_options.size() > 1 || always_display) {
 
 		units_list_preview_pane unit_preview(gui,&map,sample_units);
 		std::vector<gui::preview_pane*> preview_panes;
@@ -124,10 +132,10 @@ void advance_unit(const game_data& info,
 	recorder.choose_option(res);
 
 	LOG_DP << "animating advancement...\n";
-	animate_unit_advancement(info,units,loc,gui,size_t(res));
-	
-	// In some rare cases the unit can have enough XP to advance again, 
-	// so try to do that. 
+	animate_unit_advancement(units,loc,gui,size_t(res));
+
+	// In some rare cases the unit can have enough XP to advance again,
+	// so try to do that.
 	// Make sure that we don't enter an infinite level loop.
 	u = units.find(loc);
 	if(u != units.end()) {
@@ -135,7 +143,7 @@ void advance_unit(const game_data& info,
 		if(u->second.experience() < 81) {
 			// For all leveling up we have to add advancement to replay here because replay
 			// doesn't handle multi advancemnet
-			advance_unit(info, map, units, loc, gui, random_choice, true);
+			advance_unit(map, units, loc, gui, random_choice, true);
 		} else {
 			LOG_STREAM(err, config) << "Unit has an too high amount of " << u->second.experience()
 				<< " XP left, cascade leveling disabled\n";
@@ -145,7 +153,7 @@ void advance_unit(const game_data& info,
 	}
 }
 
-bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::location loc, game_display& gui, size_t choice)
+bool animate_unit_advancement(unit_map& units, map_location loc, game_display& gui, size_t choice)
 {
 	const events::command_disabler cmd_disabler;
 
@@ -161,7 +169,7 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 		return false;
 	}
 
-	// When the unit advances, it fades to white, and then switches 
+	// When the unit advances, it fades to white, and then switches
 	// to the new unit, then fades back to the normal colour
 
 	if(!gui.video().update_locked()) {
@@ -173,16 +181,17 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 
 	if(choice < options.size()) {
 		const std::string& chosen_unit = options[choice];
-		::advance_unit(info,units,loc,chosen_unit);
+		::advance_unit(units,loc,chosen_unit);
 	} else {
 		unit amla_unit(u->second);
+		config mod_option(*mod_options[choice - options.size()]);
 
 		LOG_NG << "firing advance event (AMLA)\n";
 		game_events::fire("advance",loc);
 
 		amla_unit.get_experience(-amla_unit.max_experience()); // subtract xp required
-		amla_unit.add_modification("advance",*mod_options[choice - options.size()]);
-		units.replace(new std::pair<gamemap::location,unit>(loc,amla_unit));
+		amla_unit.add_modification("advance",mod_option);
+		units.replace(loc, amla_unit);
 
 		LOG_NG << "firing post_advance event (AMLA)\n";
 		game_events::fire("post_advance",loc);
@@ -196,7 +205,7 @@ bool animate_unit_advancement(const game_data& info,unit_map& units, gamemap::lo
 		animator.add_animation(&u->second,"levelin",u->first);
 		animator.start_animations();
 		animator.wait_for_end();
-		u->second.set_standing(u->first);
+		animator.set_all_standing();
 		gui.invalidate(loc);
 		gui.draw();
 		events::pump();
@@ -224,13 +233,12 @@ void show_objectives(game_display& disp, const config& level, const std::string&
 bool is_illegal_file_char(char c)
 {
 	return c == '/' || c == '\\' || c == ':'
- 	#ifdef WIN32
+ 	#ifdef _WIN32
 	|| c == '?' || c == '|' || c == '<' || c == '>' || c == '*' || c == '"'
 	#endif
 	;
 }
 
-
 int get_save_name(display & disp,const std::string& message, const std::string& txt_label,
 				  std::string* fname, gui::DIALOG_TYPE dialog_type, const std::string& title,
 				  const bool has_exit_button, const bool ask_for_filename)
@@ -272,7 +280,7 @@ int get_save_name(display & disp,const std::string& message, const std::string&
 		}
 
 		if (std::count_if(fname->begin(),fname->end(),is_illegal_file_char)) {
-			gui::message_dialog(disp, _("Error"), 
+			gui::message_dialog(disp, _("Error"),
 				_("Save names may not contain colons, slashes, or backslashes. "
 				"Please choose a different name.")).show();
 			overwrite = 1;
@@ -280,7 +288,7 @@ int get_save_name(display & disp,const std::string& message, const std::string&
 		}
 
 		if (is_gzip_file(*fname)) {
-			gui::message_dialog(disp, _("Error"), 
+			gui::message_dialog(disp, _("Error"),
 				_("Save names should not end on '.gz'. "
 				"Please choose a different name.")).show();
 			overwrite = 1;
@@ -304,74 +312,25 @@ int get_save_name(display & disp,const std::string& message, const std::string&
 	return res;
 }
 
-//! Class to handle deleting a saved game.
 namespace {
 
-class load_game_filter_textbox : public gui::dialog_textbox {
-public:
-	load_game_filter_textbox(CVideo& video, const std::vector<std::string>& items, gui::dialog& dialog)
-	  : gui::dialog_textbox(new gui::label(video, _("Filter: ")), video, 250),
-	    items_(items),
-	    dialog_(dialog),
-		first_time_(true)
-	{
-		sorter_.set_alpha_sort(0).set_id_sort(1);
-		set_text("");
-	}
-
-	int get_save_index(int index) const {
-		//we must add one to the index to ignore the header row, and
-		//then subtract one from the result to return the index not including
-		//the header row.
-		++index;
-		if(size_t(index) >= index_map_.size()) {
-			return -1;
-		}
-		return index_map_[index]-1;
-	}
-private:
-	std::vector<std::string> items_, filtered_items_;
-	std::vector<int> index_map_;
-	gui::dialog& dialog_;
-	gui::menu::basic_sorter sorter_;
-	bool first_time_;
-	virtual void handle_text_changed(const wide_string& text) {
-		filtered_items_.clear();
-		index_map_.clear();
-		const std::string t = utils::wstring_to_string(text);
-		for(size_t n = 0; n != items_.size(); ++n) {
-			if(n == 0 || std::search(items_[n].begin(), items_[n].end(),
-			                         t.begin(), t.end(), chars_equal_insensitive) != items_[n].end()) {
-				filtered_items_.push_back(items_[n]);
-				index_map_.push_back(n);
-			}
-		}
-
-		if(first_time_) {
-			dialog_.set_menu(filtered_items_, &sorter_);
-			first_time_ = false;
-		} else {
-			dialog_.set_menu_items(filtered_items_);
-		}
-	}
-};
-
+/** Class to handle deleting a saved game. */
 class delete_save : public gui::dialog_button_action
 {
 public:
-	delete_save(display& disp, load_game_filter_textbox& filter, std::vector<save_info>& saves, std::vector<config*>& save_summaries) : disp_(disp), saves_(saves), summaries_(save_summaries), filter_(filter) {}
+	delete_save(display& disp, gui::filter_textbox& filter, std::vector<save_info>& saves, std::vector<config*>& save_summaries) : disp_(disp), saves_(saves), summaries_(save_summaries), filter_(filter) {}
 private:
 	gui::dialog_button_action::RESULT button_pressed(int menu_selection);
 
 	display& disp_;
 	std::vector<save_info>& saves_;
 	std::vector<config*>& summaries_;
-	load_game_filter_textbox& filter_;
+	gui::filter_textbox& filter_;
 };
 
 gui::dialog_button_action::RESULT delete_save::button_pressed(int menu_selection)
 {
-	const size_t index = size_t(filter_.get_save_index(menu_selection));
+	const size_t index = size_t(filter_.get_index(menu_selection));
 	if(index < saves_.size()) {
 
 		// See if we should ask the user for deletion confirmation
@@ -379,7 +338,7 @@ gui::dialog_button_action::RESULT delete_save::button_pressed(int menu_selection
 			gui::dialog dmenu(disp_,"",
 					       _("Do you really want to delete this game?"),
 					       gui::YES_NO);
-			dmenu.add_option(_("Don't ask me again!"), true);
+			dmenu.add_option(_("Don't ask me again!"), false);
 			const int res = dmenu.show();
 			// See if the user doesn't want to be asked this again
 			if(dmenu.option_checked()) {
@@ -391,6 +350,9 @@ gui::dialog_button_action::RESULT delete_save::button_pressed(int menu_selection
 			}
 		}
 
+		// Remove the item from filter_textbox memory
+		filter_.delete_item(menu_selection);
+
 		// Delete the file
 		delete_game(saves_[index].name);
 
@@ -412,17 +374,25 @@ static const int save_preview_border = 10;
 class save_preview_pane : public gui::preview_pane
 {
 public:
-	save_preview_pane(CVideo &video, const config& game_config, gamemap* map, const game_data& data,
-	                  const std::vector<save_info>& info, const std::vector<config*>& summaries, const load_game_filter_textbox& textbox)
-		: gui::preview_pane(video), game_config_(&game_config), map_(map), data_(&data), info_(&info), summaries_(&summaries), index_(0), textbox_(textbox)
+	save_preview_pane(CVideo &video, const config& game_config, gamemap* map,
+			const std::vector<save_info>& info,
+			const std::vector<config*>& summaries,
+			const gui::filter_textbox& textbox) :
+		gui::preview_pane(video),
+		game_config_(&game_config),
+		map_(map), info_(&info),
+		summaries_(&summaries),
+		index_(0),
+		map_cache_(),
+		textbox_(textbox)
 	{
-		set_measurements(minimum<int>(200,video.getx()/4),
-				 minimum<int>(400,video.gety() * 4/5));
+		set_measurements(std::min<int>(200,video.getx()/4),
+				 std::min<int>(400,video.gety() * 4/5));
 	}
 
 	void draw_contents();
 	void set_selection(int index) {
-		index_ = textbox_.get_save_index(index);
+		index_ = textbox_.get_index(index);
 		set_dirty();
 	}
 
@@ -431,12 +401,11 @@ public:
 private:
 	const config* game_config_;
 	gamemap* map_;
-	const game_data* data_;
 	const std::vector<save_info>* info_;
 	const std::vector<config*>* summaries_;
 	int index_;
 	std::map<std::string,surface> map_cache_;
-	const load_game_filter_textbox& textbox_;
+	const gui::filter_textbox& textbox_;
 };
 
 void save_preview_pane::draw_contents()
@@ -466,8 +435,8 @@ void save_preview_pane::draw_contents()
 
 	int ypos = area.y;
 
-	const game_data::unit_type_map::const_iterator leader = data_->unit_types.find(summary["leader"]);
-	if(leader != data_->unit_types.end()) {
+	const unit_type_data::unit_type_map::const_iterator leader = unit_type_data::types().find_unit_type(summary["leader"]);
+	if(unit_type_data::types().unit_type_exists(summary["leader"])) {
 
 #ifdef LOW_MEM
 		const surface image(image::get_image(leader->second.image()));
@@ -517,7 +486,7 @@ void save_preview_pane::draw_contents()
 				if(map_surf != NULL) {
 					map_cache_.insert(std::pair<std::string,surface>(map_data,surface(map_surf)));
 				}
-			} catch(gamemap::incorrect_format_exception& e) {
+			} catch(incorrect_map_format_exception& e) {
 				ERR_CF << "map could not be loaded: " << e.msg_ << '\n';
 			} catch(twml_exception& e) {
 				ERR_CF << "map could not be loaded: " << e.dev_message << '\n';
@@ -527,15 +496,21 @@ void save_preview_pane::draw_contents()
 
 	if(map_surf != NULL) {
 		SDL_Rect map_rect = {area.x + area.w - map_surf->w,area.y,map_surf->w,map_surf->h};
-		ypos = maximum<int>(ypos,map_rect.y + map_rect.h + save_preview_border);
+		ypos = std::max<int>(ypos,map_rect.y + map_rect.h + save_preview_border);
 		SDL_BlitSurface(map_surf,NULL,screen,&map_rect);
 	}
 
-	char* old_locale= std::setlocale(LC_TIME, get_locale().localename.c_str());
-	char time_buf[256];
-	const size_t res = strftime(time_buf,sizeof(time_buf),_("%a %b %d %H:%M %Y"),localtime(&((*info_)[index_].time_modified)));
-	if(res == 0) {
-		time_buf[0] = 0;
+	char* old_locale = std::setlocale(LC_TIME, get_locale().localename.c_str());
+	char time_buf[256] = {0};
+	const save_info& save = (*info_)[index_];
+	tm* tm_l = localtime(&save.time_modified);
+	if (tm_l) {
+		const size_t res = strftime(time_buf,sizeof(time_buf),_("%a %b %d %H:%M %Y"),tm_l);
+		if(res == 0) {
+			time_buf[0] = 0;
+		}
+	} else {
+		LOG_NG << "localtime() returned null for time " << save.time_modified << ", save " << save.name;
 	}
 
 	if(old_locale) {
@@ -549,7 +524,7 @@ void save_preview_pane::draw_contents()
 	str << font::BOLD_TEXT << utils::escape(name) << "\n" << time_buf;
 
 	const std::string& campaign_type = summary["campaign_type"];
-	if(summary["corrupt"] == "yes") {
+	if(utils::string_bool(summary["corrupt"], false)) {
 		str << "\n" << _("#(Invalid)");
 	} else if (!campaign_type.empty()) {
 		str << "\n";
@@ -580,7 +555,7 @@ void save_preview_pane::draw_contents()
 
 		str << "\n";
 
-		if(summary["snapshot"] == "no" && summary["replay"] == "yes") {
+		if(utils::string_bool(summary["replay"], false) && !utils::string_bool(summary["snapshot"], true)) {
 			str << _("replay");
 		} else if (!summary["turn"].empty()) {
 			str << _("Turn") << " " << summary["turn"];
@@ -644,7 +619,7 @@ std::string format_time_summary(time_t t)
 
 } // end anon namespace
 
-std::string load_game_dialog(display& disp, const config& game_config, const game_data& data, bool* show_replay)
+std::string load_game_dialog(display& disp, const config& game_config, bool* show_replay, bool* cancel_orders)
 {
 	std::vector<save_info> games;
 	{
@@ -675,7 +650,7 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 
 	for(i = games.begin(); i != games.end(); ++i) {
 		std::string name = i->name;
-		utils::truncate_as_wstring(name, minimum<size_t>(name.size(), 40));
+		utils::truncate_as_wstring(name, std::min<size_t>(name.size(), 40));
 
 		std::ostringstream str;
 		str << name << COLUMN_SEPARATOR << format_time_summary(i->time_modified);
@@ -683,9 +658,6 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 		items.push_back(str.str());
 	}
 
-	gui::menu::basic_sorter sorter;
-	sorter.set_alpha_sort(0).set_id_sort(1);
-
 	gamemap map_obj(game_config, "");
 
 
@@ -693,29 +665,33 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 			  _("Load Game"),
 			  _("Choose the game to load"), gui::NULL_DIALOG);
 	lmenu.set_basic_behavior(gui::OK_CANCEL);
-	load_game_filter_textbox* filter = new load_game_filter_textbox(disp.video(), items, lmenu);
-	save_preview_pane save_preview(disp.video(),game_config,&map_obj,data,games,summaries,*filter);
+
+	gui::menu::basic_sorter sorter;
+	sorter.set_alpha_sort(0).set_id_sort(1);
+	lmenu.set_menu(items, &sorter);
+
+	gui::filter_textbox* filter = new gui::filter_textbox(disp.video(), _("Filter: "), items, items, 1, lmenu);
 	lmenu.set_textbox(filter);
+
+	save_preview_pane save_preview(disp.video(),game_config,&map_obj,games,summaries,*filter);
 	lmenu.add_pane(&save_preview);
 	// create an option for whether the replay should be shown or not
 	if(show_replay != NULL) {
-		#ifdef USE_SMALL_GUI
-			lmenu.add_option(_("Show replay"), false, gui::dialog::BUTTON_STANDARD);
-		#else
-			lmenu.add_option(_("Show replay"), false);
-		#endif
+		lmenu.add_option(_("Show replay"), false,
+			game_config::small_gui ? gui::dialog::BUTTON_CHECKBOX : gui::dialog::BUTTON_STANDARD);
+	}
+	if(cancel_orders != NULL) {
+		lmenu.add_option(_("Cancel orders"), false,
+			game_config::small_gui ? gui::dialog::BUTTON_STANDARD : gui::dialog::BUTTON_EXTRA);
 	}
 	lmenu.add_button(new gui::standard_dialog_button(disp.video(),_("OK"),0,false), gui::dialog::BUTTON_STANDARD);
 	lmenu.add_button(new gui::standard_dialog_button(disp.video(),_("Cancel"),1,true), gui::dialog::BUTTON_STANDARD);
 
 	delete_save save_deleter(disp,*filter,games,summaries);
 	gui::dialog_button_info delete_button(&save_deleter,_("Delete Save"));
-	#ifdef USE_SMALL_GUI
-		//placing the buttons in one line so that none is coverd by any of the others
-		lmenu.add_button(delete_button,gui::dialog::BUTTON_HELP);
-	#else
-		lmenu.add_button(delete_button);
-	#endif
+
+	lmenu.add_button(delete_button,
+		game_config::small_gui ? gui::dialog::BUTTON_HELP : gui::dialog::BUTTON_EXTRA);
 
 	int res = lmenu.show();
 
@@ -724,16 +700,19 @@ std::string load_game_dialog(display& disp, const config& game_config, const gam
 	if(res == -1)
 		return "";
 
-	res = filter->get_save_index(res);
-
+	res = filter->get_index(res);
+	int option_index = 0;
 	if(show_replay != NULL) {
-	  *show_replay = lmenu.option_checked();
+	  *show_replay = lmenu.option_checked(option_index++);
 
 		const config& summary = *summaries[res];
-		if(summary["replay"] == "yes" && summary["snapshot"] == "no") {
+		if(utils::string_bool(summary["replay"], false) && !utils::string_bool(summary["snapshot"], true)) {
 			*show_replay = true;
 		}
 	}
+	if (cancel_orders != NULL) {
+		*cancel_orders = lmenu.option_checked(option_index++);
+	}
 
 	return games[res].name;
 }
@@ -742,12 +721,33 @@ namespace {
 	static const int unit_preview_border = 10;
 }
 
-//! Show unit-stats in a side-pane to unit-list, recall-list, etc.
+unit_preview_pane::details::details() :
+	image(),
+	name(),
+	type_name(),
+	race(),
+	level(0),
+	alignment(),
+	traits(),
+	abilities(),
+	hitpoints(0),
+	max_hitpoints(0),
+	experience(0),
+	max_experience(0),
+	hp_color(),
+	xp_color(),
+	movement_left(0),
+	total_movement(0),
+	attacks()
+{
+}
 
-unit_preview_pane::unit_preview_pane(game_display& disp, const gamemap* map, TYPE type, bool on_left_side)
+unit_preview_pane::unit_preview_pane(game_display& disp, const gamemap* map,
+		const gui::filter_textbox* filter, TYPE type, bool on_left_side)
 				    : gui::preview_pane(disp.video()), disp_(disp), map_(map), index_(0),
-				      details_button_(disp.video(),_("Profile"),gui::button::TYPE_PRESS,"lite_small",gui::button::MINIMUM_SPACE),
-				      left_(on_left_side), weapons_(type == SHOW_ALL)
+				      details_button_(disp.video(), _("Profile"),
+				      gui::button::TYPE_PRESS,"lite_small", gui::button::MINIMUM_SPACE),
+				      filter_(filter), weapons_(type == SHOW_ALL), left_(on_left_side)
 {
 	unsigned w = font::relative_size(weapons_ ? 200 : 190);
 	unsigned h = font::relative_size(weapons_ ? 370 : 140);
@@ -774,11 +774,14 @@ bool unit_preview_pane::left_side() const
 
 void unit_preview_pane::set_selection(int index)
 {
-	index = minimum<int>(int(size()-1),index);
-	if(index != index_ && index >= 0) {
+	index = std::min<int>(static_cast<int>(size())-1,index);
+	if (filter_) {
+		index = filter_->get_index(index);
+	}
+	if(index != index_) {
 		index_ = index;
 		set_dirty();
-		if(map_ != NULL) {
+		if(map_ != NULL && index >= 0) {
 			details_button_.set_dirty();
 		}
 	}
@@ -824,29 +827,30 @@ void unit_preview_pane::draw_contents()
 
 	SDL_Rect description_rect = {image_rect.x,image_rect.y+image_rect.h+details_button_.location().h,0,0};
 
-	if(det.description.empty() == false) {
+	if(det.name.empty() == false) {
 		std::stringstream desc;
-		desc << font::NORMAL_TEXT << det.description;
+		desc << font::NORMAL_TEXT << det.name;
 		const std::string description = desc.str();
 		description_rect = font::text_area(description, font::SIZE_NORMAL);
-		description_rect = font::draw_text(&video(), area, 
-							font::SIZE_NORMAL, font::NORMAL_COLOUR, 
-							desc.str(), right_align ?  image_rect.x : 
-							image_rect.x + image_rect.w - description_rect.w, 
+		description_rect = font::draw_text(&video(), area,
+							font::SIZE_NORMAL, font::NORMAL_COLOUR,
+							desc.str(), right_align ?  image_rect.x :
+							image_rect.x + image_rect.w - description_rect.w,
 							image_rect.y + image_rect.h + details_button_.location().h);
 	}
 
 	std::stringstream text;
-	text << det.name << "\n"
+	text << det.type_name << "\n"
+		<< det.race << "\n"
 		<< font::BOLD_TEXT << _("level") << " " << det.level << "\n"
 		<< det.alignment << "\n"
 		<< det.traits << "\n";
 
-	for(std::vector<std::string>::const_iterator a = det.abilities.begin(); a != det.abilities.end(); a++) {
+	for(std::vector<t_string>::const_iterator a = det.abilities.begin(); a != det.abilities.end(); a++) {
 		if(a != det.abilities.begin()) {
 			text << ", ";
 		}
-		text << gettext(a->c_str());
+		text << (*a);
 	}
 	text << "\n";
 
@@ -864,7 +868,7 @@ void unit_preview_pane::draw_contents()
 		for(std::vector<attack_type>::const_iterator at_it = det.attacks.begin();
 		    at_it != det.attacks.end(); ++at_it) {
 			// specials_context seems not needed here
-			//at_it->set_specials_context(gamemap::location(),u);
+			//at_it->set_specials_context(map_location(),u);
 
 			// see generate_report() in generate_report.cpp
 			text << "<245,230,193>" << at_it->name()
@@ -874,8 +878,13 @@ void unit_preview_pane::draw_contents()
 			if (!special.empty()) {
 				text << "<166,146,117>  " << special << "\n";
 			}
+			std::string accuracy = at_it->accuracy_parry_description();
+			if(accuracy.empty() == false) {
+				accuracy += " ";
+			}
+
 			text << "<166,146,117>  " << at_it->damage() << "-" << at_it->num_attacks()
-				<< " -- " << _(at_it->range().c_str()) << "\n";
+				<< " " << accuracy << "-- " << _(at_it->range().c_str()) << "\n";
 		}
 	}
 
@@ -901,24 +910,28 @@ void unit_preview_pane::draw_contents()
 	}
 }
 
-units_list_preview_pane::units_list_preview_pane(game_display& disp, const gamemap* map, const unit& u, TYPE type, bool on_left_side)
-					: unit_preview_pane(disp, map, type, on_left_side),
-					  units_(&unit_store_)
+units_list_preview_pane::units_list_preview_pane(game_display& disp,
+		const gamemap* map, const unit& u, TYPE type, bool on_left_side) :
+	unit_preview_pane(disp, map, NULL, type, on_left_side),
+	units_(&unit_store_),
+	unit_store_(1, u)
 {
-	unit_store_.push_back(u);
 }
 
-units_list_preview_pane::units_list_preview_pane(game_display& disp, const gamemap* map, std::vector<unit>& units, TYPE type, bool on_left_side)
-					: unit_preview_pane(disp, map, type, on_left_side),
-					  units_(&units)
-{}
+units_list_preview_pane::units_list_preview_pane(game_display& disp,
+		const gamemap* map, std::vector<unit>& units,
+		const gui::filter_textbox* filter, TYPE type, bool on_left_side) :
+	unit_preview_pane(disp, map, filter, type, on_left_side),
+	units_(&units),
+	unit_store_()
+{
+}
 
 size_t units_list_preview_pane::size() const
 {
 	return (units_!=NULL) ? units_->size() : 0;
 }
 
-//unit_preview_pane::
 const unit_preview_pane::details units_list_preview_pane::get_details() const
 {
 	unit& u = (*units_)[index_];
@@ -926,10 +939,13 @@ const unit_preview_pane::details units_list_preview_pane::get_details() const
 
 	det.image = u.still_image();
 
-	det.description = u.description();
-	det.name = u.language_name();
+	det.name = u.name();
+	det.type_name = u.type_name();
+	if(u.race() != NULL) {
+		det.race = u.race()->name(u.gender());
+	}
 	det.level = u.level();
-	det.alignment = unit_type::alignment_description(u.alignment());
+	det.alignment = unit_type::alignment_description(u.alignment(), u.gender());
 	det.traits = u.traits_description();
 
 	//we filter to remove the tooltips (increment by 2)
@@ -938,11 +954,11 @@ const unit_preview_pane::details units_list_preview_pane::get_details() const
 		 a != abilities.end(); a+=2) {
 		det.abilities.push_back(*a);
 	}
-	
+
 	det.hitpoints = u.hitpoints();
 	det.max_hitpoints = u.max_hitpoints();
 	det.hp_color = font::color2markup(u.hp_color());
-	
+
 	det.experience = u.experience();
 	det.max_experience = u.max_experience();
 	det.xp_color = font::color2markup(u.xp_color());
@@ -961,8 +977,10 @@ void units_list_preview_pane::process_event()
 	}
 }
 
-unit_types_preview_pane::unit_types_preview_pane(game_display& disp, const gamemap* map, std::vector<const unit_type*>& unit_types, int side, TYPE type, bool on_left_side)
-					: unit_preview_pane(disp, map, type, on_left_side),
+unit_types_preview_pane::unit_types_preview_pane(game_display& disp, const gamemap* map,
+					std::vector<const unit_type*>& unit_types, const gui::filter_textbox* filter,
+					int side, TYPE type, bool on_left_side)
+					: unit_preview_pane(disp, map, filter, type, on_left_side),
 					  unit_types_(&unit_types), side_(side)
 {}
 
@@ -979,13 +997,23 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 	if (t==NULL)
 		return det;
 
+    //FIXME: There should be a better way to deal with this
+    unit_type_data::types().find_unit_type(t->id(), unit_type::WITHOUT_ANIMATIONS);
+
 	std::string mod = "~RC(" + t->flag_rgb() + ">" + team::get_side_colour_index(side_) + ")";
 	det.image = image::get_image(t->image()+mod);
 
-	det.description = "";
-	det.name = t->language_name();
+	det.name = "";
+	det.type_name = t->type_name();
 	det.level = t->level();
-	det.alignment = unit_type::alignment_description(t->alignment());
+	det.alignment = unit_type::alignment_description(t->alignment(), t->genders().front());
+
+	race_map const& rcm = unit_type_data::types().races();
+	race_map::const_iterator ri = rcm.find(t->race());
+	if(ri != rcm.end()) {
+		assert(t->genders().empty() != true);
+		det.race = (*ri).second.name(t->genders().front());
+	}
 
 	//FIXME: This probably must be move into a unit_type function
 	const std::vector<config*> traits = t->possible_traits();
@@ -1006,7 +1034,7 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 	}
 
 	det.abilities = t->abilities();
-	
+
 	det.hitpoints = t->hitpoints();
 	det.max_hitpoints = t->hitpoints();
 	det.hp_color = "<33,225,0>"; // from unit::hp_color()
@@ -1035,7 +1063,7 @@ const unit_types_preview_pane::details unit_types_preview_pane::get_details() co
 
 void unit_types_preview_pane::process_event()
 {
-	if(map_ != NULL && details_button_.pressed() && index_ >= 0 && index_ < int(size())) { 
+	if(map_ != NULL && details_button_.pressed() && index_ >= 0 && index_ < int(size())) {
 		const unit_type* type = (*unit_types_)[index_];
 		if (type != NULL)
 			show_unit_description(disp_, *type);
@@ -1050,7 +1078,7 @@ void show_unit_description(game_display &disp, const unit& u)
 		show_unit_description(disp, *t);
 	else
 		// can't find type, try open the id page to have feedback and unit error page
-		help::show_unit_help(disp, u.id());
+	  help::show_unit_help(disp, u.type_id());
 }
 
 void show_unit_description(game_display &disp, const unit_type& t)
@@ -1078,7 +1106,7 @@ bool campaign_preview_pane::left_side() const { return false; }
 
 void campaign_preview_pane::set_selection(int index)
 {
-	index = minimum<int>(descriptions_->size()-1,index);
+	index = std::min<int>(descriptions_->size()-1,index);
 	if(index != index_ && index >= 0) {
 		index_ = index;
 		set_dirty();
@@ -1130,8 +1158,8 @@ void campaign_preview_pane::draw_contents()
 		int max_height = area.h-(txt_area.h+txt_area.y-area.y);
 
 		src_rect.x = src_rect.y = 0;
-		src_rect.w = minimum<int>(area.w,img->w);
-		src_rect.h = minimum<int>(max_height,img->h);
+		src_rect.w = std::min<int>(area.w,img->w);
+		src_rect.h = std::min<int>(max_height,img->h);
 		dst_rect.x = area.x+(area.w-src_rect.w)/2;
 		dst_rect.y = txt_area.y+((max_height-src_rect.h)*8)/13;
 		if(dst_rect.y - txt_area.h - txt_area.y >= 120) {
@@ -1144,7 +1172,7 @@ void campaign_preview_pane::draw_contents()
 	}
 }
 
-network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num, network::statistics (*get_stats)(network::connection handle))
+static network::connection network_data_dialog(display& disp, const std::string& msg, config& cfg, network::connection connection_num, network::statistics (*get_stats)(network::connection handle))
 {
 #ifdef USE_TINY_GUI
 	const size_t width = 200;
@@ -1163,11 +1191,16 @@ network::connection network_data_dialog(display& disp, const std::string& msg, c
 	gui::button cancel_button(disp.video(),_("Cancel"));
 	std::vector<gui::button*> buttons_ptr(1,&cancel_button);
 
-	gui::dialog_frame frame(disp.video(), msg, gui::dialog_frame::default_style, false, &buttons_ptr);
-	frame.layout(left,top,width,height);
+	gui::dialog_frame frame(disp.video(), msg, gui::dialog_frame::default_style, true, &buttons_ptr);
+	SDL_Rect centered_layout = frame.layout(left,top,width,height).interior;
+	centered_layout.x = disp.w() / 2 - centered_layout.w / 2;
+	centered_layout.y = disp.h() / 2 - centered_layout.h / 2;
+	// HACK: otherwise we get an empty useless space in the dialog below the progressbar
+	centered_layout.h = height;
+	frame.layout(centered_layout);
 	frame.draw();
 
-	const SDL_Rect progress_rect = {left+border,top+border,width-border*2,height-border*2};
+	const SDL_Rect progress_rect = {centered_layout.x+border,centered_layout.y+border,centered_layout.w-border*2,centered_layout.h-border*2};
 	gui::progress_bar progress(disp.video());
 	progress.set_location(progress_rect);
 
@@ -1191,7 +1224,7 @@ network::connection network_data_dialog(display& disp, const std::string& msg, c
 		events::raise_draw_event();
 		disp.flip();
 		events::pump();
-		
+
 		if(res != 0) {
 			return res;
 		}
@@ -1265,7 +1298,7 @@ network::connection network_connect_dialog(display& disp, const std::string& msg
 	gui::button cancel_button(disp.video(),_("Cancel"));
 	std::vector<gui::button*> buttons_ptr(1,&cancel_button);
 
-	gui::dialog_frame frame(disp.video(), msg, gui::dialog_frame::default_style, false, &buttons_ptr);
+	gui::dialog_frame frame(disp.video(), msg, gui::dialog_frame::default_style, true, &buttons_ptr);
 	frame.layout(left,top,width,height);
 	frame.draw();
 
