@@ -1,6 +1,6 @@
-/* $Id: filesystem.cpp 26448 2008-05-07 17:35:33Z mordante $ */
+/* $Id: filesystem.cpp 33585 2009-03-13 00:28:36Z soliton $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,250 +12,28 @@
    See the COPYING file for more details.
 */
 
-//! @file filesystem.cpp 
-//! File-IO
+/**
+ * @file filesystem.cpp
+ * File-IO
+ */
 
 #include "global.hpp"
 
-// Include files for opendir(3), readdir(3), etc. 
-// These files may vary from platform to platform, 
-// since these functions are NOT ANSI-conforming functions. 
+// Include files for opendir(3), readdir(3), etc.
+// These files may vary from platform to platform,
+// since these functions are NOT ANSI-conforming functions.
 // They may have to be altered to port to new platforms
-#include <sys/types.h>
 
 //for mkdir
 #include <sys/stat.h>
-#include <sys/types.h>
 
 #ifdef _WIN32
-
-/* /////////////////////////////////////////////////////////////////////////
- * This code swiped from dirent.c in the unixem library, version 1.7.3.
- * See http://synesis.com.au/software/unixem.html for full sources.
- * It's under BSD license.
- */
-
-#include <direct.h>
-#include <io.h>
-#include <errno.h>
-#include <stdlib.h>
-#include <windows.h>
-
-
-/* /////////////////////////////////////////////////////////////////////////
- * Compiler differences
- */
-
-#if defined(__BORLANDC__)
-# define DIRENT_PROVIDED_BY_COMPILER
-#elif defined(__DMC__)
-# define DIRENT_PROVIDED_BY_COMPILER
-#elif defined(__GNUC__)
-# define DIRENT_PROVIDED_BY_COMPILER
-#elif defined(__INTEL_COMPILER)
-#elif defined(_MSC_VER)
-#elif defined(__MWERKS__)
-#elif defined(__WATCOMC__)
-#else
-# error Compiler not discriminated
-#endif /* compiler */
-
-#if defined(DIRENT_PROVIDED_BY_COMPILER)
-#include <dirent.h>
-#else
-
-/* ////////////////////////////////////////////////////////////////////// */
-
-#include <stddef.h>
-
-#ifndef NAME_MAX
-# define NAME_MAX   (260)
-#endif /* !NAME_MAX */
-
-struct dirent
-{
-	char	d_name[NAME_MAX + 1];	   /*!< file name (null-terminated) */
-	int 	d_mode;
-};
-
-struct DIR
-{
-	char			  directory[_MAX_DIR+1];   /* . */
-	WIN32_FIND_DATAA  find_data;			   /* The Win32 FindFile data. */
-	HANDLE			  hFind;				   /* The Win32 FindFile handle. */
-	struct dirent	  dirent;				   /* The handle's entry. */
-};
-
-#ifndef FILE_ATTRIBUTE_ERROR
-# define FILE_ATTRIBUTE_ERROR			(0xFFFFFFFF)
-#endif /* FILE_ATTRIBUTE_ERROR */
-
-/* /////////////////////////////////////////////////////////////////////////
- * Helper functions
- */
-
-static HANDLE dirent__findfile_directory(char const *name, LPWIN32_FIND_DATAA data)
-{
-	char	search_spec[_MAX_PATH +1];
-
-	// Simply add the *.*, ensuring the path separator is included.
-	(void)lstrcpyA(search_spec, name);
-	if( '\\' != search_spec[lstrlenA(search_spec) - 1] &&
-		'/' != search_spec[lstrlenA(search_spec) - 1])
-	{
-		(void)lstrcatA(search_spec, "\\*.*");
-	}
-	else
-	{
-		(void)lstrcatA(search_spec, "*.*");
-	}
-
-	return FindFirstFileA(search_spec, data);
-}
-
-/* /////////////////////////////////////////////////////////////////////////
- * API functions
- */
-
-DIR *opendir(char const *name)
-{
-	DIR 	*result =	NULL;
-	DWORD	dwAttr;
-
-	// Must be a valid name 
-	if( !name ||
-		!*name ||
-		(dwAttr = GetFileAttributes(name)) == 0xFFFFFFFF)
-	{
-		errno = ENOENT;
-	}
-	// Must be a directory 
-	else if(!(dwAttr & FILE_ATTRIBUTE_DIRECTORY))
-	{
-		errno = ENOTDIR;
-	}
-	else
-	{
-		result = (DIR*)malloc(sizeof(DIR));
-
-		if(result == NULL)
-		{
-			errno = ENOMEM;
-		}
-		else
-		{
-			result->hFind=dirent__findfile_directory(name, &result->find_data);
-
-			if(result->hFind == INVALID_HANDLE_VALUE)
-			{
-				free(result);
-
-				result = NULL;
-			}
-			else
-			{
-				// Save the directory, in case of rewind. 
-				(void)lstrcpyA(result->directory, name);
-				(void)lstrcpyA(result->dirent.d_name, result->find_data.cFileName);
-				result->dirent.d_mode	=	(int)result->find_data.dwFileAttributes;
-			}
-		}
-	}
-
-	return result;
-}
-
-int closedir(DIR *dir)
-{
-	int ret;
-
-	if(dir == NULL)
-	{
-		errno = EBADF;
-
-		ret = -1;
-	}
-	else
-	{
-		// Close the search handle, if not already done.
-		if(dir->hFind != INVALID_HANDLE_VALUE)
-		{
-			(void)FindClose(dir->hFind);
-		}
-
-		free(dir);
-
-		ret = 0;
-	}
-
-	return ret;
-}
-
-struct dirent *readdir(DIR *dir)
-{
-	// The last find exhausted the matches, so return NULL.
-	if(dir->hFind == INVALID_HANDLE_VALUE)
-	{
-		if(FILE_ATTRIBUTE_ERROR == dir->find_data.dwFileAttributes)
-		{
-			errno = EBADF;
-		}
-		else
-		{
-			dir->find_data.dwFileAttributes = FILE_ATTRIBUTE_ERROR;
-		}
-
-		return NULL;
-	}
-	else
-	{
-		// Copy the result of the last successful match to dirent.
-		(void)lstrcpyA(dir->dirent.d_name, dir->find_data.cFileName);
-
-		// Attempt the next match.
-		if(!FindNextFileA(dir->hFind, &dir->find_data))
-		{
-			// Exhausted all matches, so close and null the handle.
-			(void)FindClose(dir->hFind);
-			dir->hFind = INVALID_HANDLE_VALUE;
-		}
-
-		return &dir->dirent;
-	}
-}
-
-/*
- * Microsoft C uses _stat instead of stat,
- * for both the function name and the structure name.
- * See <http://svn.ghostscript.com:8080/ghostscript/trunk/gs/src/stat_.h>
- */
-#ifdef _MSC_VER
-#  define stat _stat
-namespace {
-	typedef int mode_t;
-}
-#endif
-
-#ifndef S_IFMT
-#define S_IFMT	(S_IFDIR|S_IFREG)
-#endif
-#ifndef S_ISREG
-#define S_ISREG(x) (((x) & S_IFMT) == S_IFREG)
-#endif
-#ifndef S_ISDIR
-#define S_ISDIR(x) (((x) & S_IFMT) == S_IFDIR)
-#endif
-
-#endif /* !DIRENT_PROVIDED_BY_COMPILER */
-
-#define mkdir(a,b) (_mkdir(a))
-
+#include "filesystem_win32.ii"
+#include <cctype>
 #else /* !_WIN32 */
-
 #include <unistd.h>
-
 #include <dirent.h>
-
+#include <libgen.h>
 #endif /* !_WIN32 */
 
 #ifdef __BEOS__
@@ -266,28 +44,25 @@ BPath be_path;
 #endif
 
 // for getenv
-#include <cstdlib>
 #include <cerrno>
-#include <string>
-#include <cstring>
-#include <algorithm>
 #include <fstream>
-#include <iostream>
 #include <iomanip>
-#include <sstream>
 #include <set>
+#include <boost/algorithm/string.hpp>
+
+// for strerror
+#include <cstring>
 
-#include "wesconfig.h"
 #include "config.hpp"
 #include "filesystem.hpp"
+#include "foreach.hpp"
 #include "game_config.hpp"
 #include "log.hpp"
-#include "scoped_resource.hpp"
-#include "util.hpp"
 #include "loadscreen.hpp"
 
 #define DBG_FS LOG_STREAM(debug, filesystem)
 #define LOG_FS LOG_STREAM(info, filesystem)
+#define WRN_FS LOG_STREAM(warn, filesystem)
 #define ERR_FS LOG_STREAM(err, filesystem)
 
 namespace {
@@ -306,26 +81,28 @@ bool ends_with(const std::string& str, const std::string& suffix)
 }
 
 // These are the filenames that get special processing
-#define MAINCFG "_main.cfg"
+#define MAINCFG 	"_main.cfg"
 #define FINALCFG	"_final.cfg"
+#define INITIALCFG	"_initial.cfg"
 
-// Don't pass directory as reference, it seems to break on 
+// Don't pass directory as reference, it seems to break on
 // arklinux with GCC-4.3.
 void get_files_in_dir(const std::string directory,
 					  std::vector<std::string>* files,
 					  std::vector<std::string>* dirs,
-					  FILE_NAME_MODE mode,
-					  FILE_FILTER filter,
-					  FILE_REORDER_OPTION reorder)
+					  file_name_option mode,
+					  file_filter_option filter,
+					  file_reorder_option reorder,
+					  file_tree_checksum* checksum)
 {
-	// If we have a path to find directories in, 
-	// then convert relative pathnames to be rooted 
+	// If we have a path to find directories in,
+	// then convert relative pathnames to be rooted
 	// on the wesnoth path
 #ifndef __AMIGAOS4__
 	if(!directory.empty() && directory[0] != '/' && !game_config::path.empty()){
 		const std::string& dir = game_config::path + "/" + directory;
 		if(is_directory(dir)) {
-			get_files_in_dir(dir,files,dirs,mode,filter,reorder);
+			get_files_in_dir(dir,files,dirs,mode,filter,reorder,checksum);
 			return;
 		}
 	}
@@ -334,7 +111,7 @@ void get_files_in_dir(const std::string directory,
 	struct stat st;
 
 	if (reorder == DO_REORDER) {
-		LOG_FS << "searching _main.cfg in directory " << directory << '\n';
+		LOG_FS << "searching for _main.cfg in directory " << directory << '\n';
 		std::string maincfg;
 		if (directory.empty() || directory[directory.size()-1] == '/'
 #ifdef __AMIGAOS4__
@@ -368,7 +145,7 @@ void get_files_in_dir(const std::string directory,
 		if(entry->d_name[0] == '.')
 			continue;
 #ifdef __APPLE__
-		// HFS Mac OS X decomposes filenames using combining unicode characters. 
+		// HFS Mac OS X decomposes filenames using combining unicode characters.
 		// Try to get the precomposed form.
 		char macname[MAXNAMLEN+1];
 		CFStringRef cstr = CFStringCreateWithCString(NULL,
@@ -384,7 +161,7 @@ void get_files_in_dir(const std::string directory,
 		CFRelease(mut_str);
 		const std::string basename = macname;
 #else
-		// generic Unix 
+		// generic Unix
 		const std::string basename = entry->d_name;
 #endif /* !APPLE */
 
@@ -406,11 +183,18 @@ void get_files_in_dir(const std::string directory,
 					else
 						files->push_back(basename);
 				}
+				if (checksum != NULL) {
+					if(st.st_mtime > checksum->modified) {
+						checksum->modified = st.st_mtime;
+					}
+					checksum->sum_size += st.st_size;
+					checksum->nfiles++;
+				}
 			} else if (S_ISDIR(st.st_mode)) {
 				if (filter == SKIP_MEDIA_DIR
 						&& (basename == "images"|| basename == "sounds"))
 					continue;
-			
+
 				if (reorder == DO_REORDER &&
 						::stat((fullname+"/"+MAINCFG).c_str(), &st)!=-1 &&
 						S_ISREG(st.st_mode)) {
@@ -446,12 +230,27 @@ void get_files_in_dir(const std::string directory,
 		std::sort(dirs->begin(),dirs->end());
 
 	if (files != NULL && reorder == DO_REORDER) {
-		for (unsigned int i = 0; i < files->size(); i++)
-			if (ends_with((*files)[i], FINALCFG)) {
+		// move FINALCFG, if present, to the end of the vector
+		for (unsigned int i = 0; i < files->size(); i++) {
+			if (ends_with((*files)[i], "/" FINALCFG)) {
 				files->push_back((*files)[i]);
 				files->erase(files->begin()+i);
 				break;
 			}
+		}
+		// move INITIALCFG, if present, to the beginning of the vector
+		int foundit = -1;
+		for (unsigned int i = 0; i < files->size(); i++)
+			if (ends_with((*files)[i], "/" INITIALCFG)) {
+				foundit = i;
+				break;
+			}
+		if (foundit > 0) {
+			std::string initialcfg = (*files)[foundit];
+			for (unsigned int i = foundit; i > 0; i--)
+				(*files)[i] = (*files)[i-1];
+			(*files)[0] = initialcfg;
+		}
 	}
 }
 
@@ -462,7 +261,7 @@ std::string get_prefs_file()
 
 std::string get_save_index_file()
 {
-	return get_user_data_dir() + "/save_index";
+	return get_user_data_dir() + "/save_index.gz";
 }
 
 std::string get_saves_dir()
@@ -477,11 +276,26 @@ std::string get_cache_dir()
 	return get_dir(dir_path);
 }
 
+std::string get_addon_campaigns_dir()
+{
+	const std::string dir_path = get_user_data_dir() + "/data/campaigns";
+	return get_dir(dir_path);
+}
+
+std::vector< std::string > get_addon_dirs()
+{
+	std::vector< std::string > ret;
+
+	ret.push_back(get_addon_campaigns_dir());
+
+	return ret;
+}
+
 std::string get_intl_dir()
 {
 #ifdef _WIN32
-	return get_cwd() + "/po";
-#endif
+	return get_cwd() + "/translations";
+#else
 
 #ifdef USE_INTERNAL_DATA
 	return get_cwd() + "/" LOCALEDIR;
@@ -494,6 +308,7 @@ std::string get_intl_dir()
 #endif
 
 	return res;
+#endif
 }
 
 std::string get_screenshot_dir()
@@ -502,6 +317,26 @@ std::string get_screenshot_dir()
 	return get_dir(dir_path);
 }
 
+std::string get_next_filename(const std::string& name, const std::string& extension)
+{
+	std::string next_filename;
+	int counter = 0;
+
+	do {
+		std::stringstream filename;
+
+		filename << name;
+		filename.width(3);
+		filename.fill('0');
+		filename.setf(std::ios_base::right);
+		filename << counter << extension;
+		counter++;
+		next_filename = filename.str();
+	} while(file_exists(next_filename) && counter < 1000);
+	return next_filename;
+}
+
+
 std::string get_upload_dir()
 {
 	const std::string dir_path = get_user_data_dir() + "/upload";
@@ -561,6 +396,14 @@ bool delete_directory(const std::string& path)
 	}
 
 	errno = 0;
+#ifdef _WIN32
+	// remove() doesn't delete directories on windows.
+	int (*remove)(const char*);
+	if(is_directory(path))
+		remove = rmdir;
+	else
+		remove = ::remove;
+#endif
 	if(remove(path.c_str()) != 0) {
 		LOG_FS << "remove(" << path << "): " << strerror(errno) << "\n";
 		ret = false;
@@ -575,7 +418,7 @@ std::string get_cwd()
 	if(res != NULL) {
 		std::string str(res);
 
-#ifdef WIN32
+#ifdef _WIN32
 		std::replace(str.begin(),str.end(),'\\','/');
 #endif
 
@@ -585,36 +428,118 @@ std::string get_cwd()
 	}
 }
 
-std::string get_user_data_dir()
+std::string get_exe_dir()
 {
+#ifndef _WIN32
+	char buf[1024];
+	size_t path_size = readlink("/proc/self/exe", buf, 1024);
+	if(path_size == static_cast<size_t>(-1))
+		return std::string();
+	buf[path_size] = 0;
+	return std::string(dirname(buf));
+#else
+	return get_cwd();
+#endif
+}
+
+bool create_directory_if_missing(const std::string& dirname)
+{
+	if(is_directory(dirname)) {
+		DBG_FS << "directory " << dirname << " exists, not creating\n";
+		return true;
+	} else if(file_exists(dirname)) {
+		ERR_FS << "cannot create directory " << dirname << "; file exists\n";
+		return false;
+	}
+	DBG_FS << "creating missing directory " << dirname << '\n';
+	return make_directory(dirname);
+}
+
+
+namespace {
+	std::string user_data_dir;
+}
+
+static std::string setup_user_data_dir();
+
+void set_preferences_dir(std::string path)
+{
+#ifndef PREFERENCES_DIR
+const std::string PREFERENCES_DIR = ".wesnoth" + std::string(game_config::version).substr(0,3);
+#endif
 #ifdef _WIN32
-
-	static bool inited_dirs = false;
-
-	if(!inited_dirs) {
-		_mkdir("userdata");
-		_mkdir("userdata/editor");
-		_mkdir("userdata/editor/maps");
-		_mkdir("userdata/data");
-		_mkdir("userdata/data/ais");
-		_mkdir("userdata/data/scenarios");
-		_mkdir("userdata/data/scenarios/multiplayer");
-		_mkdir("userdata/data/maps");
-		_mkdir("userdata/data/maps/multiplayer");
-		_mkdir("userdata/saves");
-		inited_dirs = true;
-	}
-
-	char buf[256];
-	const char* const res = getcwd(buf,sizeof(buf));
-
-	if(res != NULL) {
-		std::string cur_path(res);
-		std::replace(cur_path.begin(),cur_path.end(),'\\','/');
-		return cur_path + "/userdata";
+	if(path.empty()) {
+		game_config::preferences_dir = get_cwd() + "/userdata";
+	} else if (path.size() > 2 && path[1] == ':') {
+		//allow absolute path override
+		game_config::preferences_dir = path;
 	} else {
-		return "userdata";
+		BOOL (*SHGetSpecialFolderPath)(HWND, LPTSTR, int, BOOL);
+		HMODULE module = LoadLibrary("shell32");
+		SHGetSpecialFolderPath = (BOOL (*)(HWND, LPTSTR, int, BOOL))GetProcAddress(module, "SHGetSpecialFolderPathA");
+		if(SHGetSpecialFolderPath) {
+			LOG_FS << "Using SHGetSpecialFolderPath to find My Documents\n";
+			char my_documents_path[MAX_PATH];
+			if(SHGetSpecialFolderPath(NULL, my_documents_path, 5, 1)) {
+				std::string mygames_path = std::string(my_documents_path) + "/" + "My Games";
+				boost::algorithm::replace_all(mygames_path, std::string("\\"), std::string("/"));
+				create_directory_if_missing(mygames_path);
+				game_config::preferences_dir = mygames_path + "/" + path;
+			} else {
+				WRN_FS << "SHGetSpecialFolderPath failed\n";
+				game_config::preferences_dir = get_cwd() + "/" + path;
+			}
+		} else {
+			LOG_FS << "Failed to load SHGetSpecialFolderPath function\n";
+			game_config::preferences_dir = get_cwd() + "/" + path;
+		}
 	}
+
+#else /*_WIN32*/
+	if (path.empty()) {
+		path = PREFERENCES_DIR;
+	}
+#ifndef __AMIGAOS4__
+	const char* const current_dir = ".";
+	const char* home_str = getenv("HOME");
+#else
+	const char* const current_dir = " ";
+	const char* home_str = "PROGDIR:";
+#endif
+	if(home_str == NULL)
+		home_str = current_dir;
+
+	const std::string home(home_str);
+
+#ifndef __AMIGAOS4__
+	game_config::preferences_dir = home + std::string("/") + path;
+#else
+	game_config::preferences_dir = home + path;
+#endif
+
+#endif /*_WIN32*/
+	user_data_dir = setup_user_data_dir();
+}
+
+
+static std::string setup_user_data_dir()
+{
+	if (game_config::preferences_dir.empty())
+		set_preferences_dir(std::string());
+#ifdef _WIN32
+	_mkdir((game_config::preferences_dir).c_str());
+	_mkdir((game_config::preferences_dir + "/editor").c_str());
+	_mkdir((game_config::preferences_dir + "/editor/maps").c_str());
+	_mkdir((game_config::preferences_dir + "/data").c_str());
+	_mkdir((game_config::preferences_dir + "/data/ais").c_str());
+	_mkdir((game_config::preferences_dir + "/data/campaigns").c_str());
+	_mkdir((game_config::preferences_dir + "/data/multiplayer").c_str());
+	_mkdir((game_config::preferences_dir + "/data/maps").c_str());
+	_mkdir((game_config::preferences_dir + "/data/maps/multiplayer").c_str());
+	_mkdir((game_config::preferences_dir + "/data/units").c_str());
+	_mkdir((game_config::preferences_dir + "/saves").c_str());
+
+	return game_config::preferences_dir;
 #elif defined(__BEOS__)
 	if (be_path.InitCheck() != B_OK) {
 		BPath tpath;
@@ -623,64 +548,64 @@ std::string get_user_data_dir()
 		} else {
 			be_path.SetTo("/boot/home/config/settings/wesnoth");
 		}
-		tpath = be_path;
-		tpath.Append("editor/maps");
-		create_directory(tpath.Path(), 0775);
+	#define BEOS_CREATE_PREFERENCES_SUBDIR(subdir) \
+			tpath = be_path;                       \
+			tpath.Append(subdir);                  \
+			create_directory(tpath.Path(), 0775);
+
+		BEOS_CREATE_PREFERENCES_SUBDIR("editor");
+		BEOS_CREATE_PREFERENCES_SUBDIR("editor/maps");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/ais");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/campaigns");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/multiplayer");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/maps");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/maps/multiplayer");
+		BEOS_CREATE_PREFERENCES_SUBDIR("data/units");
+		BEOS_CREATE_PREFERENCES_SUBDIR("saves");
+	#undef BEOS_CREATE_PREFERENCES_SUBDIR
 	}
 	return be_path.Path();
 #else
+	const std::string& dir_path = game_config::preferences_dir;
 
-#ifndef __AMIGAOS4__
-	static const char* const current_dir = ".";
-	const char* home_str = getenv("HOME");
-#else
-	static const char* const current_dir = " ";
-	const char* home_str = "PROGDIR:";
-#endif
-	if(home_str == NULL)
-		home_str = current_dir;
-
-	const std::string home(home_str);
-
-#ifndef PREFERENCES_DIR
-#define PREFERENCES_DIR ".wesnoth"
-#endif
-
-#ifndef __AMIGAOS4__
-	const std::string dir_path = home + std::string("/") + PREFERENCES_DIR;
-#else
-	const std::string dir_path = home + PREFERENCES_DIR;
-#endif
-	DIR* dir = opendir(dir_path.c_str());
+	const bool res = create_directory_if_missing(dir_path);
+	// probe read permissions (if we could make the directory)
+	DIR* const dir = res ? opendir(dir_path.c_str()) : NULL;
 	if(dir == NULL) {
-		const int res = mkdir(dir_path.c_str(),AccessMode);
-
-		// Also create the maps directory
-		mkdir((dir_path + "/editor").c_str(),AccessMode);
-		mkdir((dir_path + "/editor/maps").c_str(),AccessMode);
-		mkdir((dir_path + "/data").c_str(),AccessMode);
-		mkdir((dir_path + "/data/ais").c_str(),AccessMode);
-		mkdir((dir_path + "/data/scenarios").c_str(),AccessMode);
-		mkdir((dir_path + "/data/scenarios/multiplayer").c_str(),AccessMode);
-		mkdir((dir_path + "/data/maps").c_str(),AccessMode);
-		mkdir((dir_path + "/data/maps/multiplayer").c_str(),AccessMode);
-		mkdir((dir_path + "/saves").c_str(),AccessMode);
-		if(res == 0) {
-			dir = opendir(dir_path.c_str());
-		} else {
-			ERR_FS << "could not open or create directory: " << dir_path << '\n';
-		}
-	}
-
-	if(dir == NULL)
+		ERR_FS << "could not open or create preferences directory at " << dir_path << '\n';
 		return "";
-
+	}
 	closedir(dir);
 
+	// Create user data and add-on directories
+	create_directory_if_missing(dir_path + "/editor");
+	create_directory_if_missing(dir_path + "/editor/maps");
+	create_directory_if_missing(dir_path + "/data");
+	create_directory_if_missing(dir_path + "/data/ais");
+	create_directory_if_missing(dir_path + "/data/campaigns");
+	create_directory_if_missing(dir_path + "/data/multiplayer");
+	create_directory_if_missing(dir_path + "/data/maps");
+	create_directory_if_missing(dir_path + "/data/maps/multiplayer");
+	create_directory_if_missing(dir_path + "/data/units");
+	create_directory_if_missing(dir_path + "/saves");
+
 	return dir_path;
 #endif
 }
 
+const std::string& get_user_data_dir()
+{
+	// ensure setup gets called only once per session
+	// FIXME: this is okay and optimized, but how should we react
+	// if the user deletes a dir while we are running?
+	if (user_data_dir.empty())
+	{
+		user_data_dir = setup_user_data_dir();
+	}
+	return user_data_dir;
+}
+
 static std::string read_stream(std::istream& s)
 {
 	std::stringstream ss;
@@ -688,27 +613,37 @@ static std::string read_stream(std::istream& s)
 	return ss.str();
 }
 
-std::istream *istream_file(std::string const &fname)
+std::istream *istream_file(const std::string &fname)
 {
-	LOG_FS << "streaming " << fname << " for reading.\n";
-	if (!fname.empty() && fname[0] != '/' && !game_config::path.empty()) {
-		std::ifstream *s = new std::ifstream((game_config::path + "/" + fname).c_str(),std::ios_base::binary);
-		if (s->is_open())
-			return s;
-		LOG_FS << "could not open " << fname << " for reading.\n";
-		delete s;
+	LOG_FS << "Streaming " << fname << " for reading.\n";
+	if (fname.empty())
+	{
+		ERR_FS << "Trying to open file with empty name.\n";
+		std::ifstream *s = new std::ifstream();
+		s->clear(std::ios_base::failbit);
+		return s;
 	}
-
-	//! @todo FIXME: why do we rely on this even with relative paths ?
-	// Still useful with zipios, for things like cache and prefs.
-	// NOTE zipios has been removed - not sure what to do with this code.
-	std::istream *s = new std::ifstream(fname.c_str(), std::ios_base::binary);
-	if (s->fail())
-		LOG_FS << "streaming " << fname << " failed.\n";
+#ifndef _WIN32
+	// TODO: Should also be done for Windows; but *nix systems should
+	// already be sufficient to catch most offenders.
+	if (fname[0] != '/') {
+		WRN_FS << "Trying to open file with relative path: '" << fname << "'.\n";
+#if 0
+		std::ifstream *s = new std::ifstream();
+		s->clear(std::ios_base::failbit);
+		return s;
+#endif
+	}
+#endif
+	std::ifstream *s = new std::ifstream(fname.c_str(),std::ios_base::binary);
+	if (s->is_open())
+		return s;
+	ERR_FS << "Could not open '" << fname << "' for reading.\n";
 	return s;
+
 }
 
-std::string read_file(std::string const &fname)
+std::string read_file(const std::string &fname)
 {
 	scoped_istream s = istream_file(fname);
 	return read_stream(*s);
@@ -733,7 +668,7 @@ void write_file(const std::string& fname, const std::string& data)
 	char buf[block_size];
 
 	for(size_t i = 0; i < data.size(); i += block_size) {
-		const size_t bytes = minimum<size_t>(block_size,data.size() - i);
+		const size_t bytes = std::min<size_t>(block_size,data.size() - i);
 		std::copy(data.begin() + i, data.begin() + i + bytes,buf);
 		const size_t res = fwrite(buf,1,bytes,file.get());
 		if(res != bytes) {
@@ -745,12 +680,9 @@ void write_file(const std::string& fname, const std::string& data)
 
 std::string read_map(const std::string& name)
 {
-	std::string res = read_file("data/maps/" + name);
-	if(res == "") {
-		res = read_file(get_user_data_dir() + "/data/maps/" + name);
-	}
+	std::string res = read_file(get_wml_location("maps/" + name));
 
-	if(res == "") {
+	if (res.empty()) {
 		res = read_file(get_user_data_dir() + "/editor/maps/" + name);
 	}
 
@@ -774,7 +706,6 @@ static bool is_directory_internal(const std::string& fname)
 	if(::stat(fname.c_str(), &dir_stat) == -1) {
 		return false;
 	}
-
 	return S_ISDIR(dir_stat.st_mode);
 #endif
 }
@@ -794,12 +725,13 @@ bool is_directory(const std::string& fname)
 
 bool file_exists(const std::string& name)
 {
-	std::ifstream file(name.c_str(),std::ios_base::binary);
-	if (file.rdstate() != 0) {
-		return false;
-	}
-	file.close();
-	return true;
+#ifdef _WIN32
+       struct stat st;
+       return (::stat(name.c_str(), &st) == 0);
+#else
+	struct stat st;
+	return (::stat(name.c_str(), &st) != -1);
+#endif
 }
 
 time_t file_create_time(const std::string& fname)
@@ -811,7 +743,6 @@ time_t file_create_time(const std::string& fname)
 	return buf.st_mtime;
 }
 
-//! Return the next ordered full filename within this directory.
 std::string next_filename(const std::string &dirname, unsigned int max)
 {
 	std::vector<std::string> files;
@@ -844,13 +775,15 @@ std::string next_filename(const std::string &dirname, unsigned int max)
 	return dirname + "/" + fname.str();
 }
 
-//! Returns true if the file ends with '.gz'.
-//! 
-//! @param filename     The name to test.
+/**
+ * Returns true if the file ends with '.gz'.
+ *
+ * @param filename                The name to test.
+ */
 bool is_gzip_file(const std::string& filename)
-{ 
-	return (filename.length() > 3 
-		&& filename.substr(filename.length() - 3) == ".gz"); 
+{
+	return (filename.length() > 3
+		&& filename.substr(filename.length() - 3) == ".gz");
 }
 
 file_tree_checksum::file_tree_checksum()
@@ -884,30 +817,21 @@ bool operator!=(const file_tree_checksum& lhs, const file_tree_checksum& rhs)
 
 static void get_file_tree_checksum_internal(const std::string& path, file_tree_checksum& res)
 {
-	std::vector<std::string> files, dirs;
-	get_files_in_dir(path,&files,&dirs, ENTIRE_FILE_PATH, SKIP_MEDIA_DIR);
+
+	std::vector<std::string> dirs;
+	get_files_in_dir(path,NULL,&dirs, ENTIRE_FILE_PATH, SKIP_MEDIA_DIR, DONT_REORDER, &res);
 	increment_filesystem_progress();
-	for(std::vector<std::string>::const_iterator i = files.begin(); i != files.end(); ++i) {
-		++res.nfiles;
-
-		struct stat buf;
-		if(::stat(i->c_str(),&buf) != -1) {
-			if(buf.st_mtime > res.modified) {
-				res.modified = buf.st_mtime;
-			}
-
-			res.sum_size += buf.st_size;
-		}
-	}
 
 	for(std::vector<std::string>::const_iterator j = dirs.begin(); j != dirs.end(); ++j) {
 		get_file_tree_checksum_internal(*j,res);
 	}
 }
 
-const file_tree_checksum& data_tree_checksum()
+const file_tree_checksum& data_tree_checksum(bool reset)
 {
 	static file_tree_checksum checksum;
+	if (reset)
+		checksum.reset();
 	if(checksum.nfiles == 0) {
 		get_file_tree_checksum_internal("data/",checksum);
 		get_file_tree_checksum_internal(get_user_data_dir() + "/data/",checksum);
@@ -998,9 +922,13 @@ void binary_paths_manager::set_paths(const config& cfg)
 	cleanup();
 	init_binary_paths();
 
-	const config::child_list& items = cfg.get_children("binary_path");
-	for(config::child_list::const_iterator i = items.begin(); i != items.end(); ++i) {
-		std::string path = (**i)["path"].str();
+	foreach (const config *item, cfg.get_children("binary_path"))
+	{
+		std::string path = (*item)["path"].str();
+		if (path.find("..") != std::string::npos) {
+			ERR_FS << "Invalid binary path '" << path << "'\n";
+			continue;
+		}
 		if (!path.empty() && path[path.size()-1] != '/') path += "/";
 		if(binary_paths.count(path) == 0) {
 			binary_paths.insert(path);
@@ -1030,45 +958,226 @@ const std::vector<std::string>& get_binary_paths(const std::string& type)
 		return itor->second;
 	}
 
+	if (type.find("..") != std::string::npos) {
+		// Not an assertion, as language.cpp is passing user data as type.
+		ERR_FS << "Invalid WML type '" << type << "' for binary paths\n";
+		static std::vector<std::string> dummy;
+		return dummy;
+	}
+
 	std::vector<std::string>& res = binary_paths_cache[type];
 
 	init_binary_paths();
 
-	for(std::set<std::string>::const_iterator i = binary_paths.begin(); i != binary_paths.end(); ++i) {
-		res.push_back(get_user_data_dir() + "/" + *i + type + "/");
+	foreach (const std::string &path, binary_paths)
+	{
+		res.push_back(get_user_data_dir() + "/" + path + type + "/");
 
 		if(!game_config::path.empty()) {
-			res.push_back(game_config::path + "/" + *i + type + "/");
+			res.push_back(game_config::path + "/" + path + type + "/");
 		}
-
-		res.push_back(*i + type + "/");
 	}
 
+	// not found in "/type" directory, try main directory
+	res.push_back(get_user_data_dir());
+
+	if(!game_config::path.empty())
+		res.push_back(game_config::path+"/");
+
 	return res;
 }
 
 std::string get_binary_file_location(const std::string& type, const std::string& filename)
 {
-	const std::vector<std::string>& paths = get_binary_paths(type);
-	if(!filename.empty()) {
-		DBG_FS << "Looking for " << filename << " in  '.'\n";
-		if(file_exists(filename) || is_directory(filename)) {
-		  DBG_FS << "  Found at " << filename << "\n";
-			return filename;
-		}
+	DBG_FS << "Looking for '" << filename << "'.\n";
+
+	if (filename.empty()) {
+		LOG_FS << "  invalid filename (type: " << type <<")\n";
+		return std::string();
 	}
 
-	for(std::vector<std::string>::const_iterator i = paths.begin(); i != paths.end(); ++i) {
-		const std::string file = *i + filename;
-		DBG_FS << "  Checking " << *i << "\n";
-		if(file_exists(file) || is_directory(file)) {
-		  DBG_FS << "  Found at " << file << "\n";
+	// Some parts of Wesnoth enjoy putting ".." inside filenames. This is
+	// bad and should be fixed. But in the meantime, deal with them in a dumb way.
+	std::string::size_type pos = filename.rfind("../");
+	if (pos != std::string::npos) {
+		std::string nf = filename.substr(pos + 3);
+		WRN_FS << "Illegal path '" << filename << "' replaced by '" << nf << "'\n";
+		return get_binary_file_location(type, nf);
+	}
+
+	if (filename.find("..") != std::string::npos) {
+		ERR_FS << "Illegal path '" << filename << "' (\"..\" not allowed).\n";
+		return std::string();
+	}
+
+	foreach (const std::string &path, get_binary_paths(type))
+	{
+		const std::string file = path + filename;
+		DBG_FS << "  checking '" << path << "'\n";
+		if(file_exists(file)) {
+			DBG_FS << "  found at '" << file << "'\n";
 			return file;
 		}
 	}
 
-	DBG_FS << "  " << filename << " not found.\n";
-	return "";
+	DBG_FS << "  not found\n";
+	return std::string();
+}
+
+std::string get_binary_dir_location(const std::string &type, const std::string &filename)
+{
+	DBG_FS << "Looking for '" << filename << "'.\n";
+
+	if (filename.empty()) {
+		LOG_FS << "  invalid filename (type: " << type <<")\n";
+		return std::string();
+	}
+
+	if (filename.find("..") != std::string::npos) {
+		ERR_FS << "Illegal path '" << filename << "' (\"..\" not allowed).\n";
+		return std::string();
+	}
+
+	foreach (const std::string &path, get_binary_paths(type))
+	{
+		const std::string file = path + filename;
+		DBG_FS << "  checking '" << path << "'\n";
+		if (is_directory(file)) {
+			DBG_FS << "  found at '" << file << "'\n";
+			return file;
+		}
+	}
+
+	DBG_FS << "  not found\n";
+	return std::string();
+}
+
+std::string get_wml_location(const std::string &filename, const std::string &current_dir)
+{
+	DBG_FS << "Looking for '" << filename << "'.\n";
+
+	std::string result;
+
+	if (filename.empty()) {
+		LOG_FS << "  invalid filename\n";
+		return result;
+	}
+
+	if (filename.find("..") != std::string::npos) {
+		ERR_FS << "Illegal path '" << filename << "' (\"..\" not allowed).\n";
+		return result;
+	}
+
+	bool already_found = false;
+
+	if (filename[0] == '~' || filename[0] == '@')
+	{
+		// If the filename starts with '~' or '@', look in the user data directory.
+		result = get_user_data_dir() + "/data/" + filename.substr(1);
+		DBG_FS << "  trying '" << result << "'\n";
+
+		already_found = file_exists(result) || is_directory(result);
+
+		// If it starts with '@', look also in the standard data directory.
+		if (filename[0] == '@' && !already_found && !game_config::path.empty())
+			result = game_config::path + "/data/" + filename.substr(1);
+	}
+	else if (filename.size() >= 2 && filename[0] == '.' && filename[1] == '/')
+	{
+		// If the filename begins with a "./", look in the same directory
+		// as the file currrently being preprocessed.
+		result = current_dir + filename.substr(2);
+	}
+	else if (!game_config::path.empty())
+		result = game_config::path + "/data/" + filename;
+
+	DBG_FS << "  trying '" << result << "'\n";
+
+	if (result.empty() ||
+	    (!already_found && !file_exists(result) && !is_directory(result)))
+	{
+		DBG_FS << "  not found\n";
+		result.clear();
+	}
+	else
+		DBG_FS << "  found: '" << result << "'\n";
+
+	return result;
+}
+
+static bool is_path_sep(char c)
+{
+#ifdef _WIN32
+	if (c == '/' || c == '\\') return true;
+#else
+	if (c == '/') return true;
+#endif
+	return false;
+}
+
+std::string normalize_path(const std::string &p1)
+{
+	if (p1.empty()) return p1;
+
+	std::string p2;
+#ifdef _WIN32
+	if (p1.size() >= 2 && p1[1] == ':')
+		// Windows relative paths with explicit drive name are not handled.
+		p2 = p1;
+	else
+#endif
+	if (!is_path_sep(p1[0]))
+		p2 = get_cwd() + "/" + p1;
+	else
+		p2 = p1;
+
+#ifdef _WIN32
+	std::string drive;
+	if (p2.size() >= 2 && p2[1] == ':') {
+		drive = p2.substr(0, 2);
+		p2.erase(0, 2);
+	}
+#endif
+
+	std::vector<std::string> components(1);
+	for (int i = 0, i_end = p2.size(); i <= i_end; ++i)
+	{
+		std::string &last = components[components.size() - 1];
+		char c = p2[i];
+		if (is_path_sep(c) || c == 0)
+		{
+			if (last == ".")
+				last.clear();
+			else if (last == "..")
+			{
+				if (components.size() >= 2) {
+					components.pop_back();
+					components[components.size() - 1].clear();
+				} else
+					last.clear();
+			}
+			else if (!last.empty())
+				components.push_back(std::string());
+		}
+		else
+			last += c;
+	}
+
+	std::ostringstream p4;
+	components.pop_back();
+
+#ifdef _WIN32
+	p4 << drive;
+#endif
+
+	foreach (const std::string &s, components)
+	{
+		p4 << '/' << s;
+	}
+
+	DBG_FS << "Normalizing '" << p2 << "' to '" << p4.str() << "'\n";
+
+	return p4.str();
 }
 
 scoped_istream& scoped_istream::operator=(std::istream *s)
