@@ -1,7 +1,7 @@
-/* $Id: font.cpp 24636 2008-03-14 11:13:37Z shadowmaster $ */
+/* $Id: font.cpp 33687 2009-03-15 16:50:42Z mordante $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,8 @@
    See the COPYING file for more details.
 */
 
-#include "wesconfig.h"
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "config.hpp"
@@ -21,39 +22,32 @@
 #include "font.hpp"
 #include "game_config.hpp"
 #include "log.hpp"
-#include "sdl_utils.hpp"
 #include "tooltips.hpp"
-#include "util.hpp"
 #include "video.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "serialization/string_utils.hpp"
 #include "marked-up_text.hpp"
+#include "foreach.hpp"
 
-#include <algorithm>
-#include <cstdio>
-#include <iostream>
 #include <list>
-#include <map>
 #include <set>
-#include <sstream>
 #include <stack>
-#include <string>
+
+#include <cairo-features.h>
+
+#ifdef CAIRO_HAS_FT_FONT
+#include <fontconfig/fontconfig.h>
+#endif
+
+#ifdef CAIRO_HAS_WIN32_FONT
+#include <windows.h>
+#endif
 
 #define DBG_FT LOG_STREAM(debug, display)
 #define LOG_FT LOG_STREAM(info, display)
 #define WRN_FT LOG_STREAM(warn, display)
 #define ERR_FT LOG_STREAM(err, display)
 
-//Deliberately breaking compilation with the original SDL_ttf library. Remove
-//the lines below to be able to do this anyway, however this is buggy and may
-//cause random memory corruption errors depending on the OS / Version of
-//Freetype / Language you are using!
-//
-#ifndef SDL_TTF_WESNOTH
-#error Please use the SDL_ttf files in the sdl_ttf directory, and not the original SDL_ttf library.
-#endif
-
 #ifdef	HAVE_FRIBIDI
 #include <fribidi/fribidi.h>
 
@@ -75,7 +69,7 @@ struct font_id
 	};
 	bool operator<(const font_id& o) const
 	{
-		return subset < o.subset || subset == o.subset && size < o.size;
+		return subset < o.subset || (subset == o.subset && size < o.size);
 	};
 
 	subset_id subset;
@@ -87,10 +81,33 @@ std::vector<std::string> font_names;
 
 struct text_chunk
 {
-	text_chunk(subset_id subset) : subset(subset) {}
-	text_chunk(subset_id subset, std::string const & text) : subset(subset), text(text) {}
-	text_chunk(subset_id subset, ucs2_string const & ucs2_text) : subset(subset), ucs2_text(ucs2_text) {}
-	text_chunk(subset_id subset, std::string const & text, ucs2_string const & ucs2_text) : subset(subset), text(text), ucs2_text(ucs2_text) {}
+	text_chunk(subset_id subset) :
+		subset(subset),
+		text(),
+		ucs2_text()
+	{
+	}
+
+	text_chunk(subset_id subset, std::string const & text) :
+		subset(subset),
+		text(text),
+		ucs2_text()
+	{
+	}
+
+	text_chunk(subset_id subset, ucs2_string const & ucs2_text) :
+		subset(subset),
+		text(),
+		ucs2_text(ucs2_text)
+	{
+	}
+
+	text_chunk(subset_id subset, std::string const & text, ucs2_string const & ucs2_text) :
+		subset(subset),
+		text(text),
+		ucs2_text(ucs2_text)
+	{
+	}
 
 	bool operator==(text_chunk const & t) const { return subset == t.subset && ucs2_text == t.ucs2_text; }
 	bool operator!=(text_chunk const & t) const { return !operator==(t); }
@@ -274,18 +291,74 @@ manager::manager()
 	} else {
 		LOG_FT << "Initialized true type fonts\n";
 	}
+
+	init();
 }
 
 manager::~manager()
 {
+	deinit();
+
 	clear_fonts();
 	TTF_Quit();
 }
 
+void manager::update_font_path() const
+{
+	deinit();
+	init();
+}
+
+void manager::init() const
+{
+#ifdef CAIRO_HAS_FT_FONT
+	if(!FcConfigAppFontAddDir( FcConfigGetCurrent(),
+			reinterpret_cast<const FcChar8*>
+			((game_config::path + "/fonts/").c_str()))) {
+
+		ERR_FT << "Could not load the  true type fonts\n";
+		throw error();
+	}
+#endif
+
+#if CAIRO_HAS_WIN32_FONT
+	foreach(const std::string& path, get_binary_paths("fonts")) {
+		std::vector<std::string> files;
+		get_files_in_dir(path, &files, NULL, ENTIRE_FILE_PATH);
+		foreach(const std::string& file, files)
+			if(file.substr(file.length() - 4) == ".ttf")
+				AddFontResource(file.c_str());
+	}
+#endif
+}
+
+void manager::deinit() const
+{
+#ifdef CAIRO_HAS_FT_FONT
+	FcConfigAppFontClear(FcConfigGetCurrent());
+#endif
+
+#if CAIRO_HAS_WIN32_FONT
+	foreach(const std::string& path, get_binary_paths("fonts")) {
+		std::vector<std::string> files;
+		get_files_in_dir(path, &files, NULL, ENTIRE_FILE_PATH);
+		foreach(const std::string& file, files)
+			if(file.substr(file.length() - 4) == ".ttf")
+				RemoveFontResource(file.c_str());
+	}
+#endif
+}
+
 //structure used to describe a font, and the subset of the Unicode character
 //set it covers.
 struct subset_descriptor
 {
+	subset_descriptor() :
+		name(),
+		present_codepoints()
+	{
+	}
+
 	std::string name;
 	std::vector<std::pair<size_t, size_t> > present_codepoints;
 };
@@ -323,7 +396,7 @@ static void set_font_list(const std::vector<subset_descriptor>& fontlist)
 		for(cp_range = itor->present_codepoints.begin();
 				cp_range != itor->present_codepoints.end(); ++cp_range) {
 
-			size_t cp_max = maximum<size_t>(cp_range->first, cp_range->second);
+			size_t cp_max = std::max<size_t>(cp_range->first, cp_range->second);
 			if(cp_max >= font_map.size()) {
 				font_map.resize(cp_max+1, -1);
 			}
@@ -412,9 +485,21 @@ void text_surface::bidi_cvt()
 }
 #endif
 
-text_surface::text_surface(std::string const &str, int size, SDL_Color color, int style)
-  : font_size_(size), color_(color), style_(style), w_(-1), h_(-1), str_(str),
-  initialized_(false)
+text_surface::text_surface(std::string const &str, int size,
+		SDL_Color color, int style) :
+	hash_(0),
+	font_size_(size),
+	color_(color),
+	style_(style),
+	w_(-1),
+	h_(-1),
+	str_(str),
+	initialized_(false),
+	chunks_(),
+	surfs_()
+#ifdef	HAVE_FRIBIDI
+	,is_rtl_(false)
+#endif
 {
 #ifdef	HAVE_FRIBIDI
 	bidi_cvt();
@@ -422,8 +507,20 @@ text_surface::text_surface(std::string const &str, int size, SDL_Color color, in
 	hash();
 }
 
-text_surface::text_surface(int size, SDL_Color color, int style)
-  : hash_(0), font_size_(size), color_(color), style_(style), w_(-1), h_(-1), initialized_(false)
+text_surface::text_surface(int size, SDL_Color color, int style) :
+	hash_(0),
+	font_size_(size),
+	color_(color),
+	style_(style),
+	w_(-1),
+	h_(-1),
+	str_(),
+	initialized_(false),
+	chunks_(),
+	surfs_()
+#ifdef	HAVE_FRIBIDI
+	,is_rtl_(false)
+#endif
 {
 }
 
@@ -467,10 +564,10 @@ void text_surface::measure() const
 			itor->ucs2_text.push_back(0);
 		}
 
-		TTF_SizeUNICODE(ttfont, 
+		TTF_SizeUNICODE(ttfont,
 			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), &w, &h);
 		w_ += w;
-		h_ = maximum<int>(h_, h);
+		h_ = std::max<int>(h_, h);
 	}
 }
 
@@ -513,7 +610,7 @@ std::vector<surface> const &text_surface::get_surfaces() const
 			continue;
 		font_style_setter const style_setter(ttfont, style_);
 
-		surface s = surface(TTF_RenderUNICODE_Blended(ttfont, 
+		surface s = surface(TTF_RenderUNICODE_Blended(ttfont,
 			static_cast<Uint16 const *>(&(itor->ucs2_text.front())), color_));
 		if(!s.null())
 			surfs_.push_back(s);
@@ -570,25 +667,43 @@ text_surface &text_cache::find(text_surface const &t)
 
 }
 
-static surface render_text(const std::string& text, int fontsize, const SDL_Color& colour, int style)
+static surface render_text(const std::string& text, int fontsize, const SDL_Color& colour, int style, bool use_markup)
 {
-	const std::vector<std::string> lines = utils::split(text, '\n', utils::REMOVE_EMPTY);
+	// we keep blank lines and spaces (may be wanted for indentation)
+	const std::vector<std::string> lines = utils::split(text, '\n', 0);
 	std::vector<std::vector<surface> > surfaces;
 	surfaces.reserve(lines.size());
 	size_t width = 0, height = 0;
-	text_surface txt_surf(fontsize, colour, style);
 
 	for(std::vector< std::string >::const_iterator ln = lines.begin(), ln_end = lines.end(); ln != ln_end; ++ln) {
-		if (!ln->empty()) {
-			txt_surf.set_text(*ln);
-			const text_surface& cached_surf = text_cache::find(txt_surf);
-			const std::vector<surface>&res = cached_surf.get_surfaces();
 
-			if (!res.empty()) {
-				surfaces.push_back(res);
-				width = maximum<size_t>(cached_surf.width(), width);
-				height += cached_surf.height();
-			}
+		SDL_Color col = colour;
+		int sz = fontsize;
+		int text_style = style;
+
+		std::string::const_iterator after_markup = use_markup ?
+			parse_markup(ln->begin(),ln->end(),&sz,&col,&text_style) : ln->begin();
+		text_surface txt_surf(sz, col, text_style);
+
+		if (after_markup == ln->end() && ln+1 != ln_end) {
+			// we replace empty line by a space (to have a line height)
+			// except for the blank following the last '\n'
+			txt_surf.set_text(" ");
+		} else if (after_markup == ln->begin()) {
+		 	// simple case, no markup to skip
+			txt_surf.set_text(*ln);
+		} else  {
+			const std::string line(after_markup,ln->end());
+			txt_surf.set_text(line);
+		}
+
+		const text_surface& cached_surf = text_cache::find(txt_surf);
+		const std::vector<surface>&res = cached_surf.get_surfaces();
+
+		if (!res.empty()) {
+			surfaces.push_back(res);
+			width = std::max<size_t>(cached_surf.width(), width);
+			height += cached_surf.height();
 		}
 	}
 
@@ -616,7 +731,7 @@ static surface render_text(const std::string& text, int fontsize, const SDL_Colo
 				SDL_Rect dstrect = {xpos, ypos, 0, 0};
 				SDL_BlitSurface(*j, NULL, res, &dstrect);
 				xpos += (*j)->w;
-				height = maximum<size_t>((*j)->h, height);
+				height = std::max<size_t>((*j)->h, height);
 			}
 			ypos += height;
 		}
@@ -628,22 +743,29 @@ static surface render_text(const std::string& text, int fontsize, const SDL_Colo
 
 surface get_rendered_text(const std::string& str, int size, const SDL_Color& colour, int style)
 {
-	return render_text(str, size, colour, style);
+	// TODO maybe later also to parse markup here, but a lot to check
+	return render_text(str, size, colour, style, false);
 }
 
 SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 		   const SDL_Color& colour, const std::string& text,
 		   int x, int y, bool use_tooltips, int style)
 {
-	const std::string etext = make_text_ellipsis(text, size, area.w);
-
 	if (gui_surface.null()) {
 		text_surface const &u = text_cache::find(text_surface(text, size, colour, style));
 		SDL_Rect res = {0, 0, u.width(), u.height()};
 		return res;
 	}
 
-	surface surface(render_text(etext,size,colour,style));
+	if(area.w == 0) {  // no place to draw
+		SDL_Rect res = {0,0,0,0};
+		return res;
+	}
+
+	const std::string etext = make_text_ellipsis(text, size, area.w);
+
+	// for the main current use, we already parsed markup
+	surface surface(render_text(etext,size,colour,style,false));
 	if(surface == NULL) {
 		SDL_Rect res = {0,0,0,0};
 		return res;
@@ -695,21 +817,6 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 	return dest;
 }
 
-SDL_Rect draw_text_line(CVideo* gui, const SDL_Rect& area, int size,
-                        const SDL_Color& colour, const std::string& text,
-                        int x, int y, bool use_tooltips, int style)
-{
-	surface surface;
-
-	if(gui == NULL) {
-		surface = NULL;
-	} else {
-		surface = gui->getSurface();
-	}
-
-	return draw_text_line(surface, area, size, colour, text, x, y, use_tooltips, style);
-}
-
 int get_max_height(int size)
 {
 	// Only returns the maximal size of the first font
@@ -746,13 +853,19 @@ SDL_Rect line_size(const std::string& line, int font_size, int style)
 	return res;
 }
 
-std::string make_text_ellipsis(const std::string &text, int font_size, int max_width, bool with_tags)
+std::string make_text_ellipsis(const std::string &text, int font_size,
+		int max_width, bool with_tags, bool parse_for_style)
 {
 	static const std::string ellipsis = "...";
 
-	if(line_width(with_tags ? text : del_tags(text), font_size) <= max_width)
+	SDL_Color unused_color;
+	int unused_int;
+	int style = TTF_STYLE_NORMAL;
+	if(parse_for_style) parse_markup(text.begin(), text.end(), &unused_int, &unused_color, &style);
+
+	if(line_width(with_tags ? text : del_tags(text), font_size, style) <= max_width)
 		return text;
-	if(line_width(ellipsis, font_size) > max_width)
+	if(line_width(ellipsis, font_size, style) > max_width)
 		return "";
 
 	std::string current_substring;
@@ -764,7 +877,7 @@ std::string make_text_ellipsis(const std::string &text, int font_size, int max_w
 		tmp.append(itor.substr().first, itor.substr().second);
 		tmp += ellipsis;
 
-		if (line_width(with_tags ? tmp : del_tags(tmp), font_size) > max_width) {
+		if (line_width(with_tags ? tmp : del_tags(tmp), font_size, style) > max_width) {
 			return current_substring + ellipsis;
 		}
 
@@ -785,7 +898,7 @@ public:
 	floating_label(const std::string& text, int font_size, const SDL_Color& colour, const SDL_Color& bgcolour,
 			double xpos, double ypos, double xmove, double ymove, int lifetime, const SDL_Rect& clip_rect,
 			font::ALIGN align, int border_size, bool scroll_with_map)
-		: surf_(NULL), buf_(NULL), foreground_(NULL), text_(text), font_size_(font_size), colour_(colour),
+		: surf_(NULL), buf_(NULL), text_(text), font_size_(font_size), colour_(colour),
 		bgcolour_(bgcolour), bgalpha_(bgcolour.unused), xpos_(xpos), ypos_(ypos),
 		xmove_(xmove), ymove_(ymove), lifetime_(lifetime), clip_rect_(clip_rect),
 		alpha_change_(-255/lifetime), visible_(true), align_(align), border_(border_size), scroll_(scroll_with_map)
@@ -809,7 +922,7 @@ private:
 
 	int xpos(size_t width) const;
 
-	surface surf_, buf_, foreground_;
+	surface surf_, buf_;
 	std::string text_;
 	int font_size_;
 	SDL_Color colour_, bgcolour_;
@@ -851,30 +964,56 @@ int floating_label::xpos(size_t width) const
 surface floating_label::create_surface()
 {
 	if (surf_.null()) {
-		foreground_ = font::render_text(text_, font_size_, colour_, 0);
+		surface foreground = font::render_text(text_, font_size_, colour_, 0, true);
 
-		if(foreground_ == NULL) {
+		if(foreground == NULL) {
+			ERR_FT << "could not create floating label's text" << std::endl;
 			return NULL;
 		}
 
-		//if the surface has to be created onto some kind of background, then do that here
+		// combine foreground text with its background
 		if(bgalpha_ != 0) {
-			surface tmp(create_compatible_surface(foreground_,foreground_->w+border_*2,foreground_->h+border_*2));
-			if(tmp == NULL) {
-				return NULL;
+			// background is a dark tootlip box
+			surface background = create_neutral_surface(foreground->w + border_*2, foreground->h + border_*2);
+
+			if (background == NULL) {
+				ERR_FT << "could not create tooltip box" << std::endl;
+				surf_ = create_optimized_surface(foreground);
+				return surf_;
 			}
 
-			SDL_FillRect(tmp,NULL,SDL_MapRGBA(tmp->format,bgcolour_.r,bgcolour_.g,bgcolour_.b, bgalpha_));
+			Uint32 color = SDL_MapRGBA(foreground->format, bgcolour_.r,bgcolour_.g, bgcolour_.b, bgalpha_);
+			SDL_FillRect(background,NULL, color);
 
-			surf_.assign(tmp);
-		} else {
-			surface background = font::render_text(text_, font_size_, font::BLACK_COLOUR, 0);
-			background = blur_alpha_surface(background,2);
-			background = adjust_surface_alpha(background, ftofxp(4.0));
+			// we make the text less transparent, because the blitting on the
+			// dark background will darken the anti-aliased part.
+			// This 1.13 value seems to restore the brightness of version 1.4
+			// (where the text was blitted directly on screen)
+			foreground = adjust_surface_alpha(foreground, ftofxp(1.13), false);
 
-			surf_.assign(background);
+			SDL_Rect r = { border_, border_, 0, 0 };
+			SDL_SetAlpha(foreground,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
+			blit_surface(foreground, NULL, background, &r);
+
+			surf_ = create_optimized_surface(background);
+			// RLE compression seems less efficient for big semi-transparent area
+			// so, remove it for this case, but keep the optimized display format
+			SDL_SetAlpha(surf_,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
 		}
+		else {
+			// background is blurred shadow of the text
+			//TODO add a little extra space for the blur of letters with a lower part
+			surface background = shadow_image(foreground, false);
 
+			if (background == NULL) {
+				ERR_FT << "could not create floating label's shadow" << std::endl;
+				surf_ = create_optimized_surface(foreground);
+				return surf_;
+			}
+			SDL_SetAlpha(foreground,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
+			blit_surface(foreground, NULL, background, NULL);
+			surf_ = create_optimized_surface(background);
+		}
 	}
 
 	return surf_;
@@ -908,11 +1047,6 @@ void floating_label::draw(surface screen)
 	SDL_BlitSurface(screen,&rect,buf_,NULL);
 	SDL_BlitSurface(surf_,NULL,screen,&rect);
 
-	if(foreground_ != NULL) {
-		SDL_Rect rect = {xpos(surf_->w)+border_,int(ypos_)+border_,foreground_->w,foreground_->h};
-		SDL_BlitSurface(foreground_,NULL,screen,&rect);
-	}
-
 	update_rect(rect);
 }
 
@@ -931,14 +1065,10 @@ void floating_label::undraw(surface screen)
 	move(xmove_,ymove_);
 	if(lifetime_ > 0) {
 		--lifetime_;
-		if(alpha_change_ != 0 && (xmove_ != 0.0 || ymove_ != 0.0)) {
-			// we don't compress these surfaces since they will always change
-			if (!surf_.null()) {
-				surf_.assign(adjust_surface_alpha_add(surf_,alpha_change_,false));
-			}
-			if (!foreground_.null()) {
-				foreground_.assign(adjust_surface_alpha_add(foreground_,alpha_change_,false));
-			}
+		if(alpha_change_ != 0 && (xmove_ != 0.0 || ymove_ != 0.0) && surf_ != NULL) {
+			// fade out moving floating labels
+			// note that we don't optimize these surfaces since they will always change
+			surf_.assign(adjust_surface_alpha_add(surf_,alpha_change_,false));
 		}
 	}
 }
@@ -1121,13 +1251,17 @@ static bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_
 
 namespace font {
 
+namespace {
+	t_string font_order;
+} // namespace
+
 bool load_font_config()
 {
 	//read font config separately, so we do not have to re-read the whole
 	//config when changing languages
 	config cfg;
 	try {
-		scoped_istream stream = preprocess_file("data/hardwired/fonts.cfg");
+		scoped_istream stream = preprocess_file(get_wml_location("hardwired/fonts.cfg"));
 		read(cfg, *stream);
 	} catch(config::error &e) {
 		ERR_FT << "could not read fonts.cfg:\n"
@@ -1145,6 +1279,7 @@ bool load_font_config()
 		known_fonts.insert((**child)["name"]);
 	}
 
+	font_order = (*fonts_config)["order"];
 	const std::vector<std::string> font_order = utils::split((*fonts_config)["order"]);
 	std::vector<font::subset_descriptor> fontlist;
 	std::vector<std::string>::const_iterator font;
@@ -1164,6 +1299,11 @@ bool load_font_config()
 	return true;
 }
 
+const t_string& get_fonts()
+{
+	return font_order;
+}
+
 void cache_mode(CACHE mode)
 {
 	if(mode == CACHE_LOBBY) {
