@@ -1,6 +1,6 @@
-/* $Id: font.hpp 23985 2008-02-21 08:15:14Z mythological $ */
+/* $Id: font.hpp 32635 2009-02-07 13:00:40Z baufo $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,9 +15,10 @@
 #define FONT_HPP_INCLUDED
 
 #include "SDL.h"
-#include "sdl_ttf/SDL_ttf.h"
+#include "SDL_ttf.h"
 
 class CVideo;
+class t_string;
 #include "sdl_utils.hpp"
 #include "serialization/string_utils.hpp"
 
@@ -30,7 +31,21 @@ namespace font {
 struct manager {
 	manager();
 	~manager();
+
+	/**
+	 * Updates the font path, when initialized it sets the fontpath to
+	 * game_config::path. When this path is updated, this function should be
+	 * called.
+	 */
+	void update_font_path() const;
+
 	struct error {};
+private:
+	/** Initializes the font path. */
+	void init() const;
+
+	/** Deinitializes the font path. */
+	void deinit() const;
 };
 
 //various standard colours
@@ -79,10 +94,6 @@ SDL_Rect draw_text_line(surface gui_surface, const SDL_Rect& area, int size,
 						const SDL_Color& colour, const std::string& text,
 						int x, int y, bool use_tooltips, int style);
 
-SDL_Rect draw_text_line(CVideo* gui, const SDL_Rect& area, int size,
-                        const SDL_Color& colour, const std::string& text,
-                        int x, int y, bool use_tooltips, int style);
-
 // Returns the maximum height of a font, in pixels
 int get_max_height(int size);
 
@@ -101,8 +112,11 @@ SDL_Rect line_size(const std::string& line, int font_size, int style=TTF_STYLE_N
 ///
 /// If the text excedes the specified max width, end it with an ellipsis (...)
 /// The with_tags can probably always be set to false
+/// If parse_for_style is true we look for the style of the text (bold, etc.)
+/// before deleting its tags.
 ///
-std::string make_text_ellipsis(const std::string& text, int font_size, int max_width, bool with_tags = true);
+std::string make_text_ellipsis(const std::string& text, int font_size, int max_width,
+		bool with_tags = true, bool parse_for_style = false);
 
 
 /// structure which will hide all current floating labels, and cause floating labels
@@ -152,6 +166,9 @@ void undraw_floating_labels(surface screen);
 
 bool load_font_config();
 
+/** Returns the currently defined fonts. */
+const t_string& get_fonts();
+
 enum CACHE { CACHE_LOBBY, CACHE_GAME };
 void cache_mode(CACHE mode);
 
