@@ -1,6 +1,6 @@
-/* $Id: game_config.cpp 26448 2008-05-07 17:35:33Z mordante $ */
+/* $Id: game_config.cpp 33403 2009-03-07 22:14:56Z ai0867 $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,19 +13,19 @@
 */
 
 #include "global.hpp"
+#include "game_config.hpp"
 
 #include "config.hpp"
-#include "log.hpp"
 #include "gettext.hpp"
-#include "game_config.hpp"
+#include "log.hpp"
 #include "util.hpp"
-#include "serialization/string_utils.hpp"
+#include "version.hpp"
 #include "wesconfig.h"
+#include "serialization/string_utils.hpp"
+#ifdef HAVE_REVISION
+#include "revision.hpp"
+#endif /* HAVE_REVISION */
 
-#include <cstdlib>
-#include <string>
-#include <sstream>
-#include <ostream>
 
 #define DBG_NG LOG_STREAM(debug, engine)
 #define ERR_NG LOG_STREAM(err, engine)
@@ -42,21 +42,26 @@ namespace game_config
 	const int gold_carryover_percentage = 80;
 	const bool gold_carryover_add = false;
 	const std::string version = VERSION;
-#ifdef SVNREV
-	const std::string svnrev = SVNREV;
+#ifdef REVISION
+	const std::string revision = VERSION " (" REVISION ")";
 #else
-	const std::string svnrev = "";
+	const std::string revision = VERSION;
 #endif
-	bool debug = false, editor = false, ignore_replay_errors = false, mp_debug = false, exit_at_end = false, no_delay = false, disable_autosave = false;
+	std::string wesnothd_name;
+	bool debug = false, editor = false, ignore_replay_errors = false, mp_debug = false, exit_at_end = false, no_delay = false, small_gui = false, disable_autosave = false;
 
-	std::string game_icon = "wesnoth-icon.png", game_title, game_logo, title_music, lobby_music;
+	bool use_dummylocales = false;
+
+	int cache_compression_level = 6;
+
+	std::string game_icon = "wesnoth-icon-small.png", game_title, game_logo, title_music, lobby_music;
 	int title_logo_x = 0, title_logo_y = 0, title_buttons_x = 0, title_buttons_y = 0, title_buttons_padding = 0,
 	    title_tip_x = 0, title_tip_width = 0, title_tip_padding = 0;
 
 	std::string terrain_mask_image = "terrain/alphamask.png";
 	std::string grid_image = "terrain/grid.png";
-	std::string unreachable_image = "terrain/darken.png";    //!< overlay image for unreachable tiles
-	std::string linger_image = "terrain/darken-linger.png";  //!< overlay image for tiles in linger mode
+	std::string unreachable_image = "terrain/darken.png";    /**< overlay image for unreachable tiles. */
+	std::string linger_image = "terrain/darken-linger.png";  /**< overlay image for tiles in linger mode. */
 
 	std::string energy_image = "misc/bar-energy.png";
 	std::string moved_ball_image = "misc/ball-moved.png";
@@ -87,14 +92,17 @@ namespace game_config
 	std::string level_image;
 	std::string ellipsis_image;
 
+	std::string default_victory_music;
+	std::string default_defeat_music;
+
 	std::map<std::string, color_range > team_rgb_range;
 	std::map<std::string, t_string > team_rgb_name;
 
 	std::map<std::string, std::vector<Uint32> > team_rgb_colors;
 
-	const struct game_version wesnoth_version(VERSION);
-	const struct game_version min_savegame_version(MIN_SAVEGAME_VERSION);
-	const struct game_version test_version("test");
+	const version_info wesnoth_version(VERSION);
+	const version_info min_savegame_version(MIN_SAVEGAME_VERSION);
+	const std::string  test_version("test");
 
 	const std::string observer_team_name = "observer";
 
@@ -131,6 +139,9 @@ namespace game_config
 	std::string path = "";
 #endif
 #endif
+
+	std::string preferences_dir = "";
+
 	std::vector<server_info> server_list;
 
 	void load_config(const config* cfg)
@@ -191,6 +202,8 @@ namespace game_config
 
 		level_image = v["level_image"];
 		ellipsis_image = v["ellipsis_image"];
+		default_victory_music = v["default_victory_music"];
+		default_defeat_music = v["default_defeat_music"];
 
 		add_color_info(v);
 
@@ -212,9 +225,13 @@ namespace game_config
 			sinf.address = (**server)["address"];
 			server_list.push_back(sinf);
 		}
+		if (!v["wesnothd_name"].empty())
+		{
+			game_config::wesnothd_name = v["wesnothd_name"];
+		}
 	}
 
-	const void add_color_info(const config& v){
+	void add_color_info(const config& v){
 		const config::child_list& team_colors = v.get_children("color_range");
 		for(config::child_list::const_iterator teamC = team_colors.begin(); teamC != team_colors.end(); ++teamC) {
 			if(!(**teamC)["id"].empty() && !(**teamC)["rgb"].empty()){
@@ -290,59 +307,22 @@ namespace game_config
 		return i->second;
 	}
 
-game_version::game_version(std::string str) :
-		major_nr(0), minor_nr(0), patch(0), extra(""), full(str)
-{
-
-	size_t offset = str.find_first_not_of("0123456789");
-	major_nr = lexical_cast_default<unsigned int>(std::string(str, 0, offset), 0);
-	str.erase(0, offset + 1);
-
-	if(! str.empty()) {
-		offset = str.find_first_not_of("0123456789");
-		minor_nr = lexical_cast_default<unsigned int>(std::string(str, 0, offset ), 0);
-		str.erase(0, offset + 1);
-	}
-
-	if(! str.empty()) {
-		offset = str.find_first_not_of("0123456789");
-		patch = lexical_cast_default<unsigned int>(std::string(str, 0, offset ), 0);
-		if(offset != std::string::npos) {
-			extra = std::string(str, offset, std::string::npos);
+	bool is_compatible_savegame_version(const std::string& v)
+	{
+		bool ret = v==version;
+		if(ret) {
+			return true;
+		} else {
+			// do not load if too old, if either the savegame or the current game
+			// has the version 'test' allow loading
+			try {
+				ret = !(v < min_savegame_version && test_version != v && test_version != version);
+			} catch(version_info::not_sane_exception&) {
+				ERR_NG << "invalid version_info from saved game: '" << v << "'\n";
+				ret = false;
+			}
 		}
+		return ret;
 	}
-}
-
-bool operator<(const struct game_version& a, const struct game_version& b)
-{
-	if(a.major_nr != b.major_nr) return a.major_nr < b.major_nr;
-	if(a.minor_nr != b.minor_nr) return a.minor_nr < b.minor_nr;
-	return a.patch < b.patch;
-}
-
-bool operator<=(const struct game_version& a, const struct game_version& b)
-{
-	return a < b || a == b;
-}
-
-bool operator>(const struct game_version& a, const struct game_version& b)
-{
-	return !(a <= b);
-}
-
-bool operator>=(const struct game_version& a, const struct game_version& b)
-{
-	return !(a < b);
-}
-
-bool operator==(const struct game_version& a, const struct game_version& b)
-{
-	return a.full == b.full;
-}
-
-bool operator!=(const struct game_version& a, const struct game_version& b)
-{
-	return a.full != b.full;
-}
 
 } // game_config
