@@ -1,6 +1,6 @@
-/* $Id: game_config.hpp 24103 2008-02-27 00:14:53Z soliton $ */
+/* $Id: game_config.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,6 +15,7 @@
 #define GAME_CONFIG_H_INCLUDED
 
 class config;
+class version_info;
 
 #include "color_range.hpp"
 #include "tstring.hpp"
@@ -34,24 +35,33 @@ namespace game_config
 	extern int kill_experience;
 	extern unsigned lobby_refresh;
 	extern const std::string version;
-	extern const std::string svnrev;
+	extern const std::string revision;
 
-	//! Default percentage gold carried over to the next scenario.
+	extern std::string wesnothd_name;
+
+	/** Default percentage gold carried over to the next scenario. */
 	extern const int gold_carryover_percentage;
 
-	//! If true the carried over gold is added to the start gold
-	//! in the next scenario, otherwise it uses the maximum of
-	//! starting gold and carryover gold.
+	/**
+	 * If true the carried over gold is added to the start gold in the next
+	 * scenario, otherwise it uses the maximum of starting gold and carryover
+	 * gold.
+	 */
 	extern const bool gold_carryover_add;
 
-	extern bool debug, editor, ignore_replay_errors, mp_debug, exit_at_end, no_delay, disable_autosave;
+	extern bool debug, editor, ignore_replay_errors, mp_debug, exit_at_end, no_delay, small_gui, disable_autosave;
+
+	extern bool use_dummylocales;
+
+	extern int cache_compression_level;
 
 	extern std::string path;
+	extern std::string preferences_dir;
 
 	struct server_info {
 		server_info() : name(""), address("") { }
 		std::string name;
-		std::string address; //!< may include ':' followed by port number
+		std::string address; /**< may include ':' followed by port number */
 	};
 	extern std::vector<server_info> server_list;
 
@@ -62,7 +72,7 @@ namespace game_config
 		terrain_mask_image, grid_image, unreachable_image, linger_image,
 		observer_image, tod_bright_image,
 		checked_menu_image, unchecked_menu_image, wml_menu_image, level_image,
-		ellipsis_image;
+		ellipsis_image, default_victory_music, default_defeat_music;
 
 	extern double hp_bar_scaling, xp_bar_scaling;
 
@@ -78,11 +88,13 @@ namespace game_config
 	extern std::map<std::string, t_string> team_rgb_name;
 	extern std::map<std::string, std::vector<Uint32> > team_rgb_colors;
 
-	//! observer team name used for observer team chat
+	/** observer team name used for observer team chat */
 	extern const std::string observer_team_name;
 
-	//! The maximum number of hexes on a map and items in an array and also
-	//! used as maximum in wml loops.
+	/**
+	 * The maximum number of hexes on a map and items in an array and also used
+	 * as maximum in wml loops.
+	 */
 	extern const size_t max_loop;
 
 	namespace sounds {
@@ -96,36 +108,20 @@ namespace game_config
 
 	void load_config(const config* cfg);
 
-	const void add_color_info(const config& v);
+	void add_color_info(const config& v);
 	const std::vector<Uint32>& tc_info(const std::string& name);
 	const color_range& color_info(const std::string& name);
 
-	struct game_version {
-		game_version(std::string str);
-
-		//Note gcc 4.1.2(prerelease) as shipped in Debian etch doesn't
-		//like the name major and minor :( so make gcc happy
-		unsigned int major_nr;
-		unsigned int minor_nr;
-		unsigned int patch;
-		std::string extra;
-
-		std::string full;
-	};
-
-	// Note the < <= > and >= operator ignore the extra version the == and != do
-	// use the extra version.
-	bool operator<(const struct game_version& a, const struct game_version& b);
-	bool operator<=(const struct game_version& a, const struct game_version& b);
-	bool operator>(const struct game_version& a, const struct game_version& b);
-	bool operator>=(const struct game_version& a, const struct game_version& b);
-	bool operator==(const struct game_version& a, const struct game_version& b);
-	bool operator!=(const struct game_version& a, const struct game_version& b);
-
-	extern const struct game_version wesnoth_version;
-	extern const struct game_version min_savegame_version;
-	extern const struct game_version test_version;
+	extern const version_info wesnoth_version;
+	extern const version_info min_savegame_version;
+	extern const std::string  test_version;
 
+	/**
+	 * Tests whether the given version is compatible with the
+	 * running version, i.e. >= MIN_SAVEGAME_VERSION.
+	 * See wesconfig.h for the criteria used for this evaluation.
+	 */
+	bool is_compatible_savegame_version(const std::string& v);
 }
 
 #endif
