@@ -1,6 +1,6 @@
-/* $Id: game_preferences_display.cpp 24144 2008-02-28 17:36:51Z soliton $ */
+/* $Id: game_preferences_display.cpp 33157 2009-02-28 16:12:49Z ivanovic $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,26 +15,15 @@
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
-#include "cursor.hpp"
 #include "display.hpp"
-#include "game_preferences.hpp"
+#include "filesystem.hpp"
+#include "filechooser.hpp"
 #include "gettext.hpp"
-#include "hotkeys.hpp"
-#include "marked-up_text.hpp"
 #include "preferences_display.hpp"
-#include "construct_dialog.hpp"
-#include "video.hpp"
 #include "wml_separators.hpp"
-#include "widgets/button.hpp"
-#include "widgets/label.hpp"
-#include "widgets/menu.hpp"
 #include "widgets/slider.hpp"
-#include "widgets/textbox.hpp"
-#include "scoped_resource.hpp"
-#include "theme.hpp"
+#include "formula_string_utils.hpp"
 
-#include <vector>
-#include <string>
 
 namespace preferences {
 
@@ -103,22 +92,23 @@ private:
 //
 	// change
 	gui::slider music_slider_, sound_slider_, UI_sound_slider_, bell_slider_,
-	            scroll_slider_, gamma_slider_, chat_lines_slider_,
+	            scroll_slider_, chat_lines_slider_,
 	  buffer_size_slider_, idle_anim_slider_, autosavemax_slider_;
 	gui::list_slider<double> turbo_slider_;
-	gui::button fullscreen_button_, turbo_button_, show_ai_moves_button_,
+	gui::button fullscreen_button_, scroll_to_action_button_,turbo_button_, show_ai_moves_button_,
 			show_grid_button_, save_replays_button_, delete_saves_button_,
 			show_lobby_joins_button1_,
 			show_lobby_joins_button2_,
 			show_lobby_joins_button3_,
 			sort_list_by_group_button_, iconize_list_button_,
+			remember_pw_button_, mp_server_search_button_,
 			friends_list_button_, friends_back_button_,
 			friends_add_friend_button_, friends_add_ignore_button_,
 			friends_remove_button_, show_floating_labels_button_,
 			turn_dialog_button_, turn_bell_button_,
 			show_team_colours_button_, show_colour_cursors_button_,
 			show_haloing_button_, video_mode_button_,
-			theme_button_, hotkeys_button_, gamma_button_,
+			theme_button_, hotkeys_button_,
 			flip_time_button_, advanced_button_, sound_button_,
 			music_button_, chat_timestamp_button_,
 			advanced_sound_button_, normal_sound_button_,
@@ -126,7 +116,7 @@ private:
 			sample_rate_button2_, sample_rate_button3_,
 			confirm_sound_button_, idle_anim_button_;
 	gui::label music_label_, sound_label_, UI_sound_label_, bell_label_,
-	           scroll_label_, gamma_label_, chat_lines_label_,
+	           scroll_label_, chat_lines_label_,
 	           turbo_slider_label_, sample_rate_label_, buffer_size_label_,
 	  idle_anim_slider_label_, autosavemax_slider_label_;
 	gui::textbox sample_rate_input_, friends_input_;
@@ -149,15 +139,17 @@ public:
 //change
 preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	: gui::preview_pane(disp.video()),
+	  friends_names_(),
 	  music_slider_(disp.video()), sound_slider_(disp.video()),
 	  UI_sound_slider_(disp.video()), bell_slider_(disp.video()),
-	  scroll_slider_(disp.video()), gamma_slider_(disp.video()),
+	  scroll_slider_(disp.video()),
 	  chat_lines_slider_(disp.video()), buffer_size_slider_(disp.video()),
-	  idle_anim_slider_(disp.video()), autosavemax_slider_(disp.video()), 
+	  idle_anim_slider_(disp.video()), autosavemax_slider_(disp.video()),
 	  turbo_slider_(disp.video()),
 
 
 	  fullscreen_button_(disp.video(), _("Toggle Full Screen"), gui::button::TYPE_CHECK),
+	  scroll_to_action_button_(disp.video(), _("Enable scroll tracking of unit actions"), gui::button::TYPE_CHECK),
 	  turbo_button_(disp.video(), _("Accelerated Speed"), gui::button::TYPE_CHECK),
 	  show_ai_moves_button_(disp.video(), _("Skip AI Moves"), gui::button::TYPE_CHECK),
 	  show_grid_button_(disp.video(), _("Show Grid"), gui::button::TYPE_CHECK),
@@ -168,6 +160,8 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	  show_lobby_joins_button3_(disp.video(), _("Show All Lobby Joins"), gui::button::TYPE_CHECK),
 	  sort_list_by_group_button_(disp.video(), _("Sort Lobby List"), gui::button::TYPE_CHECK),
 	  iconize_list_button_(disp.video(), _("Iconize Lobby List"), gui::button::TYPE_CHECK),
+	  remember_pw_button_(disp.video(), _("Save password to preferences (clear text)"), gui::button::TYPE_CHECK),
+	  mp_server_search_button_(disp.video(), _("Set path to wesnothd")),
 	  friends_list_button_(disp.video(), _("Friends List")),
 	  friends_back_button_(disp.video(), _("Multiplayer Options")),
 	  friends_add_friend_button_(disp.video(), _("Add As Friend")),
@@ -182,7 +176,6 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	  video_mode_button_(disp.video(), _("Change Resolution")),
 	  theme_button_(disp.video(), _("Theme")),
 	  hotkeys_button_(disp.video(), _("Hotkeys")),
-	  gamma_button_(disp.video(), _("Adjust Gamma"), gui::button::TYPE_CHECK),
 	  flip_time_button_(disp.video(), _("Reverse Time Graphics"), gui::button::TYPE_CHECK),
 	  advanced_button_(disp.video(), "", gui::button::TYPE_CHECK),
 	  sound_button_(disp.video(), _("Sound effects"), gui::button::TYPE_CHECK),
@@ -200,7 +193,7 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	  music_label_(disp.video(), _("Music Volume:")), sound_label_(disp.video(), _("SFX Volume:")),
 	  UI_sound_label_(disp.video(), _("UI Sound Volume:")),
 	  bell_label_(disp.video(), _("Bell Volume:")), scroll_label_(disp.video(), _("Scroll Speed:")),
-	  gamma_label_(disp.video(), _("Gamma:")), chat_lines_label_(disp.video(), ""),
+	  chat_lines_label_(disp.video(), ""),
 	  turbo_slider_label_(disp.video(), "", font::SIZE_SMALL ),
 	  sample_rate_label_(disp.video(), _("Sample Rate (Hz):")), buffer_size_label_(disp.video(), ""),
 	  idle_anim_slider_label_(disp.video(), _("Frequency:")),
@@ -292,17 +285,12 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	chat_timestamp_button_.set_check(chat_timestamping());
 	chat_timestamp_button_.set_help_string(_("Add a timestamp to chat messages"));
 
-	gamma_button_.set_check(adjust_gamma());
-	gamma_button_.set_help_string(_("Change the brightness of the display"));
-
-	gamma_slider_.set_min(50);
-	gamma_slider_.set_max(200);
-	gamma_slider_.set_value(gamma());
-	gamma_slider_.set_help_string(_("Change the brightness of the display"));
-
 	fullscreen_button_.set_check(fullscreen());
 	fullscreen_button_.set_help_string(_("Choose whether the game should run full screen or in a window"));
 
+	scroll_to_action_button_.set_check(scroll_to_action());
+	scroll_to_action_button_.set_help_string(_("Should the map view scroll to a unit when an action or move is animated."));
+
 	turbo_button_.set_check(turbo());
 	turbo_button_.set_help_string(_("Make units move and fight faster"));
 
@@ -355,6 +343,9 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	iconize_list_button_.set_check(iconize_list());
 	iconize_list_button_.set_help_string(_("Show icons in front of the player names in the lobby."));
 
+	remember_pw_button_.set_check(remember_password());
+	remember_pw_button_.set_help_string(_("Uncheck to delete the saved password (on exit)."));
+
 	show_lobby_joins_button1_.set_check(lobby_joins() == SHOW_NONE);
 	show_lobby_joins_button1_.set_help_string(_("Do not show messages about players joining the multiplayer lobby"));
 	show_lobby_joins_button2_.set_check(lobby_joins() == SHOW_FRIENDS);
@@ -362,6 +353,7 @@ preferences_dialog::preferences_dialog(display& disp, const config& game_cfg)
 	show_lobby_joins_button3_.set_check(lobby_joins() == SHOW_ALL);
 	show_lobby_joins_button3_.set_help_string(_("Show messages about all players joining the multiplayer lobby"));
 
+	mp_server_search_button_.set_help_string(_("Find and set path to MP server to host LAN games."));
 	friends_list_button_.set_help_string(_("View and edit your friends and ignores list"));
 	friends_back_button_.set_help_string(_("Back to the multiplayer options"));
 	friends_add_friend_button_.set_help_string(_("Add this username to your friends list"));
@@ -409,13 +401,13 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&bell_slider_);
 	h.push_back(&UI_sound_slider_);
 	h.push_back(&scroll_slider_);
-	h.push_back(&gamma_slider_);
 	h.push_back(&chat_lines_slider_);
 	h.push_back(&turbo_slider_);
 	h.push_back(&idle_anim_slider_);
 	h.push_back(&autosavemax_slider_);
 	h.push_back(&buffer_size_slider_);
 	h.push_back(&fullscreen_button_);
+	h.push_back(&scroll_to_action_button_);
 	h.push_back(&turbo_button_);
 	h.push_back(&idle_anim_button_);
 	h.push_back(&show_ai_moves_button_);
@@ -424,9 +416,11 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&show_grid_button_);
 	h.push_back(&sort_list_by_group_button_);
 	h.push_back(&iconize_list_button_);
+	h.push_back(&remember_pw_button_);
 	h.push_back(&show_lobby_joins_button1_);
 	h.push_back(&show_lobby_joins_button2_);
 	h.push_back(&show_lobby_joins_button3_);
+	h.push_back(&mp_server_search_button_);
 	h.push_back(&friends_list_button_);
 	h.push_back(&friends_back_button_);
 	h.push_back(&friends_add_friend_button_);
@@ -443,7 +437,6 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&video_mode_button_);
 	h.push_back(&theme_button_);
 	h.push_back(&hotkeys_button_);
-	h.push_back(&gamma_button_);
 	h.push_back(&flip_time_button_);
 	h.push_back(&advanced_button_);
 	h.push_back(&sound_button_);
@@ -460,7 +453,6 @@ handler_vector preferences_dialog::handler_members()
 	h.push_back(&bell_label_);
 	h.push_back(&UI_sound_label_);
 	h.push_back(&scroll_label_);
-	h.push_back(&gamma_label_);
 	h.push_back(&turbo_slider_label_);
 	h.push_back(&idle_anim_slider_label_);
 	h.push_back(&autosavemax_slider_label_);
@@ -520,13 +512,8 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 
 	// Display tab
 	ypos = rect.y + top_border;
-	gamma_button_.set_location(rect.x, ypos);
-	ypos += short_interline;
-	gamma_label_.set_location(rect.x, ypos);
-	SDL_Rect gamma_rect = { rect.x + gamma_label_.width(), ypos,
-							rect.w - gamma_label_.width() - right_border, 0 };
-	gamma_slider_.set_location(gamma_rect);
-	ypos += item_interline; fullscreen_button_.set_location(rect.x, ypos);
+	fullscreen_button_.set_location(rect.x, ypos);
+	ypos += item_interline; scroll_to_action_button_.set_location(rect.x, ypos);
 	ypos += item_interline; show_colour_cursors_button_.set_location(rect.x, ypos);
 	ypos += item_interline; flip_time_button_.set_location(rect.x,ypos);
 	ypos += item_interline; show_floating_labels_button_.set_location(rect.x, ypos);
@@ -542,9 +529,9 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 	                           bottom_row_y - theme_button_.height());
 
 	// Sound tab
-	slider_label_width_ = maximum<unsigned>(music_label_.width(), sound_label_.width());
-	slider_label_width_ = maximum<unsigned>(slider_label_width_, bell_label_.width());
-	slider_label_width_ = maximum<unsigned>(slider_label_width_, UI_sound_label_.width());
+	slider_label_width_ = std::max<unsigned>(music_label_.width(), sound_label_.width());
+	slider_label_width_ = std::max<unsigned>(slider_label_width_, bell_label_.width());
+	slider_label_width_ = std::max<unsigned>(slider_label_width_, UI_sound_label_.width());
 	ypos = rect.y + top_border;
 	sound_button_.set_location(rect.x, ypos);
 
@@ -614,6 +601,7 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 								rect.w - horizontal_padding - right_border, 0 };
 	chat_lines_slider_.set_location(chat_lines_rect);
 	ypos += item_interline; chat_timestamp_button_.set_location(rect.x, ypos);
+	ypos += item_interline; remember_pw_button_.set_location(rect.x, ypos);
 	ypos += item_interline; sort_list_by_group_button_.set_location(rect.x, ypos);
 	ypos += item_interline; iconize_list_button_.set_location(rect.x, ypos);
 
@@ -623,6 +611,8 @@ void preferences_dialog::update_location(SDL_Rect const &rect)
 
 	friends_list_button_.set_location(rect.x, bottom_row_y - friends_list_button_.height());
 
+	mp_server_search_button_.set_location(rect.x + horizontal_padding + friends_list_button_.width(), bottom_row_y - mp_server_search_button_.height());
+
 	//Friends tab
 	ypos = rect.y + top_border;
 	friends_input_.set_location(rect.x,ypos);
@@ -711,16 +701,12 @@ void preferences_dialog::process_event()
 			throw video_mode_change_exception(fullscreen_button_.checked()
 											? video_mode_change_exception::MAKE_FULLSCREEN
 											: video_mode_change_exception::MAKE_WINDOWED);
+		if (scroll_to_action_button_.pressed())
+			set_scroll_to_action(scroll_to_action_button_.checked());
 		if (show_colour_cursors_button_.pressed())
 			set_colour_cursors(show_colour_cursors_button_.checked());
 		if (show_haloing_button_.pressed())
 			set_show_haloes(show_haloing_button_.checked());
-		if (gamma_button_.pressed()) {
-			set_adjust_gamma(gamma_button_.checked());
-			const bool enable_gamma = adjust_gamma();
-			gamma_slider_.enable(enable_gamma);
-			gamma_label_.enable(enable_gamma);
-		}
 		if (flip_time_button_.pressed())
 			set_flip_time(flip_time_button_.checked());
 		if (idle_anim_button_.pressed()) {
@@ -732,7 +718,6 @@ void preferences_dialog::process_event()
 				idle_anim_slider_.set_value(0);
 		}
 
-		set_gamma(gamma_slider_.value());
 		set_idle_anim_rate(idle_anim_slider_.value());
 
 		return;
@@ -854,11 +839,23 @@ void preferences_dialog::process_event()
 			set_sort_list(sort_list_by_group_button_.checked());
 		if (iconize_list_button_.pressed())
 			set_iconize_list(iconize_list_button_.checked());
+		if (remember_pw_button_.pressed())
+			set_remember_password(remember_pw_button_.checked());
 		if (chat_timestamp_button_.pressed())
 			set_chat_timestamping(chat_timestamp_button_.checked());
 		if (friends_list_button_.pressed())
 			set_selection(FRIENDS_TAB);
 
+		if (mp_server_search_button_.pressed())
+		{
+			std::string path = show_wesnothd_server_search(disp_);
+			if (!path.empty())
+			{
+				preferences::set_mp_server_program_name(path);
+			}
+			parent->clear_buttons();
+		}
+
 		set_chat_lines(chat_lines_slider_.value());
 
 		//display currently select amount of chat lines
@@ -902,7 +899,7 @@ void preferences_dialog::process_event()
 				to_remove = friends_names_[friends_.selection()];
 			}
 			if(!to_remove.empty()) {
-				//! @todo Better to remove from a specific relation.
+				/** @todo Better to remove from a specific relation. */
 				preferences::remove_friend(to_remove);
 				preferences::remove_ignore(to_remove);
 				friends_input_.clear();
@@ -979,12 +976,14 @@ void preferences_dialog::set_advanced_menu()
 
 void preferences_dialog::set_friends_menu()
 {
-	const std::vector<std::string>& friends = utils::split(preferences::get_friends());
-	const std::vector<std::string>& ignores = utils::split(preferences::get_ignores());
+	const std::set<std::string>& friends = preferences::get_friends();
+	const std::set<std::string>& ignores = preferences::get_ignores();
+
 	std::vector<std::string> friends_items;
 	std::vector<std::string> friends_names;
 	std::string const imgpre = IMAGE_PREFIX + std::string("misc/status-");
-	std::vector<std::string>::const_iterator i;
+
+	std::set<std::string>::const_iterator i;
 	for (i = friends.begin(); i != friends.end(); ++i)
 	{
 		friends_items.push_back(imgpre + "friend.png" + COLUMN_SEPARATOR
@@ -1032,15 +1031,11 @@ void preferences_dialog::set_selection(int index)
 	autosavemax_slider_.enable(!hide_general);
 
 	const bool hide_display = tab_ != DISPLAY_TAB;
-	gamma_label_.hide(hide_display);
-	gamma_slider_.hide(hide_display);
-	gamma_label_.enable(adjust_gamma());
-	gamma_slider_.enable(adjust_gamma());
-	gamma_button_.hide(hide_display);
 	show_floating_labels_button_.hide(hide_display);
 	show_colour_cursors_button_.hide(hide_display);
 	show_haloing_button_.hide(hide_display);
 	fullscreen_button_.hide(hide_display);
+	scroll_to_action_button_.hide(hide_display);
 	idle_anim_button_.hide(hide_display);
 	idle_anim_slider_label_.hide(hide_display);
 	idle_anim_slider_label_.enable(idle_anim());
@@ -1090,10 +1085,12 @@ void preferences_dialog::set_selection(int index)
 	chat_timestamp_button_.hide(hide_multiplayer);
 	sort_list_by_group_button_.hide(hide_multiplayer);
 	iconize_list_button_.hide(hide_multiplayer);
+	remember_pw_button_.hide(hide_multiplayer);
 	show_lobby_joins_button1_.hide(hide_multiplayer);
 	show_lobby_joins_button2_.hide(hide_multiplayer);
 	show_lobby_joins_button3_.hide(hide_multiplayer);
 	friends_list_button_.hide(hide_multiplayer);
+	mp_server_search_button_.hide(hide_multiplayer);
 
 	const bool hide_friends = tab_ != FRIENDS_TAB;
 	friends_.hide(hide_friends);
@@ -1170,4 +1167,51 @@ bool show_theme_dialog(display& disp)
 	return(0);
 }
 
+std::string show_wesnothd_server_search(display& disp)
+{
+	// Showing file_chooser so user can search the wesnothd
+	std::string old_path = preferences::get_mp_server_program_name();
+	size_t offset = old_path.rfind("/");
+	if (offset != std::string::npos)
+	{
+		old_path = old_path.substr(0, offset);
+	}
+	else
+	{
+		old_path = "";
+	}
+#ifndef _WIN32
+
+#ifndef WESNOTH_PREFIX
+#define WESNOTH_PREFIX "/usr"
+#endif
+	const std::string filename = "wesnothd";
+	std::string path = WESNOTH_PREFIX + std::string("/bin");
+	if (!is_directory(path))
+		path = get_cwd();
+
+#else
+	const std::string filename = "wesnothd.exe";
+	std::string path = get_cwd();
+#endif
+	if (!old_path.empty()
+			&& is_directory(old_path))
+	{
+		path = old_path;
+	}
+
+	utils::string_map symbols;
+
+	symbols["filename"] = filename;
+
+	const std::string title =  vgettext("Find $filename server binary to host networked games", symbols);
+
+	int res = dialogs::show_file_chooser_dialog(disp, path, title, false, filename);
+	if (res == 0)
+		return path;
+	else
+		return "";
+}
+
+
 }
