@@ -1,6 +1,6 @@
-/* $Id: gamestatus.cpp 26453 2008-05-07 23:31:24Z shadowmaster $ */
+/* $Id: gamestatus.cpp 33687 2009-03-15 16:50:42Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,33 +12,28 @@
    See the COPYING file for more details.
 */
 
-//! @file gamestatus.cpp
-//! Maintain status of a game, load&save games.
+/**
+ * @file gamestatus.cpp
+ * Maintain status of a game, load&save games.
+ */
 
 #include "global.hpp"
 
-#include "filesystem.hpp"
-#include "game_config.hpp"
-#include "game_errors.hpp"
-#include "gamestatus.hpp"
 #include "gettext.hpp"
-#include "language.hpp"
 #include "log.hpp"
 #include "game_preferences.hpp"
+#include "replay.hpp"
 #include "statistics.hpp"
-#include "util.hpp"
+#include "unit_id.hpp"
 #include "wesconfig.h"
 #include "serialization/binary_or_text.hpp"
-#include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
-#include "serialization/string_utils.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
 
-#include <algorithm>
-#include <cstdio>
-#include <iostream>
-#include <iterator>
-#include <sstream>
+#ifndef _MSC_VER
+#include <sys/time.h>
+#endif
 
 #define DBG_NG lg::debug(lg::engine)
 #define LOG_NG lg::info(lg::engine)
@@ -48,13 +43,15 @@
 #ifdef _WIN32
 #include <windows.h>
 
-//! conv_ansi_utf8()
-//!   - Convert a string between ANSI encoding (for Windows filename) and UTF-8
-//!  string &name
-//!     - filename to be converted
-//!  bool a2u
-//!     - if true, convert the string from ANSI to UTF-8.
-//!     - if false, reverse. (convert it from UTF-8 to ANSI)
+/**
+ * conv_ansi_utf8()
+ *   - Convert a string between ANSI encoding (for Windows filename) and UTF-8
+ *  string &name
+ *     - filename to be converted
+ *  bool a2u
+ *     - if true, convert the string from ANSI to UTF-8.
+ *     - if false, reverse. (convert it from UTF-8 to ANSI)
+ */
 void conv_ansi_utf8(std::string &name, bool a2u) {
 	int wlen = MultiByteToWideChar(a2u ? CP_ACP : CP_UTF8, 0,
                                    name.c_str(), -1, NULL, 0);
@@ -127,57 +124,58 @@ player_info* game_state::get_player(const std::string& id) {
 		return &found->second;
 }
 
-time_of_day::time_of_day(const config& cfg)
-                 : lawful_bonus(atoi(cfg["lawful_bonus"].c_str())),
-                   bonus_modified(0),
-                   image(cfg["image"]), name(cfg["name"]), id(cfg["id"]),
-			       image_mask(cfg["mask"]),
-                   red(atoi(cfg["red"].c_str())),
-                   green(atoi(cfg["green"].c_str())),
-                   blue(atoi(cfg["blue"].c_str())),
-		   sounds(cfg["sound"])
+#ifdef __UNUSED__
+std::string generate_game_uuid()
 {
+	struct timeval ts;
+	std::stringstream uuid;
+	gettimeofday(&ts, NULL);
+
+	uuid << preferences::login() << "@" << ts.tv_sec << "." << ts.tv_usec;
+
+	return uuid.str();
+}
+#endif
+
+void gamestatus::add_time_area(const config& cfg)
+{
+	areas_.push_back(area_time_of_day());
+	area_time_of_day &area = areas_.back();
+	area.id   = cfg["id"];
+	area.xsrc = cfg["x"];
+	area.ysrc = cfg["y"];
+	std::vector<map_location> const& locs = parse_location_range(area.xsrc, area.ysrc);
+	std::copy(locs.begin(), locs.end(), std::inserter(area.hexes, area.hexes.end()));
+	time_of_day::parse_times(cfg, area.times);
 }
 
-void time_of_day::write(config& cfg) const
+void gamestatus::add_time_area(const std::string& id, const std::set<map_location>& locs,
+                               const config& time_cfg)
 {
-	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",lawful_bonus);
-	cfg["lawful_bonus"] = buf;
-
-	snprintf(buf,sizeof(buf),"%d",red);
-	cfg["red"] = buf;
-
-	snprintf(buf,sizeof(buf),"%d",green);
-	cfg["green"] = buf;
-
-	snprintf(buf,sizeof(buf),"%d",blue);
-	cfg["blue"] = buf;
-
-	cfg["image"] = image;
-	cfg["name"] = name;
-	cfg["id"] = id;
-	cfg["mask"] = image_mask;
+	areas_.push_back(area_time_of_day());
+	area_time_of_day& area = areas_.back();
+	area.id = id;
+	area.hexes = locs;
+	time_of_day::parse_times(time_cfg, area.times);
 }
 
-static void parse_times(const config& cfg, std::vector<time_of_day>& normal_times)
+void gamestatus::remove_time_area(const std::string& area_id)
 {
-	const config::child_list& times = cfg.get_children("time");
-	config::child_list::const_iterator t;
-	for(t = times.begin(); t != times.end(); ++t) {
-		normal_times.push_back(time_of_day(**t));
-	}
-
-	if(normal_times.empty())
-	{
-		// Make sure we have at least default time
-		config dummy_cfg;
-		normal_times.push_back(time_of_day(dummy_cfg));
+	if(area_id.empty()) {
+		areas_.clear();
+	} else {
+		// search for all time areas that match the id.
+		std::vector<area_time_of_day>::iterator i = areas_.begin();
+		while(i != areas_.end()) {
+			if((*i).id == area_id) {
+				i = areas_.erase(i);
+			} else {
+				++i;
+			}
+		}
 	}
 }
 
-//! Reads turns and time information from parameters.
-//! It sets random starting ToD and current_tod to config.
 gamestatus::gamestatus(const config& time_cfg, int num_turns, game_state* s_o_g) :
 		teams(0),
 		times_(),
@@ -201,20 +199,13 @@ gamestatus::gamestatus(const config& time_cfg, int num_turns, game_state* s_o_g)
 		turn_ = atoi(turn_at.c_str());
 	}
 
-	parse_times(time_cfg,times_);
+	time_of_day::parse_times(time_cfg,times_);
 
 	set_start_ToD(const_cast<config&>(time_cfg),s_o_g);
 
 	const config::child_list& times_range = time_cfg.get_children("time_area");
-	for(config::child_list::const_iterator t = times_range.begin(); t != times_range.end(); ++t) {
-		const std::vector<gamemap::location> locs = parse_location_range((**t)["x"],(**t)["y"]);
-		area_time_of_day area;
-		area.xsrc = (**t)["x"];
-		area.ysrc = (**t)["y"];
-		std::copy(locs.begin(),locs.end(),std::inserter(area.hexes,area.hexes.end()));
-		parse_times(**t,area.times);
-		areas_.push_back(area);
-	}
+	for(config::child_list::const_iterator t = times_range.begin(); t != times_range.end(); ++t)
+		this->add_time_area(**t);
 }
 
 void gamestatus::write(config& cfg) const
@@ -246,8 +237,6 @@ void gamestatus::write(config& cfg) const
 	}
 }
 
-//! Returns time of day object in the turn.
-//! Correct time is calculated from current time.
 time_of_day gamestatus::get_time_of_day_turn(int nturn) const
 {
 	VALIDATE(times_.size(), _("No time of day has been defined."));
@@ -262,7 +251,6 @@ time_of_day gamestatus::get_time_of_day_turn(int nturn) const
 	return times_[time];
 }
 
-//! ~eturns time of day object for current turn.
 time_of_day gamestatus::get_time_of_day() const
 {
 	VALIDATE(times_.size(), _("No time of day has been defined."));
@@ -275,24 +263,22 @@ time_of_day gamestatus::get_previous_time_of_day() const
 	return get_time_of_day_turn(turn()-1);
 }
 
-//! Returns time of day object in the turn.
-//! It first tries to look for specified.
-//! If no area time specified in location, it returns global time.
-time_of_day gamestatus::get_time_of_day(int illuminated, const gamemap::location& loc, int n_turn) const
+time_of_day gamestatus::get_time_of_day(int illuminated, const map_location& loc, int n_turn) const
 {
 	time_of_day res = get_time_of_day_turn(n_turn);
 
-	for(std::vector<area_time_of_day>::const_iterator i = areas_.begin(); i != areas_.end(); ++i) {
-		if(i->hexes.count(loc) == 1) {
+	if(loc.valid()) {
+		for(std::vector<area_time_of_day>::const_iterator i = areas_.begin(); i != areas_.end(); ++i) {
+			if(i->hexes.count(loc) == 1) {
 
-			VALIDATE(i->times.size(), _("No time of day has been defined."));
+				VALIDATE(i->times.size(), _("No time of day has been defined."));
 
-			res = i->times[(n_turn-1)%i->times.size()];
-			break;
+				res = i->times[(n_turn-1)%i->times.size()];
+				break;
+			}
 		}
 	}
 
-
 	if(illuminated) {
 		res.bonus_modified=illuminated;
 		res.lawful_bonus += illuminated;
@@ -300,13 +286,11 @@ time_of_day gamestatus::get_time_of_day(int illuminated, const gamemap::location
 	return res;
 }
 
-time_of_day gamestatus::get_time_of_day(int illuminated, const gamemap::location& loc) const
+time_of_day gamestatus::get_time_of_day(int illuminated, const map_location& loc) const
 {
 	return get_time_of_day(illuminated,loc,turn());
 }
 
-//! Sets global time of day in this turn.
-//! Time is a number between 0 and n-1, where n is number of ToDs.
 bool gamestatus::set_time_of_day(int newTime)
 {
 	// newTime can come from network so have to take run time test
@@ -386,13 +370,29 @@ int gamestatus::number_of_turns() const
 }
 void gamestatus::modify_turns(const std::string& mod)
 {
-	numTurns_ = maximum<int>(utils::apply_modifier(numTurns_,mod,0),-1);
+	numTurns_ = std::max<int>(utils::apply_modifier(numTurns_,mod,0),-1);
 }
 void gamestatus::add_turns(int num)
 {
-	numTurns_ = maximum<int>(numTurns_ + num,-1);
+	numTurns_ = std::max<int>(numTurns_ + num,-1);
 }
 
+void gamestatus::set_turn(unsigned int num)
+{
+	VALIDATE(times_.size(), _("No time of day has been defined."));
+	const unsigned int old_num = turn_;
+	// Correct ToD
+	currentTime_ = (num  - 1) % times_.size();
+	if (currentTime_ < 0) {
+		currentTime_ += times_.size();
+	}
+	if(static_cast<int>(num) > numTurns_ && numTurns_ != -1) {
+		this->add_turns(numTurns_ - num);
+	}
+	turn_ = num;
+
+	LOG_NG << "changed current turn number from " << old_num << " to " << num << '\n';
+}
 
 bool gamestatus::next_turn()
 {
@@ -401,7 +401,7 @@ bool gamestatus::next_turn()
 	return numTurns_ == -1 || turn_ <= size_t(numTurns_);
 }
 
-static player_info read_player(const game_data& data, const config* cfg)
+static player_info read_player(const config* cfg)
 {
 	player_info res;
 
@@ -412,7 +412,7 @@ static player_info read_player(const game_data& data, const config* cfg)
 
 	const config::child_list& units = cfg->get_children("unit");
 	for(config::child_list::const_iterator i = units.begin(); i != units.end(); ++i) {
-		res.available_units.push_back(unit(data,**i,false));
+		res.available_units.push_back(unit(**i,false));
 	}
 
 	res.can_recruit.clear();
@@ -426,17 +426,49 @@ static player_info read_player(const game_data& data, const config* cfg)
 	return res;
 }
 
-game_state::game_state(const game_data& data, const config& cfg, bool show_replay) :
+game_state::game_state()  :
+		label(),
+		version(),
+		campaign_type(),
+		campaign_define(),
+		campaign_xtra_defines(),
+		campaign(),
+		history(),
+		abbrev(),
+		scenario(),
+		next_scenario(),
+		completion(),
+		end_text(),
+		end_text_duration(),
+		players(),
+		scoped_variables(),
+		wml_menu_items(),
+		difficulty("NORMAL"),
+		replay_data(),
+		starting_pos(),
+		snapshot(),
+		last_selected(map_location::null_location),
+		rng_(),
+		variables(),
+		temporaries(),
+		generator_setter(&recorder)
+		{}
+
+
+game_state::game_state(const config& cfg, bool show_replay) :
 		label(cfg["label"]),
 		version(cfg["version"]),
 		campaign_type(cfg["campaign_type"]),
 		campaign_define(cfg["campaign_define"]),
 		campaign_xtra_defines(utils::split(cfg["campaign_extra_defines"])),
 		campaign(cfg["campaign"]),
+		history(cfg["history"]),
 		abbrev(cfg["abbrev"]),
 		scenario(cfg["scenario"]),
 		next_scenario(cfg["next_scenario"]),
 		completion(cfg["completion"]),
+		end_text(cfg["end_text"]),
+		end_text_duration(lexical_cast_default<unsigned int>(cfg["end_text_duration"])),
 		players(),
 		scoped_variables(),
 		wml_menu_items(),
@@ -444,39 +476,53 @@ game_state::game_state(const game_data& data, const config& cfg, bool show_repla
 		replay_data(),
 		starting_pos(),
 		snapshot(),
-		last_selected(gamemap::location::null_location),
+		last_selected(map_location::null_location),
+		rng_(cfg),
 		variables(),
 		temporaries(),
-		//! @todo  older savegames don't have random_seed stored, evaluate later
-		//! whether default can be removed again. Look after branching 1.5.
-		random_seed_(lexical_cast_default<int>(cfg["random_seed"], 42)),
-		random_pool_(random_seed_),
-		random_calls_(0)
+		generator_setter(&recorder)
 {
+	n_unit::id_manager::instance().set_save_id(lexical_cast_default<size_t>(cfg["next_underlying_unit_id"],0));
 	log_scope("read_game");
 
 	const config* snapshot = cfg.child("snapshot");
+	const config* replay_start = cfg.child("replay_start");
 
 	// We have to load era id for MP games so they can load correct era.
-	
+
 
 	if ((snapshot != NULL) && (!snapshot->empty()) && (!show_replay)) {
 
 		this->snapshot = *snapshot;
 
-		seed_random(random_seed_, lexical_cast_default<unsigned>((*snapshot)["random_calls"]));
+		rng_.seed_random(lexical_cast_default<unsigned>((*snapshot)["random_calls"]));
 
 		// Midgame saves have the recall list stored in the snapshot.
-		load_recall_list(data, snapshot->get_children("player"));
+		load_recall_list(snapshot->get_children("player"));
 
 	} else {
-		// Start of scenario save, replays and MP campaign network next scenario 
+		// Start of scenario save, replays and MP campaign network next scenario
 		// have the recall list stored in root of the config.
-		load_recall_list(data, cfg.get_children("player"));
+		// Edit:
+		// Unfortunately, the replay savegame format is not homogeneous. Some savegames
+		// have the player information stored in the starting position, others in the
+		// root of the config. If the starting position player information is available
+		// it will be preferred.
+		if (replay_start != NULL){
+			// Check if we find some player information in the starting position
+			const config::child_list& cfg_players = (*replay_start).get_children("player");
+			if (!cfg_players.empty())
+				load_recall_list(cfg_players);
+			else
+				load_recall_list(cfg.get_children("player"));
+		}
+		else{
+			load_recall_list(cfg.get_children("player"));
+		}
 	}
-         
-	std::cerr << "scenario: '" << scenario << "'\n";
-	std::cerr << "next_scenario: '" << next_scenario << "'\n";
+
+	LOG_NG << "scenario: '" << scenario << "'\n";
+	LOG_NG << "next_scenario: '" << next_scenario << "'\n";
 
 	if(difficulty.empty()) {
 		difficulty = "NORMAL";
@@ -497,9 +543,21 @@ game_state::game_state(const game_data& data, const config& cfg, bool show_repla
 		replay_data = *replay;
 	}
 
-	const config* replay_start = cfg.child("replay_start");
 	if(replay_start != NULL) {
 		starting_pos = *replay_start;
+		//This is a quick hack to make replays for campaigns work again:
+		//The [player] information needs to be stored somewhere within the gamestate,
+		//because we need it later on when creating the replay savegame.
+		//We therefore put it inside the starting_pos, so it doesn't get lost.
+		//See also playcampaign::play_game, where after finishing the scenario the replay
+		//will be saved.
+		if(!starting_pos.empty()) {
+			config::child_list player_list = cfg.get_children("player");
+			for (config::child_list::const_iterator p = player_list.begin(); p != player_list.end(); p++){
+				config& cfg_player = starting_pos.add_child("player");
+				cfg_player.merge_with(**p);
+			}
+		}
 	}
 
 	if(cfg.child("statistics")) {
@@ -517,14 +575,14 @@ static void write_player(const player_info& player, config& cfg)
 
 	cfg["gold"] = buf;
 
-	cfg["gold_add"] = player.gold_add ? "true" : "false";
+	cfg["gold_add"] = player.gold_add ? "yes" : "no";
 
 	for(std::vector<unit>::const_iterator i = player.available_units.begin();
 	    i != player.available_units.end(); ++i) {
 		config new_cfg;
 		i->write(new_cfg);
 		cfg.add_child("unit",new_cfg);
-		DBG_NG << "added unit '" << new_cfg["description"] << "' to player '" << player.name << "'\n";
+		DBG_NG << "added unit '" << new_cfg["id"] << "' to player '" << player.name << "'\n";
 	}
 
 	std::stringstream can_recruit;
@@ -539,6 +597,18 @@ static void write_player(const player_info& player, config& cfg)
 	cfg["can_recruit"] = can_recruit_str;
 }
 
+void write_players(game_state& gamestate, config& cfg)
+{
+	for(std::map<std::string, player_info>::const_iterator i=gamestate.players.begin();
+		i!=gamestate.players.end(); ++i)
+	{
+		config new_cfg;
+		write_player(i->second, new_cfg);
+		new_cfg["save_id"]=i->first;
+		cfg.add_child("player", new_cfg);
+	}
+}
+
 static void write_player(config_writer &out, const player_info& player)
 {
 	out.write_key_val("name", player.name);
@@ -548,12 +618,15 @@ static void write_player(config_writer &out, const player_info& player)
 
 	out.write_key_val("gold", buf);
 
+	const std::string gold_add = player.gold_add ? "true" : "false";
+	out.write_key_val("gold_add", gold_add);
+
 	for(std::vector<unit>::const_iterator i = player.available_units.begin();
 	    i != player.available_units.end(); ++i) {
 		config new_cfg;
 		i->write(new_cfg);
 		out.write_child("unit",new_cfg);
-		DBG_NG << "added unit '" << new_cfg["description"] << "' to player '" << player.name << "'\n";
+		DBG_NG << "added unit '" << new_cfg["id"] << "' to player '" << player.name << "'\n";
 	}
 
 	std::stringstream can_recruit;
@@ -568,12 +641,12 @@ static void write_player(config_writer &out, const player_info& player)
 	out.write_key_val("can_recruit", can_recruit_str);
 }
 
-
-//! @deprecated, use other write_game below.
+/** @deprecated, use other write_game below. */
 void write_game(const game_state& gamestate, config& cfg, WRITE_GAME_MODE mode)
 {
 	log_scope("write_game");
 	cfg["label"] = gamestate.label;
+	cfg["history"] = gamestate.history;
 	cfg["abbrev"] = gamestate.abbrev;
 	cfg["version"] = game_config::version;
 
@@ -590,9 +663,13 @@ void write_game(const game_state& gamestate, config& cfg, WRITE_GAME_MODE mode)
 
 	cfg["campaign_define"] = gamestate.campaign_define;
 	cfg["campaign_extra_defines"] = utils::join(gamestate.campaign_xtra_defines);
+	cfg["next_underlying_unit_id"] = lexical_cast<std::string>(n_unit::id_manager::instance().get_save_id());
 
-	cfg["random_seed"] = lexical_cast<std::string>(gamestate.get_random_seed());
-	cfg["random_calls"] = lexical_cast<std::string>(gamestate.get_random_calls());
+	cfg["random_seed"] = lexical_cast<std::string>(gamestate.rng().get_random_seed());
+	cfg["random_calls"] = lexical_cast<std::string>(gamestate.rng().get_random_calls());
+
+	cfg["end_text"] = gamestate.end_text;
+	cfg["end_text_duration"] = str_cast<unsigned int>(gamestate.end_text_duration);
 
 	cfg.add_child("variables",gamestate.get_variables());
 
@@ -636,6 +713,7 @@ void write_game(config_writer &out, const game_state& gamestate, WRITE_GAME_MODE
 	log_scope("write_game");
 
 	out.write_key_val("label", gamestate.label);
+	out.write_key_val("history", gamestate.history);
 	out.write_key_val("abbrev", gamestate.abbrev);
 	out.write_key_val("version", game_config::version);
 	out.write_key_val("scenario", gamestate.scenario);
@@ -646,8 +724,11 @@ void write_game(config_writer &out, const game_state& gamestate, WRITE_GAME_MODE
 	out.write_key_val("difficulty", gamestate.difficulty);
 	out.write_key_val("campaign_define", gamestate.campaign_define);
 	out.write_key_val("campaign_extra_defines", utils::join(gamestate.campaign_xtra_defines));
-	out.write_key_val("random_seed", lexical_cast<std::string>(gamestate.get_random_seed()));
-	out.write_key_val("random_calls", lexical_cast<std::string>(gamestate.get_random_calls()));
+	out.write_key_val("random_seed", lexical_cast<std::string>(gamestate.rng().get_random_seed()));
+	out.write_key_val("random_calls", lexical_cast<std::string>(gamestate.rng().get_random_calls()));
+	out.write_key_val("next_underlying_unit_id", lexical_cast<std::string>(n_unit::id_manager::instance().get_save_id()));
+	out.write_key_val("end_text", gamestate.end_text);
+	out.write_key_val("end_text_duration", str_cast<unsigned int>(gamestate.end_text_duration));
 	out.write_child("variables", gamestate.get_variables());
 
 	for(std::map<std::string, wml_menu_item *>::const_iterator j=gamestate.wml_menu_items.begin();
@@ -687,8 +768,10 @@ void write_game(config_writer &out, const game_state& gamestate, WRITE_GAME_MODE
 	}
 }
 
-//! A structure for comparing to save_info objects based on their modified time.
-//! If the times are equal, will order based on the name.
+/**
+ * A structure for comparing to save_info objects based on their modified time.
+ * If the times are equal, will order based on the name.
+ */
 struct save_info_less_time {
 	bool operator()(const save_info& a, const save_info& b) const {
 		if (a.time_modified > b.time_modified) {
@@ -775,19 +858,19 @@ void read_save_file(const std::string& name, config& cfg, std::string* error_log
 		}
 	} catch (config::error &err)
 	{
-		std::cerr << err.message;
+		ERR_NG << err.message;
 		throw game::load_game_failed();
 	}
 
 	if(cfg.empty()) {
-		std::cerr << "Could not parse file data into config\n";
+		ERR_NG << "Could not parse file data into config\n";
 		throw game::load_game_failed();
 	}
 }
 
-void copy_era(config &cfg)
+static void copy_era(config &cfg)
 {
-	if (cfg.child("replay_start") 
+	if (cfg.child("replay_start")
 		&& cfg.child("replay_start")->child("era")
 		&& cfg.child("snapshot"))
 	{
@@ -796,7 +879,7 @@ void copy_era(config &cfg)
 	}
 }
 
-void load_game(const game_data& data, const std::string& name, game_state& gamestate, std::string* error_log)
+void load_game(const std::string& name, game_state& gamestate, std::string* error_log)
 {
 	log_scope("load_game");
 
@@ -804,7 +887,7 @@ void load_game(const game_data& data, const std::string& name, game_state& games
 	read_save_file(name,cfg,error_log);
 	copy_era(cfg);
 
-	gamestate = game_state(data,cfg);
+	gamestate = game_state(cfg);
 }
 
 void load_game_summary(const std::string& name, config& cfg_summary, std::string* error_log){
@@ -853,15 +936,22 @@ void finish_save_game(config_writer &out, const game_state& gamestate, const std
 // Throws game::save_game_failed
 void save_game(const game_state& gamestate)
 {
-	std::string filename = gamestate.label; 
+	std::string filename = gamestate.label;
 	if(preferences::compress_saves()) {
 		filename += ".gz";
 	}
 
 	scoped_ostream os(open_save_game(filename));
-	config_writer out(*os, preferences::compress_saves(), PACKAGE);
-	write_game(out, gamestate);
-	finish_save_game(out, gamestate, gamestate.label);
+	std::stringstream ss;
+	{
+		config_writer out(ss, preferences::compress_saves());
+		write_game(out, gamestate);
+		finish_save_game(out, gamestate, gamestate.label);
+	}
+	(*os) << ss.str();
+	if (!os->good()) {
+		throw game::save_game_failed(_("Could not write to file"));
+	}
 }
 
 namespace {
@@ -876,9 +966,9 @@ static config& save_index()
 			scoped_istream stream = istream_file(get_save_index_file());
 			detect_format_and_read(save_index_cfg, *stream);
 		} catch(io_exception& e) {
-			std::cerr << "error reading save index: '" << e.what() << "'\n";
+			ERR_NG << "error reading save index: '" << e.what() << "'\n";
 		} catch(config::error&) {
-			std::cerr << "error parsing save index config file\n";
+			ERR_NG << "error parsing save index config file\n";
 			save_index_cfg.clear();
 		}
 
@@ -888,8 +978,17 @@ static config& save_index()
 	return save_index_cfg;
 }
 
-config& save_summary(const std::string& save)
+config& save_summary(std::string save)
 {
+	/*
+	 * All saves are .gz files now so make sure we use that name when opening
+	 * a file. If not some parts of the code use the name with and some parts
+	 * without the .gz suffix.
+	 */
+	if(save.length() < 3 || save.substr(save.length() - 3) != ".gz") {
+		save += ".gz";
+	}
+
 	config& cfg = save_index();
 	config* res = cfg.find_child("save","save",save);
 	if(res == NULL) {
@@ -905,9 +1004,9 @@ void write_save_index()
 	log_scope("write_save_index()");
 	try {
 		scoped_ostream stream = ostream_file(get_save_index_file());
-		write_compressed(*stream, save_index());
+		write(*stream, save_index());
 	} catch(io_exception& e) {
-		std::cerr << "error writing to save index file: '" << e.what() << "'\n";
+		ERR_NG << "error writing to save index file: '" << e.what() << "'\n";
 	}
 }
 
@@ -936,14 +1035,14 @@ void extract_summary_data_from_save(const game_state& gamestate, config& out)
 
 	// Find the first human leader so we can display their icon in the load menu.
 
-	//! @todo Ideally we should grab all leaders if there's more than 1 human player?
+	/** @todo Ideally we should grab all leaders if there's more than 1 human player? */
 	std::string leader;
 
 	for(std::map<std::string, player_info>::const_iterator p = gamestate.players.begin();
 	    p!=gamestate.players.end(); ++p) {
 		for(std::vector<unit>::const_iterator u = p->second.available_units.begin(); u != p->second.available_units.end(); ++u) {
 			if(u->can_recruit()) {
-				leader = u->id();
+				leader = u->type_id();
 			}
 		}
 	}
@@ -959,7 +1058,7 @@ void extract_summary_data_from_save(const game_state& gamestate, config& out)
 				continue;
 			}
 
-			if((**s)["shroud"] == "yes") {
+			if(utils::string_bool((**s)["shroud"])) {
 				shrouded = true;
 			}
 
@@ -994,7 +1093,7 @@ void extract_summary_from_config(config& cfg_save, config& cfg_summary)
 	const config* cfg_snapshot = cfg_save.child("snapshot");
 	const config* cfg_replay_start = cfg_save.child("replay_start");
 
-	const bool has_replay = cfg_save.child("replay") != NULL;
+	const bool has_replay = (cfg_save.child("replay") != NULL && !cfg_save.child("replay")->empty());
 	const bool has_snapshot = (cfg_snapshot != NULL) && (cfg_snapshot->child("side") != NULL);
 
 	cfg_summary["replay"] = has_replay ? "yes" : "no";
@@ -1017,7 +1116,7 @@ void extract_summary_from_config(config& cfg_save, config& cfg_summary)
 
 	// Find the first human leader so we can display their icon in the load menu.
 
-	//! @todo Ideally we should grab all leaders if there's more than 1 human player?
+	/** @todo Ideally we should grab all leaders if there's more than 1 human player? */
 	std::string leader;
 
 	const config::child_list& players = cfg_save.get_children("player");
@@ -1040,7 +1139,7 @@ void extract_summary_from_config(config& cfg_save, config& cfg_summary)
 					continue;
 				}
 
-				if((**s)["shroud"] == "yes") {
+				if(utils::string_bool((**s)["shroud"])) {
 					shrouded = true;
 				}
 
@@ -1088,6 +1187,11 @@ config& game_state::get_variable_cfg(const std::string& key)
 	return variable_info(key, true, variable_info::TYPE_CONTAINER).as_container();
 }
 
+variable_info::array_range game_state::get_variable_cfgs(const std::string& key)
+{
+	return variable_info(key, true, variable_info::TYPE_ARRAY).as_array();
+}
+
 void game_state::set_variable(const std::string& key, const t_string& value)
 {
 	get_variable(key) = value;
@@ -1122,58 +1226,16 @@ void game_state::clear_variable(const std::string& varname)
 	}
 }
 
-//! Get a new random number.
-int game_state::get_random()
-{
-	random_next();
-	++random_calls_;
-	DBG_NG << "pulled user random " << random_pool_ 
-		<< " for call " << random_calls_ << '\n';
-
-	return (static_cast<unsigned>(random_pool_ / 65536) % 32768);
-}
-
-//! Seeds the random pool.
-//!
-//! @param seed         The initial value for the random engine.
-//! @param call_count   Upon loading we need to restore the state at saving
-//!                     so set the number of times a random number is generated
-//!                     for replays the orginal value is required.
-void game_state::seed_random(const int seed, const unsigned call_count)
-{
-	random_pool_ = seed;
-	random_seed_ = seed;
-	for(random_calls_ = 0; random_calls_ < call_count; ++random_calls_) {
-		random_next();
-	}
-	DBG_NG << "Seeded random with " << random_seed_ << " with " 
-		<< random_calls_ << " calls, pool is now at " 
-		<< random_pool_ << '\n';
-}
-
-//! Sets the next random number in the pool.
-void game_state::random_next()
-{
-	// Use the simple random generator as shown in man rand(3).
-	// The division is done separately since we also want to 
-	// quickly go the the wanted index in the random list.
-	random_pool_ = random_pool_ * 1103515245 + 12345;
-}
-
-//! Loads the recall list.
-//!
-//! @param game_data    data parameter of game_state::game_state.
-//! @param players      Reference to the players section to load.
-void game_state::load_recall_list(const game_data& data, const config::child_list& players)
+void game_state::load_recall_list(const config::child_list& players)
 {
 	if(!players.empty()) {
 		for(config::child_list::const_iterator i = players.begin(); i != players.end(); ++i) {
 			std::string save_id = (**i)["save_id"];
 
 			if(save_id.empty()) {
-				std::cerr << "Corrupted player entry: NULL save_id" << std::endl;
+				ERR_NG << "Corrupted player entry: NULL save_id" << std::endl;
 			} else {
-				player_info player = read_player(data, *i);
+				player_info player = read_player(*i);
 				this->players.insert(std::pair<std::string, player_info>(save_id,player));
 			}
 		}
@@ -1188,7 +1250,34 @@ static void clear_wmi(std::map<std::string, wml_menu_item*>& gs_wmi) {
 	gs_wmi.clear();
 }
 
-game_state::game_state(const game_state& state) : variable_set(/*silences gcc warning*/)
+game_state::game_state(const game_state& state) :
+	/* default construct everything to silence compiler warnings. */
+	variable_set(),
+	label(),
+	version(),
+	campaign_type(),
+	campaign_define(),
+	campaign_xtra_defines(),
+	campaign(),
+	history(),
+	abbrev(),
+	scenario(),
+	next_scenario(),
+	completion(),
+	end_text(),
+	end_text_duration(),
+	players(),
+	scoped_variables(),
+	wml_menu_items(),
+	difficulty(),
+	replay_data(),
+	starting_pos(),
+	snapshot(),
+	last_selected(),
+	rng_(),
+	variables(),
+	temporaries(),
+	generator_setter(&recorder)
 {
 	*this = state;
 }
@@ -1199,6 +1288,7 @@ game_state& game_state::operator=(const game_state& state)
 		return *this;
 	}
 
+	history = state.history;
 	abbrev = state.abbrev;
 	label = state.label;
 	version = state.version;
@@ -1208,9 +1298,9 @@ game_state& game_state::operator=(const game_state& state)
 	campaign = state.campaign;
 	scenario = state.scenario;
 	completion = state.completion;
-	random_seed_ = state.random_seed_;
-	random_pool_ = state.random_pool_;
-	random_calls_ = state.random_calls_;
+	end_text = state.end_text;
+	end_text_duration = state.end_text_duration;
+	rng_ = state.rng_;
 	players = state.players;
 	scoped_variables = state.scoped_variables;
 
@@ -1262,12 +1352,12 @@ void player_info::debug(){
 	LOG_NG << "\tGold: " << gold << "\n";
 	LOG_NG << "\tAvailable units:\n";
 	for (std::vector<unit>::const_iterator u = available_units.begin(); u != available_units.end(); u++){
-		LOG_NG << "\t\t" + u->description() + "\n";
+		LOG_NG << "\t\t" + u->name() + "\n";
 	}
 	LOG_NG << "\tEnd available units\n";
 }
 
-wml_menu_item::wml_menu_item(const std::string& id, const config* cfg) : 
+wml_menu_item::wml_menu_item(const std::string& id, const config* cfg) :
 		name(),
 		image(),
 		description(),
