@@ -1,6 +1,6 @@
-/* $Id: help.hpp 24050 2008-02-25 18:58:29Z mordante $ */
+/* $Id: help.hpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,6 @@
 
 class config;
 class display;
-class game_data;
 class gamemap;
 
 #include "hotkeys.hpp"
@@ -28,7 +27,7 @@ class gamemap;
 namespace help {
 
 struct help_manager {
-	help_manager(const config *game_config, const game_data *game_info, gamemap *map);
+	help_manager(const config *game_config, gamemap *map);
 	~help_manager();
 };
 
@@ -52,7 +51,7 @@ class help_button : public gui::dialog_button, public hotkey::command_executor {
 public:
 	help_button(display& disp, const std::string &help_topic);
 	int action(gui::dialog_process_info &info);
-	const std::string topic() const { return topic_; }
+	std::string topic() const { return topic_; }
 	void join();
 	void leave();
 private:
