@@ -1,6 +1,6 @@
-/* $Id: hotkeys.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: hotkeys.cpp 33339 2009-03-04 13:38:05Z silene $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,27 +16,17 @@
 
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
-#include "config.hpp"
 #include "construct_dialog.hpp"
 #include "display.hpp"
-#include "events.hpp"
 #include "hotkeys.hpp"
-#include "game_config.hpp"
-#include "game_errors.hpp"
+#include "game_end_exceptions.hpp"
 #include "gettext.hpp"
+#include "filesystem.hpp"
 #include "log.hpp"
-#include "menu_events.hpp"
 #include "preferences_display.hpp"
-#include "show_dialog.hpp"
-#include "util.hpp"
-#include "video.hpp"
 #include "wesconfig.h"
 #include "wml_separators.hpp"
-#include "SDL.h"
 
-#include <algorithm>
-#include <cstdlib>
-#include <map>
 
 #define ERR_G LOG_STREAM(err, general)
 #define LOG_G LOG_STREAM(info, general)
@@ -50,125 +40,218 @@ const struct {
 	const char* command;
 	const char* description;
 	bool hidden;
+	hotkey::scope scope;
 } hotkey_list_[] = {
-	{ hotkey::HOTKEY_CYCLE_UNITS, "cycle", N_("Next Unit"), false },
-	{ hotkey::HOTKEY_CYCLE_BACK_UNITS, "cycleback", N_("Previous Unit"), false },
-	{ hotkey::HOTKEY_UNIT_HOLD_POSITION, "holdposition", N_("Hold Position"), false},
-	{ hotkey::HOTKEY_END_UNIT_TURN, "endunitturn", N_("End Unit Turn"), false },
-	{ hotkey::HOTKEY_LEADER, "leader", N_("Leader"), false },
-	{ hotkey::HOTKEY_UNDO, "undo", N_("Undo"), false },
-	{ hotkey::HOTKEY_REDO, "redo", N_("Redo"), false },
-	{ hotkey::HOTKEY_ZOOM_IN, "zoomin", N_("Zoom In"), false },
-	{ hotkey::HOTKEY_ZOOM_OUT, "zoomout", N_("Zoom Out"), false },
-	{ hotkey::HOTKEY_ZOOM_DEFAULT, "zoomdefault", N_("Default Zoom"), false },
-	{ hotkey::HOTKEY_FULLSCREEN, "fullscreen", N_("Toggle Full Screen"), false },
-	{ hotkey::HOTKEY_SCREENSHOT, "screenshot", N_("Screenshot"), false },
-	{ hotkey::HOTKEY_ACCELERATED, "accelerated", N_("Accelerated"), false },
-	{ hotkey::HOTKEY_UNIT_DESCRIPTION, "describeunit", N_("Unit Description"), false },
-	{ hotkey::HOTKEY_RENAME_UNIT, "renameunit", N_("Rename Unit"), false },
-	{ hotkey::HOTKEY_SAVE_GAME, "save", N_("Save Game"), false },
-	{ hotkey::HOTKEY_SAVE_REPLAY, "savereplay", N_("Save Replay"), false },
-	{ hotkey::HOTKEY_SAVE_MAP, "savemap", N_("Save The Map"), false },
-	{ hotkey::HOTKEY_LOAD_GAME, "load", N_("Load Game"), false },
-	{ hotkey::HOTKEY_RECRUIT, "recruit", N_("Recruit"), false },
-	{ hotkey::HOTKEY_REPEAT_RECRUIT, "repeatrecruit", N_("Repeat Recruit"), false },
-	{ hotkey::HOTKEY_RECALL, "recall", N_("Recall"), false },
-	{ hotkey::HOTKEY_ENDTURN, "endturn", N_("End Turn"), false },
-	{ hotkey::HOTKEY_TOGGLE_GRID, "togglegrid", N_("Toggle Grid"), false },
-	{ hotkey::HOTKEY_MOUSE_SCROLL, "mousescroll", N_("Mouse Scrolling"), false },
-	{ hotkey::HOTKEY_STATUS_TABLE, "statustable", N_("Status Table"), false },
-	{ hotkey::HOTKEY_MUTE, "mute", N_("Mute"), false },
-	{ hotkey::HOTKEY_SPEAK, "speak", N_("Speak"), false },
-	{ hotkey::HOTKEY_CREATE_UNIT, "createunit", N_("Create Unit (Debug!)"), false },
-	{ hotkey::HOTKEY_CHANGE_UNIT_SIDE, "changeside", N_("Change Unit Side (Debug!)"), false },
-	{ hotkey::HOTKEY_PREFERENCES, "preferences", N_("Preferences"), false },
-	{ hotkey::HOTKEY_OBJECTIVES, "objectives", N_("Scenario Objectives"), false },
-	{ hotkey::HOTKEY_UNIT_LIST, "unitlist", N_("Unit List"), false },
-	{ hotkey::HOTKEY_STATISTICS, "statistics", N_("Statistics"), false },
-	{ hotkey::HOTKEY_QUIT_GAME, "quit", N_("Quit Game"), false },
-	{ hotkey::HOTKEY_LABEL_TEAM_TERRAIN, "labelteamterrain", N_("Set Team Label"), false },
-	{ hotkey::HOTKEY_LABEL_TERRAIN, "labelterrain", N_("Set Label"), false },
-	{ hotkey::HOTKEY_CLEAR_LABELS, "clearlabels", N_("Clear Labels"), false },
-	{ hotkey::HOTKEY_SHOW_ENEMY_MOVES, "showenemymoves", N_("Show Enemy Moves"), false },
-	{ hotkey::HOTKEY_BEST_ENEMY_MOVES, "bestenemymoves", N_("Best Possible Enemy Moves"), false },
-	{ hotkey::HOTKEY_PLAY_REPLAY, "playreplay", N_("Play"), false },
-	{ hotkey::HOTKEY_RESET_REPLAY, "resetreplay", N_("Reset"), false },
-	{ hotkey::HOTKEY_STOP_REPLAY, "stopreplay", N_("Stop"), false },
-	{ hotkey::HOTKEY_REPLAY_NEXT_TURN, "replaynextturn", N_("Next Turn"), false },
-	{ hotkey::HOTKEY_REPLAY_NEXT_SIDE, "replaynextside", N_("Next Side"), false },
+	{ hotkey::HOTKEY_CYCLE_UNITS, "cycle", N_("Next Unit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CYCLE_BACK_UNITS, "cycleback", N_("Previous Unit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_UNIT_HOLD_POSITION, "holdposition", N_("Hold Position"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_END_UNIT_TURN, "endunitturn", N_("End Unit Turn"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_LEADER, "leader", N_("Leader"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_UNDO, "undo", N_("Undo"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_REDO, "redo", N_("Redo"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_ZOOM_IN, "zoomin", N_("Zoom In"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_ZOOM_OUT, "zoomout", N_("Zoom Out"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_ZOOM_DEFAULT, "zoomdefault", N_("Default Zoom"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_FULLSCREEN, "fullscreen", N_("Toggle Full Screen"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_SCREENSHOT, "screenshot", N_("Screenshot"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_MAP_SCREENSHOT, "mapscreenshot", N_("Map Screenshot"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_ACCELERATED, "accelerated", N_("Accelerated"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_UNIT_DESCRIPTION, "describeunit", N_("Unit Description"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_RENAME_UNIT, "renameunit", N_("Rename Unit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SAVE_GAME, "save", N_("Save Game"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SAVE_REPLAY, "savereplay", N_("Save Replay"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SAVE_MAP, "savemap", N_("Save The Map"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_LOAD_GAME, "load", N_("Load Game"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_RECRUIT, "recruit", N_("Recruit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_REPEAT_RECRUIT, "repeatrecruit", N_("Repeat Recruit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_RECALL, "recall", N_("Recall"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_ENDTURN, "endturn", N_("End Turn"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_TOGGLE_GRID, "togglegrid", N_("Toggle Grid"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_MOUSE_SCROLL, "mousescroll", N_("Mouse Scrolling"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_STATUS_TABLE, "statustable", N_("Status Table"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_MUTE, "mute", N_("Mute"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_SPEAK, "speak", N_("Speak"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CREATE_UNIT, "createunit", N_("Create Unit (Debug!)"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CHANGE_UNIT_SIDE, "changeside", N_("Change Unit Side (Debug!)"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_PREFERENCES, "preferences", N_("Preferences"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_OBJECTIVES, "objectives", N_("Scenario Objectives"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_UNIT_LIST, "unitlist", N_("Unit List"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_STATISTICS, "statistics", N_("Statistics"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_STOP_NETWORK, "stopnetwork", N_("Stop"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_START_NETWORK, "startnetwork", N_("Play"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_QUIT_GAME, "quit", N_("Quit Game"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_LABEL_TEAM_TERRAIN, "labelteamterrain", N_("Set Team Label"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_LABEL_TERRAIN, "labelterrain", N_("Set Label"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CLEAR_LABELS, "clearlabels", N_("Clear Labels"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SHOW_ENEMY_MOVES, "showenemymoves", N_("Show Enemy Moves"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_BEST_ENEMY_MOVES, "bestenemymoves", N_("Best Possible Enemy Moves"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_PLAY_REPLAY, "playreplay", N_("Play"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_RESET_REPLAY, "resetreplay", N_("Reset"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_STOP_REPLAY, "stopreplay", N_("Stop"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_REPLAY_NEXT_TURN, "replaynextturn", N_("Next Turn"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_REPLAY_NEXT_SIDE, "replaynextside", N_("Next Side"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_REPLAY_SHOW_EVERYTHING, "replayshoweverything",
-	  N_("Full map"), false },
+	  N_("Full map"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_REPLAY_SHOW_EACH, "replayshoweach",
-	  N_("Each team"), false },
+	  N_("Each team"), false, hotkey::SCOPE_GAME },
 	{ hotkey::HOTKEY_REPLAY_SHOW_TEAM1, "replayshowteam1",
-	  N_("Team 1"), false },
-	{ hotkey::HOTKEY_REPLAY_SKIP_ANIMATION, "replayskipanimation", N_("Skip animation"), false },
+	  N_("Team 1"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_REPLAY_SKIP_ANIMATION, "replayskipanimation", N_("Skip animation"), false, hotkey::SCOPE_GAME },
 
-	{ hotkey::HOTKEY_EDIT_SET_TERRAIN, "editsetterrain", N_("Set Terrain"),true },
-	{ hotkey::HOTKEY_EDIT_QUIT, "editquit", N_("Quit Editor"),true },
-	{ hotkey::HOTKEY_EDIT_NEW_MAP, "editnewmap", N_("New Map"),true },
-	{ hotkey::HOTKEY_EDIT_LOAD_MAP, "editloadmap", N_("Load Map"),true },
-	{ hotkey::HOTKEY_EDIT_SAVE_MAP, "editsavemap", N_("Save Map"),true },
-	{ hotkey::HOTKEY_EDIT_SAVE_AS, "editsaveas", N_("Save As"),true },
-	{ hotkey::HOTKEY_EDIT_SET_START_POS, "editsetstartpos", N_("Set Player's keep"),true },
-	{ hotkey::HOTKEY_EDIT_FLOOD_FILL, "editfloodfill", N_("Flood Fill"),true },
-	{ hotkey::HOTKEY_EDIT_FILL_SELECTION, "editfillselection", N_("Fill Selection"),true },
-	{ hotkey::HOTKEY_EDIT_ROTATE_SELECTION, "editrotateselection", N_("Rotate Selection"),true },
-	{ hotkey::HOTKEY_EDIT_CUT, "editcut", N_("Cut"),true },
-	{ hotkey::HOTKEY_EDIT_COPY, "editcopy", N_("Copy"),true },
-	{ hotkey::HOTKEY_EDIT_PASTE, "editpaste", N_("Paste"),true },
-	{ hotkey::HOTKEY_EDIT_REVERT, "editrevert", N_("Revert from Disk"),true },
-	{ hotkey::HOTKEY_EDIT_RESIZE, "editresize", N_("Resize Map"),true },
-	{ hotkey::HOTKEY_EDIT_FLIP, "editflip", N_("Flip Map"),true },
-	{ hotkey::HOTKEY_EDIT_SELECT_ALL, "editselectall", N_("Select All"),true },
-	{ hotkey::HOTKEY_EDIT_DRAW, "editdraw", N_("Draw Terrain"),true },
-	{ hotkey::HOTKEY_EDIT_REFRESH, "editrefresh", N_("Refresh Image Cache"), true },
-	{ hotkey::HOTKEY_EDIT_AUTO_UPDATE, "editautoupdate", N_("Delay transition updates"), true },
-	{ hotkey::HOTKEY_EDIT_UPDATE, "editupdate", N_("Update transitions"), true },
+#ifndef DISABLE_EDITOR2
+	{ hotkey::HOTKEY_EDITOR_QUIT_TO_DESKTOP, "editor-quit-to-desktop", N_("Quit to Desktop"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_CLOSE_MAP, "editor-close-map", N_("Close Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SWITCH_MAP, "editor-switch-map", N_("Switch Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SETTINGS, "editor-settings", N_("Editor Settings"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_PARTIAL_UNDO, "editor-partial-undo", N_("Partial Undo"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_NEW, "editor-map-new", N_("New Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_LOAD, "editor-map-load", N_("Load Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_SAVE, "editor-map-save", N_("Save Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_SAVE_AS, "editor-map-save-as", N_("Save Map As"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_REVERT, "editor-map-revert", N_("Revert All Changes"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_INFO, "editor-map-info", N_("Map Information"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_TERRAIN_PALETTE_SWAP, "editor-terrain-palette-swap",
+		N_("Swap fore- and background terrains"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_TOOL_NEXT, "editor-tool-next", N_("Next Tool"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_TOOL_PAINT, "editor-tool-paint", N_("Paint Tool"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_TOOL_FILL, "editor-tool-fill", N_("Fill Tool"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_TOOL_SELECT, "editor-tool-select", N_("Selection Tool"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_TOOL_STARTING_POSITION, "editor-tool-starting-position",
+		N_("Set Starting Positions Tool"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_BRUSH_NEXT, "editor-brush-next", N_("Next Brush"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_BRUSH_DEFAULT, "editor-brush-default", N_("Default Brush"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_CUT, "editor-cut", N_("Cut"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_COPY, "editor-copy", N_("Copy"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_PASTE, "editor-paste", N_("Paste"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECT_ALL, "editor-select-all",
+		 N_("Select All"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECT_INVERSE, "editor-select-inverse",
+		 N_("Select Inverse"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECT_NONE, "editor-select-none",
+		 N_("Select None"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_CLIPBOARD_ROTATE_CW, "editor-clipboard-rotate-cw",
+		 N_("Rotate Clipboard Clockwise"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_CLIPBOARD_ROTATE_CCW, "editor-clipboard-rotate-ccw",
+		 N_("Rotate Clipboard Counter-Clockwise"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_CLIPBOARD_FLIP_HORIZONTAL, "editor-clipboard-flip-horizontal",
+		N_("Flip Clipboard Horizontally"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_CLIPBOARD_FLIP_VERTICAL, "editor-clipboard-flip-vertical",
+		N_("Flip Clipboard Vertically"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECTION_ROTATE, "editor-selection-rotate",
+		N_("Rotate Selection"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECTION_FLIP, "editor-selection-flip",
+		N_("Flip Selection"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECTION_FILL, "editor-selection-fill",
+		N_("Fill Selection"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECTION_GENERATE, "editor-selection-generate",
+		N_("Generate Tiles In Selection"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_SELECTION_RANDOMIZE, "editor-selection-randomize",
+		N_("Randomize Tiles In Selection"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_RESIZE, "editor-map-resize",
+		N_("Resize Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_ROTATE, "editor-map-rotate",
+		N_("Rotate Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_GENERATE, "editor-map-generate",
+		 N_("Generate Map"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_APPLY_MASK, "editor-map-apply-mask",
+		 N_("Apply a Mask"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_MAP_CREATE_MASK_TO, "editor-map-create-mask-to",
+		 N_("Create Mask"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_REFRESH, "editor-refresh",
+		N_("Refresh Display"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_UPDATE_TRANSITIONS, "editor-update-transitions",
+		N_("Update Terrain Transitions"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_AUTO_UPDATE_TRANSITIONS, "editor-auto-update-transitions",
+		N_("Auto-update Terrain Transitions"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_REFRESH_IMAGE_CACHE, "editor-refresh-image-cache",
+		N_("Refresh Image Cache"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_DRAW_COORDINATES, "editor-draw-coordinates",
+		N_("Draw Hex Coordinates"), false, hotkey::SCOPE_EDITOR },
+	{ hotkey::HOTKEY_EDITOR_DRAW_TERRAIN_CODES, "editor-draw-terrain-codes",
+		N_("Draw Terrain Codes"), false, hotkey::SCOPE_EDITOR },
 
-	{ hotkey::HOTKEY_DELAY_SHROUD, "delayshroud", N_("Delay Shroud Updates"), false },
-	{ hotkey::HOTKEY_UPDATE_SHROUD, "updateshroud", N_("Update Shroud Now"), false },
-	{ hotkey::HOTKEY_CONTINUE_MOVE, "continue", N_("Continue Move"), false },
-	{ hotkey::HOTKEY_SEARCH, "search", N_("Find Label or Unit"), false },
-	{ hotkey::HOTKEY_SPEAK_ALLY, "speaktoally", N_("Speak to Ally"), false },
-	{ hotkey::HOTKEY_SPEAK_ALL, "speaktoall", N_("Speak to All"), false },
-	{ hotkey::HOTKEY_HELP, "help", N_("Help"), false },
-	{ hotkey::HOTKEY_CHAT_LOG, "chatlog", N_("View Chat Log"), false },
-	{ hotkey::HOTKEY_LANGUAGE, "changelanguage", N_("Change the language"), true },
 
-	{ hotkey::HOTKEY_USER_CMD, "command", N_("Enter user command"), false },
-	{ hotkey::HOTKEY_CLEAR_MSG, "clearmessages", N_("Clear messages"), false },
-#ifdef USRCMD2
-	{ hotkey::HOTKEY_USER_CMD_2, "usercommand#2", N_("User-Command#2"), false },
-	{ hotkey::HOTKEY_USER_CMD_3, "usercommand#3", N_("User-Command#3"), false },
 #endif
-	{ hotkey::HOTKEY_NULL, NULL, NULL, true }
+
+	{ hotkey::HOTKEY_DELAY_SHROUD, "delayshroud", N_("Delay Shroud Updates"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_UPDATE_SHROUD, "updateshroud", N_("Update Shroud Now"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CONTINUE_MOVE, "continue", N_("Continue Move"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SEARCH, "search", N_("Find Label or Unit"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SPEAK_ALLY, "speaktoally", N_("Speak to Ally"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_SPEAK_ALL, "speaktoall", N_("Speak to All"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_HELP, "help", N_("Help"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_CHAT_LOG, "chatlog", N_("View Chat Log"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_LANGUAGE, "changelanguage", N_("Change the language"), true, hotkey::SCOPE_GENERAL },
+
+	{ hotkey::HOTKEY_USER_CMD, "command", N_("Enter user command"), false, hotkey::SCOPE_GENERAL },
+	{ hotkey::HOTKEY_CUSTOM_CMD, "customcommand", N_("Custom command"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_AI_FORMULA, "aiformula", N_("Run AI formula"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_CLEAR_MSG, "clearmessages", N_("Clear messages"), false, hotkey::SCOPE_GAME },
+#ifdef USRCMD2
+	{ hotkey::HOTKEY_USER_CMD_2, "usercommand#2", N_("User-Command#2"), false, hotkey::SCOPE_GAME },
+	{ hotkey::HOTKEY_USER_CMD_3, "usercommand#3", N_("User-Command#3"), false, hotkey::SCOPE_GAME },
+#endif
+	{ hotkey::HOTKEY_NULL, NULL, NULL, true, hotkey::SCOPE_GENERAL }
 };
 
 std::vector<hotkey::hotkey_item> hotkeys_;
 hotkey::hotkey_item null_hotkey_;
 
+std::string hotkey_tag_name = "hotkey";
+
+const std::string scope_strings_[] = {"general", "game", "editor"};
+const std::string scope_labels_[] = {"Common", "Game", "Editor"};
+std::vector<bool> scope_active_(hotkey::SCOPE_COUNT, false);
 }
 
 namespace hotkey {
 
-const std::string CLEARED_TEXT = "__none__";
+
+void deactivate_all_scopes()
+{
+	for (int i = 0; i < hotkey::SCOPE_COUNT; ++i) {
+		scope_active_[i] = false;
+	}
+}
+
+void set_scope_active(scope s, bool set)
+{
+	scope_active_[s] = set;
+}
+
+bool is_scope_active(scope s)
+{
+	return scope_active_[s];
+}
+
+const std::string& get_scope_string(scope s)
+{
+	return scope_strings_[s];
+}
+
+const std::string& get_scope_label(scope s)
+{
+	return scope_labels_[s];
+}
 
 static void key_event_execute(display& disp, const SDL_KeyboardEvent& event, command_executor* executor);
 
+const std::string CLEARED_TEXT = "__none__";
 
-hotkey_item::hotkey_item(HOTKEY_COMMAND id, 
-		const std::string& command, const std::string& description, bool hidden) :
-	id_(id), 
-	command_(command), 
-	description_(description), 
+hotkey_item::hotkey_item(HOTKEY_COMMAND id,
+		const std::string& command, const t_string &description, bool hidden,
+		scope s) :
+	id_(id),
+	command_(command),
+	description_(description),
+	scope_(s),
 	type_(UNBOUND),
 	character_(0),
-	ctrl_(false), 
-	alt_(false), 
+	ctrl_(false),
+	alt_(false),
 	cmd_(false),
 	keycode_(0),
-	shift_(false), 
+	shift_(false),
 	hidden_(hidden)
 {
 }
@@ -184,10 +267,10 @@ void hotkey_item::load_from_config(const config& cfg)
 {
 	const std::string& key = cfg["key"];
 
-	alt_ = (cfg["alt"] == "yes");
-	cmd_ = (cfg["cmd"] == "yes");
-	ctrl_ = (cfg["ctrl"] == "yes");
-	shift_ = (cfg["shift"] == "yes");
+	alt_ = utils::string_bool(cfg["alt"]);
+	cmd_ = utils::string_bool(cfg["cmd"]);
+	ctrl_ = utils::string_bool(cfg["ctrl"]);
+	shift_ = utils::string_bool(cfg["shift"]);
 
 	if (!key.empty()) {
 		// They may really want a specific key on the keyboard: we assume
@@ -254,7 +337,7 @@ std::string hotkey_item::get_name() const
 	return str.str();
 }
 
-void hotkey_item::set_description(const std::string& description)
+void hotkey_item::set_description(const t_string &description)
 {
 	description_ = description;
 }
@@ -308,17 +391,48 @@ void hotkey_item::set_key(int character, int keycode, bool shift, bool ctrl, boo
 }
 
 manager::manager()
+{
+	init();
+}
+
+void manager::init()
 {
 	for (int i = 0; hotkey_list_[i].command; ++i) {
 		hotkeys_.push_back(hotkey_item(hotkey_list_[i].id, hotkey_list_[i].command,
-				"", hotkey_list_[i].hidden));
+				"", hotkey_list_[i].hidden, hotkey_list_[i].scope));
 	}
 }
 
+void manager::wipe()
+{
+	hotkeys_.clear();
+}
 
 manager::~manager()
 {
-	hotkeys_.clear();
+	wipe();
+}
+
+scope_changer::scope_changer(const config& cfg, const std::string& hotkey_tag)
+: cfg_(cfg)
+, prev_tag_name_(hotkey_tag_name)
+, prev_scope_active_(scope_active_)
+{
+	manager::wipe();
+	manager::init();
+	hotkey::load_descriptions();
+	load_hotkeys(cfg_);
+	set_hotkey_tag_name(hotkey_tag);
+}
+
+scope_changer::~scope_changer()
+{
+	scope_active_.swap(prev_scope_active_);
+	manager::wipe();
+	manager::init();
+	hotkey::load_descriptions();
+	set_hotkey_tag_name(prev_tag_name_);
+	load_hotkeys(cfg_);
 }
 
 void load_descriptions()
@@ -327,16 +441,20 @@ void load_descriptions()
 		if (i >= hotkeys_.size()) {
 			ERR_G << "Hotkey list too short: " << hotkeys_.size() << "\n";
 		}
-		hotkeys_[i].set_description(dsgettext(PACKAGE "-lib", hotkey_list_[i].description));
+		hotkeys_[i].set_description(t_string(hotkey_list_[i].description, PACKAGE "-lib"));
 	}
 }
 
+void set_hotkey_tag_name(const std::string& name)
+{
+	hotkey_tag_name = name;
+}
+
 void load_hotkeys(const config& cfg)
 {
-	const config::child_list& children = cfg.get_children("hotkey");
+	const config::child_list& children = cfg.get_children(hotkey_tag_name);
 	for(config::child_list::const_iterator i = children.begin(); i != children.end(); ++i) {
 		hotkey_item& h = get_hotkey((**i)["command"]);
-
 		if(h.get_id() != HOTKEY_NULL) {
 			h.load_from_config(**i);
 		}
@@ -345,22 +463,20 @@ void load_hotkeys(const config& cfg)
 
 void save_hotkeys(config& cfg)
 {
-	cfg.clear_children("hotkey");
+	cfg.clear_children(hotkey_tag_name);
 
 	for(std::vector<hotkey_item>::iterator i = hotkeys_.begin(); i != hotkeys_.end(); ++i) {
-		if (i->hidden() || i->get_type() == hotkey_item::UNBOUND)
+		if (i->hidden() || i->get_type() == hotkey_item::UNBOUND || !i->is_in_active_scope())
 			continue;
-		config& item = cfg.add_child("hotkey");
 
+		config& item = cfg.add_child(hotkey_tag_name);
 		item["command"] = i->get_command();
-
 		if (i->get_type() == hotkey_item::CLEARED)
 		{
 			item["key"] = CLEARED_TEXT;
 			continue;
 		}
 
-
 		if (i->get_type() == hotkey_item::BY_KEYCODE) {
 			item["key"] = SDL_GetKeyName(SDLKey(i->get_keycode()));
 			item["shift"] = i->get_shift() ? "yes" : "no";
@@ -403,7 +519,8 @@ hotkey_item& get_hotkey(const std::string& command)
 	return *itor;
 }
 
-hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool alt, bool cmd)
+hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl,
+	bool alt, bool cmd)
 {
 	std::vector<hotkey_item>::iterator itor;
 
@@ -432,21 +549,29 @@ hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool
 		if (itor->get_type() == hotkey_item::BY_CHARACTER) {
 			if (character == itor->get_character()) {
 				if (ctrl == itor->get_ctrl()
-					&& alt == itor->get_alt()
-					&& cmd == itor->get_cmd()) {
-					DBG_G << "Could match by character..." << "yes\n";
-					break;
+						&& alt == itor->get_alt()
+						&& cmd == itor->get_cmd()) {
+					if (itor->is_in_active_scope()) {
+						DBG_G << "Could match by character..." << "yes\n";
+						break;
+					} else {
+						DBG_G << "Could match by character..." << "yes, but scope is inactive\n";
+					}
 				}
 				DBG_G << "Could match by character..." << "but modifiers different\n";
 			}
 		} else if (itor->get_type() == hotkey_item::BY_KEYCODE) {
 			if (keycode == itor->get_keycode()) {
 				if (shift == itor->get_shift()
-					&& ctrl == itor->get_ctrl()
-					&& alt == itor->get_alt()
-					&& cmd == itor->get_cmd()) {
-					DBG_G << "Could match by keycode..." << "yes\n";
-					break;
+						&& ctrl == itor->get_ctrl()
+						&& alt == itor->get_alt()
+						&& cmd == itor->get_cmd()) {
+					if (itor->is_in_active_scope()) {
+						DBG_G << "Could match by keycode..." << "yes\n";
+						break;
+					} else {
+						DBG_G << "Could match by keycode..." << "yes, but scope is inactive\n";
+					}
 				}
 				DBG_G << "Could match by keycode..." << "but modifiers different\n";
 			}
@@ -471,11 +596,12 @@ hotkey_item& get_hotkey(const SDL_KeyboardEvent& event)
 #endif
 			);
 }
-void _get_visible_hotkey_itor(int index, std::vector<hotkey_item>::iterator& itor)
+
+static void _get_visible_hotkey_itor(int index, std::vector<hotkey_item>::iterator& itor)
 {
 	int counter = 0;
 	for (itor = hotkeys_.begin(); itor != hotkeys_.end(); ++itor) {
-		if (itor->hidden())
+		if (itor->hidden() || !itor->is_in_active_scope())
 			continue;
 
 		if (index == counter)
@@ -638,6 +764,12 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 		case HOTKEY_STATISTICS:
 			show_statistics();
 			break;
+		case HOTKEY_STOP_NETWORK:
+			stop_network();
+			break;
+		case HOTKEY_START_NETWORK:
+			start_network();
+			break;
 		case HOTKEY_LABEL_TEAM_TERRAIN:
 			label_terrain(true);
 			break;
@@ -674,7 +806,14 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 		case HOTKEY_USER_CMD:
 			user_command();
 			break;
+		case HOTKEY_CUSTOM_CMD:
+			custom_command();
+			break;
 //%%
+		case HOTKEY_AI_FORMULA:
+			std::cerr <<" run ai formula\n";
+			ai_formula();
+			break;
 		case HOTKEY_CLEAR_MSG:
 			clear_messages();
 			break;
@@ -688,70 +827,6 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 			break;
 #endif
 
-		case HOTKEY_EDIT_SET_TERRAIN:
-			edit_set_terrain();
-			break;
-		case HOTKEY_EDIT_QUIT:
-			edit_quit();
-			break;
-		 case HOTKEY_EDIT_NEW_MAP:
-			edit_new_map();
-			break;
-		 case HOTKEY_EDIT_LOAD_MAP:
-			edit_load_map();
-			break;
-		 case HOTKEY_EDIT_SAVE_MAP:
-			edit_save_map();
-			break;
-		 case HOTKEY_EDIT_SAVE_AS:
-			edit_save_as();
-			break;
-		 case HOTKEY_EDIT_SET_START_POS:
-			edit_set_start_pos();
-			break;
-		 case HOTKEY_EDIT_FLOOD_FILL:
-			edit_flood_fill();
-			break;
-		 case HOTKEY_EDIT_FILL_SELECTION:
-			edit_fill_selection();
-			break;
-		 case HOTKEY_EDIT_ROTATE_SELECTION:
-			edit_rotate_selection();
-			break;
-		 case HOTKEY_EDIT_CUT:
-			edit_cut();
-			break;
-		 case HOTKEY_EDIT_PASTE:
-			edit_paste();
-			break;
-		 case HOTKEY_EDIT_COPY:
-			edit_copy();
-			break;
-		 case HOTKEY_EDIT_REVERT:
-			edit_revert();
-			break;
-		 case HOTKEY_EDIT_RESIZE:
-			edit_resize();
-			break;
-		 case HOTKEY_EDIT_FLIP:
-			edit_flip();
-			break;
-		 case HOTKEY_EDIT_SELECT_ALL:
-			edit_select_all();
-			break;
-		 case HOTKEY_EDIT_DRAW:
-			edit_draw();
-			break;
-		 case HOTKEY_EDIT_REFRESH:
-			edit_refresh();
-			break;
-		 case HOTKEY_EDIT_UPDATE:
-			edit_update();
-			break;
-		 case HOTKEY_EDIT_AUTO_UPDATE:
-			edit_auto_update();
-			break;
-
 		 case HOTKEY_LANGUAGE:
 			change_language();
 			break;
@@ -791,6 +866,7 @@ bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
 void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* executor, int index)
 {
 	const int zoom_amount = 4;
+	bool map_screenshot = false;
 
 	if(executor != NULL) {
 		if(!executor->can_execute_command(command, index) || executor->execute_command(command, index))
@@ -809,12 +885,26 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 		case HOTKEY_FULLSCREEN:
 			preferences::set_fullscreen(!preferences::fullscreen());
 			break;
+		case HOTKEY_MAP_SCREENSHOT:
+			if (!disp.in_game() && !disp.in_editor())
+				break;
+			map_screenshot = true;
+		case HOTKEY_SCREENSHOT: {
+			std::string name = map_screenshot ? _("Map-Screenshot") : _("Screenshot");
+			std::string filename = get_screenshot_dir() + "/" + name + "_";
+			filename = get_next_filename(filename, ".bmp");
+			int size = disp.screenshot(filename, map_screenshot);
+			if (size > 0) {
+				std::stringstream res;
+				res << filename << " ( " << size/1000000 <<" "<< (size/1000)%1000 << " kB )";
+				gui::dialog(disp,_("Screenshot done"),res.str(),gui::MESSAGE).show();
+			} else
+				gui::dialog(disp,_("Screenshot failed"),"",gui::MESSAGE).show();
+			break;
+		}
 		case HOTKEY_MOUSE_SCROLL:
 			preferences::enable_mouse_scroll(!preferences::mouse_scroll_enabled());
 			break;
-		case HOTKEY_SCREENSHOT:
-			disp.screenshot();
-			break;
 		case HOTKEY_ACCELERATED:
 			preferences::set_turbo(!preferences::turbo());
 			break;
@@ -887,7 +977,7 @@ void command_executor::show_menu(const std::vector<std::string>& items_arg, int
 }
 
 std::string command_executor::get_menu_image(hotkey::HOTKEY_COMMAND command, int index) const {
-	switch(get_action_state(command)) {
+	switch(get_action_state(command, index)) {
 		case ACTION_ON: return game_config::checked_menu_image;
 		case ACTION_OFF: return game_config::unchecked_menu_image;
 		default: return get_action_image(command, index);
