@@ -1,6 +1,6 @@
-/* $Id: hotkeys.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: hotkeys.hpp 33339 2009-03-04 13:38:05Z silene $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,6 +15,7 @@
 #define HOTKEYS_HPP_INCLUDED
 
 #include "events.hpp"
+#include "tstring.hpp"
 #include "SDL.h"
 
 #include <string>
@@ -28,18 +29,29 @@ class display;
 //refers to a hotkey command being executed.
 namespace hotkey {
 
+/** Available hotkey scopes. The scope is used to allow command from
+ * non-overlapping areas of the game share the same key
+ */
+enum scope {
+	SCOPE_GENERAL,
+	SCOPE_GAME,
+	SCOPE_EDITOR,
+	SCOPE_COUNT
+};
+
+
 enum HOTKEY_COMMAND {
 	HOTKEY_CYCLE_UNITS,HOTKEY_CYCLE_BACK_UNITS, HOTKEY_UNIT_HOLD_POSITION,
 	HOTKEY_END_UNIT_TURN, HOTKEY_LEADER,
 	HOTKEY_UNDO, HOTKEY_REDO,
 	HOTKEY_ZOOM_IN, HOTKEY_ZOOM_OUT, HOTKEY_ZOOM_DEFAULT,
-	HOTKEY_FULLSCREEN, HOTKEY_SCREENSHOT, HOTKEY_ACCELERATED,
+	HOTKEY_FULLSCREEN, HOTKEY_SCREENSHOT, HOTKEY_MAP_SCREENSHOT, HOTKEY_ACCELERATED,
 	HOTKEY_UNIT_DESCRIPTION, HOTKEY_RENAME_UNIT,
 	HOTKEY_SAVE_GAME, HOTKEY_SAVE_REPLAY, HOTKEY_SAVE_MAP, HOTKEY_LOAD_GAME,
 	HOTKEY_RECRUIT, HOTKEY_REPEAT_RECRUIT, HOTKEY_RECALL, HOTKEY_ENDTURN,
 	HOTKEY_TOGGLE_GRID, HOTKEY_STATUS_TABLE, HOTKEY_MUTE, HOTKEY_MOUSE_SCROLL,
 	HOTKEY_SPEAK, HOTKEY_CREATE_UNIT, HOTKEY_CHANGE_UNIT_SIDE, HOTKEY_PREFERENCES,
-	HOTKEY_OBJECTIVES, HOTKEY_UNIT_LIST, HOTKEY_STATISTICS, HOTKEY_QUIT_GAME,
+	HOTKEY_OBJECTIVES, HOTKEY_UNIT_LIST, HOTKEY_STATISTICS, HOTKEY_STOP_NETWORK, HOTKEY_START_NETWORK, HOTKEY_QUIT_GAME,
 	HOTKEY_LABEL_TEAM_TERRAIN, HOTKEY_LABEL_TERRAIN, HOTKEY_CLEAR_LABELS,HOTKEY_SHOW_ENEMY_MOVES, HOTKEY_BEST_ENEMY_MOVES,
 	HOTKEY_DELAY_SHROUD, HOTKEY_UPDATE_SHROUD, HOTKEY_CONTINUE_MOVE,
 	HOTKEY_SEARCH, HOTKEY_SPEAK_ALLY, HOTKEY_SPEAK_ALL, HOTKEY_HELP,
@@ -49,18 +61,38 @@ enum HOTKEY_COMMAND {
 	HOTKEY_REPLAY_SHOW_EACH, HOTKEY_REPLAY_SHOW_TEAM1,
 	HOTKEY_REPLAY_SKIP_ANIMATION,
 
-	//editing specific commands
-	HOTKEY_EDIT_SET_TERRAIN,
-	HOTKEY_EDIT_QUIT, HOTKEY_EDIT_SAVE_MAP,
-	HOTKEY_EDIT_SAVE_AS, HOTKEY_EDIT_SET_START_POS,
-	HOTKEY_EDIT_NEW_MAP, HOTKEY_EDIT_LOAD_MAP, HOTKEY_EDIT_FLOOD_FILL,
-	HOTKEY_EDIT_FILL_SELECTION, HOTKEY_EDIT_ROTATE_SELECTION, HOTKEY_EDIT_CUT, HOTKEY_EDIT_COPY,
-	HOTKEY_EDIT_PASTE, HOTKEY_EDIT_REVERT, HOTKEY_EDIT_RESIZE,
-	HOTKEY_EDIT_FLIP, HOTKEY_EDIT_SELECT_ALL, HOTKEY_EDIT_DRAW,
-	HOTKEY_EDIT_REFRESH, HOTKEY_EDIT_UPDATE, HOTKEY_EDIT_AUTO_UPDATE,
+#ifndef DISABLE_EDITOR2
+	HOTKEY_EDITOR_QUIT_TO_DESKTOP,
+	HOTKEY_EDITOR_CLOSE_MAP, HOTKEY_EDITOR_SWITCH_MAP,
+	HOTKEY_EDITOR_SETTINGS,
+	HOTKEY_EDITOR_PARTIAL_UNDO,
+	HOTKEY_EDITOR_MAP_NEW, HOTKEY_EDITOR_MAP_LOAD, HOTKEY_EDITOR_MAP_SAVE,
+	HOTKEY_EDITOR_MAP_SAVE_AS, HOTKEY_EDITOR_MAP_REVERT, HOTKEY_EDITOR_MAP_INFO,
+	HOTKEY_EDITOR_TERRAIN_PALETTE_SWAP,
+	HOTKEY_EDITOR_TOOL_NEXT, HOTKEY_EDITOR_TOOL_PAINT, HOTKEY_EDITOR_TOOL_FILL,
+	HOTKEY_EDITOR_TOOL_SELECT, HOTKEY_EDITOR_TOOL_STARTING_POSITION,
+	HOTKEY_EDITOR_BRUSH_NEXT, HOTKEY_EDITOR_BRUSH_DEFAULT,
+	HOTKEY_EDITOR_CUT, HOTKEY_EDITOR_COPY, HOTKEY_EDITOR_PASTE,
+	HOTKEY_EDITOR_SELECT_ALL, HOTKEY_EDITOR_SELECT_INVERSE,
+	HOTKEY_EDITOR_SELECT_NONE,
+	HOTKEY_EDITOR_CLIPBOARD_ROTATE_CW, HOTKEY_EDITOR_CLIPBOARD_ROTATE_CCW,
+	HOTKEY_EDITOR_CLIPBOARD_FLIP_HORIZONTAL, HOTKEY_EDITOR_CLIPBOARD_FLIP_VERTICAL,
+	HOTKEY_EDITOR_SELECTION_ROTATE, HOTKEY_EDITOR_SELECTION_FLIP,
+	HOTKEY_EDITOR_SELECTION_FILL,
+	HOTKEY_EDITOR_SELECTION_GENERATE, HOTKEY_EDITOR_SELECTION_RANDOMIZE,
+	HOTKEY_EDITOR_MAP_RESIZE, HOTKEY_EDITOR_MAP_ROTATE,
+	HOTKEY_EDITOR_MAP_GENERATE, HOTKEY_EDITOR_MAP_APPLY_MASK,
+	HOTKEY_EDITOR_MAP_CREATE_MASK_TO,
+	HOTKEY_EDITOR_REFRESH, HOTKEY_EDITOR_UPDATE_TRANSITIONS,
+	HOTKEY_EDITOR_AUTO_UPDATE_TRANSITIONS,
+	HOTKEY_EDITOR_REFRESH_IMAGE_CACHE,
+	HOTKEY_EDITOR_DRAW_COORDINATES, HOTKEY_EDITOR_DRAW_TERRAIN_CODES,
+#endif
 
 	//misc.
 	HOTKEY_USER_CMD,
+	HOTKEY_CUSTOM_CMD,
+	HOTKEY_AI_FORMULA,
 	HOTKEY_CLEAR_MSG,
 #ifdef USRCMD2
 	HOTKEY_USER_CMD_2,
@@ -69,12 +101,19 @@ enum HOTKEY_COMMAND {
 	HOTKEY_NULL
 };
 
+void deactivate_all_scopes();
+void set_scope_active(scope s, bool set = true);
+bool is_scope_active(scope s);
+const std::string& get_scope_string(scope s);
+const std::string& get_scope_label(scope s);
+
 class hotkey_item {
 public:
-	hotkey_item() : 
+	hotkey_item() :
 		id_(HOTKEY_NULL),
 		command_(),
 		description_(),
+		scope_(SCOPE_GENERAL),
 		type_(UNBOUND),
 		character_(0),
 		ctrl_(false),
@@ -85,15 +124,17 @@ public:
 		hidden_(false)
 		{}
 
-	hotkey_item(HOTKEY_COMMAND id, const std::string& command, const std::string& description, bool hidden=false);
+	hotkey_item(HOTKEY_COMMAND id, const std::string& command,
+		const t_string &description, bool hidden = false,
+		scope s=SCOPE_GENERAL);
 
 	HOTKEY_COMMAND get_id() const { return id_; };
 	const std::string& get_command() const { return command_; };
-	const std::string& get_description() const { return description_; };
+	const t_string &get_description() const { return description_; };
 
 	void load_from_config(const config& cfg);
 
-	void set_description(const std::string& description);
+	void set_description(const t_string &description);
 	void clear_hotkey();
 	void set_key(int character, int keycode, bool shift, bool ctrl, bool alt, bool cmd);
 
@@ -106,6 +147,12 @@ public:
 
 	enum type get_type() const { return type_; }
 
+
+	/** @return the scope of this hotkey */
+	scope get_scope() const { return scope_; }
+
+	bool is_in_active_scope() const { return is_scope_active(get_scope()); }
+
 	// Returns unicode value of keypress.
 	int get_character() const { return character_; }
 	bool get_alt() const { return alt_; }
@@ -124,7 +171,8 @@ public:
 private:
 	HOTKEY_COMMAND id_;
 	std::string command_;
-	std::string description_;
+	t_string description_;
+	scope scope_;
 
 	// UNBOUND means unset, CHARACTER means see character_, KEY means keycode_.
 	enum type type_;
@@ -141,23 +189,40 @@ private:
 	bool shift_;
 
 	bool hidden_;
+
 };
 
+
+
 class manager {
 public:
 	manager();
+	static void init();
+	static void wipe();
 	~manager();
 };
 
+class scope_changer {
+public:
+	scope_changer(const config& cfg, const std::string& hotkey_tag);
+	~scope_changer();
+private:
+	const config& cfg_;
+	std::string prev_tag_name_;
+	std::vector<bool> prev_scope_active_;
+};
+
 void load_descriptions();
 
+void set_hotkey_tag_name(const std::string& name);
 void load_hotkeys(const config& cfg);
 void save_hotkeys(config& cfg);
 
 hotkey_item& get_hotkey(HOTKEY_COMMAND id);
 hotkey_item& get_hotkey(const std::string& command);
 
-hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool alt, bool cmd);
+hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl,
+	bool alt, bool cmd);
 hotkey_item& get_hotkey(const SDL_KeyboardEvent& event);
 
 hotkey_item& get_visible_hotkey(int index);
@@ -201,6 +266,8 @@ public:
 	virtual void objectives() {}
 	virtual void unit_list() {}
 	virtual void show_statistics() {}
+	virtual void stop_network() {}
+	virtual void start_network() {}
 	virtual void label_terrain(bool /*team_only*/) {}
 	virtual void clear_labels() {}
 	virtual void show_enemy_moves(bool /*ignore_units*/) {}
@@ -211,6 +278,8 @@ public:
 	virtual void show_help() {}
 	virtual void show_chat_log() {}
 	virtual void user_command() {}
+	virtual void custom_command() {}
+	virtual void ai_formula() {}
 	virtual void clear_messages() {}
 #ifdef USRCMD2
 	virtual void user_command_2() {}
@@ -227,33 +296,10 @@ public:
 	virtual void replay_show_team1() {}
 	virtual void replay_skip_animation() {}
 
-	// Map editor stuff.
-	virtual void edit_set_terrain() {}
-	virtual void edit_quit() {}
-	virtual void edit_new_map() {}
-	virtual void edit_load_map() {}
-	virtual void edit_save_map() {}
-	virtual void edit_save_as() {}
-	virtual void edit_set_start_pos() {}
-	virtual void edit_flood_fill() {}
-	virtual void edit_fill_selection() {}
-	virtual void edit_rotate_selection() {}
-	virtual void edit_cut() {}
-	virtual void edit_copy() {}
-	virtual void edit_paste() {}
-	virtual void edit_revert() {}
-	virtual void edit_resize() {}
-	virtual void edit_flip() {}
-	virtual void edit_select_all() {}
-	virtual void edit_draw() {}
-	virtual void edit_refresh() {}
-	virtual void edit_update() {}
-	virtual void edit_auto_update() {}
-
 	//Gets the action's image (if any). Displayed left of the action text in menus.
 	virtual std::string get_action_image(hotkey::HOTKEY_COMMAND /*command*/, int /*index*/) const { return ""; }
 	//Does the action control a toggle switch? If so, return the state of the action (on or off)
-	virtual ACTION_STATE get_action_state(hotkey::HOTKEY_COMMAND /*command*/) const { return ACTION_STATELESS; }
+	virtual ACTION_STATE get_action_state(hotkey::HOTKEY_COMMAND /*command*/, int /*index*/) const { return ACTION_STATELESS; }
 	//Returns the appropriate menu image. Checkable items will get a checked/unchecked image.
 	std::string get_menu_image(hotkey::HOTKEY_COMMAND command, int index=-1) const;
 	//Returns a vector of images for a given menu
