@@ -1,7 +1,6 @@
-/* $Id: language.cpp 23842 2008-02-16 08:47:16Z mordante $ */
-/* vim:set encoding=utf-8: */
+/* $Id: language.cpp 33428 2009-03-08 14:50:09Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -15,33 +14,74 @@
 
 #include "global.hpp"
 
-#include "config.hpp"
 #include "filesystem.hpp"
-#include "game_config.hpp"
+#include "foreach.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
 #include "log.hpp"
 #include "preferences.hpp"
-#include "util.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "wesconfig.h"
 
-#include <algorithm>
-#include <cassert>
-#include <cctype>
-#include <cerrno>
+#include <stdexcept>
 #include <clocale>
-#include <cstdlib>
-#include <cstring>
-#include <iostream>
+#include <boost/scoped_array.hpp>
+
+#ifdef _WIN32
+#include <windows.h>
+#ifndef _MSC_VER
+extern "C" int _putenv(const char*);
+#endif
+#endif
+
+#ifdef __APPLE__
+#include <cerrno>
+#endif
+
+#define DBG_FS LOG_STREAM(debug, filesystem)
+#define DBG_GENERAL LOG_STREAM(debug, general)
+#define LOG_GENERAL LOG_STREAM(debug, general)
+#define WRN_GENERAL LOG_STREAM(debug, general)
+
+
+/** Tests one locale to be available. */
+static bool has_locale(const char* s) {
+	try {
+		// The way to find out whether a locale is available is to set it and
+		// hope not runtime error gets thrown.
+		std::locale dummy(s);
+		return true;
+	} catch (std::runtime_error&) {
+		return false;
+	}
+}
+
+/** Test the locale for a language and it's utf-8 variations. */
+static bool has_language(const std::string& language)
+{
+	if(has_locale(language.c_str())) {
+		return true;
+	}
+
+	std::string utf = language + ".utf-8";
+	if(has_locale(utf.c_str())) {
+		return true;
+	}
+
+	utf = language + ".UTF-8";
+	if(has_locale(utf.c_str())) {
+		return true;
+	}
+
+	return false;
+}
 
 namespace {
 	language_def current_language;
 	string_map strings_;
 }
 
-static std::vector<language_def> known_languages;
+static language_list known_languages;
 
 std::string languagedef_name (const language_def& def)
 {
@@ -58,6 +98,33 @@ bool language_def::operator== (const language_def& a) const
 	return ((language == a.language) /* && (localename == a.localename) */ );
 }
 
+bool language_def::available() const
+{
+#if defined(_WIN32) || defined(__APPLE__)
+	// Under windows and OSX all locales are available and testing for it seems
+	// to fail so just return true.
+	return true;
+#endif
+
+	if (game_config::use_dummylocales)
+	{
+		// Dummy has every language available.
+		return true;
+	} else {
+		if(has_language(localename)) {
+			return true;
+		} else {
+			foreach(const std::string& lang, alternates) {
+				if(has_language(lang)) {
+					return true;
+				}
+			}
+		}
+
+		return false;
+	}
+}
+
 symbol_table string_table;
 
 const t_string& symbol_table::operator[](const std::string& key) const
@@ -85,7 +152,7 @@ bool load_language_list()
 {
 	config cfg;
 	try {
-		scoped_istream stream = preprocess_file("data/hardwired/language.cfg");
+		scoped_istream stream = preprocess_file(get_wml_location("hardwired/language.cfg"));
 		read(cfg, *stream);
 	} catch(config::error &) {
 		return false;
@@ -93,27 +160,31 @@ bool load_language_list()
 
 	known_languages.clear();
 	known_languages.push_back(
-		language_def("", t_string(N_("System default language"), "wesnoth"), "ltr"));
+		language_def("", t_string(N_("System default language"), "wesnoth"), "ltr", "", "A"));
 
 	config::const_child_itors langs = cfg.child_range("locale");
 	for(;langs.first != langs.second; ++langs.first) {
 		known_languages.push_back(
 			language_def((**langs.first)["locale"], (**langs.first)["name"], (**langs.first)["dir"],
-				(**langs.first)["alternates"]));
+				(**langs.first)["alternates"], (**langs.first)["sort_name"]));
 	}
 
 	return true;
 }
 
-std::vector<language_def> get_languages()
+language_list get_languages()
 {
+	// We sort every time, the local might have changed which can modify the
+	// sort order.
+	std::sort(known_languages.begin(), known_languages.end());
 	return known_languages;
 }
 
-static void wesnoth_setlocale(int category, std::string const &slocale,
+static void wesnoth_setlocale(int category, std::string slocale,
 	std::vector<std::string> const *alternates)
 {
-	char const *locale = slocale.c_str();
+	const char *locale = slocale.c_str();
+	std::string extra;
 	// FIXME: ideally we should check LANGUAGE and on first invocation
 	// use that value, so someone with es would get the game in Spanish
 	// instead of en_US the first time round
@@ -139,59 +210,92 @@ static void wesnoth_setlocale(int category, std::string const &slocale,
 #endif
 
 #ifdef _WIN32
-	const std::string env = "LANG=" + slocale;
-	putenv(env.c_str());
+	std::string env = "LANG=" + slocale;
+	_putenv(env.c_str());
+	env = "LC_ALL=" + slocale;
+	_putenv(env.c_str());
+	std::string win_locale = locale;
+	win_locale = win_locale.substr(0,2);
+	#include "language_win32.ii"
+	SetEnvironmentVariable("LANG", win_locale.c_str());
+	SetEnvironmentVariable("LC_ALL", win_locale.c_str());
+	if(category == LC_MESSAGES)
+	    category = LC_ALL;
+	locale = win_locale.c_str();
 #endif
 
-#ifdef USE_DUMMYLOCALES
-	static enum { UNINIT, NONE, PRESENT } status = UNINIT;
-	static std::string locpath;
-	if (status == UNINIT)
-		if (char const *p = getenv("LOCPATH")) {
-			locpath = p;
-			status = PRESENT;
-		} else status = NONE;
-	if (slocale.empty())
-		if (status == NONE)
-			unsetenv("LOCPATH");
-		else
-			setenv("LOCPATH", locpath.c_str(), 1);
-	else setenv("LOCPATH", (game_config::path + "/locales").c_str(), 1);
-	std::string xlocale;
-	if (!slocale.empty()) {
-		// dummy suffix to prevent locale aliasing from kicking in
-		xlocale = slocale + "@wesnoth";
-		locale = xlocale.c_str();
+#ifndef _WIN32
+	// dummy locales aren't working in windows
+	if (game_config::use_dummylocales)
+	{
+		static enum { UNINIT, NONE, PRESENT } status = UNINIT;
+		static std::string locpath;
+		if (status == UNINIT) {
+			if (char const *p = getenv("LOCPATH")) {
+				locpath = p;
+				status = PRESENT;
+			} else status = NONE;
+		}
+		if (slocale.empty())
+			if (status == NONE)
+				unsetenv("LOCPATH");
+			else
+				setenv("LOCPATH", locpath.c_str(), 1);
+		else {
+			setenv("LOCPATH", (game_config::path + "/locales").c_str(), 1);
+			DBG_GENERAL << "LOCPATH set to '" << (game_config::path + "/locales") << "'\n";
+		}
+		std::string xlocale;
+		if (!slocale.empty()) {
+			// dummy suffix to prevent locale aliasing from kicking in
+			extra = "@wesnoth";
+			xlocale = slocale + "@wesnoth";
+			locale = xlocale.c_str();
+		}
 	}
 #endif
 
 	char *res = NULL;
-	char const *try_loc = locale;
+	std::string orig_locale;
+	orig_locale.assign(locale);
+
+	typedef boost::scoped_array<char> char_array;
+	size_t length = orig_locale.length()+1;
+	char_array try_loc(new char[length]);
+	orig_locale.copy(try_loc.get(), orig_locale.length());
+	try_loc[orig_locale.length()] = 0;
 	std::vector<std::string>::const_iterator i;
 	if (alternates) i = alternates->begin();
 	while (true) {
-		res = std::setlocale(category, try_loc);
+		res = std::setlocale(category, try_loc.get());
 		if (res) break;
 
-		std::string utf8 = std::string(try_loc) + std::string(".utf-8");
+		std::string utf8 = orig_locale + std::string(".utf-8");
 		res = std::setlocale(category, utf8.c_str());
 		if (res) break;
 
-		utf8 = std::string(try_loc) + std::string(".UTF-8");
+		utf8 = orig_locale + std::string(".UTF-8");
 		res = std::setlocale(category, utf8.c_str());
 		if (res) break;
 
 		if (!alternates) break;
 		if (i == alternates->end()) break;
-		try_loc = i->c_str();
+		orig_locale = *i + extra;
+		if (length < orig_locale.length()+1)
+		{
+			length = orig_locale.length()+1;
+			try_loc.reset(new char[length]);
+		}
+		orig_locale.copy(try_loc.get(), orig_locale.length());
+		try_loc[orig_locale.length()] = 0;
 		i++;
 	}
 
 	if (res == NULL)
-		std::cerr << "WARNING: setlocale() failed for '"
+		WRN_GENERAL << "WARNING: setlocale() failed for '"
 			  << locale << "'.\n";
 	else
-		std::cerr << "set locale to '" << try_loc << "'\n";
+		LOG_GENERAL << "set locale to '" << (try_loc.get()) << "' result: '" << res <<"'\n";
 }
 
 bool set_language(const language_def& locale)
@@ -210,7 +314,7 @@ bool set_language(const language_def& locale)
 
 	// fill string_table (should be moved somwhere else some day)
 	try {
-		scoped_istream stream = preprocess_file("data/hardwired/english.cfg");
+		scoped_istream stream = preprocess_file(get_wml_location("hardwired/english.cfg"));
 		read(cfg, *stream);
 	} catch(config::error& e) {
 		std::cerr << "Could not read english.cfg\n";
@@ -229,7 +333,7 @@ bool set_language(const language_def& locale)
 	// end of string_table fill
 
 	// Reset translations for the name of current languages
-	for (std::vector<language_def>::iterator itor = known_languages.begin();
+	for (language_list::iterator itor = known_languages.begin();
 			itor != known_languages.end(); ++itor) {
 
 		itor->language.reset_translation();
@@ -249,7 +353,7 @@ const language_def& get_locale()
 	const std::string& prefs_locale = preferences::language();
 	if(prefs_locale.empty() == false) {
 		wesnoth_setlocale(LC_MESSAGES, prefs_locale, NULL);
-		for(std::vector<language_def>::const_iterator i = known_languages.begin();
+		for(language_list::const_iterator i = known_languages.begin();
 				i != known_languages.end(); ++i) {
 			if (prefs_locale == i->localename)
 				return *i;
@@ -260,6 +364,11 @@ const language_def& get_locale()
 
 #if 0
 	const char* const locale = getenv("LANG");
+	#ifdef _WIN32
+	    std::string win_locale = locale
+		#include "language_win32.ii"
+		return win_locale;
+	#endif
 	if(locale != NULL && strlen(locale) >= 2) {
 		//we can't pass pointers into the string to the std::string
 		//constructor because some STL implementations don't support
@@ -286,12 +395,13 @@ void init_textdomains(const config& cfg)
 		if(path.empty()) {
 			t_string::add_textdomain(name, get_intl_dir());
 		} else {
-			const std::string& location = get_binary_file_location(path, "");
+			std::string location = get_binary_dir_location("", path);
 
-			//if location is empty, this causes a crash on Windows, so we
-			//disallow adding empty domains
-			if(location.empty()) {
-				std::cerr << "no location found for '" << path << "', not adding textdomain\n";
+			if (location.empty()) {
+				//if location is empty, this causes a crash on Windows, so we
+				//disallow adding empty domains
+				LOG_STREAM(err, general) << "no location found for '" << path
+					<< "', not adding textdomain\n";
 			} else {
 				t_string::add_textdomain(name, location);
 			}
@@ -299,3 +409,4 @@ void init_textdomains(const config& cfg)
 	}
 }
 
+/* vim:set encoding=utf-8: */
