@@ -1,6 +1,6 @@
-/* $Id: language.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: language.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -33,25 +33,39 @@ struct language_def
 		localename(),
 		alternates(),
 		language(),
+		sort_name(),
 		rtl(false)
 		{}
 
 	language_def(const std::string& name, const t_string& lang, const std::string& dir,
-	    const std::string &salternates = "") :
-		localename(name), 
-		alternates(),
-		language(lang), 
+		    const std::string &salternates = "", const std::string& sort_name = "") :
+		localename(name),
+		alternates(utils::split(salternates)),
+		language(lang),
+		sort_name(sort_name.empty() ? std::string(lang) : sort_name),
 		rtl(dir == "rtl")
 	{
-            alternates = utils::split(salternates);
-        }
+    }
+
 	std::string localename;
 	std::vector<std::string> alternates;
 	t_string language;
+	std::string sort_name;
 	bool rtl;		// A right to left language? (e.g: Hebrew)
 	bool operator== (const language_def&) const;
+	bool operator< (const language_def& a) const
+		{ return sort_name < a.sort_name; }
+
+	/**
+	 * Is the locale available on the system?
+	 *
+	 * If the dummy locales are selected we always return true.
+	 */
+	bool available() const;
 };
 
+typedef std::vector<language_def> language_list;
+
 std::string languagedef_name (const language_def& def);
 
 struct symbol_table
