@@ -1,6 +1,6 @@
-/* $Id: mapgen_dialog.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: mapgen_dialog.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -28,12 +28,14 @@ public:
 
 	std::string name() const;
 
+	std::string config_name() const;
+
 	std::string create_map(const std::vector<std::string>& args);
 	config create_scenario(const std::vector<std::string>& args);
 
 private:
 
-	std::string generate_map(const std::vector<std::string>& args, std::map<gamemap::location,std::string>* labels=NULL);
+	std::string generate_map(const std::vector<std::string>& args, std::map<map_location,std::string>* labels=NULL);
 
 	size_t default_width_, default_height_, width_, height_, island_size_, iterations_, hill_size_, max_lakes_, nvillages_, castle_size_, nplayers_;
 	bool link_castles_;
