@@ -1,6 +1,6 @@
-/* $Id: marked-up_text.cpp 29185 2008-09-02 09:40:29Z suokko $ */
+/* $Id: marked-up_text.cpp 32725 2009-02-09 19:31:04Z baufo $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,16 +12,19 @@
    See the COPYING file for more details.
 */
 
-//! @file marked-up_text.cpp
-//! Support for simple markup in text (fonts, colors, images).
-//! E.g. "@Victory" will be shown in green.
+/**
+ * @file marked-up_text.cpp
+ * Support for simple markup in text (fonts, colors, images).
+ * E.g. "@Victory" will be shown in green.
+ */
+
+#define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "global.hpp"
 
 #include "font.hpp"
 #include "gettext.hpp"
 #include "marked-up_text.hpp"
-#include "team.hpp"
 #include "video.hpp"
 #include "wml_exception.hpp"
 
@@ -35,8 +38,7 @@ const char LARGE_TEXT='*', SMALL_TEXT='`',
            GREEN_TEXT='@', RED_TEXT='#',
            COLOR_TEXT='<', IMAGE='&';
 
-//! Parses the markup-tags at the front of a string.
-static std::string::const_iterator parse_markup(std::string::const_iterator i1,
+std::string::const_iterator parse_markup(std::string::const_iterator i1,
 												std::string::const_iterator i2,
 												int* font_size,
 												SDL_Color* colour, int* style)
@@ -127,8 +129,6 @@ static std::string::const_iterator parse_markup(std::string::const_iterator i1,
 	return i1;
 }
 
-
-//! Copy string, but without tags at the beginning
 std::string del_tags(const std::string& text){
 	int ignore_int;
 	SDL_Color ignore_color;
@@ -137,62 +137,38 @@ std::string del_tags(const std::string& text){
 	for(line = lines.begin(); line != lines.end(); ++line) {
 		std::string::const_iterator i1 = line->begin(),
 			i2 = line->end();
-		*line = std::string(parse_markup(i1,i2,&ignore_int,&ignore_color,&ignore_int),i2); 
+		*line = std::string(parse_markup(i1,i2,&ignore_int,&ignore_color,&ignore_int),i2);
 	}
 	return utils::join(lines, '\n');
 }
 
-//! Copy string, but with NULL MARKUP tag at the beginning of each line
 std::string nullify_markup(const std::string& text) {
 	std::vector<std::string> lines = utils::split(text, '\n', 0);
 	std::vector<std::string>::iterator line;
 	for(line = lines.begin(); line != lines.end(); ++line) {
-		*line = std::string() + NULL_MARKUP + *line; 
+		*line = std::string() + NULL_MARKUP + *line;
 	}
 	return utils::join(lines, '\n');
 }
 
-
-//! Create string of color-markup, such as "<255,255,0>" for yellow.
 std::string color2markup(const SDL_Color color) {
 	std::stringstream markup;
 	// The RGB of SDL_Color are Uint8, we need to cast them to int.
 	// Without cast, it gives their char equivalent.
-	markup << "<" 
-		   << static_cast<int>(color.r) << "," 
-		   << static_cast<int>(color.g) << "," 
+	markup << "<"
+		   << static_cast<int>(color.r) << ","
+		   << static_cast<int>(color.g) << ","
 		   << static_cast<int>(color.b) << ">";
 	return markup.str();
 }
 
-//! Calculate the size of a text (in pixels) if it were to be drawn.
 SDL_Rect text_area(const std::string& text, int size, int style)
 {
 	const SDL_Rect area = {0,0,10000,10000};
 	return draw_text(NULL, area, size, font::NORMAL_COLOUR, text, 0, 0, false, style);
 }
 
-//! Draw text on the screen, clip text to area. Supports simple markup.
-//!
-//! If the text runs outside of area horizontally, 
-//! an ellipsis will be displayed at the end of it.
-//! If use_tooltips is true, then text with an ellipsis will have a 
-//! tooltip set for it equivalent to the entire contents of the text.
-//!
-//! Some very basic 'markup' will be done on the text:
-//! - any line beginning in # will be displayed in BAD_COLOUR  (red)
-//! - any line beginning in @ will be displayed in GOOD_COLOUR (green)
-//! - any line beginning in + will be displayed with size increased by 2
-//! - any line beginning in - will be displayed with size decreased by 2
-//! - any line beginning with 0x0n will be displayed in the colour of side n
-//!
-//! The above special characters can be quoted using a C-style backslash.
-//!
-//! A bounding rectangle of the text is returned. 
-//! If gui is NULL, then the text will not be drawn, 
-//! and only a bounding rectangle will be returned.
-//! 
-SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
+SDL_Rect draw_text(surface dst, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& txt,
                    int x, int y, bool use_tooltips, int style)
 {
@@ -221,7 +197,7 @@ SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
 
 			utils::unescape(new_string);
 
-			const SDL_Rect rect = draw_text_line(gui, area, sz, col, new_string, x, y, use_tooltips, text_style);
+			const SDL_Rect rect = draw_text_line(dst, area, sz, col, new_string, x, y, use_tooltips, text_style);
 			if(rect.w > res.w) {
 				res.w = rect.w;
 			}
@@ -241,9 +217,13 @@ SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
 	return res;
 }
 
-//! Determine if char is one of the special chars used as markup.
-//! @retval true	input-char is a markup-char
-//! @retval false	input-char is a normal char
+SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
+                   const SDL_Color& colour, const std::string& txt,
+                   int x, int y, bool use_tooltips, int style)
+{
+	return draw_text(gui != NULL ? gui->getSurface() : NULL, area, size, colour, txt, x, y, use_tooltips, style);
+}
+
 bool is_format_char(char c)
 {
 	switch(c) {
@@ -347,11 +327,6 @@ inline bool break_after(wchar_t ch)
 
 } // end of anon namespace
 
-//! Wrap text.
-//!
-//! If the text exceedes the specified max width, wrap it one a word basis.
-//! If this is not possible, e.g. the word is too big to fit, 
-//! wrap it on a char basis.
 std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height, int max_lines)
 {
 	VALIDATE(max_width > 0, _("The maximum text width is less than 1."));
@@ -386,7 +361,7 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 			font_sz = font_size;
 			style = TTF_STYLE_NORMAL;
 			parse_markup(format_string.begin(),format_string.end(),&font_sz,&color,&style);
-			current_line = format_string;
+			current_line.clear();
 			start_of_line = false;
 		}
 
@@ -443,7 +418,7 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 			}
 		}
 
-		if(line_break || current_word.empty() && ch == end) {
+		if(line_break || (current_word.empty() && ch == end)) {
 			SDL_Rect size = line_size(current_line, font_sz, style);
 			if(max_height > 0 && current_height + size.h >= size_t(max_height)) {
 				return wrapped_text;
@@ -468,16 +443,6 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 	return wrapped_text;
 }
 
-//! Draw text on the screen, fit text to maximum width, no markup, no tooltips.
-//!
-//! This method makes sure that the text fits within a given maximum width. 
-//! If a line exceedes this width, it will be wrapped 
-//! on a word basis if possible, otherwise on a char basis.
-//! This method is otherwise similar to the draw_text method,
-//! but it doesn't support special markup or tooltips.
-//!
-//! @return 	a bounding rectangle of the text.
-//!
 SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 		     const SDL_Color& colour, const std::string& text,
 		     int x, int y, int max_width)
@@ -486,8 +451,6 @@ SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 	return font::draw_text(gui, area, font_size, colour, wrapped_text, x, y, false);
 }
 
-
-//! Chop up one long string of text into lines.
 size_t text_to_lines(std::string& message, size_t max_length)
 {
 	std::string starting_markup;
@@ -497,7 +460,7 @@ size_t text_to_lines(std::string& message, size_t max_length)
 	for(std::string::iterator i = message.begin(); i != message.end(); ++i) {
 		if(at_start) {
 			if(font::is_format_char(*i)) {
-				push_back(starting_markup,*i);
+				starting_markup.push_back(*i);
 			} else {
 				at_start = false;
 			}
