@@ -1,6 +1,6 @@
-/* $Id: menu_events.hpp 23984 2008-02-21 03:03:24Z sapient $ */
+/* $Id: menu_events.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -17,62 +17,26 @@
 #define MENU_EVENTS_H_INCLUDED
 
 #include "global.hpp"
-
+#include "actions.hpp"
 #include "show_dialog.hpp"
 #include "display.hpp"
 #include "floating_textbox.hpp"
-#include "mouse_events.hpp"
-#include "statistics.hpp"
 #include "widgets/textbox.hpp"
 
 class game_state;
 class gamestatus;
 
-enum LEVEL_RESULT { VICTORY, DEFEAT, QUIT, LEVEL_CONTINUE, LEVEL_CONTINUE_NO_SAVE, OBSERVER_END, SKIP_TO_LINGER };
+namespace events {
+	class mouse_handler;
+}
 
-#define DELAY_END_LEVEL(end_ptr, code) try { \
-	code; \
-	} catch ( end_level_exception &e) { \
-		if (end_ptr == 0) { \
-			end_ptr = new end_level_exception(e); \
-		} \
-	}
-
-#define THROW_END_LEVEL_DELETE(end_ptr) if (end_ptr) {\
-	end_level_exception temp_exception(*end_ptr);\
-	delete end_ptr; \
-	throw temp_exception; \
-	}
-
-#define THROW_END_LEVEL(end_ptr) if (end_ptr) {\
-	throw end_level_exception(*end_ptr); \
-	}
-
-struct end_level_exception {
-	end_level_exception(LEVEL_RESULT res, const int percentage = -1, 
-			const bool add = false, const bool bonus=true) :
-   		result(res), 
-		gold_bonus(bonus),
-		carryover_percentage(percentage),
-		carryover_add(add)
-	{}
-
-	LEVEL_RESULT result;
-	bool gold_bonus;
-	int carryover_percentage;
-	bool carryover_add;
-};
-
-struct end_turn_exception {
-	end_turn_exception(unsigned int r = 0): redo(r) {}
-	unsigned int redo;
-};
+struct fallback_ai_to_human_exception {};
 
 namespace events{
 
 class chat_handler {
 public:
-	chat_handler() {}
+	chat_handler();
 	virtual ~chat_handler();
 
 protected:
@@ -85,12 +49,13 @@ protected:
 	virtual void send_chat_message(const std::string& message, bool allies_only=false)=0;
 	void send_command(const std::string& cmd, const std::string& args="");
 	void change_logging(const std::string& data);
+	friend class chat_command_handler;
 };
 
 class menu_handler : private chat_handler {
 public:
 	menu_handler(game_display* gui, unit_map& units, std::vector<team>& teams,
-		const config& level, const game_data& gameinfo, const gamemap& map,
+		const config& level, const gamemap& map,
 		const config& game_config, const gamestatus& status, game_state& gamestate,
 		undo_list& undo_stack, undo_list& redo_stack);
 	virtual ~menu_handler();
@@ -113,9 +78,9 @@ public:
 	void speak();
 	void whisper();
 	void shout();
-	void recruit(const bool browse, const unsigned int team_num, const gamemap::location& last_hex);
-	void repeat_recruit(const unsigned int team_num, const gamemap::location& last_hex);
-	void recall(const unsigned int team_num, const gamemap::location& last_hex);
+	void recruit(const bool browse, const unsigned int team_num, const map_location& last_hex);
+	void repeat_recruit(const unsigned int team_num, const map_location& last_hex);
+	void recall(const unsigned int team_num, const map_location& last_hex);
 	void undo(const unsigned int team_num);
 	void redo(const unsigned int team_num);
 	void show_enemy_moves(bool ignore_units, const unsigned int team_num);
@@ -135,6 +100,8 @@ public:
 	void end_unit_turn(mouse_handler& mousehandler, const unsigned int team_num);
 	void search();
 	void user_command();
+	void custom_command(mouse_handler& mousehandler, const unsigned int team_num);
+	void ai_formula();
 	void clear_messages();
 #ifdef USRCMD2
 	void user_command_2();
@@ -143,11 +110,12 @@ public:
 
 	unit_map::iterator current_unit(mouse_handler& mousehandler);
 	unit_map::const_iterator current_unit(const mouse_handler& mousehandler) const;
-	void move_unit_to_loc(const unit_map::const_iterator& ui, const gamemap::location& target,
+	void move_unit_to_loc(const unit_map::const_iterator& ui, const map_location& target,
 		bool continue_move, const unsigned int team_num, mouse_handler& mousehandler);
 	void do_speak();
 	void do_search(const std::string& new_search);
 	void do_command(const std::string& str, const unsigned int team_num, mouse_handler& mousehandler);
+	void do_ai_formula(const std::string& str, const unsigned int team_num, mouse_handler& mousehandler);
 	void clear_undo_stack(const unsigned int team_num);
 	void autosave(const std::string &label, unsigned turn, const config &starting_pos) const;
 	bool has_team() const;
@@ -157,12 +125,17 @@ protected:
 			game_display::MESSAGE_TYPE type=game_display::MESSAGE_PRIVATE);
 	void send_chat_message(const std::string& message, bool allies_only=false);
 private:
+	//console_handler is basically a sliced out part of menu_handler
+	//and as such needs access to menu_handler's privates
+	friend class console_handler;
+
 	//void do_speak(const std::string& message, bool allies_only);
-	void do_recruit(const std::string& name, const unsigned int team_num, const gamemap::location& last_hex);
+	void do_recruit(const std::string& name, const unsigned int team_num, const map_location& last_hex);
 //	std::vector<std::string> create_unit_table(const statistics::stats::str_int_map& m,unsigned int team);
 	void write_game_snapshot(config& start) const;
 	bool has_friends() const;
 	bool clear_shroud(const unsigned int team_num);
+	static void change_controller(const std::string& side, const std::string& controller);
 	static void change_side_controller(const std::string& side, const std::string& player, bool own_side=false);
 	void scenario_settings_table(int selected=0);
 
@@ -170,7 +143,6 @@ private:
 	unit_map& units_;
 	std::vector<team>& teams_;
 	const config& level_;
-	const game_data& gameinfo_;
 	const gamemap& map_;
 	const config& game_config_;
 	const gamestatus& status_;
@@ -180,7 +152,7 @@ private:
 	undo_list& redo_stack_;
 	gui::floating_textbox textbox_info_;
 	std::string last_search_;
-	gamemap::location last_search_hit_;
+	map_location last_search_hit_;
 
 	std::string last_recruit_;
 };
