@@ -1,6 +1,6 @@
-/* $Id: minimap.cpp 23963 2008-02-20 00:16:11Z esr $ */
+/* $Id: minimap.cpp 32647 2009-02-07 15:31:18Z ilor $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,16 +12,21 @@
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
+#include "minimap.hpp"
 
 #include "gettext.hpp"
 #include "image.hpp"
 #include "log.hpp"
-#include "minimap.hpp"
+#include "map.hpp"
 #include "team.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
 
 #define DBG_DP LOG_STREAM(debug, display)
+#define WRN_DP LOG_STREAM(warn, display)
 
 namespace image {
 
@@ -37,13 +42,7 @@ surface getMinimap(int w, int h, const gamemap& map, const viewpoint* vw)
 		return surface(NULL);
 	}
 
-	surface minimap(SDL_CreateRGBSurface(SDL_SWSURFACE,
-	                               map_width,map_height,
-	                               pixel_format->BitsPerPixel,
-	                               pixel_format->Rmask,
-	                               pixel_format->Gmask,
-	                               pixel_format->Bmask,
-	                               pixel_format->Amask));
+	surface minimap(create_neutral_surface(map_width, map_height));
 	if(minimap == NULL)
 		return surface(NULL);
 
@@ -56,12 +55,12 @@ surface getMinimap(int w, int h, const gamemap& map, const viewpoint* vw)
 
 			surface surf(NULL);
 
-			const gamemap::location loc(x,y);
+			const map_location loc(x,y);
 			if(map.on_board(loc)) {
 				const bool shrouded = vw != NULL && vw->shrouded(loc);
 				// shrouded hex are not considered fogged (no need to fog a black image)
 				const bool fogged = vw != NULL && !shrouded && vw->fogged(loc);
-				const t_translation::t_terrain terrain = shrouded ? 
+				const t_translation::t_terrain terrain = shrouded ?
 					t_translation::VOID_TERRAIN : map[loc];
 
 				bool need_fogging = false;
@@ -83,11 +82,34 @@ surface getMinimap(int w, int h, const gamemap& map, const viewpoint* vw)
 					if(tile == 0) {
 						utils::string_map symbols;
 						symbols["terrain"] = t_translation::write_terrain_code(terrain);
-						const std::string msg = 
+						const std::string msg =
 							vgettext("Could not get image for terrain: $terrain.", symbols);
 						VALIDATE(false, msg);
 					}
 
+					//Compose images of base and overlay if neccessary
+					if(map.get_terrain_info(terrain).is_combined()) {
+						surface overlay(get_image("terrain/" + map.get_terrain_info(terrain).minimap_image_overlay() + ".png", image::HEXED));
+						if(overlay != 0 && overlay != tile) {
+							surface combined = create_compatible_surface(tile, tile->w, tile->h);
+							SDL_Rect r;
+							r.x = 0;
+							r.y = 0;
+							SDL_BlitSurface(tile, NULL, combined, &r);
+							r.x = std::max(0, (tile->w - overlay->w)/2);
+							r.y = std::max(0, (tile->h - overlay->h)/2);
+                            if ((overlay->flags & SDL_RLEACCEL) == 0) {
+                                blit_surface(overlay, NULL, combined, &r);
+                            } else {
+                                WRN_DP << map.get_terrain_info(terrain).minimap_image_overlay() << ".png overlay is RLE-encoded, creating a neutral surface\n";
+                                surface overlay_neutral = make_neutral_surface(overlay);
+							    blit_surface(overlay_neutral, NULL, combined, &r);
+                            }
+							tile = combined;
+						}
+
+					}
+
 					surf = surface(scale_surface_blended(tile,scale,scale));
 
 					VALIDATE(surf != NULL, _("Error creating or aquiring an image."));
@@ -96,7 +118,7 @@ surface getMinimap(int w, int h, const gamemap& map, const viewpoint* vw)
 				}
 
 				surf = i->second;
-				
+
 				if (need_fogging) {
 					surf = surface(adjust_surface_colour(surf,-50,-50,-50));
 					fog_cache->insert(cache_map::value_type(terrain,surf));
@@ -120,8 +142,8 @@ surface getMinimap(int w, int h, const gamemap& map, const viewpoint* vw)
 
 	double wratio = w*1.0 / minimap->w;
 	double hratio = h*1.0 / minimap->h;
-	double ratio = minimum<double>(wratio, hratio);
-	
+	double ratio = std::min<double>(wratio, hratio);
+
 	minimap = scale_surface(minimap,
 		static_cast<int>(minimap->w * ratio), static_cast<int>(minimap->h * ratio));
 
