@@ -1,6 +1,6 @@
-/* $Id: mouse_events.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: mouse_events.hpp 33255 2009-03-01 17:58:38Z alink $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -22,42 +22,32 @@
 #include "game_display.hpp"
 #include "pathfind.hpp"
 #include "unit_map.hpp"
-
+#include "mouse_handler_base.hpp"
 class gamestatus;
 
 #include "SDL.h"
 
 namespace events{
 
-struct command_disabler
-{
-	command_disabler();
-	~command_disabler();
-};
-
-class mouse_handler{
+class mouse_handler : public mouse_handler_base {
 public:
 	mouse_handler(game_display* gui, std::vector<team>& teams, unit_map& units, gamemap& map,
-		gamestatus& status, const game_data& gameinfo, undo_list& undo_stack, undo_list& redo_stack);
+		gamestatus& status, undo_list& undo_stack, undo_list& redo_stack);
 	~mouse_handler();
 	static mouse_handler* get_singleton() { return singleton_ ;}
 	void set_team(const int team_number);
-	void mouse_motion(const SDL_MouseMotionEvent& event, const bool browse);
-	// update the mouse with a fake mouse motion
-	void mouse_update(const bool browse);
 	void mouse_press(const SDL_MouseButtonEvent& event, const bool browse);
 	void cycle_units(const bool browse, const bool reverse = false);
 	void cycle_back_units(const bool browse) { cycle_units(browse, true); }
 
 	int get_path_turns() const { return path_turns_; }
 	paths get_current_paths() { return current_paths_; }
-	const gamemap::location& get_last_hex() const { return last_hex_; }
-	gamemap::location get_selected_hex() const { return selected_hex_; }
-	const bool get_undo() const { return undo_; }
-	const bool get_show_menu() const { return show_menu_; }
+	const map_location& get_last_hex() const { return last_hex_; }
+	map_location get_selected_hex() const { return selected_hex_; }
+	bool get_undo() const { return undo_; }
 	void set_path_turns(const int path_turns) { path_turns_ = path_turns; }
 	void set_current_paths(paths new_paths);
-	void set_selected_hex(gamemap::location hex) { selected_hex_ = hex; }
+	void set_selected_hex(map_location hex) { selected_hex_ = hex; }
 	void deselect_hex();
 	void invalidate_reachmap() { reachmap_invalid_ = true; }
 
@@ -65,19 +55,28 @@ public:
 	void set_undo(const bool undo) { undo_ = undo; }
 
 	unit_map::iterator selected_unit();
-	paths::route get_route(unit_map::const_iterator un, gamemap::location go_to, team &team);
-private:
-	team& viewing_team() { return teams_[(*gui_).viewing_team()]; }
-	const team& viewing_team() const { return teams_[(*gui_).viewing_team()]; }
+	paths::route get_route(unit_map::const_iterator un, map_location go_to, team &team);
+protected:
+	/**
+	 * Due to the way this class is constructed we can assume that the
+	 * display* gui_ member actually points to a game_display (derived class)
+	 */
+	game_display& gui() { return *gui_; }
+	/** Const version */
+	const game_display& gui() const { return *gui_; }
+
+	team& viewing_team() { return teams_[gui().viewing_team()]; }
+	const team& viewing_team() const { return teams_[gui().viewing_team()]; }
 	team& current_team() { return teams_[team_num_-1]; }
 
-	// use update to force an update of the mouse state
+	int drag_threshold() const;
+	/**
+	 * Use update to force an update of the mouse state.
+	 */
 	void mouse_motion(int x, int y, const bool browse, bool update=false);
-	bool is_left_click(const SDL_MouseButtonEvent& event);
-	bool is_middle_click(const SDL_MouseButtonEvent& event);
-	bool is_right_click(const SDL_MouseButtonEvent& event);
-	void left_click(const SDL_MouseButtonEvent& event, const bool browse);
-	void select_hex(const gamemap::location& hex, const bool browse);
+	bool right_click_show_menu(int x, int y, const bool browse);
+	bool left_click(int x, int y, const bool browse);
+	void select_hex(const map_location& hex, const bool browse);
 	void clear_undo_stack();
 	bool move_unit_along_current_route(bool check_shroud, bool attackmove=false);
 	// wrapper to catch bad_alloc so this should be called
@@ -85,34 +84,25 @@ private:
 	// the real function but can throw bad_alloc
 	bool attack_enemy_(unit_map::iterator attacker, unit_map::iterator defender);
 	void show_attack_options(unit_map::const_iterator u);
-	gamemap::location current_unit_attacks_from(const gamemap::location& loc);
-	unit_map::const_iterator find_unit(const gamemap::location& hex) const;
-	unit_map::iterator find_unit(const gamemap::location& hex);
+	map_location current_unit_attacks_from(const map_location& loc);
+	unit_map::const_iterator find_unit(const map_location& hex) const;
+	unit_map::iterator find_unit(const map_location& hex);
 	bool unit_in_cycle(unit_map::const_iterator it);
-
+private:
+	gamemap& map_;
 	game_display* gui_;
 	std::vector<team>& teams_;
 	unit_map& units_;
-	gamemap& map_;
 	gamestatus& status_;
-	const game_data& gameinfo_;
 	undo_list& undo_stack_;
 	undo_list& redo_stack_;
 
-	bool minimap_scrolling_;
-	bool dragging_;
-	bool dragging_started_;
-	int drag_from_x_;
-	int drag_from_y_;
-
-	// last highlighted hex
-	gamemap::location last_hex_;
 	// previous highlighted hexes
 	// the hex of the selected unit and empty hex are "free"
-	gamemap::location previous_hex_;
-	gamemap::location previous_free_hex_;
-	gamemap::location selected_hex_;
-	gamemap::location next_unit_;
+	map_location previous_hex_;
+	map_location previous_free_hex_;
+	map_location selected_hex_;
+	map_location next_unit_;
 	paths::route current_route_;
 	paths current_paths_;
 	bool enemy_paths_;
@@ -123,7 +113,6 @@ private:
 	//computed with enemies_visible()
 	bool enemies_visible_;
 	bool undo_;
-	bool show_menu_;
 	bool over_route_;
 	bool attackmove_;
 	bool reachmap_invalid_;
@@ -132,7 +121,6 @@ private:
 	static mouse_handler * singleton_;
 };
 
-extern int commands_disabled;
 }
 
 #endif
