@@ -1,6 +1,6 @@
-/* $Id: multiplayer_connect.cpp 29171 2008-09-01 20:39:08Z suokko $ */
+/* $Id: multiplayer_connect.cpp 33040 2009-02-23 07:05:50Z dfranke $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,25 +12,25 @@
    See the COPYING file for more details.
 */
 
-//! @file multiplayer_connect.cpp 
-//! Prepare to join a multiplayer-game.
+/**
+ * @file multiplayer_connect.cpp
+ * Prepare to join a multiplayer-game.
+ */
 
 #include "global.hpp"
 
 #include "ai.hpp"
 #include "dialogs.hpp"
-#include "font.hpp"
-#include "game_config.hpp"
-#include "game_events.hpp"
-#include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
+#include "map.hpp"
 #include "multiplayer_connect.hpp"
 #include "statistics.hpp"
-#include "show_dialog.hpp"
-#include "serialization/string_utils.hpp"
+#include "unit_id.hpp"
+#include "wml_separators.hpp"
+#include "formula_string_utils.hpp"
 
-#include <cassert>
+#include <boost/bind.hpp>
 
 #define LOG_NW LOG_STREAM(info, network)
 #define ERR_NW LOG_STREAM(err, network)
@@ -43,7 +43,8 @@ namespace {
 		"network",
 		"human",
 		"ai",
-		"null"
+		"null",
+		"reserved"
 	};
 }
 
@@ -55,6 +56,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	index_(index),
 	id_(""), // Id is reset, and not imported from loading savegames
 	save_id_(cfg_.get_attribute("save_id")),
+	current_player_(cfg_.get_attribute("current_player")),
 	controller_(),
 	faction_(lexical_cast_default<int>(cfg_.get_attribute("faction"), 0)),
 	team_(0),
@@ -66,10 +68,10 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	ai_algorithm_(),
 	player_number_(parent.video(), lexical_cast_default<std::string>(index+1, ""),
 	               font::SIZE_LARGE, font::LOBBY_COLOUR),
-	combo_controller_(parent.disp(), parent.player_types_),
-	orig_controller_(parent.video(), cfg.get_attribute("description"), font::SIZE_SMALL),
+	combo_controller_(new gui::combo_drag(parent.disp(), parent.player_types_, parent.combo_control_group_)),
+	orig_controller_(parent.video(), current_player_, font::SIZE_SMALL),
 	combo_ai_algorithm_(parent.disp(), std::vector<std::string>()),
-	combo_faction_(parent.disp(), parent.player_factions_),
+	combo_faction_(parent.disp(), std::vector<std::string>()),
 	combo_leader_(parent.disp(), std::vector<std::string>()),
 	combo_gender_(parent.disp(), std::vector<std::string>()),
 	combo_team_(parent.disp(), parent.player_teams_),
@@ -80,8 +82,12 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	label_income_(parent.video(), _("Normal"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 	allow_player_(utils::string_bool(cfg_.get_attribute("allow_player"), true)),
 	enabled_(!parent_->params_.saved_game), changed_(false),
-	llm_(parent.era_sides_, &parent.game_data_, enabled_ ? &combo_leader_ : NULL, enabled_ ? &combo_gender_ : NULL)
+	llm_(parent.era_sides_, enabled_ ? &combo_leader_ : NULL, enabled_ ? &combo_gender_ : NULL)
 {
+	// convert ai controllers
+	if (cfg_["controller"] == "human_ai"
+			|| cfg_["controller"] == "network_ai")
+		cfg_["controller"] = "ai";
 	if(allow_player_ && enabled_) {
 		controller_ = parent_->default_controller_;
 	} else {
@@ -94,6 +100,12 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 		}
 		else
 		{
+			if (cfg_["controller"] == "network"
+					|| cfg_["controller"] == "human")
+			{
+				cfg_["controller"] = "reserved";
+
+			}
 			for(; i != CNTR_LAST; ++i) {
 				if(cfg_["controller"] == controller_names[i]) {
 					controller_ = static_cast<mp::controller>(i);
@@ -125,7 +137,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	label_gold_.hide(!enabled_);
 	label_income_.hide(!enabled_);
 
-	std::vector<std::string>::const_iterator itor = 
+	std::vector<std::string>::const_iterator itor =
 			std::find(parent_->team_names_.begin(), parent_->team_names_.end(),
 			cfg_.get_attribute("team_name"));
 	if(itor == parent_->team_names_.end()) {
@@ -137,6 +149,10 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 	if(!cfg_.get_attribute("colour").empty()) {
 		colour_ = game_config::color_info(cfg_["colour"]).index() - 1;
 	}
+	llm_.set_colour(colour_);
+
+	update_faction_combo();
+
 	config *ai = cfg_.child("ai");
 	if (ai)
 		ai_algorithm_ = lexical_cast_default<std::string>((*ai)["ai_algorithm"], "default");
@@ -152,7 +168,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 		combo_faction_.set_items(pseudo_factions);
 		combo_faction_.set_selected(0);
 
-		// Hack: if there is a unit which can recruit, use it as a leader. 
+		// Hack: if there is a unit which can recruit, use it as a leader.
 		// Necessary to display leader information when loading saves.
 		config::const_child_itors side_units = cfg.child_range("unit");
 		std::string leader_type;
@@ -167,17 +183,17 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 		if(leader_type.empty()) {
 			leader_name_pseudolist.push_back("-");
 		} else {
-			game_data::unit_type_map::const_iterator leader_name =
-				parent_->game_data_.unit_types.find(leader_type);
-			if(leader_name == parent_->game_data_.unit_types.end()) {
+			unit_type_data::unit_type_map::const_iterator leader_name =
+				unit_type_data::types().find_unit_type(leader_type);
+			if(leader_name == unit_type_data::types().end()) {
 				leader_name_pseudolist.push_back("-");
 			} else {
 				if (gender_ == "female")
-					leader_name_pseudolist.push_back(leader_name->second.get_gender_unit_type(unit_race::FEMALE).language_name());
+					leader_name_pseudolist.push_back(leader_name->second.get_gender_unit_type(unit_race::FEMALE).type_name());
 				else if (gender_ == "male")
-					leader_name_pseudolist.push_back(leader_name->second.get_gender_unit_type(unit_race::MALE).language_name());
+					leader_name_pseudolist.push_back(leader_name->second.get_gender_unit_type(unit_race::MALE).type_name());
 				else
-					leader_name_pseudolist.push_back(leader_name->second.language_name());
+					leader_name_pseudolist.push_back(leader_name->second.type_name());
 			}
 		}
 		combo_leader_.set_items(leader_name_pseudolist);
@@ -186,8 +202,8 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 
 		if (!gender_.empty()) {
 			if (leader_type.empty()
-					|| parent_->game_data_.unit_types.find(leader_type)
-					== parent_->game_data_.unit_types.end())
+					|| unit_type_data::types().find_unit_type(leader_type)
+					== unit_type_data::types().end())
 			{
 				gender_name_pseudolist.push_back("-");
 			} else {
@@ -230,17 +246,17 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 			llm_.set_leader_combo(NULL);
 			llm_.set_gender_combo(NULL);
 			std::vector<std::string> leader_name_pseudolist;
-			game_data::unit_type_map::const_iterator leader_name = parent_->game_data_.unit_types.find(leader_);
-			if(leader_name == parent_->game_data_.unit_types.end()) {
+			unit_type_data::unit_type_map::const_iterator leader_name = unit_type_data::types().find_unit_type(leader_);
+			if(leader_name == unit_type_data::types().end()) {
 				leader_name_pseudolist.push_back("?");
 			} else {
-				leader_name_pseudolist.push_back(leader_name->second.language_name());
+				leader_name_pseudolist.push_back(leader_name->second.type_name());
 			}
 			combo_leader_.set_items(leader_name_pseudolist);
 			combo_leader_.set_selected(0);
 			std::vector<std::string> gender_name_pseudolist;
 			if (!gender_.empty()) {
-				if(leader_name == parent_->game_data_.unit_types.end()) {
+				if(leader_name == unit_type_data::types().end()) {
 					gender_name_pseudolist.push_back("?");
 				} else {
 					if (gender_ == "female")
@@ -256,7 +272,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 			combo_gender_.set_selected(0);
 		}
 
-		// Try to pick a faction for the sake of appearance 
+		// Try to pick a faction for the sake of appearance
 		// and for filling in the blanks
 		if(faction_ == 0) {
 			std::vector<std::string> find;
@@ -265,7 +281,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 				// Choose based on faction
 				find.push_back(cfg_["faction"]);
 				search_field = "id";
-			} else if(!cfg_.get_attribute("recruit").empty()) {
+			} else if(utils::string_bool(cfg["faction_from_recruit"]) && !cfg_.get_attribute("recruit").empty()) {
 				// Choose based on recruit
 				find = utils::split(cfg_["recruit"]);
 				search_field = "recruit";
@@ -274,31 +290,36 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 				find.push_back(leader_);
 				search_field = "leader";
 			}
-
-			std::vector<std::string>::const_iterator search = find.begin();
-			while(search != find.end()) {
-				int faction_index = 0;
-				std::vector<config*>::const_iterator faction = parent.era_sides_.begin();
-				while(faction_ == 0 && faction != parent.era_sides_.end()) {
-					const config& side = (**faction);
-					std::vector<std::string> recruit;
-					recruit = utils::split(side[search_field]);
+			// Pick the first faction with the greater amount of data matching the criteria
+			int faction_index = 0;
+			int best_score = 0;
+			std::vector<config*>::const_iterator faction = parent.era_sides_.begin();
+			while(faction != parent.era_sides_.end()) {
+				int faction_score = 0;
+				const config& side = (**faction);
+				std::vector<std::string> recruit;
+				recruit = utils::split(side[search_field]);
+				std::vector<std::string>::const_iterator search = find.begin();
+				while(search != find.end()) {
 					for(itor = recruit.begin(); itor != recruit.end(); ++itor) {
 						if(*itor == *search) {
-							faction_ = faction_index;
-							llm_.update_leader_list(faction_);
-							llm_.update_gender_list(llm_.get_leader());
-							combo_faction_.enable(false);
+							faction_score++;
+							break;
 						}
 					}
-					++faction;
-					faction_index++;
+					++search;
 				}
-				// Exit outmost loop if we've found a faction
-				if(!combo_faction_.enabled()) {
-					break;
+				if(faction_score > best_score) {
+					best_score = faction_score;
+					faction_ = faction_index;
 				}
-				++search;
+				++faction;
+				faction_index++;
+			}
+			if (faction_) {
+				llm_.update_leader_list(faction_);
+				llm_.update_gender_list(llm_.get_leader());
+				combo_faction_.enable(false);
 			}
 		} else {
 			combo_faction_.enable(false);
@@ -311,6 +332,7 @@ connect::side::side(connect& parent, const config& cfg, int index) :
 connect::side::side(const side& a) :
 	parent_(a.parent_), cfg_(a.cfg_),
 	index_(a.index_), id_(a.id_),  save_id_(a.save_id_),
+	current_player_(a.current_player_),
 	controller_(a.controller_),
 	faction_(a.faction_), team_(a.team_), colour_(a.colour_),
 	gold_(a.gold_), income_(a.income_), leader_(a.leader_), /* taken_(a.taken_), */
@@ -326,6 +348,7 @@ connect::side::side(const side& a) :
 	allow_player_(a.allow_player_), enabled_(a.enabled_),
 	changed_(a.changed_), llm_(a.llm_)
 {
+	llm_.set_colour(colour_);
 	llm_.set_leader_combo((enabled_ && leader_.empty()) ? &combo_leader_ : NULL);
 	llm_.set_gender_combo((enabled_ && leader_.empty()) ? &combo_gender_ : NULL);
 	// FIXME: this is an ugly hack to force updating the gender list when the side
@@ -336,8 +359,8 @@ connect::side::side(const side& a) :
 void connect::side::add_widgets_to_scrollpane(gui::scrollpane& pane, int pos)
 {
 	pane.add_widget(&player_number_,     0, 5 + pos);
-	pane.add_widget(&combo_controller_, 20, 5 + pos);
-	pane.add_widget(&orig_controller_,  20 + (combo_controller_.width() - orig_controller_.width()) / 2,
+	pane.add_widget(combo_controller_.get(), 20, 5 + pos);
+	pane.add_widget(&orig_controller_,  20 + (combo_controller_->width() - orig_controller_.width()) / 2,
 									    35 + pos + (combo_leader_.height() - orig_controller_.height()) / 2);
 	pane.add_widget(&combo_ai_algorithm_, 20, 35 + pos);
 	pane.add_widget(&combo_faction_, 135, 5 + pos);
@@ -351,43 +374,57 @@ void connect::side::add_widgets_to_scrollpane(gui::scrollpane& pane, int pos)
 	pane.add_widget(&label_income_,  475 + slider_gold_.width(), 35 + pos);
 }
 
+bool connect::side::is_owned_by(const std::string& name) const
+{
+	return name == id_;
+}
+
 void connect::side::process_event()
 {
-	if(combo_controller_.changed() && combo_controller_.selected() >= 0) {
-		if (combo_controller_.selected() == CNTR_LAST) {
+	int drop_target;
+	if ( ( drop_target = combo_controller_->get_drop_target() )> -1)
+	{
+		const std::string target_id = parent_->sides_[drop_target].get_id();
+		const mp::controller target_controller = parent_->sides_[drop_target].get_controller();
+		const std::string target_ai = parent_->sides_[drop_target].ai_algorithm_;
+
+		parent_->sides_[drop_target].ai_algorithm_ = ai_algorithm_;
+		if (id_.empty())
+		{
+			parent_->sides_[drop_target].set_controller(controller_);
+		} else {
+			parent_->sides_[drop_target].set_id(id_);
+		}
+
+		ai_algorithm_ = target_ai;
+		if (target_id.empty())
+		{
+			set_controller(target_controller);
+		} else {
+			set_id(target_id);
+		}
+		changed_ = true;
+		parent_->sides_[drop_target].changed_ = true;
+		init_ai_algorithm_combo();
+		update_ui();
+		parent_->sides_[drop_target].init_ai_algorithm_combo();
+		parent_->sides_[drop_target].update_ui();
+	}
+	else if(combo_controller_->changed() && combo_controller_->selected() >= 0) {
+		const int cntr_last = (save_id_.empty() ? CNTR_LAST-1 : CNTR_LAST) - (parent_->local_only_ ? 1 : 0);
+		if (combo_controller_->selected() == cntr_last) {
 			update_controller_ui();
-		} else if (combo_controller_.selected() < CNTR_LAST) {
-			// If the current side corresponds to an existing user,
-			// we must kick it!
-
-			// Update controller first, or else kick will reset it.
-			controller_ = mp::controller(combo_controller_.selected());
-
-			// Don't kick an empty player or the game creator
-			if(!id_.empty()) {
-				if (id_ != preferences::login()) {
-					parent_->kick_player(id_);
-				}
-				id_ = "";
-			}
+		} else if (combo_controller_->selected() < cntr_last) {
+			// Correct entry number if CNTR_NETWORK is not allowed for combo_controller_
+			controller_ = mp::controller(combo_controller_->selected() + (parent_->local_only_ ? 1 : 0));
+			id_ = "";
 			changed_ = true;
 		} else {
-			size_t user = combo_controller_.selected() - CNTR_LAST - 1;
+			// give user second side
+			size_t user = combo_controller_->selected() - cntr_last - 1;
 
-			// If the selected user already was attributed to
-			// another side, find its side, and switch users.
-			//! todo Let one player get several sides like it is
-			//! already possible once the game started.
 			const std::string new_id = parent_->users_[user].name;
 			if (new_id != id_) {
-				int old_side = parent_->find_player_side(new_id);
-				if (old_side != -1) {
-					if (id_.empty()) {
-						parent_->sides_[old_side].set_controller(parent_->default_controller_);
-					} else {
-						parent_->sides_[old_side].set_id(id_);
-					}
-				}
 				id_ = new_id;
 				controller_ = parent_->users_[user].controller;
 				changed_ = true;
@@ -396,9 +433,19 @@ void connect::side::process_event()
 		update_ai_algorithm_combo();
 	}
 
+	if (combo_controller_->hidden())
+		combo_controller_->hide(false);
 	if(!enabled_)
 		return;
 
+	if (combo_colour_.changed() && combo_colour_.selected() >= 0) {
+		colour_ = combo_colour_.selected();
+		llm_.set_colour(colour_);
+		update_faction_combo();
+		llm_.set_leader_combo(&combo_leader_);
+		llm_.set_gender_combo(&combo_gender_);
+		changed_ = true;
+	}
 	if (combo_faction_.changed() && combo_faction_.selected() >= 0) {
 		faction_ = combo_faction_.selected();
 		llm_.update_leader_list(faction_);
@@ -421,10 +468,6 @@ void connect::side::process_event()
 		team_ = combo_team_.selected();
 		changed_ = true;
 	}
-	if (combo_colour_.changed() && combo_colour_.selected() >= 0) {
-		colour_ = combo_colour_.selected();
-		changed_ = true;
-	}
 	if (slider_gold_.value() != gold_) {
 		gold_ = slider_gold_.value();
 		label_gold_.set_text(lexical_cast_default<std::string>(gold_, "0"));
@@ -453,9 +496,17 @@ bool connect::side::changed()
 	return res;
 }
 
-bool connect::side::available() const
+bool connect::side::available(const std::string& name) const
 {
-	return allow_player_ && controller_ == CNTR_NETWORK && id_.empty();
+	if (name.empty())
+	{
+		return allow_player_
+			&& ((controller_ == CNTR_NETWORK && id_.empty())
+					|| controller_ == CNTR_RESERVED);
+	}
+	return allow_player_
+		&& ((controller_ == CNTR_NETWORK && id_.empty())
+			|| (controller_ == CNTR_RESERVED && current_player_ == name));
 }
 
 bool connect::side::allow_player() const
@@ -466,14 +517,16 @@ bool connect::side::allow_player() const
 void connect::side::update_controller_ui()
 {
 	if (id_.empty()) {
-		combo_controller_.set_selected(controller_);
+		combo_controller_->set_selected(controller_ - (parent_->local_only_ ? 1 : 0));
 	} else {
 		connected_user_list::iterator player = parent_->find_player(id_);
 
 		if (player != parent_->users_.end()) {
-			combo_controller_.set_selected(CNTR_LAST + 1 + (player - parent_->users_.begin()));
+			const int no_reserve = save_id_.empty()?-1:0;
+			combo_controller_->set_selected(CNTR_LAST + no_reserve + 1 + (player - parent_->users_.begin()) - (parent_->local_only_ ? 1 : 0));
 		} else {
-			combo_controller_.set_selected(CNTR_NETWORK);
+			assert(parent_->local_only_ != true);
+			combo_controller_->set_selected(CNTR_NETWORK);
 		}
 	}
 
@@ -517,6 +570,27 @@ void connect::side::init_ai_algorithm_combo()
 	combo_ai_algorithm_.set_selected(sel);
 }
 
+void connect::side::update_faction_combo()
+{
+	std::vector<std::string> factions;
+	for(std::vector<config*>::const_iterator faction = parent_->era_sides_.begin();
+		   faction != parent_->era_sides_.end(); ++faction) {
+		const std::string& name = (**faction)["name"];
+		const std::string& icon = (**faction)["image"];
+		if (!icon.empty()) {
+			std::string rgb = (**faction)["flag_rgb"];
+			if (rgb.empty())
+				rgb = "magenta";
+
+			factions.push_back(IMAGE_PREFIX + icon + "~RC(" + rgb + ">" + lexical_cast<std::string>(colour_+1) + ")" + COLUMN_SEPARATOR + name);
+		} else {
+			factions.push_back(name);
+		}
+	}
+	combo_faction_.set_items(factions);
+	combo_faction_.set_selected(faction_);
+}
+
 void connect::side::update_ui()
 {
 	update_controller_ui();
@@ -543,22 +617,23 @@ void connect::side::update_ui()
 
 config connect::side::get_config() const
 {
-	config res(cfg_);
+	config res;
 
-	// If the user is allowed to change type, faction, leader etc, 
+	// If the user is allowed to change type, faction, leader etc,
 	// then import their new values in the config.
 	if(enabled_ && !parent_->era_sides_.empty()) {
 		// Merge the faction data to res
 		res.append(*(parent_->era_sides_[faction_]));
 	}
+	res.append(cfg_);
 	if (cfg_.get_attribute("side").empty()
 			|| cfg_["side"] != lexical_cast<std::string>(index_ + 1))
 	{
 		res["side"] = lexical_cast<std::string>(index_ + 1);
 	}
 	res["controller"] = controller_names[controller_];
-	res["description"] = id_;
-	res["current_player"] = id_;
+	res["current_player"] = id_.empty() ? current_player_ : id_;
+	res["id"] = id_;
 
 	if (id_.empty()) {
 		char const *description;
@@ -568,8 +643,10 @@ config connect::side::get_config() const
 			break;
 		case CNTR_LOCAL:
 			if(enabled_ && cfg_.get_attribute("save_id").empty()) {
-				res["save_id"] = "local" + res["side"].str();
+				res["save_id"] = preferences::login() + res["side"].str();
 			}
+			res["id"] = preferences::login() + res["side"].str();
+			res["current_player"] = preferences::login();
 			description = N_("Anonymous local player");
 			break;
 		case CNTR_COMPUTER:
@@ -577,19 +654,11 @@ config connect::side::get_config() const
 				res["save_id"] = "ai" + res["side"].str();
 			}
 			{
+				res["id"] = res["save_id"];
 				config *ai = res.child("ai");
 				if (!ai) ai = &res.add_child("ai");
-				#ifdef HAVE_PYTHON
-				if (ai_algorithm_.substr(ai_algorithm_.length() - 3) == ".py") {
-					(*ai)["ai_algorithm"] = "python_ai";
-					(*ai)["python_script"] = ai_algorithm_;
-				}
-				else
-				#endif
-				{
-					if (ai_algorithm_ != "default")
-						(*ai)["ai_algorithm"] = ai_algorithm_;
-				}
+				if (ai_algorithm_ != "default")
+				     (*ai)["ai_algorithm"] = ai_algorithm_;
 			}
 			description = N_("Computer player");
 			break;
@@ -597,17 +666,26 @@ config connect::side::get_config() const
 			description = N_("(Empty slot)");
 			res["no_leader"] = "yes";
 			break;
+		case CNTR_RESERVED:
+			{
+				utils::string_map symbols;
+				symbols["playername"] = current_player_;
+				description = vgettext("(Reserved for $playername)",symbols).c_str();
+			}
+			break;
+		case CNTR_LAST:
 		default:
 			assert(false);
 			break;
 		}
-		res["user_description"] = t_string(description, "wesnoth");
+		res["name"] = t_string(description, "wesnoth");
 	} else {
+		res["id"] = id_ + res["side"];
 		if(enabled_ && cfg_.get_attribute("save_id").empty()) {
-			res["save_id"] = id_;
+			res["save_id"] = id_ + res["side"].str();
 		}
 
-		res["user_description"] = id_;
+		res["name"] = id_;
 	}
 
 	if(enabled_) {
@@ -674,7 +752,7 @@ config connect::side::get_config() const
 		config trimmed = cfg_;
 		trimmed["side"] = "";
 		trimmed["controller"] = "";
-		trimmed["description"] = "";
+		trimmed["id"] = "";
 		trimmed["team_name"] = "";
 		trimmed["user_team_name"] = "";
 		trimmed["colour"] = "";
@@ -693,7 +771,7 @@ config connect::side::get_config() const
 */
 		if(controller_ != CNTR_COMPUTER) {
 			// Only override names for computer controlled players
-			trimmed["user_description"] = "";
+			trimmed["name"] = "";
 		}
 		trimmed.prune();
 		res.merge_with(trimmed);
@@ -718,8 +796,12 @@ void connect::side::update_user_list()
 {
 	bool name_present = false;
 
-	std::vector<std::string> list = parent_->player_types_;
-	list.push_back("----");
+	typedef std::vector<std::string> name_list;
+
+	name_list list = parent_->player_types_;
+	if (!save_id_.empty())
+		list.push_back(_("Reserved"));
+	list.push_back(_("--give--"));
 
 	connected_user_list::const_iterator itor;
 	for (itor = parent_->users_.begin(); itor != parent_->users_.end();
@@ -733,7 +815,7 @@ void connect::side::update_user_list()
 		id_ = "";
 	}
 
-	combo_controller_.set_items(list);
+	combo_controller_->set_items(list);
 	update_controller_ui();
 }
 
@@ -819,7 +901,7 @@ void connect::side::resolve_random()
 		if(faction_excepts.size() == 1 && faction_excepts.front() == "") {
 			faction_excepts.clear();
 		}
-		
+
 		// Builds the list of sides eligible for choice (nonrandom factions)
 		std::vector<int> nonrandom_sides;
 		for(config::child_iterator itor = parent_->era_sides_.begin();
@@ -829,12 +911,12 @@ void connect::side::resolve_random()
 				if (
 					!faction_choices.empty() &&
 					std::find(faction_choices.begin(),faction_choices.end(),faction_id) == faction_choices.end()
-				) 
+				)
 					continue;
 				if (
 					!faction_excepts.empty() &&
 					std::find(faction_excepts.begin(),faction_excepts.end(),faction_id) != faction_excepts.end()
-				) 
+				)
 					continue;
 				nonrandom_sides.push_back(itor - parent_->era_sides_.begin());
 			}
@@ -872,9 +954,9 @@ void connect::side::resolve_random()
 	}
 	// Resolve random genders "very much" like standard unit code
 	if (llm_.get_gender() == "random" || solved_random_leader) {
-		game_data::unit_type_map::const_iterator ut = parent_->game_data_.unit_types.find(leader_.empty() ? llm_.get_leader() : leader_);
+		unit_type_data::unit_type_map::const_iterator ut = unit_type_data::types().find_unit_type(leader_.empty() ? llm_.get_leader() : leader_);
 
-		if (ut != parent_->game_data_.unit_types.end()) {
+		if (ut != unit_type_data::types().end()) {
 			const std::vector<unit_race::GENDER> glist = ut->second.genders();
 			if (!glist.empty()) {
 				const int gchoice = rand() % glist.size();
@@ -899,11 +981,11 @@ void connect::side::resolve_random()
 	}
 }
 
-connect::connect(game_display& disp, const config& game_config, const game_data& data,
+connect::connect(game_display& disp, const config& game_config,
 		chat& c, config& gamelist, const create::parameters& params,
-		mp::controller default_controller) :
+		mp::controller default_controller, bool local_players_only) :
 	mp::ui(disp, _("Game Lobby: ") + params.name, game_config, c, gamelist),
-	game_data_(data),
+	local_only_(local_players_only),
 	level_(),
 	state_(),
 	params_(params),
@@ -930,7 +1012,8 @@ connect::connect(game_display& disp, const config& game_config, const game_data&
 	income_title_label_(video(), _("Income"), font::SIZE_SMALL, font::LOBBY_COLOUR),
 
 	launch_(video(), _("I'm Ready")),
-	cancel_(video(), _("Cancel"))
+	cancel_(video(), _("Cancel")),
+	combo_control_group_(new gui::drop_group_manager())
 {
 	load_game();
 
@@ -949,7 +1032,7 @@ connect::connect(game_display& disp, const config& game_config, const game_data&
 		create_game["password"] = params.password;
 	}
 /*
-	// The number of human-controlled sides is important to know 
+	// The number of human-controlled sides is important to know
 	// to let the server decide how many players can join this game
 	int human_sides = 0;
 	config::child_list cfg_sides = current_config()->get_children("side");
@@ -970,14 +1053,15 @@ connect::connect(game_display& disp, const config& game_config, const game_data&
 		if (s->allow_player()) {
 			if (side_choice == -1)
 				side_choice = s - sides_.begin();
-			if(s->get_save_id() == preferences::login()) {
-				side_choice = s - sides_.begin();
-				break;
+			if(s->get_current_player() == preferences::login()) {
+				sides_[s - sides_.begin()].set_id(preferences::login());
+				side_choice = gamemap::MAX_PLAYERS;
 			}
 		}
 	}
 
-	if (side_choice != -1)
+	if (side_choice != -1
+			&& side_choice != gamemap::MAX_PLAYERS)
 	{
 		sides_[side_choice].set_id(preferences::login());
 	}
@@ -1016,7 +1100,7 @@ void connect::process_event()
 			set_result(mp::ui::PLAY);
 	}
 
-	// If something has changed in the level config, 
+	// If something has changed in the level config,
 	// send it to the network:
 	if (changed) {
 		update_playerlist_state();
@@ -1044,14 +1128,8 @@ void connect::start_game()
 	update_and_send_diff(true);
 
 	// Build the gamestate object after updating the level
-	
-	level_to_gamestate(level_, state_, params_.saved_game);
-	
-	// add era events only if not save
-	const config* const era_cfg = level_.child("era");
-	if (era_cfg != NULL && level_["savegame"] != "yes") {
-		game_events::add_events(era_cfg->get_children("event"),"era_events");
-	}
+
+	level_to_gamestate(level_, state_);
 
 	network::send_data(config("start_game"), 0, true);
 }
@@ -1097,6 +1175,7 @@ void connect::process_network_data(const config& data, const network::connection
 				update_user_combos();
 			}
 			update_and_send_diff();
+			update_playerlist_state(true);
 			return;
 		}
 	}
@@ -1116,8 +1195,10 @@ void connect::process_network_data(const config& data, const network::connection
 
 		connected_user_list::iterator player = find_player(name);
 		if(player != users_.end()) {
-			//! @todo Seems like a needless limitation to only allow one side
-			//! per player.
+			/**
+			 * @todo Seems like a needless limitation to only allow one side
+			 * per player.
+			 */
 			if(find_player_side(name) != -1) {
 				config response;
 				response.values["failed"] = "yes";
@@ -1135,7 +1216,7 @@ void connect::process_network_data(const config& data, const network::connection
 
 		// Assigns this user to a side
 		if(side_taken >= 0 && side_taken < int(sides_.size())) {
-			if(!sides_[side_taken].available()) {
+			if(!sides_[side_taken].available(name)) {
 				// This side is already taken.
 				// Try to reassing the player to a different position.
 				side_list::const_iterator itor;
@@ -1170,6 +1251,10 @@ void connect::process_network_data(const config& data, const network::connection
 
 			// sides_[side_taken].set_connection(sock);
 			sides_[side_taken].import_network_user(data);
+
+			// Go thought and check if more sides are reserved
+			// For this player
+			std::for_each(sides_.begin(), sides_.end(), boost::bind(&connect::take_reserved_side, this,_1, data));
 			update_playerlist_state(false);
 			update_and_send_diff();
 
@@ -1219,6 +1304,15 @@ void connect::process_network_data(const config& data, const network::connection
 	}
 }
 
+void connect::take_reserved_side(connect::side& side, const config& data)
+{
+	if (side.available(data["name"])
+			&& side.get_current_player() == data["name"])
+	{
+		side.import_network_user(data);
+	}
+}
+
 void connect::process_network_error(network::error& error)
 {
 	// If the problem isn't related to any specific connection,
@@ -1319,13 +1413,15 @@ void connect::layout_children(const SDL_Rect& rect)
 void connect::lists_init()
 {
 	// Options
-	player_types_.push_back(_("Network Player"));
+	if(!local_only_) {
+		player_types_.push_back(_("Network Player"));
+	}
 	player_types_.push_back(_("Local Player"));
 	player_types_.push_back(_("Computer Player"));
 	player_types_.push_back(_("Empty"));
 
-	for(std::vector<config*>::const_iterator faction = era_sides_.begin(); 
-													   faction != era_sides_.end(); 
+	for(std::vector<config*>::const_iterator faction = era_sides_.begin();
+													   faction != era_sides_.end();
 													   ++faction) {
 		player_factions_.push_back((**faction)["name"]);
 	}
@@ -1352,11 +1448,11 @@ void connect::lists_init()
 				user_team_name = team_name;
 			}
 
-			std::vector<std::string>::const_iterator itor = std::find(team_names_.begin(), 
+			std::vector<std::string>::const_iterator itor = std::find(team_names_.begin(),
 															team_names_.end(), team_name);
 			if(itor == team_names_.end()) {
 				team_names_.push_back(team_name);
-				user_team_names_.push_back(user_team_name);
+				user_team_names_.push_back(user_team_name.to_serialized());
 				player_teams_.push_back(user_team_name.str());
 			}
 		}
@@ -1369,7 +1465,7 @@ void connect::lists_init()
 			if(team_name.empty())
 				team_name = side_num;
 
-			std::vector<std::string>::const_iterator itor = std::find(map_team_names.begin(), 
+			std::vector<std::string>::const_iterator itor = std::find(map_team_names.begin(),
 															map_team_names.end(), team_name);
 			if(itor == map_team_names.end()) {
 				map_team_names.push_back(team_name);
@@ -1400,7 +1496,7 @@ void connect::lists_init()
 	for(side_list::iterator s = sides_.begin(); s != sides_.end(); ++s) {
 		const int side_num = s - sides_.begin();
 		const int spos = 60 * (side_num-offset);
-		if(!s->allow_player()) {                
+		if(!s->allow_player()) {
 			offset++;
 			continue;
 		}
@@ -1409,13 +1505,15 @@ void connect::lists_init()
 	}
 }
 
-//! Called by the constructor to initialize the game from a create::parameters structure.
 void connect::load_game()
 {
 	if(params_.saved_game) {
 		bool show_replay = false;
-		const std::string game = dialogs::load_game_dialog(disp(), 
-								 game_config(), game_data_, &show_replay);
+		//bool cancel_orders = false;
+		const std::string game = dialogs::load_game_dialog(disp(),
+								 game_config(), &show_replay,
+								 NULL);
+
 		if(game.empty()) {
 			set_result(CREATE);
 			return;
@@ -1424,7 +1522,7 @@ void connect::load_game()
 		std::string error_log;
 		{
 			cursor::setter cur(cursor::WAIT);
-			::load_game(game_data_, game, state_, &error_log);
+			::load_game(game, state_, &error_log);
 		}
 		if(!error_log.empty()) {
 			gui::show_error_message(disp(),
@@ -1443,12 +1541,9 @@ void connect::load_game()
 		}
 
 		if(state_.version != game_config::version) {
-			// Do not load if too old, but if either the savegame or 
+			// Do not load if too old, but if either the savegame or
 			// the current game has the version 'test' allow loading.
-			if(state_.version < game_config::min_savegame_version &&
-					game_config::test_version.full != state_.version &&
-					game_config::test_version.full != game_config::version) {
-
+			if(!game_config::is_compatible_savegame_version(state_.version)) {
 				/* GCC-3.3 needs a temp var otherwise compilation fails */
 				gui::message_dialog dlg2(disp(), "", _("This save is from a version too old to be loaded."));
 				dlg2.show();
@@ -1473,6 +1568,7 @@ void connect::load_game()
 		level_["id"] = start_data["id"];
 		level_["name"] = start_data["name"];
 		level_["completion"] = start_data["completion"];
+		level_["next_underlying_unit_id"] = start_data["next_underlying_unit_id"];
 		// Probably not needed.
 		level_["turn"] = start_data["turn_at"];
 		level_["turn_at"] = start_data["turn_at"];
@@ -1487,15 +1583,26 @@ void connect::load_game()
 		level_["mp_countdown_action_bonus"] = start_data["mp_countdown_action_bonus"];
 		level_["experience_modifier"] = start_data["experience_modifier"];
 		level_["observer"] = start_data["observer"];
-		level_.add_child("snapshot") = start_data;
+		//Start-of-scenario save
+		if(state_.snapshot.empty()){
+			//For a start-of-scenario-save, write the data to the starting_pos and not the snapshot, since
+			//there should only be snapshots for midgame reloads
+			if(level_.child("replay_start") == NULL)
+			{
+				level_.add_child("replay_start") = start_data;
+			} else {
+				level_.child("replay_data")->merge_with(start_data);
+			}
+			level_.add_child("snapshot") = config();
+		} else {
+			level_.add_child("snapshot") = start_data;
+		}
 		level_["random_seed"] = start_data["random_seed"];
 		level_["random_calls"] = start_data["random_calls"];
 
-		// Adds the replay data, and the replay start, to the level, 
+		// Adds the replay data, and the replay start, to the level,
 		// so clients can receive it.
 		level_.add_child("replay") = state_.replay_data;
-		if(!state_.starting_pos.empty())
-			level_.add_child("replay_start") = state_.starting_pos;
 		level_.add_child("statistics") = statistics::write_stats();
 
 	} else {
@@ -1509,6 +1616,8 @@ void connect::load_game()
 		level_["mp_countdown_turn_bonus"] = lexical_cast_default<std::string>(params_.mp_countdown_turn_bonus, "35");
 		level_["mp_countdown_reservoir_time"] = lexical_cast_default<std::string>(params_.mp_countdown_reservoir_time, "330");
 		level_["mp_countdown_action_bonus"] = lexical_cast_default<std::string>(params_.mp_countdown_action_bonus, "13");
+		level_["next_underlying_unit_id"] = lexical_cast<std::string>(0);
+		n_unit::id_manager::instance().clear();
 
 		if (params_.random_start_time)
 		{
@@ -1524,7 +1633,7 @@ void connect::load_game()
 
 		level_["scenario"] = params_.name;
 		level_["experience_modifier"] = lexical_cast<std::string>(params_.xp_modifier);
-		level_["random_seed"] = lexical_cast<std::string>(state_.get_random_seed());
+		level_["random_seed"] = lexical_cast<std::string>(state_.rng().get_random_seed());
 		level_["mp_village_gold"] = lexical_cast<std::string>(params_.village_gold);
 		level_["mp_fog"] = params_.fog_game ? "yes" : "no";
 		level_["mp_shroud"] = params_.shroud_game ? "yes" : "no";
@@ -1534,7 +1643,7 @@ void connect::load_game()
 	// Add the map name to the title.
 	append_to_title(" - " + level_["name"]);
 
-	
+
 	std::string era;
 	if (params_.saved_game
 		&& level_.child("snapshot")->child("era"))
@@ -1557,7 +1666,7 @@ void connect::load_game()
 		}
 		// FIXME: @todo We should tell user about missing era but still load game
 		WRN_CF << "Missing era in MP load game " << era << "\n";
-		
+
 	}
 	if (era_cfg)
 	{
@@ -1582,11 +1691,15 @@ void connect::load_game()
 config* connect::current_config(){
 	config* cfg_level = NULL;
 
-	if (level_.child("snapshot") != NULL){
+	//It might make sense to invent a mechanism of some sort to check whether a config node contains information
+	//that you can load from(side information, specifically)
+	if (level_.child("snapshot") != NULL && level_.child("snapshot")->child("side") != NULL){
 		// Savegame
 		cfg_level = level_.child("snapshot");
-	}
-	else{
+	} else if(level_.child("side") == NULL) {
+		// Start-of-scenario save, the info has to be taken from the starting_pos
+		cfg_level = &state_.starting_pos;
+	} else {
 		// Fresh game, no snapshot available
 		cfg_level = &level_;
 	}
@@ -1637,15 +1750,15 @@ void connect::update_playerlist_state(bool silent)
 	waiting_label_.set_text(sides_available() ? _("Waiting for players to join...") : "");
 	launch_.enable(!sides_available());
 
-	// If the "gamelist_" variable has users, use it. 
+	// If the "gamelist_" variable has users, use it.
 	// Else, extracts the user list from the actual player list.
 	if (gamelist().child("user") != NULL) {
 		ui::gamelist_updated(silent);
 	} else {
 		// Updates the player list
 		std::vector<std::string> playerlist;
-		for(connected_user_list::const_iterator itor = users_.begin(); 
-													   itor != users_.end(); 
+		for(connected_user_list::const_iterator itor = users_.begin();
+													   itor != users_.end();
 													   ++itor) {
 			playerlist.push_back(itor->name);
 		}
@@ -1680,7 +1793,7 @@ int connect::find_player_side(const std::string& id) const
 
 void connect::update_user_combos()
 {
-	for (side_list::iterator itor = sides_.begin(); 
+	for (side_list::iterator itor = sides_.begin();
 									itor != sides_.end(); ++itor) {
 		itor->update_user_list();
 	}
@@ -1695,7 +1808,7 @@ void connect::kick_player(const std::string& name)
 	if(player->controller != CNTR_NETWORK)
 		return;
 
-	// If we are the server, kick the user ourselves; 
+	// If we are the server, kick the user ourselves;
 	// else, ask the server to do so.
 	if(network::is_server()) {
 		network::disconnect(player->connection);
