@@ -1,6 +1,6 @@
-/* $Id: multiplayer_create.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: multiplayer_create.cpp 33835 2009-03-18 16:02:24Z shadowmaster $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,31 +12,28 @@
    See the COPYING file for more details.
 */
 
-/** @file multiplayer_create.cpp 
- *  Create a multiplayer-game: select map, players, options etc.
+/**
+ * @file multiplayer_create.cpp
+ * Create a multiplayer-game: select map, players, options etc.
  */
 
 #include "global.hpp"
 
-#include "game_display.hpp"
-#include "game_preferences.hpp"
 #include "gettext.hpp"
-#include "image.hpp"
 #include "construct_dialog.hpp"
 #include "settings.hpp"
-#include "show_dialog.hpp"
+#include "map.hpp"
+#include "map_exception.hpp"
 #include "map_create.hpp"
+#include "gui/dialogs/message.hpp"
 #include "minimap.hpp"
 #include "multiplayer_create.hpp"
 #include "filesystem.hpp"
 #include "log.hpp"
-#include "video.hpp"
-#include "serialization/string_utils.hpp"
 #include "wml_exception.hpp"
 #include "wml_separators.hpp"
+#include "formula_string_utils.hpp"
 
-#include <cassert>
-#include <memory>
 
 #define ERR_CF LOG_STREAM(err, config)
 
@@ -246,7 +243,7 @@ create::~create()
 	preferences::set_countdown_turn_bonus(parameters_.mp_countdown_turn_bonus);
 	preferences::set_countdown_reservoir_time(parameters_.mp_countdown_reservoir_time);
 	preferences::set_countdown_action_bonus(parameters_.mp_countdown_action_bonus);
-	preferences::set_era(era_combo_.selected()); //! @todo FIXME: may be broken if new eras are added
+	preferences::set_era(era_combo_.selected()); /** @todo FIXME: may be broken if new eras are added. */
 	preferences::set_map(map_selection_);
 
 	// When using map settings, the following variables are determined by the map,
@@ -278,7 +275,7 @@ create::parameters& create::get_parameters()
 	if(mp_countdown_reservoir_time_val > 0 && mp_countdown_init_time_val > mp_countdown_reservoir_time_val)
 		mp_countdown_init_time_val = mp_countdown_reservoir_time_val;
 
-	// Updates the values in the "parameters_" member to match 
+	// Updates the values in the "parameters_" member to match
 	// the values selected by the user with the widgets:
 	parameters_.name = name_entry_.text();
 	if (size_t(era_combo_.selected()) >= era_list.size()) {
@@ -468,7 +465,10 @@ void create::process_event()
 		parameters_.scenario_data = generator_->create_scenario(std::vector<std::string>());
 		map_changed = true;
 
-		// Set the scenario to have placing of sides 
+		if (!parameters_.scenario_data["error_message"].empty())
+			gui2::show_message(disp().video(), "map generation error", parameters_.scenario_data["error_message"]);
+
+		// Set the scenario to have placing of sides
 		// based on the terrain they prefer
 		parameters_.scenario_data["modify_placing"] = "true";
 	}
@@ -482,7 +482,7 @@ void create::process_event()
 		std::auto_ptr<gamemap> map(NULL);
 		try {
 			map = std::auto_ptr<gamemap>(new gamemap(game_config(), map_data));
-		} catch(gamemap::incorrect_format_exception& e) {
+		} catch(incorrect_map_format_exception& e) {
 			ERR_CF << "map could not be loaded: " << e.msg_ << "\n";
 
 #ifndef USE_TINY_GUI
@@ -495,7 +495,7 @@ void create::process_event()
 
 		launch_game_.enable(map.get() != NULL);
 
-		// If there are less sides in the configuration than there are 
+		// If there are less sides in the configuration than there are
 		// starting positions, then generate the additional sides
 		const int map_positions = map.get() != NULL ? map->num_valid_starting_positions() : 0;
 
@@ -504,7 +504,7 @@ void create::process_event()
 			side["enemy"] = "1";
 			side["side"] = lexical_cast<std::string>(pos+1);
 			side["team_name"] = lexical_cast<std::string>(pos+1);
-			side["canrecruit"] = "1";
+			side["canrecruit"] = "yes";
 			side["controller"] = "human";
 		}
 
@@ -519,7 +519,7 @@ void create::process_event()
 		int nsides = parameters_.scenario_data.get_children("side").size();
 		const config::child_list& player_sides = parameters_.scenario_data.get_children("side");
 		for(config::child_list::const_iterator k = player_sides.begin(); k != player_sides.end(); ++k) {
-			if((**k)["allow_player"] == "no") {
+			if(!utils::string_bool((**k)["allow_player"],true)) {
 				nsides--;
 			}
 		}
@@ -555,10 +555,12 @@ void create::process_event()
 			preferences::random_start_time());
 
 		// These are per player, always show values of player 1.
-		//! @todo This might not be 100% correct, but at the moment 
-		//! it is not possible to show the fog and shroud per player.
-		// This might change in the future.
-		// NOTE when 'load game' is selected there are no sides.
+		/**
+		 * @todo This might not be 100% correct, but at the moment
+		 * it is not possible to show the fog and shroud per player.
+		 * This might change in the future.
+		 * NOTE when 'load game' is selected there are no sides.
+		 */
 		if(parameters_.scenario_data.get_children("side").size()) {
 
 			village_gold_slider_.set_value(map_settings ?
@@ -645,7 +647,7 @@ void create::hide_children(bool hide)
 			SDL_Color back_color = {0,0,0,255};
 			draw_centered_on_background(mini, minimap_rect_, back_color, video().getSurface());
 #endif
-		} catch(gamemap::incorrect_format_exception& e) {
+		} catch(incorrect_map_format_exception& e) {
 			ERR_CF << "map could not be loaded: " << e.msg_ << "\n";
 		} catch(twml_exception& e) {
 			ERR_CF <<  "map could not be loaded: " << e.dev_message << '\n';
@@ -673,7 +675,7 @@ void create::layout_children(const SDL_Rect& rect)
 	name_entry_label_.set_location(xpos, ypos);
 	name_entry_.set_location(xpos + name_entry_label_.width() + border_size, ypos);
 	name_entry_.set_width(ca.w - name_entry_label_.width() - border_size);
-	ypos += maximum<int>(name_entry_.height(), name_entry_label_.height()) + border_size;
+	ypos += std::max<int>(name_entry_.height(), name_entry_label_.height()) + border_size;
 
 	// Save ypos here (column top)
 	int ypos_columntop = ypos;
@@ -821,7 +823,7 @@ void create::layout_children(const SDL_Rect& rect)
 	name_entry_label_.set_location(xpos, ypos);
 	name_entry_.set_location(xpos + name_entry_label_.width() + border_size, ypos);
 	name_entry_.set_width(ca.w - name_entry_label_.width() - border_size);
-	ypos += maximum<int>(name_entry_.height(), name_entry_label_.height()) + border_size;
+	ypos += std::max<int>(name_entry_.height(), name_entry_label_.height()) + border_size;
 
 	// Save ypos here (column top)
 	int ypos_columntop = ypos;
