@@ -1,6 +1,6 @@
-/* $Id: multiplayer_lobby.cpp 25007 2008-03-23 09:21:06Z martinxyz $ */
+/* $Id: multiplayer_lobby.cpp 33741 2009-03-17 03:35:09Z soliton $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -12,27 +12,22 @@
    See the COPYING file for more details.
 */
 
-/** @file multiplayer_lobby.cpp 
+/** @file multiplayer_lobby.cpp
  * A section on the server where players can chat, create and join games.
  */
 
 #include "global.hpp"
 
-#include "config.hpp"
-#include "filesystem.hpp"
-#include "game_display.hpp"
+#include "map_exception.hpp"
 #include "marked-up_text.hpp"
 #include "minimap.hpp"
 #include "multiplayer_lobby.hpp"
-#include "replay.hpp"
-#include "wml_separators.hpp"
-#include "game_config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
 #include "playmp_controller.hpp"
-#include "show_dialog.hpp"
 #include "sound.hpp"
 #include "wml_exception.hpp"
+#include "formula_string_utils.hpp"
 
 #define ERR_CF LOG_STREAM(err, config)
 
@@ -41,17 +36,28 @@ std::vector<std::string> empty_string_vector;
 }
 
 namespace mp {
-	gamebrowser::gamebrowser(CVideo& video, const config* map_hashes) :
+gamebrowser::gamebrowser(CVideo& video, const config* map_hashes) :
 	menu(video, empty_string_vector, false, -1, -1, NULL, &menu::bluebg_style),
 	gold_icon_locator_("themes/gold.png"),
 	xp_icon_locator_("themes/units.png"),
 	vision_icon_locator_("misc/invisible.png"),
 	time_limit_icon_locator_("themes/sand-clock.png"),
 	observer_icon_locator_(game_config::observer_image),
-	no_observer_icon_locator_("misc/no_observer.png"), map_hashes_(map_hashes),
-	item_height_(100), margin_(5), minimap_size_(item_height_ - 2*margin_),  h_padding_(5),
-	header_height_(20), selected_(0), visible_range_(std::pair<size_t,size_t>(0,0)),
-	double_clicked_(false), ignore_next_doubleclick_(false), last_was_doubleclick_(false)
+	no_observer_icon_locator_("misc/no_observer.png"),
+	map_hashes_(map_hashes),
+	item_height_(100),
+	margin_(5),
+	minimap_size_(item_height_ - 2*margin_),
+	h_padding_(5),
+	header_height_(20),
+	selected_(0),
+	visible_range_(std::pair<size_t,size_t>(0,0)),
+	games_(),
+	redraw_items_(),
+	widths_(),
+	double_clicked_(false),
+	ignore_next_doubleclick_(false),
+	last_was_doubleclick_(false)
 {
 	set_numeric_keypress_selection(false);
 }
@@ -68,7 +74,7 @@ void gamebrowser::scroll(unsigned int pos)
 {
 	if(pos < games_.size()) {
 		visible_range_.first = pos;
-		visible_range_.second = minimum<size_t>(pos + inner_location().h / row_height(), games_.size());
+		visible_range_.second = std::min<size_t>(pos + inner_location().h / row_height(), games_.size());
 		set_dirty();
 	}
 }
@@ -114,7 +120,7 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 	const game_item& game = games_[index];
 	int xpos = item_rect.x + margin_;
 	int ypos = item_rect.y + margin_;
-
+	std::string no_era_string = "";
 	// Draw minimaps
 	if (game.mini_map != NULL) {
 		int minimap_x = xpos + (minimap_size_ - game.mini_map->w)/2;
@@ -138,6 +144,10 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 			font_color = font::BAD_COLOUR;
 		}
 	}
+	if(!game.have_era) {
+		font_color = font::DISABLED_COLOUR;
+		no_era_string = _(" (Unknown Era)");
+	}
 
 	const surface status_text(font::get_rendered_text(game.status,
 	    font::SIZE_NORMAL, font_color));
@@ -145,7 +155,7 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 
 	// First line: draw game name
 	const surface name_surf(font::get_rendered_text(
-	    font::make_text_ellipsis(game.name, font::SIZE_PLUS,
+	    font::make_text_ellipsis(game.name + no_era_string, font::SIZE_PLUS,
 	        (item_rect.x + item_rect.w) - xpos - margin_ - status_text_width - h_padding_),
 	    font::SIZE_PLUS, font_color));
 	video().blit_surface(xpos, ypos, name_surf);
@@ -178,7 +188,7 @@ void gamebrowser::draw_row(const size_t index, const SDL_Rect& item_rect, ROW_TY
 	if(observer_icon) {
 		video().blit_surface(xpos, ypos - observer_icon->h, observer_icon);
 
-		// Set ypos to the middle of the line, so that 
+		// Set ypos to the middle of the line, so that
 		// all text and icons can be aligned symmetrical to it
 		ypos -= observer_icon->h/2;
 		xpos += observer_icon->w + 2 * h_padding_;
@@ -287,7 +297,7 @@ void gamebrowser::handle_event(const SDL_Event& event)
 				case SDLK_PAGEUP:
 				{
 					const long items_on_screen = visible_range_.second - visible_range_.first;
-					selected_ = static_cast<size_t>(maximum<long>(static_cast<long>(selected_) - items_on_screen, 0));
+					selected_ = static_cast<size_t>(std::max<long>(static_cast<long>(selected_) - items_on_screen, 0));
 					adjust_position(selected_);
 					set_dirty();
 				}
@@ -295,7 +305,7 @@ void gamebrowser::handle_event(const SDL_Event& event)
 				case SDLK_PAGEDOWN:
 				{
 					const size_t items_on_screen = visible_range_.second - visible_range_.first;
-					selected_ = minimum<size_t>(selected_ + items_on_screen, games_.size() - 1);
+					selected_ = std::min<size_t>(selected_ + items_on_screen, games_.size() - 1);
 					adjust_position(selected_);
 					set_dirty();
 				}
@@ -314,7 +324,7 @@ void gamebrowser::handle_event(const SDL_Event& event)
 					break;
 			}
 		}
-	} else if(event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT || event.type == DOUBLE_CLICK_EVENT) {
+	} else if((event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT) || event.type == DOUBLE_CLICK_EVENT) {
 		int x = 0;
 		int y = 0;
 		if(event.type == SDL_MOUSEBUTTONDOWN) {
@@ -361,6 +371,14 @@ void gamebrowser::handle_event(const SDL_Event& event)
 }
 
 struct minimap_cache_item {
+
+	minimap_cache_item() :
+		map_data(),
+		mini_map(),
+		map_info_size()
+	{
+	}
+
 	std::string map_data;
 	surface mini_map;
 	std::string map_info_size;
@@ -393,6 +411,7 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 		games_.push_back(game_item());
 		games_.back().password_required = (**game)["password"] == "yes";
 		games_.back().reloaded = (**game)["savegame"] == "yes";
+		games_.back().have_era = true;
 		if((**game)["mp_era"] != "") {
 			const config* const era_cfg = game_config.find_child("era", "id", (**game)["mp_era"]);
 			utils::string_map symbols;
@@ -400,6 +419,11 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 			if (era_cfg != NULL) {
 				games_.back().map_info = era_cfg->get_attribute("name");
 			} else {
+				if((**game)["require_era"] == "no") {
+					games_.back().have_era = true;
+				} else {
+					games_.back().have_era = false;
+				}
 				games_.back().map_info = vgettext("Unknown era: $era_id", symbols);
 				verified = false;
 			}
@@ -425,13 +449,13 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 				}
 				if (!found) {
 					// Parsing the map and generating the minimap are both cpu expensive
-					gamemap map(game_config, games_.back().map_data); 
+					gamemap map(game_config, games_.back().map_data);
 					games_.back().mini_map = image::getMinimap(minimap_size_, minimap_size_, map, 0);
 					games_.back().map_info_size = lexical_cast_default<std::string, int>(map.w(), "??")
 						+ std::string("x") + lexical_cast_default<std::string, int>(map.h(), "??");
 				}
 				games_.back().map_info += " - " + games_.back().map_info_size;
-			} catch(gamemap::incorrect_format_exception &e) {
+			} catch(incorrect_map_format_exception &e) {
 				ERR_CF << "illegal map: " << e.msg_ << "\n";
 				verified = false;
 			} catch(twml_exception& e) {
@@ -554,7 +578,7 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 		}
 	}
 	if(selected_ >= games_.size())
-		selected_ = maximum<long>(static_cast<long>(games_.size()) - 1, 0);
+		selected_ = std::max<long>(static_cast<long>(games_.size()) - 1, 0);
 
 	if (scrolled_to_max) {
 		set_position(get_max_position());
@@ -569,6 +593,19 @@ void gamebrowser::set_game_items(const config& cfg, const config& game_config)
 	set_dirty();
 }
 
+void gamebrowser::select_game(const std::string id) {
+	if (id.empty()) return;
+
+	for (unsigned int i=0; i < games_.size(); i++) {
+		if (games_[i].id == id) {
+			selected_ = i;
+			break;
+		}
+	}
+	adjust_position(selected_);
+	set_dirty();
+}
+
 lobby::lobby_sorter::lobby_sorter(const config& cfg) : cfg_(cfg)
 {
 	set_alpha_sort(1);
@@ -638,6 +675,9 @@ bool lobby::lobby_sorter::less(int column, const gui::menu::item& row1, const gu
 lobby::lobby(game_display& disp, const config& cfg, chat& c, config& gamelist) :
 	mp::ui(disp, _("Game Lobby"), cfg, c, gamelist),
 
+	game_vacant_slots_(),
+	game_observers_(),
+
 	observe_game_(disp.video(), _("Observe Game")),
 	join_game_(disp.video(), _("Join Game")),
 	create_game_(disp.video(), _("Create Game")),
@@ -647,7 +687,8 @@ lobby::lobby(game_display& disp, const config& cfg, chat& c, config& gamelist) :
 #endif
 	quit_game_(disp.video(), _("Quit")),
 	last_selected_game_(-1), sorter_(gamelist),
-	games_menu_(disp.video(),cfg.child("multiplayer_hashes"))
+	games_menu_(disp.video(),cfg.child("multiplayer_hashes")),
+	minimaps_()
 {
 	skip_replay_.set_check(preferences::skip_mp_replay());
 	skip_replay_.set_help_string(_("Skip quickly to the active turn when observing"));
@@ -747,6 +788,11 @@ void lobby::process_event()
 		last_selected_game_ = selected_game;
 	}
 
+	if(selected_user_changed()) {
+		set_selected_user_changed(false);
+		games_menu_.select_game(get_selected_user_game());
+	}
+
 	if(join || observe) {
 		const int selected = games_menu_.selection();
 		if(!games_menu_.empty() && selected >= 0) {
