@@ -1,6 +1,6 @@
-/* $Id: multiplayer_ui.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: multiplayer_ui.cpp 33296 2009-03-03 01:17:48Z soliton $ */
 /*
-   Copyright (C) 2005 - 2008
+   Copyright (C) 2005 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -14,19 +14,14 @@
 
 #include "global.hpp"
 
-#include "config.hpp"
 #include "construct_dialog.hpp"
-#include "game_display.hpp"
-#include "font.hpp"
-#include "marked-up_text.hpp"
 #include "gettext.hpp"
-#include "game_config.hpp"
-#include "image.hpp"
+#include "gui/dialogs/mp_cmd_wrapper.hpp"
 #include "log.hpp"
+#include "marked-up_text.hpp"
+#include "multiplayer.hpp"
 #include "multiplayer_ui.hpp"
-#include "network.hpp"
 #include "sound.hpp"
-#include "video.hpp"
 #include "replay.hpp"
 #include "wml_separators.hpp"
 
@@ -47,7 +42,7 @@ namespace {
 										   item_size_(empty_rect)
 		{}
 		virtual void init();
-		virtual SDL_Rect item_size(const std::string& /*item*/) { return item_size_; }
+		virtual SDL_Rect item_size(const std::string& /*item*/) const { return item_size_; }
 		void set_width(const int width) { item_size_.w = width; }
 	private:
 		SDL_Rect item_size_;
@@ -62,7 +57,7 @@ namespace {
 	}
 
 	user_menu_style umenu_style;
-	
+
 } // anon namespace
 
 namespace mp {
@@ -79,7 +74,7 @@ void check_response(network::connection res, const config& data)
 	}
 }
 
-void level_to_gamestate(config& level, game_state& state, bool saved_game)
+void level_to_gamestate(config& level, game_state& state)
 {
 	//any replay data is only temporary and should be removed from
 	//the level data in case we want to save the game later
@@ -100,7 +95,7 @@ void level_to_gamestate(config& level, game_state& state, bool saved_game)
 	const std::string seed = level["random_seed"];
 	if(! seed.empty()) {
 		const unsigned calls = lexical_cast_default<unsigned>(level["random_calls"]);
-		state.seed_random(lexical_cast<int>(seed), calls);
+		state.rng().seed_random(lexical_cast<int>(seed), calls);
 	} else {
 		ERR_NG << "No random seed found, random "
 			"events will probably be out of sync.\n";
@@ -117,6 +112,7 @@ void level_to_gamestate(config& level, game_state& state, bool saved_game)
 
 	level["campaign_type"] = "multiplayer";
 	state.campaign_type = "multiplayer";
+	state.completion = level["completion"];
 	state.version = level["version"];
 
 	const config* const vars = level.child("variables");
@@ -125,6 +121,14 @@ void level_to_gamestate(config& level, game_state& state, bool saved_game)
 	}
 	state.set_menu_items(level.get_children("menu_item"));
 
+	//Check whether it is a save-game by looking for snapshot data
+	const bool saved_game = (level.child("snapshot") && level.child("snapshot")->child("side"));
+
+	//It might be a MP campaign start-of-scenario save
+	//In this case, it's not entirely a new game, but not a save, either
+	//Check whether it is no savegame and the starting_pos contains [player] information
+	const bool start_of_scenario = (!saved_game && (state.starting_pos.child("player") != NULL));
+
 	//If we start a fresh game, there won't be any snapshot information. If however this
 	//is a savegame, we got a valid snapshot here.
 	if (saved_game){
@@ -133,17 +137,30 @@ void level_to_gamestate(config& level, game_state& state, bool saved_game)
 			state.set_variables(*state.snapshot.child("variables"));
 		}
 		state.set_menu_items(state.snapshot.get_children("menu_item"));
+	}
+
+	//If it is a start-of-scenario save, we need to load the player information from
+	//the [player] tags
+	if(start_of_scenario){
+		state.load_recall_list(state.starting_pos.get_children("player"));
+	}
+
+	//In any type of reload(normal save or start-of-scenario) the players could have
+	//changed and need to be replaced
+	if(saved_game || start_of_scenario){
+		const config::child_list& saved_sides = (saved_game ?
+			state.snapshot.get_children("side") :
+			state.starting_pos.get_children("side"));
 
-		//We also need to take into account, that the reload could take place with different players.
-		//If so, they are substituted now.
-		const config::child_list& snapshot_sides = state.snapshot.get_children("side");
 		const config::child_list& level_sides = level.get_children("side");
-		for(config::child_list::const_iterator side = snapshot_sides.begin(); side != snapshot_sides.end(); ++side) {
+		for(config::child_list::const_iterator side = saved_sides.begin(); side != saved_sides.end(); ++side) {
 			for(config::child_list::const_iterator lside = level_sides.begin(); lside != level_sides.end(); ++lside) {
-				if ( ((**side)["side"] == (**lside)["side"])
-					&& ((**side)["current_player"] != (**lside)["current_player"]) ){
+				if ( ((**side)["side"] == (**lside)["side"]) && (
+					 ((**side)["current_player"] != (**lside)["current_player"]) ||
+					 ((**side)["controller"] != (**lside)["controller"]) )){
+
 					(**side)["current_player"] = (**lside)["current_player"];
-					(**side)["description"] = (**lside)["description"];
+					(**side)["id"] = (**lside)["id"];
 					(**side)["save_id"] = (**lside)["save_id"];
 					(**side)["controller"] = (**lside)["controller"];
 					break;
@@ -172,7 +189,9 @@ std::string get_colour_string(int id)
 	}
 }
 
-chat::chat()
+chat::chat() :
+	message_history_(),
+	last_update_()
 {
 }
 
@@ -191,43 +210,54 @@ void chat::add_message(const time_t& time, const std::string& user,
 
 void chat::init_textbox(gui::textbox& textbox)
 {
-	std::string s;
-
 	for(msg_hist::const_iterator itor = message_history_.begin();
 			itor != message_history_.end(); ++itor) {
-		s.append(format_message(*itor));
+		textbox.append_text(format_message(*itor), true, color_message(*itor));
 	}
 
-	textbox.set_text(s);
 	last_update_ = message_history_.size();
-	textbox.scroll_to_bottom();
 }
 
 void chat::update_textbox(gui::textbox& textbox)
 {
-	std::string s;
-
 	for(msg_hist::const_iterator itor = message_history_.begin() + last_update_;
 			itor != message_history_.end(); ++itor) {
-		s.append(format_message(*itor));
+		textbox.append_text(format_message(*itor), true, color_message(*itor));
 	}
 
-	textbox.append_text(s,true);
-
 	last_update_ = message_history_.size();
 }
 
 std::string chat::format_message(const msg& message)
 {
+	std::string msg_text = message.message;
+	if(message.user == "server") {
+		SDL_Color c;
+		int unused;
+		std::string::const_iterator after_markup =
+			font::parse_markup(message.message.begin(), message.message.end(), &unused, &c, &unused);
+
+		msg_text = std::string(after_markup,message.message.end());
+	}
 	if(message.message.substr(0,3) == "/me") {
 		return preferences::get_chat_timestamp(message.time) + "<" + message.user
-				+ message.message.substr(3) + ">\n";
+				+ msg_text.substr(3) + ">\n";
 	} else {
 		return preferences::get_chat_timestamp(message.time) + "<" + message.user
-				+ "> " + message.message + "\n";
+				+ "> " + msg_text + "\n";
 	}
 }
 
+SDL_Color chat::color_message(const msg& message) {
+	SDL_Color c = font::NORMAL_COLOUR;
+	// Normal users are not allowed to color their messages
+	if(message.user == "server") {
+		int unused;
+		font::parse_markup(message.message.begin(), message.message.end(), &unused, &c, &unused);
+	}
+	return c;
+};
+
 ui::ui(game_display& disp, const std::string& title, const config& cfg, chat& c, config& gamelist) :
 	gui::widget(disp.video()),
 	disp_(disp),
@@ -250,7 +280,10 @@ ui::ui(game_display& disp, const std::string& title, const config& cfg, chat& c,
 	chat_textbox_(disp.video(), 100, "", false),
 	users_menu_(disp.video(), std::vector<std::string>(), false, -1, -1, NULL, &umenu_style),
 
+	user_list_(),
 	selected_game_(""),
+	selected_user_(""),
+	selected_user_changed_(false),
 
 	result_(CONTINUE),
 	gamelist_refresh_(false),
@@ -374,17 +407,46 @@ void ui::handle_event(const SDL_Event& event)
 	if(event.type == SDL_KEYDOWN) {
 		handle_key_event(event.key);
 	}
-    if(users_menu_.double_clicked()) {
+	if(users_menu_.double_clicked()) {
 		std::string usr_text = user_list_[users_menu_.selection()];
-		std::string caption = _("Send a private message to ") + usr_text;
-		gui::dialog d(disp(), _("Whisper"), caption, gui::OK_CANCEL);
-		d.set_textbox( _("Message: "));
 		Uint32 show_time = SDL_GetTicks();
-		if (!(d.show() || d.textbox_text().empty())) {
-			std::stringstream msg;
-			msg << "/msg " << usr_text << ' ' << d.textbox_text();
-			chat_handler::do_speak(msg.str());
-        }
+
+		// Hack: for some reason the help string stays visible for ever
+		/** @todo find out why the help string stays visible and fix it */
+		disp().video().clear_all_help_strings();
+
+		gui2::tmp_cmd_wrapper dlg(_("Selected user: ") + usr_text);
+		dlg.show(disp().video());
+
+		std::stringstream msg;
+		switch(dlg.get_retval()) {
+			case -1:
+				if(!dlg.message().empty()) msg << "/msg " << usr_text << ' ' << dlg.message();
+				break;
+			case 1:
+				msg << "/friend " << usr_text;
+				break;
+			case 2:
+				msg << "/ignore " << usr_text;
+				break;
+			case 3:
+				msg << "/remove " << usr_text;
+				break;
+			case 4:
+				msg << "/query status " << usr_text;
+				break;
+			case 5:
+				msg << "/query kick " << usr_text;
+				if(!dlg.reason().empty()) msg << ' ' << dlg.reason();
+				break;
+			case 6:
+				msg << "/query kban " << usr_text;
+				if(!dlg.time().empty()) msg << ' ' << dlg.time();
+				if(!dlg.reason().empty()) msg << ' ' << dlg.reason();
+		}
+
+		chat_handler::do_speak(msg.str());
+
 		if(show_time + 60000 < SDL_GetTicks()) {
 			//if the dialog has been open for a long time, refresh the lobby
 			config request;
@@ -392,6 +454,11 @@ void ui::handle_event(const SDL_Event& event)
 			network::send_data(request, 0, true);
 		}
 	}
+	if(users_menu_.selection() > 0 // -1 indicates an invalid selection
+			&& selected_user_ != user_list_[users_menu_.selection()]) {
+		selected_user_ = user_list_[users_menu_.selection()];
+		selected_user_changed_ = true;
+	}
 }
 
 void ui::add_chat_message(const time_t& time, const std::string& speaker, int /*side*/, const std::string& message, game_display::MESSAGE_TYPE /*type*/)
@@ -449,6 +516,8 @@ void ui::process_message(const config& msg, const bool whisper) {
 	if (!preferences::show_lobby_join(sender, message)) return;
 	if (preferences::is_ignored(sender)) return;
 
+	preferences::admin_authentication(sender, message);
+
 	if (whisper || utils::word_match(message, preferences::login())) {
 		sound::play_UI_sound(game_config::sounds::receive_message_highlight);
 	} else if (preferences::is_friend(sender)) {
@@ -456,7 +525,8 @@ void ui::process_message(const config& msg, const bool whisper) {
 	} else if (sender == "server") {
 		sound::play_UI_sound(game_config::sounds::receive_message_server);
 	} else {
-		sound::play_UI_sound(game_config::sounds::receive_message);
+		// too annoying and probably not any helpful
+		//sound::play_UI_sound(game_config::sounds::receive_message);
 	}
 	chat_.add_message(time(NULL), (whisper ? "whisper: " : "") + msg["sender"],
 			msg["message"]);
@@ -597,6 +667,7 @@ void ui::gamelist_updated(bool silent)
 		u_elem.game_id = "";
 		u_elem.location = "";
 		u_elem.state = (**user)["available"] == "no" ? GAME : LOBBY;
+		u_elem.registered = utils::string_bool((**user)["registered"]);
 		if(!(**user)["game_id"].empty()) {
 			u_elem.game_id = (**user)["game_id"];
 			if (u_elem.game_id == selected_game_) {
@@ -606,13 +677,11 @@ void ui::gamelist_updated(bool silent)
 		if(!(**user)["location"].empty()) {
 			u_elem.location = (**user)["location"];
 		}
-		std::vector<std::string> friends = utils::split(preferences::get("friends"));
-		std::vector<std::string> ignores = utils::split(preferences::get("ignores"));
 		if (u_elem.name == preferences::login()) {
 			u_elem.relation = ME;
-		} else if (std::find(ignores.begin(), ignores.end(), u_elem.name) != ignores.end()) {
+		} else if (preferences::is_ignored(u_elem.name)) {
 			u_elem.relation = IGNORED;
-		} else if (std::find(friends.begin(), friends.end(), u_elem.name) != friends.end()) {
+		} else if (preferences::is_friend(u_elem.name)) {
 			u_elem.relation = FRIEND;
 		} else {
 			u_elem.relation = NEUTRAL;
@@ -630,15 +699,21 @@ void ui::gamelist_updated(bool silent)
 	const std::string ingame_color_tag  = "#";
 	const std::string selgame_color_tag = "<0,191,255>";
 
+	// for now I just disregard the above till I know something better,
+	// it works for me anyways
+	const std::string registered_user_tag = "~";
+
 	std::string const imgpre = IMAGE_PREFIX + std::string("misc/status-");
 	std::vector<std::string> user_strings;
 	std::vector<std::string> menu_strings;
 
 	std::list<user_info>::const_iterator u_itor = u_list.begin();
 	while (u_itor != u_list.end()) {
-		const std::string game_str = (u_itor->state == LOBBY) ? "" : " (" + u_itor->location + ")";
+		const std::string name_str = u_itor->name +
+				((u_itor->state == LOBBY) ? "" : " (" + u_itor->location + ")");
 		std::string img_str = "";
 		std::string color_str = "";
+		std::string reg_str = "";
 		switch (u_itor->state) {
 			case LOBBY:    color_str = lobby_color_tag;   break;
 			case GAME:     color_str = ingame_color_tag;  break;
@@ -646,14 +721,15 @@ void ui::gamelist_updated(bool silent)
 		}
 		if (preferences::iconize_list()) {
 			switch (u_itor->relation) {
-				case NEUTRAL: img_str = imgpre + "neutral.png" + COLUMN_SEPARATOR; break;
-				case IGNORED: img_str = imgpre + "ignore.png"  + COLUMN_SEPARATOR; break;
-				case FRIEND:  img_str = imgpre + "friend.png"  + COLUMN_SEPARATOR; break;
-				case ME:      img_str = imgpre + "self.png"    + COLUMN_SEPARATOR; break;
+				case NEUTRAL: img_str = imgpre + "neutral.png" + IMG_TEXT_SEPARATOR; break;
+				case IGNORED: img_str = imgpre + "ignore.png"  + IMG_TEXT_SEPARATOR; break;
+				case FRIEND:  img_str = imgpre + "friend.png"  + IMG_TEXT_SEPARATOR; break;
+				case ME:      img_str = imgpre + "self.png"    + IMG_TEXT_SEPARATOR; break;
 			}
 		}
+		reg_str = u_itor->registered ? registered_user_tag : "";
 		user_strings.push_back(u_itor->name);
-		menu_strings.push_back(img_str + color_str + u_itor->name + game_str + HELP_STRING_SEPARATOR + u_itor->name + game_str);
+		menu_strings.push_back(img_str + reg_str + color_str + name_str + HELP_STRING_SEPARATOR + name_str);
 		u_itor++;
 	}
 
@@ -673,6 +749,13 @@ void ui::set_selected_game(const std::string game_id)
 void ui::set_user_menu_items(const std::vector<std::string>& list)
 {
 	users_menu_.set_items(list,true,true);
+
+	// Try to keep selected player
+	std::vector<std::string>::const_iterator i =
+			std::find(user_list_.begin(), user_list_.end(), selected_user_);
+	if(i != user_list_.end()) {
+		users_menu_.move_selection_keeping_viewport(i - user_list_.begin());
+	}
 }
 
 void ui::set_user_list(const std::vector<std::string>& list, bool silent)
@@ -688,6 +771,19 @@ void ui::set_user_list(const std::vector<std::string>& list, bool silent)
 	user_list_ = list;
 }
 
+std::string ui::get_selected_user_game() {
+	config::child_list users = gamelist_.get_children("user");
+	config::child_iterator user;
+
+	for (user = users.begin(); user != users.end(); ++user) {
+		if((**user)["name"] == selected_user_) {
+			return (**user)["game_id"];
+		}
+	}
+
+	return "";
+}
+
 void ui::append_to_title(const std::string& text) {
 	title_.set_text(title_.get_text() + text);
 }
