@@ -1,6 +1,6 @@
-/* $Id: multiplayer_wait.cpp 24329 2008-03-05 17:22:35Z shadowmaster $ */
+/* $Id: multiplayer_wait.cpp 32674 2009-02-08 13:02:29Z cib $ */
 /*
-   Copyright (C) 2007 - 2008
+   Copyright (C) 2007 - 2009
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
@@ -14,25 +14,16 @@
 
 #include "global.hpp"
 
-#include "construct_dialog.hpp"
 #include "dialogs.hpp"
-#include "game_display.hpp"
-#include "game_events.hpp"
-#include "game_preferences.hpp"
 #include "gettext.hpp"
 #include "leader_list.hpp"
 #include "log.hpp"
 #include "marked-up_text.hpp"
-#include "multiplayer.hpp"
 #include "multiplayer_wait.hpp"
-#include "replay.hpp"
 #include "statistics.hpp"
-#include "util.hpp"
-#include "video.hpp"
 #include "wml_separators.hpp"
-#include "serialization/string_utils.hpp"
+#include "formula_string_utils.hpp"
 
-#include <cassert>
 
 #define DBG_NW LOG_STREAM(debug, network)
 #define LOG_NW LOG_STREAM(info, network)
@@ -45,15 +36,17 @@ const int leader_pane_border = 10;
 
 namespace mp {
 
-wait::leader_preview_pane::leader_preview_pane(game_display& disp, const game_data* data,
-		const config::child_list& side_list) :
+wait::leader_preview_pane::leader_preview_pane(game_display& disp,
+		const config::child_list& side_list, int color) :
 	gui::preview_pane(disp.video()),
 	side_list_(side_list),
+	color_(color),
 	leader_combo_(disp, std::vector<std::string>()),
 	gender_combo_(disp, std::vector<std::string>()),
-	leaders_(side_list, data, &leader_combo_, &gender_combo_),
-	selection_(0), data_(data)
+	leaders_(side_list, &leader_combo_, &gender_combo_),
+	selection_(0)
 {
+	leaders_.set_colour(color_);
 	set_location(leader_pane_position);
 }
 
@@ -95,15 +88,15 @@ void wait::leader_preview_pane::draw_contents()
 		std::string leader = leaders_.get_leader();
 		std::string gender = leaders_.get_gender();
 
-		const game_data::unit_type_map& utypes = data_->unit_types;
+		unit_type_data::unit_type_map_wrapper& utypes = unit_type_data::types();
 		std::string leader_name;
 		std::string image;
 
 		const unit_type* ut;
 		const unit_type* utg;
 
-		if (utypes.find(leader) != utypes.end() && leader != "random") {
-			ut = &(utypes.find(leader)->second);
+		if (utypes.unit_type_exists(leader) && leader != "random") {
+			ut = &(utypes.find_unit_type(leader)->second);
 			if (!gender.empty()) {
 				if (gender == "female")
 					utg = &(ut->get_gender_unit_type(unit_race::FEMALE));
@@ -112,21 +105,17 @@ void wait::leader_preview_pane::draw_contents()
 			} else
 				utg = ut;
 
-			leader_name = utg->language_name();
-#ifdef LOW_MEM
-			image = utg->image();
-#else
-			image = utg->image() + std::string("~RC(") + std::string(utg->flag_rgb() + ">1)");
-#endif
+			leader_name = utg->type_name();
+			image = utg->image() + leaders_.get_RC_suffix(utg->flag_rgb());
 		}
 
 		for(std::vector<std::string>::const_iterator itor = recruit_list.begin();
 				itor != recruit_list.end(); ++itor) {
 
-			if (utypes.find(*itor) != utypes.end()) {
+			if (utypes.unit_type_exists(leader)) {
 				if(itor != recruit_list.begin())
 					recruit_string << ", ";
-				recruit_string << utypes.find(*itor)->second.language_name();
+				recruit_string << utypes.find_unit_type(*itor)->second.type_name();
 			}
 		}
 
@@ -189,13 +178,14 @@ handler_vector wait::leader_preview_pane::handler_members() {
 }
 
 
-wait::wait(game_display& disp, const config& cfg, const game_data& data,
+wait::wait(game_display& disp, const config& cfg,
 		mp::chat& c, config& gamelist) :
 	ui(disp, _("Game Lobby"), cfg, c, gamelist),
 	cancel_button_(disp.video(), _("Cancel")),
 	start_label_(disp.video(), _("Waiting for game to start..."), font::SIZE_SMALL, font::LOBBY_COLOUR),
 	game_menu_(disp.video(), std::vector<std::string>(), false, -1, -1, NULL, &gui::menu::bluebg_style),
-	game_data_(data),
+	level_(),
+	state_(),
 	stop_updates_(false)
 {
 	game_menu_.set_numeric_keypress_selection(false);
@@ -210,7 +200,10 @@ void wait::process_event()
 
 void wait::join_game(bool observe)
 {
-	for(;;) {
+	//if we have got valid side data
+	//the first condition is to make sure that we don't have another
+	//WML message with a side-tag in it
+	while((level_.values.find("version") == level_.values.end()) || (level_.child("side") == NULL)) {
 		network::connection data_res = dialogs::network_receive_dialog(disp(),
 				_("Getting game data..."), level_);
 		if (!data_res) {
@@ -222,39 +215,40 @@ void wait::join_game(bool observe)
 			set_result(QUIT);
 			return;
 		}
-		//if we have got valid side data
-		//the first condition is to make sure that we don't have another
-		//WML message with a side-tag in it
-		if( (level_.values.find("version") != level_.values.end()) && (level_.child("side") != NULL) )
-			break;
 	}
 
 	// Add the map name to the title.
 	append_to_title(": " + level_["name"]);
-	
+
 	if (!observe) {
 		const config::child_list& sides_list = level_.get_children("side");
 
-		if(sides_list.empty()) {
-			set_result(QUIT);
-			throw config::error(_("No multiplayer sides available in this game"));
-			return;
-		}
-
 		//search for an appropriate vacant slot. If a description is set
 		//(i.e. we're loading from a saved game), then prefer to get the side
 		//with the same description as our login. Otherwise just choose the first
 		//available side.
-		int side_choice = 0;
+		int side_choice = -1;
 		for(config::child_list::const_iterator s = sides_list.begin(); s != sides_list.end(); ++s) {
-			if((**s)["controller"] == "network" && (**s)["description"].empty()) {
-				if((**s)["save_id"] == preferences::login() || (**s)["current_player"] == preferences::login()) {
+			if((**s)["controller"] == "reserved" && (**s)["current_player"] == preferences::login())
+			{
+				side_choice = s - sides_list.begin();
+				break;
+			}
+			if((**s)["controller"] == "network" && (**s)["id"].empty()) {
+				if (side_choice < 0)  // found the first empty side
 					side_choice = s - sides_list.begin();
+				if((**s)["current_player"] == preferences::login()) {
+					side_choice = s - sides_list.begin();
+					break;  // found the prefered one
 				}
 			}
 		}
-		const bool allow_changes = (*sides_list[side_choice])["allow_changes"] != "no";
+		if (static_cast<size_t>(side_choice) >= sides_list.size()) {
+			set_result(QUIT);
+			return;
+		}
 
+		const bool allow_changes = (*sides_list[side_choice])["allow_changes"] != "no";
 
 		//if the client is allowed to choose their team, instead of having
 		//it set by the server, do that here.
@@ -265,7 +259,7 @@ void wait::join_game(bool observe)
 			events::event_context context;
 
 			const config* era = level_.child("era");
-			//! @todo Check whether we have the era. If we don't inform the user.
+			/** @todo Check whether we have the era. If we don't inform the user. */
 			if(era == NULL)
 				throw config::error(_("No era information found."));
 			const config::child_list& possible_sides =
@@ -276,19 +270,37 @@ void wait::join_game(bool observe)
 				return;
 			}
 
+			int color = side_choice;
+			const std::string color_str = (*sides_list[side_choice])["colour"];
+			if (!color_str.empty())
+				color = game_config::color_info(color_str).index() - 1;
+
 			std::vector<std::string> choices;
 			for(config::child_list::const_iterator side =
 					possible_sides.begin(); side !=
-					possible_sides.end(); ++side) {
-				choices.push_back((**side)["name"]);
+					possible_sides.end(); ++side)
+			{
+				const std::string& name = (**side)["name"];
+				const std::string& icon = (**side)["image"];
+
+				if (!icon.empty()) {
+					std::string rgb = (**side)["flag_rgb"];
+					if (rgb.empty())
+						rgb = "magenta";
+
+					choices.push_back(IMAGE_PREFIX + icon + "~RC(" + rgb + ">" +
+						lexical_cast<std::string>(color+1) + ")" + COLUMN_SEPARATOR + name);
+				} else {
+					choices.push_back(name);
+				}
 			}
 
 			std::vector<gui::preview_pane* > preview_panes;
-			leader_preview_pane leader_selector(disp(), &game_data_,
-					possible_sides);
+			leader_preview_pane leader_selector(disp(),
+					possible_sides, color);
 			preview_panes.push_back(&leader_selector);
 
-			const int res = gui::show_dialog(disp(), NULL, "", _("Choose your side:"),
+			const int res = gui::show_dialog(disp(), NULL, _("Choose your faction:"), _("Starting position: ") + lexical_cast<std::string>(side_choice+1),
 						gui::OK_CANCEL, &choices, &preview_panes);
 			if(res < 0) {
 				set_result(QUIT);
@@ -327,35 +339,30 @@ void wait::start_game()
 		statistics::read_stats(*stats);
 	}
 
-
-
-	//! @todo Instead of using level_to_gamestate reinit the state_, 
-	//! this needs more testing -- Mordante
-	//! It seems level_to_gamestate is needed for the start of game
-	//! download, but downloads of later scenarios miss certain info
-	//! and add a players section. Use players to decide between old
-	//! and new way. (Of course it would be nice to unify the data
-	//! stored.)
+	/**
+	 * @todo Instead of using level_to_gamestate reinit the state_,
+	 * this needs more testing -- Mordante
+	 * It seems level_to_gamestate is needed for the start of game
+	 * download, but downloads of later scenarios miss certain info
+	 * and add a players section. Use players to decide between old
+	 * and new way. (Of course it would be nice to unify the data
+	 * stored.)
+	 */
 	if(level_.child("player") == 0) {
-		level_to_gamestate(level_, state_, level_["savegame"] == "yes");
+		level_to_gamestate(level_, state_);
 	} else {
-	
-		state_ = game_state(game_data_, level_);
 
-		// When we observe and don't have the addon installed we still need 
-		// the old way, no clue why however. Code is a copy paste of 
+		state_ = game_state(level_);
+
+		// When we observe and don't have the addon installed we still need
+		// the old way, no clue why however. Code is a copy paste of
 		// playcampaign.cpp:576 which shows an 'Unknown scenario: '$scenario|'
 		// error. This seems to work and have no side effects....
 		if(!state_.scenario.empty() && state_.scenario != "null") {
 			DBG_NW << "Falling back to loading the old way.\n";
-			level_to_gamestate(level_, state_, level_["savegame"] == "yes");
+			level_to_gamestate(level_, state_);
 		}
 	}
-	// add era events after loaded
-	const config* const era_cfg = level_.child("era");
-	if (era_cfg != NULL && level_["savegame"] != "yes") {
-		game_events::add_events(era_cfg->get_children("event"),"era_events");
-	}
 
 	LOG_NW << "starting game\n";
 }
@@ -415,7 +422,7 @@ void wait::process_network_data(const config& data, const network::connection so
 		return;
 	} else if(data.child("scenario_diff")) {
 		LOG_NW << "received diff for scenario... applying...\n";
-		//! @todo We should catch config::error and then leave the game.
+		/** @todo We should catch config::error and then leave the game. */
 		level_.apply_diff(*data.child("scenario_diff"));
 		generate_menu();
 	} else if(data.child("side")) {
@@ -443,7 +450,7 @@ void wait::generate_menu()
 			continue;
 		}
 
-		std::string description = sd["user_description"];
+		std::string description = sd["description"];
 		const std::string faction_id = sd["id"];
 
 		t_string side_name = sd["name"];
@@ -461,17 +468,17 @@ void wait::generate_menu()
 			}
 		}
 
-		if(!sd["description"].empty())
-			playerlist.push_back(sd["description"]);
+		if(!sd["id"].empty())
+			playerlist.push_back(sd["id"]);
 
 		std::string leader_name;
 		std::string leader_image;
-		const game_data::unit_type_map& utypes = game_data_.unit_types;
+		unit_type_data::unit_type_map_wrapper& utypes = unit_type_data::types();
 		const unit_type* ut;
 		const unit_type* utg;
 
-		if (utypes.find(leader_type) != utypes.end() && leader_type != "random") {
-			ut = &(utypes.find(leader_type)->second);
+		if (utypes.unit_type_exists(leader_type) && leader_type != "random") {
+			ut = &(utypes.find_unit_type(leader_type)->second);
 			if (!gender_id.empty()) {
 				if (gender_id == "female")
 					utg = &(ut->get_gender_unit_type(unit_race::FEMALE));
@@ -482,11 +489,14 @@ void wait::generate_menu()
 			} else
 				utg = ut;
 
-			leader_name = utg->language_name();
+			leader_name = utg->type_name();
 #ifdef LOW_MEM
 			leader_image = utg->image();
 #else
-			leader_image = utg->image() + std::string("~RC(") + std::string(utg->flag_rgb() + ">" + sd["side"] + ")");
+			std::string RCcolor = sd["colour"];
+			if (RCcolor.empty())
+				RCcolor = sd["side"];
+			leader_image = utg->image() + std::string("~RC(") + std::string(utg->flag_rgb() + ">" + RCcolor + ")");
 #endif
 		} else {
 			leader_image = leader_list_manager::random_enemy_picture;
@@ -500,6 +510,10 @@ void wait::generate_menu()
 				if(p != std::string::npos && p < side_name.size()) {
 					side_name = IMAGE_PREFIX + leader_image + COLUMN_SEPARATOR + side_name.str().substr(p+1);
 				}
+			} else {
+				// no image prefix, just add the leader image
+				// (assuming that there is also no COLUMN_SEPARATOR)
+				side_name = IMAGE_PREFIX + leader_image + COLUMN_SEPARATOR + side_name.str();
 			}
 		}
 
@@ -522,7 +536,7 @@ void wait::generate_menu()
 			str << sd["income"] << _(")");
 		}
 
-		str	<< COLUMN_SEPARATOR << sd["user_team_name"];
+		str	<< COLUMN_SEPARATOR << t_string().from_serialized(sd["user_team_name"].str());
 		int disp_color = lexical_cast_default<int>(sd["colour"], 0) - 1;
 		if(!sd["colour"].empty()) {
 			try {
@@ -531,8 +545,10 @@ void wait::generate_menu()
 				//ignore
 			}
 		} else {
-			//! @todo we fall back to the side colour, but that's ugly rather
-			// make the colour mandatory in 1.5.
+			/**
+			 * @todo we fall back to the side colour, but that's ugly rather
+			 * make the colour mandatory in 1.5.
+			 */
 			disp_color = lexical_cast_default<int>(sd["side"], 0) - 1;
 		}
 		str << COLUMN_SEPARATOR << get_colour_string(disp_color);
