@@ -1,6 +1,6 @@
-/* $Id: network_worker.cpp 28347 2008-08-06 15:28:35Z suokko $ */
+/* $Id: network_worker.cpp 33687 2009-03-15 16:50:42Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,42 +12,36 @@
    See the COPYING file for more details.
 */
 
-//! Network worker handles data transfers in threads
-//! Remember to use mutexs as little as possible
-//! All global vars should be used in mutex
-//! FIXME: @todo: All code which holds a mutex should run O(1) time
-//! for scalability. Implement read/write locks.
-//!  (postponed for 1.5)
+/**
+ * Network worker handles data transfers in threads
+ * Remember to use mutexs as little as possible
+ * All global vars should be used in mutex
+ * FIXME: @todo: All code which holds a mutex should run O(1) time
+ * for scalability. Implement read/write locks.
+ *  (postponed for 1.5)
+ */
 
 #include "global.hpp"
 
+#include "scoped_resource.hpp"
 #include "log.hpp"
 #include "network_worker.hpp"
-#include "network.hpp"
-#include "thread.hpp"
 #include "filesystem.hpp"
-//#include "wesconfig.h"
+#include "thread.hpp"
 #include "serialization/binary_or_text.hpp"
 #include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
+#include "wesconfig.h"
 
-#include <algorithm>
-#include <cassert>
 #include <cerrno>
-#include <cstring>
 #include <deque>
-#include <iostream>
-#include <map>
-#include <vector>
 
-#ifdef USE_SENDFILE
+#ifdef HAVE_SENDFILE
 #include <sys/sendfile.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
-#define HAVE_POLL_H
 #endif
 
-#include <boost/iostreams/filter/gzip.hpp>
 
 #ifdef __AMIGAOS4__
 #include <unistd.h>
@@ -58,8 +52,12 @@
 #  undef INADDR_ANY
 #  undef INADDR_BROADCAST
 #  undef INADDR_NONE
+#  ifndef NOMINMAX
+#   define NOMINMAX
+#  endif
 #  include <windows.h>
 #  define USE_SELECT 1
+typedef int socklen_t;
 #else
 #  include <sys/types.h>
 #  include <sys/socket.h>
@@ -112,25 +110,31 @@ size_t max_threads = 0;
 size_t get_shard(TCPsocket sock) { return reinterpret_cast<uintptr_t>(sock)%NUM_SHARDS; }
 
 struct buffer {
-	explicit buffer(TCPsocket sock) : 
+	explicit buffer(TCPsocket sock) :
 		sock(sock),
 		config_buf(),
 		config_error(""),
-		gzipped(false)
+		stream(),
+		gzipped(false),
+		raw_buffer()
 		{}
 
 	TCPsocket sock;
 	mutable config config_buf;
 	std::string config_error;
 	std::ostringstream stream;
-	//! Do we wish to send the data gzipped, if not use binary wml.
-	//! This needs to stay until the last user of binary_wml has
-	//! been removed.
+
+	/**
+	 * Do we wish to send the data gzipped, if not use binary wml. This needs
+	 * to stay until the last user of binary_wml has been removed.
+	 */
 	bool gzipped;
 
-	//this field is used if we're sending a raw buffer instead of
-	//through a config object. It will contain the entire contents of
-	//the buffer being sent.
+	/**
+	 * This field is used if we're sending a raw buffer instead of through a
+	 * config object. It will contain the entire contents of the buffer being
+	 * sent.
+	 */
 	std::vector<char> raw_buffer;
 };
 
@@ -141,6 +145,12 @@ buffer_set outgoing_bufs[NUM_SHARDS];
 
 struct schema_pair
 {
+	schema_pair() :
+		incoming(),
+		outgoing()
+	{
+	}
+
 	compression_schema incoming, outgoing;
 };
 
@@ -148,7 +158,7 @@ typedef std::map<TCPsocket,schema_pair> schema_map;
 
 schema_map schemas; //schemas_mutex
 
-//a queue of sockets that we are waiting to receive on
+/** a queue of sockets that we are waiting to receive on */
 typedef std::vector<TCPsocket> receive_list;
 receive_list pending_receives[NUM_SHARDS];
 
@@ -171,6 +181,8 @@ threading::condition* cond[NUM_SHARDS];
 
 std::map<Uint32,threading::thread*> threads[NUM_SHARDS];
 std::vector<Uint32> to_clear[NUM_SHARDS];
+int system_send_buffer_size = 0;
+bool network_use_system_sendfile = false;
 
 int receive_bytes(TCPsocket s, char* buf, size_t nbytes)
 {
@@ -188,11 +200,31 @@ int receive_bytes(TCPsocket s, char* buf, size_t nbytes)
 #endif
 }
 
-bool receive_with_timeout(TCPsocket s, char* buf, size_t nbytes,
-		bool update_stats=false, int timeout_ms=60000)
+
+void check_send_buffer_size(TCPsocket& s)
 {
+	if (system_send_buffer_size)
+		return;
+	_TCPsocket* sock = reinterpret_cast<_TCPsocket*>(s);
+	socklen_t len = sizeof(system_send_buffer_size);
+#ifdef _WIN32
+	getsockopt(sock->channel, SOL_SOCKET, SO_RCVBUF,reinterpret_cast<char*>(&system_send_buffer_size), &len);
+#else
+	getsockopt(sock->channel, SOL_SOCKET, SO_RCVBUF,&system_send_buffer_size, &len);
+#endif
+	--system_send_buffer_size;
+	DBG_NW << "send buffer size: " << system_send_buffer_size << "\n";
+}
+
+bool receive_with_timeout(TCPsocket s, char* buf, size_t nbytes,
+		bool update_stats=false, int idle_timeout_ms=30000,
+		int total_timeout_ms=300000)
+{
+#if !defined(USE_POLL) && !defined(USE_SELECT)
 	int startTicks = SDL_GetTicks();
 	int time_used = 0;
+#endif
+	int timeout_ms = idle_timeout_ms;
 	while(nbytes > 0) {
 		const int bytes_read = receive_bytes(s, buf, nbytes);
 		if(bytes_read == 0) {
@@ -207,55 +239,112 @@ bool receive_with_timeout(TCPsocket s, char* buf, size_t nbytes,
 			if(true)
 #endif
 			{
+#ifdef USE_POLL
+				struct pollfd fd = { ((_TCPsocket*)s)->channel, POLLIN, 0 };
+				int poll_res;
+
+				//we timeout of the poll every 100ms. This lets us check to
+				//see if we have been disconnected, in which case we should
+				//abort the receive.
+				const int poll_timeout = std::min(timeout_ms, 100);
+				do {
+					poll_res = poll(&fd, 1, poll_timeout);
+
+					if(poll_res == 0) {
+						timeout_ms -= poll_timeout;
+						total_timeout_ms -= poll_timeout;
+						if(timeout_ms <= 0 || total_timeout_ms <= 0) {
+							//we've been waiting too long; abort the receive
+							//as having failed due to timeout.
+							return false;
+						}
+
+						//check to see if we've been interrupted
+						const size_t shard = get_shard(s);
+						const threading::lock lock(*shard_mutexes[shard]);
+						socket_state_map::iterator lock_it = sockets_locked[shard].find(s);
+						assert(lock_it != sockets_locked[shard].end());
+						if(lock_it->second == SOCKET_INTERRUPT) {
+							return false;
+						}
+					}
+
+				} while(poll_res == 0 || (poll_res == -1 && errno == EINTR));
+
+				if (poll_res < 1)
+					return false;
+#elif defined(USE_SELECT)
+				int retval;
+				const int select_timeout = std::min(timeout_ms, 100);
+				do {
+					fd_set readfds;
+					FD_ZERO(&readfds);
+					FD_SET(((_TCPsocket*)s)->channel, &readfds);
+					struct timeval tv;
+					tv.tv_sec = select_timeout/1000;
+					tv.tv_usec = select_timeout % 1000 * 1000;
+					retval = select(((_TCPsocket*)s)->channel + 1, &readfds, NULL, NULL, &tv);
+					DBG_NW << "select retval: " << retval << ", timeout idle " << timeout_ms
+						<< " total " << total_timeout_ms << " (ms)\n";
+					if(retval == 0) {
+						timeout_ms -= select_timeout;
+						total_timeout_ms -= select_timeout;
+						if(timeout_ms <= 0 || total_timeout_ms <= 0) {
+							//we've been waiting too long; abort the receive
+							//as having failed due to timeout.
+							return false;
+						}
+
+						//check to see if we've been interrupted
+						const size_t shard = get_shard(s);
+						const threading::lock lock(*shard_mutexes[shard]);
+						socket_state_map::iterator lock_it = sockets_locked[shard].find(s);
+						assert(lock_it != sockets_locked[shard].end());
+						if(lock_it->second == SOCKET_INTERRUPT) {
+							return false;
+						}
+					}
+				} while(retval == 0 || retval == -1 && errno == EINTR);
+
+				if (retval < 1) {
+					return false;
+				}
+#else
 				//TODO: consider replacing this with a select call
 				time_used = SDL_GetTicks() - startTicks;
 				if(time_used >= timeout_ms) {
 					return false;
 				}
-#ifdef USE_POLL
-				struct pollfd fd = { ((_TCPsocket*)s)->channel, POLLIN, 0 };
-				int poll_res;
-				do {
-					time_used = SDL_GetTicks() - startTicks;
-					poll_res = poll(&fd, 1, timeout_ms - time_used);
-				} while(poll_res == -1 && errno == EINTR);
-
-#elif defined(USE_SELECT)
-				fd_set readfds;
-				FD_ZERO(&readfds);
-				FD_SET(((_TCPsocket*)s)->channel, &readfds);
-				int retval;
-				int time_left;
-				struct timeval tv;
-
-				do {
-					time_used = SDL_GetTicks() - startTicks;
-					time_left = timeout_ms - time_used;
-					tv.tv_sec = time_left/1000;
-					tv.tv_usec = time_left % 1000;
-					retval = select(((_TCPsocket*)s)->channel + 1, &readfds, NULL, NULL, &tv);
-				} while(retval == -1 && errno == EINTR);
-
-#elif
-				SDL_Delay(5);
+				SDL_Delay(20);
 #endif
 			} else {
 				return false;
 			}
 		} else {
-
+			timeout_ms = idle_timeout_ms;
 			buf += bytes_read;
 			if(update_stats && !raw_data_only) {
 				const threading::lock lock(*stats_mutex);
 				transfer_stats[s].second.transfer(static_cast<size_t>(bytes_read));
 			}
-			
+
 			if(bytes_read > static_cast<int>(nbytes)) {
 				return false;
 			}
 			nbytes -= bytes_read;
 			// We got some data from server so reset start time so slow conenction won't timeout.
+#if !defined(USE_POLL) && !defined(USE_SELECT)
 			startTicks = SDL_GetTicks();
+#endif
+		}
+		{
+			const size_t shard = get_shard(s);
+			const threading::lock lock(*shard_mutexes[shard]);
+			socket_state_map::iterator lock_it = sockets_locked[shard].find(s);
+			assert(lock_it != sockets_locked[shard].end());
+			if(lock_it->second == SOCKET_INTERRUPT) {
+				return false;
+			}
 		}
 	}
 
@@ -265,7 +354,7 @@ bool receive_with_timeout(TCPsocket s, char* buf, size_t nbytes,
 static void output_to_buffer(TCPsocket sock, const config& cfg, std::ostringstream& compressor, bool gzipped)
 {
 	if(gzipped) {
-		config_writer writer(compressor, true, "");
+		config_writer writer(compressor, true);
 		writer.write(cfg);
 	} else {
 		compression_schema *compress;
@@ -279,27 +368,22 @@ static void output_to_buffer(TCPsocket sock, const config& cfg, std::ostringstre
 
 static void make_network_buffer(const char* input, int len, std::vector<char>& buf)
 {
-	buf.resize(4 + len + 1);
-	SDLNet_Write32(len + 1, &buf[0]);
+	buf.resize(4 + len);
+	SDLNet_Write32(len, &buf[0]);
 	memcpy(&buf[4], input, len);
-	buf.back() = 0;
 }
 
-
-
 static SOCKET_STATE send_buffer(TCPsocket sock, std::vector<char>& buf, int in_size = -1)
 {
 #ifdef __BEOS__
 	int timeout = 60000;
 #endif
+//	check_send_buffer_size(sock);
 	size_t upto = 0;
-
 	size_t size = buf.size();
 	if (in_size != -1)
-	{
 		size = in_size;
-	}
-
+	int send_len = 0;
 
 	if (!raw_data_only)
 	{
@@ -320,9 +404,11 @@ static SOCKET_STATE send_buffer(TCPsocket sock, std::vector<char>& buf, int in_s
 				return SOCKET_ERRORED;
 			}
 		}
-		const int res = SDLNet_TCP_Send(sock, &buf[upto], static_cast<int>(size - upto));
-		
-		if(res == static_cast<int>(size - upto)) {
+		send_len = static_cast<int>(size - upto);
+		const int res = SDLNet_TCP_Send(sock, &buf[upto],send_len);
+
+
+		if( res == send_len) {
 			if (!raw_data_only)
 			{
 				const threading::lock lock(*stats_mutex);
@@ -330,9 +416,8 @@ static SOCKET_STATE send_buffer(TCPsocket sock, std::vector<char>& buf, int in_s
 			}
 			return SOCKET_READY;
 		}
-	
 #if defined(EAGAIN) && !defined(__BEOS__) && !defined(_WIN32)
-		if(errno == EAGAIN)
+	        if(errno == EAGAIN)
 #elif defined(EWOULDBLOCK)
 		if(errno == EWOULDBLOCK)
 #endif
@@ -352,7 +437,7 @@ static SOCKET_STATE send_buffer(TCPsocket sock, std::vector<char>& buf, int in_s
 				poll_res = poll(&fd, 1, 60000);
 			} while(poll_res == -1 && errno == EINTR);
 
-			
+
 			if(poll_res > 0)
 				continue;
 #elif defined(USE_SELECT) && !defined(__BEOS__)
@@ -387,85 +472,115 @@ static SOCKET_STATE send_buffer(TCPsocket sock, std::vector<char>& buf, int in_s
 	}
 }
 
+#ifdef HAVE_SENDFILE
+struct cork_setter {
+	cork_setter(int socket) : cork_(1), socket_(socket)
+	{
+		setsockopt(socket_, IPPROTO_TCP, TCP_CORK, &cork_, sizeof(cork_));;
+	}
+	~cork_setter()
+	{
+		cork_ = 0;
+		setsockopt(socket_, IPPROTO_TCP, TCP_CORK, &cork_, sizeof(cork_));
+	}
+	private:
+	int cork_;
+	int socket_;
+};
+
+struct close_fd {
+	    void operator()(int fd) const { close(fd); }
+};
+typedef util::scoped_resource<int, close_fd> scoped_fd;
+#endif
+
 static SOCKET_STATE send_file(buffer* buf)
 {
 	size_t upto = 0;
-	int send_size = 0;
 	size_t filesize = file_size(buf->config_error);
-#ifdef USE_SENDFILE
-	std::vector<char> buffer;
-	buffer.reserve(4);
-	SDLNet_Write32(filesize,&buffer[0]);
-	int socket = reinterpret_cast<_TCPsocket*>(buf->sock)->channel;
-	int in_file = open(buf->config_error.c_str(),O_NOATIME | O_RDONLY);
-	int cock = 1;
-	int poll_res;
-	struct pollfd fd = {socket, POLLOUT, 0 };
-	setsockopt(socket, IPPROTO_TCP, TCP_CORK, &cock, sizeof(cock));;
-	do {
-		poll_res = poll(&fd, 1, 600000);
-	} while(poll_res == -1 && errno == EINTR);
-
-	SOCKET_STATE result;
-	if (poll_res > 0)
-		result = send_buffer(buf->sock, buffer, 4);
-	else
-		result = SOCKET_ERRORED;
-	
-	
-	if (result != SOCKET_READY)
+#ifdef HAVE_SENDFILE
+	// implements linux sendfile support
+	LOG_NW << "send_file use system sendfile: " << (network_use_system_sendfile?"yes":"no") << "\n";
+	if (network_use_system_sendfile)
 	{
-		close(in_file);
-		cock = 0;
-		setsockopt(socket, IPPROTO_TCP, TCP_CORK, &cock, sizeof(cock));
-		return result;
-	}
-	result = SOCKET_READY;
-#else
-	SDLNet_Write32(filesize,&buf->raw_buffer[0]);
-	scoped_istream file_stream = istream_file(buf->config_error);
-	SOCKET_STATE result = send_buffer(buf->sock, buf->raw_buffer, 4);
-
-	if (result != SOCKET_READY)
-	{
-		return result;
-	}
-#endif
-	while (true)
-	{
-#ifdef USE_SENDFILE
-
+		std::vector<char> buffer;
+		buffer.resize(4);
+		SDLNet_Write32(filesize,&buffer[0]);
+		int socket = reinterpret_cast<_TCPsocket*>(buf->sock)->channel;
+		const scoped_fd in_file(open(buf->config_error.c_str(),O_NOATIME | O_RDONLY));
+		cork_setter set_socket_cork(socket);
+		int poll_res;
+		struct pollfd fd = {socket, POLLOUT, 0 };
 		do {
 			poll_res = poll(&fd, 1, 600000);
 		} while(poll_res == -1 && errno == EINTR);
 
-		if (poll_res <= 0 )
-		{
+		SOCKET_STATE result;
+		if (poll_res > 0)
+			result = send_buffer(buf->sock, buffer, 4);
+		else
 			result = SOCKET_ERRORED;
-			break;
-		}
 
 
-		int bytes = ::sendfile(socket, in_file, 0, filesize);
-	
-		if (bytes == -1)
+		if (result != SOCKET_READY)
 		{
-			if (errno == EAGAIN)
-				continue;
-			result = SOCKET_ERRORED;
-			break;
+			return result;
 		}
+		result = SOCKET_READY;
 
-		upto += bytes;
-
-
-		if (upto == filesize)
+		while (true)
 		{
-			buf->raw_buffer.push_back(0);
-			result = send_buffer(buf->sock, buf->raw_buffer, 1);
-			break;
+
+			do {
+				poll_res = poll(&fd, 1, 600000);
+			} while(poll_res == -1 && errno == EINTR);
+
+			if (poll_res <= 0 )
+			{
+				result = SOCKET_ERRORED;
+				break;
+			}
+
+
+			int bytes = ::sendfile(socket, in_file, 0, filesize);
+
+			if (bytes == -1)
+			{
+				if (errno == EAGAIN)
+					continue;
+				result = SOCKET_ERRORED;
+				break;
+			}
+			upto += bytes;
+
+
+			if (upto == filesize)
+			{
+				break;
+			}
 		}
-#else
+
+		return result;
+	}
+#endif
+	// default sendfile implementation
+	// if no system implementation is enabled
+	int send_size = 0;
+	// reserve 1024*8 bytes buffer
+	buf->raw_buffer.resize(std::min<size_t>(1024*8, filesize));
+	SDLNet_Write32(filesize,&buf->raw_buffer[0]);
+	scoped_istream file_stream = istream_file(buf->config_error);
+	SOCKET_STATE result = send_buffer(buf->sock, buf->raw_buffer, 4);
+
+	if (!file_stream->good()) {
+		ERR_NW << "send_file: Couldn't open file " << buf->config_error << "\n";
+	}
+	if (result != SOCKET_READY)
+	{
+		return result;
+	}
+	while (file_stream->good())
+	{
 		// read data
 		file_stream->read(&buf->raw_buffer[0], buf->raw_buffer.size());
 		send_size = file_stream->gcount();
@@ -478,32 +593,21 @@ static SOCKET_STATE send_file(buffer* buf)
 		}
 		if (upto == filesize)
 		{
-			buf->raw_buffer[0] = 0;
-			result = send_buffer(buf->sock, buf->raw_buffer, 1);
 			break;
 		}
 
-#endif
 	}
-#ifdef USE_SENDFILE
-	close(in_file);
-	cock = 0;
-	setsockopt(socket, IPPROTO_TCP, TCP_CORK, &cock, sizeof(cock));
-#endif
+	if (upto != filesize && !file_stream->good()) {
+		ERR_NW << "send_file failed because stream not good from file "
+			<< buf->config_error << " upto: " << upto
+			<< " size: " << filesize << "\n";
+	}
 	return result;
 }
 
 static SOCKET_STATE receive_buf(TCPsocket sock, std::vector<char>& buf)
 {
-#ifdef __GNUC__
-	// The address needs to be aligned on a Sparc system, if it's not aligned
-	// the SDLNet_Read32 call will cause a SIGBUS and the server will be 
-	// terminated.
-	// http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=426318
-	char num_buf[4] __attribute__ ((aligned (4)));
-#else
-	char num_buf[4];
-#endif
+	char num_buf[4] ALIGN_4;
 	bool res = receive_with_timeout(sock,num_buf,4,false);
 
 	if(!res) {
@@ -644,7 +748,7 @@ static int process_queue(void* shard_num)
 				if(sent_buf->raw_buffer.empty()) {
 					const std::string &value = sent_buf->stream.str();
 					make_network_buffer(value.c_str(), value.size(), sent_buf->raw_buffer);
-				} 
+				}
 
 				result = send_buffer(sent_buf->sock, sent_buf->raw_buffer);
 			}
@@ -654,7 +758,7 @@ static int process_queue(void* shard_num)
 		}
 
 
-		if(result != SOCKET_READY || buf.empty()) 
+		if(result != SOCKET_READY || buf.empty())
 		{
 			check_socket_result(sock,result);
 		       	continue;
@@ -720,7 +824,7 @@ manager::manager(size_t p_min_threads,size_t p_max_threads) : active_(!managed)
 
 		for(size_t shard = 0; shard != NUM_SHARDS; ++shard) {
 			const threading::lock lock(*shard_mutexes[shard]);
-			for(size_t n = 0; n != min_threads; ++n) {
+			for(size_t n = 0; n != p_min_threads; ++n) {
 				threading::thread * tmp = new threading::thread(process_queue,(void*)uintptr_t(shard));
 				threads[shard][tmp->get_id()] = tmp;
 			}
@@ -731,7 +835,6 @@ manager::manager(size_t p_min_threads,size_t p_max_threads) : active_(!managed)
 manager::~manager()
 {
 	if(active_) {
-		
 		managed = false;
 
 		for(size_t shard = 0; shard != NUM_SHARDS; ++shard) {
@@ -740,22 +843,30 @@ manager::~manager()
 				socket_errors[shard] = 0;
 			}
 
-			cond[shard]->notify_all();
+ 			cond[shard]->notify_all();
 
 			for(std::map<Uint32,threading::thread*>::const_iterator i = threads[shard].begin(); i != threads[shard].end(); ++i) {
+
 				DBG_NW << "waiting for thread " << i->first << " to exit...\n";
 				delete i->second;
 				DBG_NW << "thread exited...\n";
 			}
 
-			threads[shard].clear();
+			// Condition variables must be deleted first as
+			// they make reference to mutexs. If the mutexs
+			// are destroyed first, the condition variables
+			// will access memory already freed by way of
+			// stale mutex. Bad things will follow. ;)
+ 			threads[shard].clear();
+			// Have to clean up to_clear so no bogus clearing of threads
+			to_clear[shard].clear();
+ 			delete cond[shard];
+ 			cond[shard] = NULL;
 			delete shard_mutexes[shard];
-			shard_mutexes[shard] = NULL;
-			delete cond[shard];
-			cond[shard] = NULL;
-		}
+ 			shard_mutexes[shard] = NULL;
+ 		}
 
-		delete  stats_mutex;
+		delete stats_mutex;
 		delete schemas_mutex;
 		delete received_mutex;
 		stats_mutex = 0;
@@ -792,6 +903,11 @@ void set_raw_data_only()
 	raw_data_only = true;
 }
 
+void set_use_system_sendfile(bool use)
+{
+	network_use_system_sendfile = use;
+}
+
 void receive_data(TCPsocket sock)
 {
 	{
@@ -806,11 +922,7 @@ void receive_data(TCPsocket sock)
 	}
 }
 
-TCPsocket get_received_data(TCPsocket sock, config& cfg, bool* gzipped
-#ifdef BANDWIDTH_MONITOR
-		,network::bandwidth_in_ptr& bandwidth_in
-#endif
-		)
+TCPsocket get_received_data(TCPsocket sock, config& cfg, bool* gzipped,network::bandwidth_in_ptr& bandwidth_in)
 {
 	assert(!raw_data_only);
 	const threading::lock lock_received(*received_mutex);
@@ -840,10 +952,8 @@ TCPsocket get_received_data(TCPsocket sock, config& cfg, bool* gzipped
 		buffer* buf = *itor;
 		if (gzipped)
 			*gzipped = buf->gzipped;
-		received_data_queue.erase(itor);
-#ifdef BANDWIDTH_MONITOR
 		bandwidth_in.reset(new network::bandwidth_in((*itor)->raw_buffer.size()));
-#endif
+		received_data_queue.erase(itor);
 		delete buf;
 		return res;
 	}
@@ -874,19 +984,7 @@ static void queue_buffer(TCPsocket sock, buffer* queued_buf)
 	if(i->second == SOCKET_READY || i->second == SOCKET_ERRORED) {
 		cond[shard]->notify_one();
 	}
-}
 
-void queue_file(TCPsocket sock, const std::string& filename)
-{
-	buffer* queued_buf = new buffer(sock);
-	queued_buf->config_error = filename;
-#ifndef USE_SENDFILE
-	// We reserve buffer in main thread
-	// this helps in memory problems with threads
-	// We use 8KB buffer
-	queued_buf->raw_buffer.resize(1024*8);
-#endif
-	queue_buffer(sock, queued_buf);
 }
 
 void queue_raw_data(TCPsocket sock, const char* buf, int len)
@@ -897,21 +995,24 @@ void queue_raw_data(TCPsocket sock, const char* buf, int len)
 	queue_buffer(sock, queued_buf);
 }
 
-size_t queue_data(TCPsocket sock,const config& buf, const bool gzipped
-#ifdef BANDWIDTH_MONITOR
-		, const std::string& packet_type
-#endif
-		)
+
+void queue_file(TCPsocket sock, const std::string& filename)
+{
+ 	buffer* queued_buf = new buffer(sock);
+ 	queued_buf->config_error = filename;
+ 	queue_buffer(sock, queued_buf);
+}
+
+size_t queue_data(TCPsocket sock,const config& buf, const bool gzipped, const std::string& packet_type)
 {
 	DBG_NW << "queuing data...\n";
 
 	buffer* queued_buf = new buffer(sock);
 	output_to_buffer(sock, buf, queued_buf->stream, gzipped);
 	queued_buf->gzipped = gzipped;
-	size_t size = queued_buf->stream.str().size();
-#ifdef BANDWIDTH_MONITOR
+	const size_t size = queued_buf->stream.str().size();
+
 	network::add_bandwidth_out(packet_type, size);
-#endif
 	queue_buffer(sock, queued_buf);
 	return size;
 }
@@ -919,7 +1020,7 @@ size_t queue_data(TCPsocket sock,const config& buf, const bool gzipped
 namespace
 {
 
-//! Caller has to make sure to own the mutex for this shard
+/** Caller has to make sure to own the mutex for this shard */
 void remove_buffers(TCPsocket sock)
 {
 	{
@@ -963,14 +1064,14 @@ bool is_locked(const TCPsocket sock) {
 	return (lock_it->second == SOCKET_LOCKED);
 }
 
-bool close_socket(TCPsocket sock, bool force)
+bool close_socket(TCPsocket sock)
 {
 	{
 		const size_t shard = get_shard(sock);
 		const threading::lock lock(*shard_mutexes[shard]);
 
 		pending_receives[shard].erase(std::remove(pending_receives[shard].begin(),pending_receives[shard].end(),sock),pending_receives[shard].end());
-		{	
+		{
 			const threading::lock lock_schemas(*schemas_mutex);
 			schemas.erase(sock);
 		}
@@ -980,7 +1081,7 @@ bool close_socket(TCPsocket sock, bool force)
 			remove_buffers(sock);
 			return true;
 		}
-		if (!(lock_it->second == SOCKET_LOCKED || lock_it->second == SOCKET_INTERRUPT) || force) {
+		if (!(lock_it->second == SOCKET_LOCKED || lock_it->second == SOCKET_INTERRUPT)) {
 			sockets_locked[shard].erase(lock_it);
 			remove_buffers(sock);
 			return true;
@@ -988,7 +1089,7 @@ bool close_socket(TCPsocket sock, bool force)
 			lock_it->second = SOCKET_INTERRUPT;
 			return false;
 		}
-	
+
 	}
 
 
