@@ -1,6 +1,6 @@
-/* $Id: pathutils.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: pathutils.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,15 +12,16 @@
    See the COPYING file for more details.
 */
 
-//! @file pathutils.cpp
-//! Various pathfinding functions and utilities.
+/**
+ * @file pathutils.cpp
+ * Various pathfinding functions and utilities.
+ */
 
 #include "global.hpp"
 
 #include "pathutils.hpp"
-#include "util.hpp"
 
-size_t distance_between(const gamemap::location& a, const gamemap::location& b)
+size_t distance_between(const map_location& a, const map_location& b)
 {
 	const size_t hdistance = abs(a.x - b.x);
 
@@ -33,10 +34,10 @@ size_t distance_between(const gamemap::location& a, const gamemap::location& b)
 	// = maximum(hdistance, vdistance+hdistance-hdistance/2-hdistance%2)
 	// = maximum(hdistance,abs(a.y-b.y)+vpenalty+hdistance/2)
 
-	return maximum<int>(hdistance, abs(a.y - b.y) + vpenalty + hdistance/2);
+	return std::max<int>(hdistance, abs(a.y - b.y) + vpenalty + hdistance/2);
 }
 
-void get_adjacent_tiles(const gamemap::location& a, gamemap::location* res)
+void get_adjacent_tiles(const map_location& a, map_location* res)
 {
 	res->x = a.x;
 	res->y = a.y-1;
@@ -57,7 +58,31 @@ void get_adjacent_tiles(const gamemap::location& a, gamemap::location* res)
 	res->y = a.y - (is_even(a.x) ? 1:0);
 }
 
-bool tiles_adjacent(const gamemap::location& a, const gamemap::location& b)
+void get_tile_ring(const map_location& a, const int r, std::vector<map_location>& res)
+{
+	if(r <= 0) {
+		return;
+	}
+
+	map_location loc = a.get_direction(map_location::SOUTH_WEST, r);
+
+	for(int n = 0; n != 6; ++n) {
+		const map_location::DIRECTION dir = static_cast<map_location::DIRECTION>(n);
+		for(int i = 0; i != r; ++i) {
+			res.push_back(loc);
+			loc = loc.get_direction(dir, 1);
+		}
+	}
+}
+
+void get_tiles_in_radius(const map_location& a, const int r, std::vector<map_location>& res)
+{
+	for(int n = 0; n <= r; ++n) {
+		get_tile_ring(a, n, res);
+	}
+}
+
+bool tiles_adjacent(const map_location& a, const map_location& b)
 {
 	// Two tiles are adjacent:
 	// if y is different by 1, and x by 0,
@@ -67,7 +92,7 @@ bool tiles_adjacent(const gamemap::location& a, const gamemap::location& b)
 
 	const int xdiff = abs(a.x - b.x);
 	const int ydiff = abs(a.y - b.y);
-	return ydiff == 1 && a.x == b.x || xdiff == 1 && a.y == b.y ||
-	       xdiff == 1 && ydiff == 1 && (a.y > b.y ? is_even(a.x) : is_even(b.x));
+	return (ydiff == 1 && a.x == b.x) || (xdiff == 1 && a.y == b.y) ||
+	       (xdiff == 1 && ydiff == 1 && (a.y > b.y ? is_even(a.x) : is_even(b.x)));
 }
 
