@@ -1,6 +1,6 @@
-/* $Id: play_controller.cpp 27256 2008-06-17 14:35:07Z alink $ */
+/* $Id: play_controller.cpp 33687 2009-03-15 16:50:42Z mordante $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -13,64 +13,90 @@
    See the COPYING file for more details.
 */
 
-//! @file play_controller.cpp 
-//! Handle input via mouse & keyboard, events, schedule commands.
+/**
+ *  @file play_controller.cpp
+ *  Handle input via mouse & keyboard, events, schedule commands.
+ */
 
 #include "play_controller.hpp"
 #include "dialogs.hpp"
 #include "config_adapter.hpp"
-#include "game_display.hpp"
-#include "game_errors.hpp"
 #include "gettext.hpp"
 #include "loadscreen.hpp"
 #include "log.hpp"
 #include "sound.hpp"
-#include "team.hpp"
+#include "unit_id.hpp"
 #include "terrain_filter.hpp"
-#include "variable.hpp"
 
-#include <cassert>
 
 #define LOG_NG LOG_STREAM(info, engine)
 
-play_controller::play_controller(const config& level, const game_data& gameinfo,
-	game_state& state_of_game, int ticks, int num_turns, const config& game_config,
-	CVideo& video, bool skip_replay, bool is_replay) :
-	verify_manager_(units_), team_manager_(teams_), labels_manager_(),
-	help_manager_(&game_config, &gameinfo, &map_), mouse_handler_(gui_, teams_,
-		units_, map_, status_, gameinfo, undo_stack_, redo_stack_),
-	menu_handler_(gui_, units_, teams_, level, gameinfo, map_, game_config,
-		status_, state_of_game, undo_stack_, redo_stack_),
-	generator_setter(&recorder), statistics_context_(level["name"]),
-	gameinfo_(gameinfo), level_(level), game_config_(game_config),
-	gamestate_(state_of_game), status_(level, num_turns, &state_of_game),
-	map_(game_config, level["map_data"]), ticks_(ticks),
+play_controller::play_controller(const config& level, game_state& state_of_game,
+		int ticks, int num_turns, const config& game_config, CVideo& video,
+		bool skip_replay) :
+	controller_base(ticks, game_config, video),
+	verify_manager_(units_),
+	team_manager_(teams_),
+	prefs_disp_manager_(),
+	tooltips_manager_(),
+	events_manager_(),
+	halo_manager_(),
+	labels_manager_(),
+	help_manager_(&game_config, &map_),
+	mouse_handler_(NULL, teams_, units_, map_, status_, undo_stack_, redo_stack_),
+	menu_handler_(NULL, units_, teams_, level, map_, game_config, status_, state_of_game, undo_stack_, redo_stack_),
+	soundsources_manager_(),
+	gui_(),
+	statistics_context_(level["name"]),
+	level_(level),
+	teams_(),
+	gamestate_(state_of_game),
+	status_(level, num_turns, &state_of_game),
+	map_(game_config, level["map_data"]),
+	units_(),
+	undo_stack_(),
+	redo_stack_(),
 	xp_mod_(atoi(level["experience_modifier"].c_str()) > 0 ? atoi(level["experience_modifier"].c_str()) : 100),
 	loading_game_(level["playing_team"].empty() == false),
-	first_human_team_(-1), player_number_(1),
-	first_player_ (lexical_cast_default<unsigned int,std::string>(level_["playing_team"], 0) + 1),
-	start_turn_(status_.turn()), is_host_(true), skip_replay_(skip_replay),
-	browse_(false), linger_(false), scrolling_(false)
+	first_human_team_(-1),
+	player_number_(1),
+	first_player_(lexical_cast_default<unsigned int, std::string>(level_["playing_team"], 0) + 1),
+	start_turn_(status_.turn()),
+	is_host_(true),
+	skip_replay_(skip_replay),
+	linger_(false),
+	first_turn_(true),
+	savenames_(),
+	wml_commands_(),
+	victory_music_(),
+	defeat_music_()
 {
+	// Setup victory and defeat music
+	set_victory_music_list(level_["victory_music"]);
+	set_defeat_music_list(level_["defeat_music"]);
+
 	status_.teams = &teams_;
 	game_config::add_color_info(level);
-
-	init(video, is_replay);
+	hotkey::deactivate_all_scopes();
+	hotkey::set_scope_active(hotkey::SCOPE_GENERAL);
+	hotkey::set_scope_active(hotkey::SCOPE_GAME);
+	init(video);
 }
 
 play_controller::~play_controller(){
-	delete halo_manager_;
-	delete prefs_disp_manager_;
-	delete tooltips_manager_;
-	delete events_manager_;
-	delete soundsources_manager_;
-	delete gui_;
 }
 
-void play_controller::init(CVideo& video, bool is_replay){
-	loadscreen::global_loadscreen_manager loadscreen_manager(video);
+void play_controller::init(CVideo& video){
+	util::scoped_resource<loadscreen::global_loadscreen_manager*, util::delete_item> scoped_loadscreen_manager;
+	loadscreen::global_loadscreen_manager* loadscreen_manager = loadscreen::global_loadscreen_manager::get();
+	if (!loadscreen_manager)
+	{
+		scoped_loadscreen_manager.assign(new loadscreen::global_loadscreen_manager(video));
+		loadscreen_manager = scoped_loadscreen_manager.get();
+	}
 
-	// If the recorder has no event, adds an "game start" event 
+	loadscreen::global_loadscreen->set_progress(50, _("Loading level"));
+	// If the recorder has no event, adds an "game start" event
 	// to the recorder, whose only goal is to initialize the RNG
 	if(recorder.empty()) {
 		recorder.add_start();
@@ -80,17 +106,24 @@ void play_controller::init(CVideo& video, bool is_replay){
 	recorder.set_skip(false);
 
 	const config::child_list& unit_cfg = level_.get_children("side");
-	bool snapshot = level_["snapshot"] == "yes";
+	const bool snapshot = utils::string_bool(level_["snapshot"]);
 
-	if(level_["modify_placing"] == "true") {
+	if(utils::string_bool(level_["modify_placing"])) {
 		LOG_NG << "modifying placing...\n";
 		place_sides_in_preferred_locations(map_,unit_cfg);
 	}
 
-	loadscreen::global_loadscreen->set_progress(70, _("Initializing teams"));
 
-	LOG_NG << "initializing teams..." << unit_cfg.size() << "\n";;
-	LOG_NG << (SDL_GetTicks() - ticks_) << "\n";
+
+	LOG_NG << "initialized teams... "    << (SDL_GetTicks() - ticks_) << "\n";
+	loadscreen::global_loadscreen->set_progress(60, _("Initializing teams"));
+
+
+	// This *needs* to be created before the show_intro and show_map_scene
+	// as that functions use the manager state_of_game
+	// Has to be done before registering any events!
+	events_manager_.reset(new game_events::manager(level_,map_,
+                                                   units_,teams_, gamestate_,status_));
 
 	std::set<std::string> seen_save_ids;
 
@@ -100,27 +133,29 @@ void play_controller::init(CVideo& video, bool is_replay){
 		if (first_human_team_ == -1){
 			first_human_team_ = get_first_human_team(ui, unit_cfg);
 		}
-		get_player_info(**ui, gamestate_, save_id, teams_, level_, gameinfo_, map_, units_, status_, snapshot, is_replay );
+		get_player_info(**ui, gamestate_, save_id, teams_, level_, map_, units_, status_, snapshot);
 	}
 
-	loadscreen::global_loadscreen->set_progress(80, _("Initializing display"));
-
+	LOG_NG << "loading units..." << (SDL_GetTicks() - ticks_) << "\n";
+	loadscreen::global_loadscreen->set_progress(70, _("Loading units"));
 	preferences::encounter_recruitable_units(teams_);
 	preferences::encounter_start_units(units_);
 	preferences::encounter_recallable_units(gamestate_);
 	preferences::encounter_map_terrain(map_);
 
-	LOG_NG << "initialized teams... "    << (SDL_GetTicks() - ticks_) << "\n";
+
+	loadscreen::global_loadscreen->set_progress(80, _("Initializing display"));
 	LOG_NG << "initializing display... " << (SDL_GetTicks() - ticks_) << "\n";
 
 	const config* theme_cfg = get_theme(game_config_, level_["theme"]);
 	if (theme_cfg)
-		gui_ = new game_display(units_,video,map_,status_,teams_,*theme_cfg, game_config_, level_);
+		gui_.reset(new game_display(units_,video,map_,status_,teams_,*theme_cfg, game_config_, level_));
 	else
-		gui_ = new game_display(units_,video,map_,status_,teams_,config(), game_config_, level_);
+		gui_.reset(new game_display(units_,video,map_,status_,teams_,config(), game_config_, level_));
 	loadscreen::global_loadscreen->set_progress(90, _("Initializing display"));
-	mouse_handler_.set_gui(gui_);
-	menu_handler_.set_gui(gui_);
+	mouse_handler_.set_gui(gui_.get());
+	menu_handler_.set_gui(gui_.get());
+	events_manager_->set_gui(*gui_);
 	theme::set_known_themes(&game_config_);
 
 	LOG_NG << "done initializing display... " << (SDL_GetTicks() - ticks_) << "\n";
@@ -143,34 +178,41 @@ void play_controller::init(CVideo& video, bool is_replay){
 		}
 	}
 
+	browse_ = true;
+
 	init_managers();
+	// add era events for MP game
+	const config* era_cfg = level_.child("era");
+	if (era_cfg != NULL) {
+		game_events::add_events(era_cfg->get_children("event"),"era_events");
+	}
+
+
+
 	loadscreen::global_loadscreen->set_progress(100, _("Starting game"));
-	loadscreen::global_loadscreen = NULL;
+	loadscreen_manager->reset();
 }
 
 void play_controller::init_managers(){
 	LOG_NG << "initializing managers... " << (SDL_GetTicks() - ticks_) << "\n";
-	prefs_disp_manager_ = new preferences::display_manager(gui_);
-	tooltips_manager_ = new tooltips::manager(gui_->video());
-	soundsources_manager_ = new soundsource::manager(*gui_);
+	prefs_disp_manager_.reset(new preferences::display_manager(gui_.get()));
+	tooltips_manager_.reset(new tooltips::manager(gui_->video()));
+	soundsources_manager_.reset(new soundsource::manager(*gui_));
 
-	// This *needs* to be created before the show_intro and show_map_scene
-	// as that functions use the manager state_of_game
-	events_manager_ = new game_events::manager(level_,*gui_,map_, *soundsources_manager_,
-                                                   units_,teams_, gamestate_,status_,gameinfo_);
+	events_manager_->set_soundsource(*soundsources_manager_);
 
-	halo_manager_ = new halo::manager(*gui_);
+	halo_manager_.reset(new halo::manager(*gui_));
 	LOG_NG << "done initializing managers... " << (SDL_GetTicks() - ticks_) << "\n";
 }
 
-static int placing_score(const config& side, const gamemap& map, const gamemap::location& pos)
+static int placing_score(const config& side, const gamemap& map, const map_location& pos)
 {
 	int positions = 0, liked = 0;
 	const t_translation::t_list terrain = t_translation::read_list(side["terrain_liked"]);
 
 	for(int i = pos.x-8; i != pos.x+8; ++i) {
 		for(int j = pos.y-8; j != pos.y+8; ++j) {
-			const gamemap::location pos(i,j);
+			const map_location pos(i,j);
 			if(map.on_board(pos)) {
 				++positions;
 				if(std::count(terrain.begin(),terrain.end(),map[pos])) {
@@ -184,8 +226,16 @@ static int placing_score(const config& side, const gamemap& map, const gamemap::
 }
 
 struct placing_info {
+
+	placing_info() :
+		side(0),
+		score(0),
+		pos()
+	{
+	}
+
 	int side, score;
-	gamemap::location pos;
+	map_location pos;
 };
 
 static bool operator<(const placing_info& a, const placing_info& b) { return a.score > b.score; }
@@ -199,7 +249,7 @@ void play_controller::place_sides_in_preferred_locations(gamemap& map, const con
 	for(config::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
 		const int side_num = s - sides.begin() + 1;
 		for(int p = 1; p <= num_pos; ++p) {
-			const gamemap::location& pos = map.starting_position(p);
+			const map_location& pos = map.starting_position(p);
 			const int score = placing_score(**s,map,pos);
 			placing_info obj;
 			obj.side = side_num;
@@ -211,7 +261,7 @@ void play_controller::place_sides_in_preferred_locations(gamemap& map, const con
 
 	std::sort(placings.begin(),placings.end());
 	std::set<int> placed;
-	std::set<gamemap::location> positions_taken;
+	std::set<map_location> positions_taken;
 
 	for(std::vector<placing_info>::const_iterator i = placings.begin(); i != placings.end() && placed.size() != sides.size(); ++i) {
 		if(placed.count(i->side) == 0 && positions_taken.count(i->pos) == 0) {
@@ -307,9 +357,6 @@ void play_controller::search(){
 	menu_handler_.search();
 }
 
-const int play_controller::get_ticks(){
-	return ticks_;
-}
 
 void play_controller::fire_prestart(bool execute){
 	// pre-start events must be executed before any GUI operation,
@@ -317,13 +364,17 @@ void play_controller::fire_prestart(bool execute){
 	if (execute){
 		update_locker lock_display(gui_->video());
 		game_events::fire("prestart");
+		// prestart event may modify start turn with WML, reflect any changes.
+		start_turn_ = status_.turn();
 	}
 }
 
 void play_controller::fire_start(bool execute){
 	if(execute) {
 		game_events::fire("start");
-		gamestate_.set_variable("turn_number", "1");
+		// start event may modify start turn with WML, reflect any changes.
+		start_turn_ = status_.turn();
+		gamestate_.set_variable("turn_number", str_cast<size_t>(start_turn_));
 		first_turn_ = true;
 	} else {
 		first_turn_ = false;
@@ -344,23 +395,22 @@ void play_controller::init_side(const unsigned int team_index, bool /*is_replay*
 	team& current_team = teams_[team_index];
 
 	mouse_handler_.set_team(team_index+1);
-	
+
 	// If we are observers we move to watch next team if it is allowed
 	if (team_manager_.is_observer()
 		&& !current_team.get_disallow_observers()) {
 		gui_->set_team(size_t(team_index));
 	}
-	
-	gui_->set_playing_team(size_t(team_index));
+		gui_->set_playing_team(size_t(team_index));
 
 	std::stringstream player_number_str;
 	player_number_str << player_number_;
 	gamestate_.set_variable("side_number",player_number_str.str());
-	gamestate_.last_selected = gamemap::location::null_location;
+	gamestate_.last_selected = map_location::null_location;
 
-	/*  
-		Normally, events must not be actively fired through replays, because 
-		they have been recorded previously and therefore will get executed anyway. 
+	/*
+		Normally, events must not be actively fired through replays, because
+		they have been recorded previously and therefore will get executed anyway.
 		Firing them in the normal code would lead to double execution.
 		However, the following events are different in that they need to be executed _now_
 		(before calculation of income and healing) or we will risk OOS errors if we manipulate
@@ -368,10 +418,10 @@ void play_controller::init_side(const unsigned int team_index, bool /*is_replay*
 	*/
 	bool real_side_change = true;
 	if(first_turn_) {
-		game_events::fire("turn 1");
+		first_turn_ = false;
+		game_events::fire("turn " + str_cast<size_t>(start_turn_));
 		game_events::fire("new turn");
 		game_events::fire("side turn");
-		first_turn_ = false;
 	} else if (team_index != (first_player_ - 1) || status_.turn() > start_turn_) {
 		// Fire side turn event only if real side change occurs,
 		// not counting changes from void to a side
@@ -380,11 +430,11 @@ void play_controller::init_side(const unsigned int team_index, bool /*is_replay*
 		real_side_change = false;
 	}
 
-	// We want to work out if units for this player should get healed, 
-	// and the player should get income now. 
-	// Healing/income happen if it's not the first turn of processing, 
+	// We want to work out if units for this player should get healed,
+	// and the player should get income now.
+	// Healing/income happen if it's not the first turn of processing,
 	// or if we are loading a game, and this is not the player it started with.
-	const bool turn_refresh = status_.turn() > start_turn_ || loading_game_ && team_index != (first_player_ - 1);
+	const bool turn_refresh = status_.turn() > start_turn_ || (loading_game_ && team_index != (first_player_ - 1));
 
 	if(turn_refresh) {
 		for(unit_map::iterator i = units_.begin(); i != units_.end(); ++i) {
@@ -422,7 +472,7 @@ void play_controller::init_side(const unsigned int team_index, bool /*is_replay*
 	}
 
 	if (!recorder.is_skipping() && !skip_replay_){
-		gui_->scroll_to_leader(units_, player_number_);
+		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
 }
 
@@ -432,7 +482,7 @@ void play_controller::finish_side_turn(){
 			uit->second.end_turn();
 	}
 
-	// This implements "delayed map sharing." 
+	// This implements "delayed map sharing."
 	// It is meant as an alternative to shared vision.
 	if(current_team().copy_ally_shroud()) {
 		gui_->recalculate_minimap();
@@ -440,6 +490,7 @@ void play_controller::finish_side_turn(){
 	}
 
 	mouse_handler_.deselect_hex();
+	n_unit::id_manager::instance().reset_fake();
 	game_events::pump();
 }
 
@@ -477,17 +528,17 @@ bool play_controller::execute_command(hotkey::HOTKEY_COMMAND command, int index)
 		unsigned i = static_cast<unsigned>(index);
 		if(i < savenames_.size() && !savenames_[i].empty()) {
 			// Load the game by throwing load_game_exception
-			throw game::load_game_exception(savenames_[i],false);
+			throw game::load_game_exception(savenames_[i],false,false);
 
 		} else if (i < wml_commands_.size() && wml_commands_[i] != NULL) {
 			if(gamestate_.last_selected.valid() && wml_commands_[i]->needs_select) {
 				recorder.add_event("select", gamestate_.last_selected);
 			}
-			gamemap::location const& menu_hex = mouse_handler_.get_last_hex();
+			map_location const& menu_hex = mouse_handler_.get_last_hex();
 			recorder.add_event(wml_commands_[i]->name, menu_hex);
 			if(game_events::fire(wml_commands_[i]->name, menu_hex)) {
 				// The event has mutated the gamestate
-				apply_shroud_changes(undo_stack_, gui_, map_,
+				apply_shroud_changes(undo_stack_, gui_.get(), map_,
 					units_, teams_, (player_number_ - 1));
 				undo_stack_.clear();
 			}
@@ -497,7 +548,6 @@ bool play_controller::execute_command(hotkey::HOTKEY_COMMAND command, int index)
 	return command_executor::execute_command(command, index);
 }
 
-//! Check if a command can be executed.
 bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int index) const
 {
 	if(index >= 0) {
@@ -518,6 +568,7 @@ bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int in
 	case hotkey::HOTKEY_ZOOM_DEFAULT:
 	case hotkey::HOTKEY_FULLSCREEN:
 	case hotkey::HOTKEY_SCREENSHOT:
+	case hotkey::HOTKEY_MAP_SCREENSHOT:
 	case hotkey::HOTKEY_ACCELERATED:
 	case hotkey::HOTKEY_SAVE_MAP:
 	case hotkey::HOTKEY_TOGGLE_GRID:
@@ -532,6 +583,8 @@ bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int in
 	case hotkey::HOTKEY_SEARCH:
 	case hotkey::HOTKEY_HELP:
 	case hotkey::HOTKEY_USER_CMD:
+	case hotkey::HOTKEY_CUSTOM_CMD:
+	case hotkey::HOTKEY_AI_FORMULA:
 	case hotkey::HOTKEY_CLEAR_MSG:
 #ifdef USRCMD2
 //%%
@@ -564,7 +617,7 @@ bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int in
 		return menu_handler_.current_unit(mouse_handler_) != units_.end();
 
 	case hotkey::HOTKEY_RENAME_UNIT:
-		return !events::commands_disabled && !browse_ &&
+		return !events::commands_disabled &&
 			menu_handler_.current_unit(mouse_handler_) != units_.end() &&
 			!(menu_handler_.current_unit(mouse_handler_)->second.unrenamable()) &&
 			menu_handler_.current_unit(mouse_handler_)->second.side() == gui_->viewing_team()+1 &&
@@ -581,36 +634,46 @@ void play_controller::enter_textbox()
 		return;
 	}
 
+	const std::string str = menu_handler_.get_textbox().box()->text();
+	const unsigned int team_num = player_number_;
+	events::mouse_handler& mousehandler = mouse_handler_;
+
 	switch(menu_handler_.get_textbox().mode()) {
 	case gui::TEXTBOX_SEARCH:
-		menu_handler_.do_search(menu_handler_.get_textbox().box()->text());
+		menu_handler_.do_search(str);
+		menu_handler_.get_textbox().close(*gui_);
 		break;
 	case gui::TEXTBOX_MESSAGE:
 		menu_handler_.do_speak();
+		menu_handler_.get_textbox().close(*gui_);  //need to close that one after executing do_speak() !
 		break;
 	case gui::TEXTBOX_COMMAND:
-		menu_handler_.do_command(menu_handler_.get_textbox().box()->text(), player_number_, mouse_handler_);
+		menu_handler_.get_textbox().close(*gui_);
+		menu_handler_.do_command(str, team_num, mousehandler);
+		break;
+	case gui::TEXTBOX_AI:
+		menu_handler_.get_textbox().close(*gui_);
+		menu_handler_.do_ai_formula(str, team_num, mousehandler);
 		break;
 	default:
+		menu_handler_.get_textbox().close(*gui_);
 		LOG_STREAM(err, display) << "unknown textbox mode\n";
 	}
 
-	menu_handler_.get_textbox().close(*gui_);
 }
 
-team& play_controller::current_team() 
-{ 
-	assert(player_number_ > 0 && player_number_ <= teams_.size()); 
-	return teams_[player_number_-1]; 
+team& play_controller::current_team()
+{
+	assert(player_number_ > 0 && player_number_ <= teams_.size());
+	return teams_[player_number_-1];
 }
-	
-const team& play_controller::current_team() const 
-{ 
-	assert(player_number_ > 0 && player_number_ <= teams_.size()); 
-	return teams_[player_number_-1]; 
+
+const team& play_controller::current_team() const
+{
+	assert(player_number_ > 0 && player_number_ <= teams_.size());
+	return teams_[player_number_-1];
 }
 
-//! Find a human team (ie one we own) starting backwards from 'team_num'.
 int play_controller::find_human_team_before(const size_t team_num) const
 {
 	if (team_num > teams_.size())
@@ -634,169 +697,75 @@ int play_controller::find_human_team_before(const size_t team_num) const
 	return human_side+1;
 }
 
-//! Process mouse- and keypress-events from SDL.
-void play_controller::handle_event(const SDL_Event& event)
-{
-	if(gui::in_dialog()) {
-		return;
-	}
-
-	switch(event.type) {
-	case SDL_KEYDOWN:
-//%%
-		// Detect key press events, unless there is a textbox present on-screen,
-		// in which case the key press events should go only to it.
-		if(menu_handler_.get_textbox().active() == false) {
-			hotkey::key_event(*gui_,event.key,this);
-		} else {
-			if(event.key.keysym.sym == SDLK_ESCAPE) {
-				menu_handler_.get_textbox().close(*gui_);
-			} else if(event.key.keysym.sym == SDLK_TAB) {
-				menu_handler_.get_textbox().tab(teams_, units_, *gui_);
-			} else if(event.key.keysym.sym == SDLK_RETURN || event.key.keysym.sym == SDLK_KP_ENTER) {
-				enter_textbox();
-			}
-			break;
-		}
-
-		// intentionally fall-through
-	case SDL_KEYUP:
-
-		// If the user has pressed 1 through 9, we want to show 
-		// how far the unit can move in that many turns
-		if(event.key.keysym.sym >= '1' && event.key.keysym.sym <= '7') {
-			const int new_path_turns = (event.type == SDL_KEYDOWN) ?
-			                           event.key.keysym.sym - '1' : 0;
-
-			if(new_path_turns != mouse_handler_.get_path_turns()) {
-				mouse_handler_.set_path_turns(new_path_turns);
-
-				const unit_map::iterator u = mouse_handler_.selected_unit();
-
-				if(u != units_.end()) {
-					const bool teleport = u->second.get_ability_bool("teleport",u->first);
-					mouse_handler_.set_current_paths(paths(map_,units_,u->first,
-					                       teams_,false,teleport, teams_[gui_->viewing_team()],
-					                       mouse_handler_.get_path_turns()));
-					gui_->highlight_reach(mouse_handler_.get_current_paths());
-				}
-			}
-		}
-//%%
-//		std::cerr << "@play_controller.cpp::handle_event : Key pressed: " << event.key.keysym.sym 
-//				<< std::endl;
-
-		break;
-	case SDL_MOUSEMOTION:
-		// Ignore old mouse motion events in the event queue
-		SDL_Event new_event;
-
-		if(SDL_PeepEvents(&new_event,1,SDL_GETEVENT,
-					SDL_EVENTMASK(SDL_MOUSEMOTION)) > 0) {
-			while(SDL_PeepEvents(&new_event,1,SDL_GETEVENT,
-						SDL_EVENTMASK(SDL_MOUSEMOTION)) > 0);
-			mouse_handler_.mouse_motion(new_event.motion, browse_);
-		} else {
-			mouse_handler_.mouse_motion(event.motion, browse_);
-		}
-		break;
-	case SDL_MOUSEBUTTONDOWN:
-	case SDL_MOUSEBUTTONUP:
-		mouse_handler_.mouse_press(event.button, browse_);
-		if (mouse_handler_.get_undo()){
-			mouse_handler_.set_undo(false);
-			menu_handler_.undo(player_number_);
-		}
-		if (mouse_handler_.get_show_menu()){
-			show_menu(gui_->get_theme().context_menu()->items(),event.button.x,event.button.y,true);
-		}
-		break;
-	default:
-		break;
-	}
+void play_controller::slice_before_scroll() {
+	soundsources_manager_->update();
 }
 
-void play_controller::play_slice()
-{
-	CKey key;
-
-	events::pump();
-	events::raise_process_event();
-
-	events::raise_draw_event();
-	soundsources_manager_->update();
-
-	const theme::menu* const m = gui_->menu_pressed();
-	if(m != NULL) {
-		const SDL_Rect& menu_loc = m->location(gui_->screen_area());
-		show_menu(m->items(),menu_loc.x+1,menu_loc.y + menu_loc.h + 1,false);
-		return;
-	}
-
-	int mousex, mousey;
-	bool middle_pressed = (SDL_GetMouseState(&mousex,&mousey)& SDL_BUTTON(2)) != 0;
-	tooltips::process(mousex, mousey);
-
-	const int scroll_threshold = (preferences::mouse_scroll_enabled()) ? 5 : 0;
-	bool was_scrolling = scrolling_;
-	scrolling_ = false;
-
-	if((key[SDLK_UP] && !menu_handler_.get_textbox().active()) || mousey < scroll_threshold) {
-		gui_->scroll(0,-preferences::scroll_speed());
-		scrolling_ = true;
-	}
-
-	if((key[SDLK_DOWN] && !menu_handler_.get_textbox().active()) || mousey > gui_->h()-scroll_threshold) {
-		gui_->scroll(0,preferences::scroll_speed());
-		scrolling_ = true;
-	}
-
-	if((key[SDLK_LEFT] && !menu_handler_.get_textbox().active()) || mousex < scroll_threshold) {
-		gui_->scroll(-preferences::scroll_speed(),0);
-		scrolling_ = true;
-	}
-
-	if((key[SDLK_RIGHT] && !menu_handler_.get_textbox().active()) || mousex > gui_->w()-scroll_threshold) {
-		gui_->scroll(preferences::scroll_speed(),0);
-		scrolling_ = true;
-	}
-
-	if (middle_pressed) {
-		const SDL_Rect& rect = gui_->map_outside_area();
-		if (point_in_rect(mousex, mousey,rect)) {
-			// relative distance from the center to the border
-			// NOTE: the view is a rectangle, so can be more sensible in one direction
-			// but seems intuitive to use and it's useful since you must
-			// more often scroll in the direction where the view is shorter
-			const double xdisp = ((1.0*mousex / rect.w) - 0.5);
-			const double ydisp = ((1.0*mousey / rect.h) - 0.5);
-
-			// 4.0 give twice the normal speed when mouse is at border (xdisp=0.5)
-			const double scroll_speed = 4.0 * preferences::scroll_speed();
-
-			const int xspeed = round_double(xdisp * scroll_speed);
-			const int yspeed = round_double(ydisp * scroll_speed);
-
-			gui_->scroll(xspeed,yspeed);
-			scrolling_ = true;
-		}
-	}
-
-	gui_->draw();
-	if (!scrolling_) {
-		if (was_scrolling) {
-			// scrolling ended, update the cursor and the brightened hex
-			mouse_handler_.mouse_update(browse_);
-		}
-		gui_->delay(20);
-	}
-
+void play_controller::slice_end() {
 	if(!browse_ && current_team().objectives_changed()) {
 		dialogs::show_objectives(*gui_, level_, current_team().objectives());
 		current_team().reset_objectives_changed();
 	}
 }
 
+events::mouse_handler& play_controller::get_mouse_handler_base() {
+	return mouse_handler_;
+}
+
+game_display& play_controller::get_display() {
+	return *gui_;
+}
+
+bool play_controller::have_keyboard_focus()
+{
+	return !menu_handler_.get_textbox().active();
+}
+
+void play_controller::process_keydown_event(const SDL_Event& event) {
+	if(event.key.keysym.sym == SDLK_ESCAPE) {
+		menu_handler_.get_textbox().close(*gui_);
+	} else if(event.key.keysym.sym == SDLK_TAB) {
+		menu_handler_.get_textbox().tab(teams_, units_, *gui_);
+	} else if(event.key.keysym.sym == SDLK_RETURN || event.key.keysym.sym == SDLK_KP_ENTER) {
+		enter_textbox();
+	}
+}
+
+void play_controller::process_keyup_event(const SDL_Event& event) {
+	// If the user has pressed 1 through 9, we want to show
+	// how far the unit can move in that many turns
+	if(event.key.keysym.sym >= '1' && event.key.keysym.sym <= '7') {
+		const int new_path_turns = (event.type == SDL_KEYDOWN) ?
+		                           event.key.keysym.sym - '1' : 0;
+
+		if(new_path_turns != mouse_handler_.get_path_turns()) {
+			mouse_handler_.set_path_turns(new_path_turns);
+
+			const unit_map::iterator u = mouse_handler_.selected_unit();
+
+			if(u != units_.end()) {
+				const bool teleport = u->second.get_ability_bool("teleport",u->first);
+
+				// if it's not the unit's turn, we reset its moves
+				unit_movement_resetter move_reset(u->second, u->second.side() != player_number_);
+
+				mouse_handler_.set_current_paths(paths(map_,units_,u->first,
+				                       teams_,false,teleport, teams_[gui_->viewing_team()],
+				                       mouse_handler_.get_path_turns()));
+				gui_->highlight_reach(mouse_handler_.get_current_paths());
+			}
+		}
+	}
+}
+
+void play_controller::post_mouse_press(const SDL_Event& /*event*/)
+{
+	if (mouse_handler_.get_undo()){
+		mouse_handler_.set_undo(false);
+		menu_handler_.undo(player_number_);
+	}
+}
+
 static void trim_items(std::vector<std::string>& newitems) {
 	if (newitems.size() > 5) {
 		std::vector<std::string> subitems;
@@ -825,11 +794,7 @@ void play_controller::expand_autosaves(std::vector<std::string>& items)
 					} else {
 						newsaves.push_back(name);
 					}
-					if (turn == 1) {
-						newitems.push_back(_("Back to start"));
-					} else {
-						newitems.push_back(_("Back to turn ") + lexical_cast<std::string>(turn));
-					}
+					newitems.push_back(_("Back to turn ") + lexical_cast<std::string>(turn));
 				}
 			}
 
@@ -858,7 +823,7 @@ void play_controller::expand_wml_commands(std::vector<std::string>& items)
 			std::vector<std::string> newitems;
 
 			char buf[50];
-			const gamemap::location& hex = mouse_handler_.get_last_hex();
+			const map_location& hex = mouse_handler_.get_last_hex();
 			snprintf(buf,sizeof(buf),"%d",hex.x+1);
 			gamestate_.set_variable("x1", buf);
 			snprintf(buf,sizeof(buf),"%d",hex.y+1);
@@ -881,7 +846,7 @@ void play_controller::expand_wml_commands(std::vector<std::string>& items)
 					std::string newitem = itor->second->description;
 
 					// Prevent accidental hotkey binding by appending a space
-					push_back<std::string, char>(newitem, ' ');
+					newitem.push_back(' ');
 					newitems.push_back(newitem);
 				}
 			}
@@ -930,8 +895,6 @@ void play_controller::show_menu(const std::vector<std::string>& items_arg, int x
 	command_executor::show_menu(items, xloc, yloc, context_menu, *gui_);
 }
 
-//! Determines whether the command should be in the context menu or not.
-//! Independant of whether or not we can actually execute the command.
 bool play_controller::in_context_menu(hotkey::HOTKEY_COMMAND command) const
 {
 	switch(command) {
@@ -964,7 +927,7 @@ std::string play_controller::get_action_image(hotkey::HOTKEY_COMMAND command, in
 	return command_executor::get_action_image(command, index);
 }
 
-hotkey::ACTION_STATE play_controller::get_action_state(hotkey::HOTKEY_COMMAND command) const
+hotkey::ACTION_STATE play_controller::get_action_state(hotkey::HOTKEY_COMMAND command, int /*index*/) const
 {
 	switch(command) {
 	case hotkey::HOTKEY_DELAY_SHROUD:
@@ -974,3 +937,42 @@ hotkey::ACTION_STATE play_controller::get_action_state(hotkey::HOTKEY_COMMAND co
 	}
 }
 
+namespace {
+	static const std::string empty_str = "";
+}
+
+const std::string& play_controller::select_victory_music() const
+{
+	if(victory_music_.empty())
+		return empty_str;
+
+	const size_t p = gamestate_.rng().get_random() % victory_music_.size();
+	assert(p < victory_music_.size());
+	return victory_music_[p];
+}
+
+const std::string& play_controller::select_defeat_music() const
+{
+	if(defeat_music_.empty())
+		return empty_str;
+
+	const size_t p = gamestate_.rng().get_random() % defeat_music_.size();
+	assert(p < defeat_music_.size());
+	return defeat_music_[p];
+}
+
+
+void play_controller::set_victory_music_list(const std::string& list)
+{
+	victory_music_ = utils::split(list);
+	if(victory_music_.empty())
+		victory_music_ = utils::split(game_config::default_victory_music);
+}
+
+void play_controller::set_defeat_music_list(const std::string& list)
+{
+	defeat_music_  = utils::split(list);
+	if(defeat_music_.empty())
+		defeat_music_ = utils::split(game_config::default_defeat_music);
+}
+
