@@ -1,6 +1,7 @@
-/* $Id: playsingle_controller.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: playsingle_controller.cpp 33833 2009-03-18 15:52:13Z shadowmaster $ */
+
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -13,42 +14,58 @@
    See the COPYING file for more details.
 */
 
-//! @file playsingle_controller.cpp
-//! Logic for single-player game.
+/**
+ *  @file playsingle_controller.cpp
+ *  Logic for single-player game.
+ */
 
 #include "playsingle_controller.hpp"
 
-#include "construct_dialog.hpp"
-#include "ai_interface.hpp"
-#include "game_errors.hpp"
+#include "ai.hpp"
+#include "game_end_exceptions.hpp"
 #include "gettext.hpp"
 #include "intro.hpp"
 #include "log.hpp"
+#include "map_label.hpp"
 #include "marked-up_text.hpp"
-#include "playturn.hpp"
 #include "sound.hpp"
 #include "upload_log.hpp"
+#include "formula_string_utils.hpp"
 
 #define ERR_NG LOG_STREAM(err, engine)
 #define LOG_NG LOG_STREAM(info, engine)
 
-playsingle_controller::playsingle_controller(const config& level, const game_data& gameinfo, game_state& state_of_game,
-											 const int ticks, const int num_turns, const config& game_config, CVideo& video,
-											 bool skip_replay)
-	: play_controller(level, gameinfo, state_of_game, ticks, num_turns, game_config, video, skip_replay, false),
-	cursor_setter(cursor::NORMAL), replay_sender_(recorder) , turn_over_(false)
+playsingle_controller::playsingle_controller(const config& level,
+		game_state& state_of_game, const int ticks, const int num_turns,
+		const config& game_config, CVideo& video, bool skip_replay) :
+	play_controller(level, state_of_game, ticks, num_turns, game_config, video, skip_replay),
+	cursor_setter(cursor::NORMAL),
+	data_backlog_(),
+	textbox_info_(),
+	replay_sender_(recorder),
+	end_turn_(false),
+	player_type_changed_(false),
+	replaying_(false),
+	turn_over_(false),
+	skip_next_turn_(false),
+	victory_music_(),
+	defeat_music_()
 {
-	end_turn_ = false;
-	replaying_ = false;
-
 	// game may need to start in linger mode
 	if (state_of_game.completion == "victory" || state_of_game.completion == "defeat")
 	{
-		std::cerr << "Setting linger mode.\n";
+		LOG_NG << "Setting linger mode.\n";
 		browse_ = linger_ = true;
 	}
 }
 
+
+playsingle_controller::~playsingle_controller()
+{
+	ai_manager::reap_ais() ;
+}
+
+
 void playsingle_controller::init_gui(){
 	LOG_NG << "Initializing GUI... " << (SDL_GetTicks() - ticks_) << "\n";
 	play_controller::init_gui();
@@ -93,7 +110,9 @@ void playsingle_controller::end_turn(){
 	if (linger_)
 		end_turn_ = true;
 	else if (!browse_){
+		browse_ = true;
 		end_turn_ = menu_handler_.end_turn(player_number_);
+		browse_ = end_turn_;
 	}
 }
 
@@ -131,6 +150,14 @@ void playsingle_controller::user_command(){
 	menu_handler_.user_command();
 }
 
+void playsingle_controller::custom_command(){
+	menu_handler_.custom_command(mouse_handler_, player_number_);
+}
+
+void playsingle_controller::ai_formula(){
+	menu_handler_.ai_formula();
+}
+
 void playsingle_controller::clear_messages(){
 	menu_handler_.clear_messages();
 }
@@ -149,23 +176,27 @@ void playsingle_controller::report_victory(
 		    std::stringstream& report,
 		    end_level_exception& end_level,
 		    int player_gold,
-		    int remaining_gold, int finishing_bonus_per_turn, 
+		    int remaining_gold, int finishing_bonus_per_turn,
 		    int turns_left, int finishing_bonus)
 {
 	report << _("Remaining gold: ")
 		   << remaining_gold << "\n";
 	if(end_level.gold_bonus) {
-		report << _("Early finish bonus: ")
-			   << finishing_bonus_per_turn
-			   << " " << _("per turn") << "\n"
-			   << font::BOLD_TEXT << _("Turns finished early: ")
-			   << turns_left << "\n"
-			   << _("Bonus: ")
-			   << finishing_bonus << "\n"
-			   << _("Gold: ")
-			   << (remaining_gold + finishing_bonus);
+		if (turns_left > -1) {
+			report << _("Early finish bonus: ")
+				   << finishing_bonus_per_turn
+				   << " " << _("per turn") << "\n"
+				   << font::BOLD_TEXT << _("Turns finished early: ")
+				   << turns_left << "\n"
+				   << _("Bonus: ")
+				   << finishing_bonus << "\n";
+	  	}
+		report << _("Gold: ")
+		       << (remaining_gold + finishing_bonus);
+	}
+	if (remaining_gold > 0) {
+		report << '\n' << _("Carry over percentage: ") << end_level.carryover_percentage;
 	}
-	report << '\n' << _("Carry over percentage: ") << end_level.carryover_percentage;
 	if(end_level.carryover_add) {
 		report << '\n' << font::BOLD_TEXT << _("Bonus Gold: ") << player_gold;
 	} else {
@@ -201,7 +232,7 @@ void playsingle_controller::report_victory(
 }
 
 LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& story, upload_log& log,
-												  bool skip_replay)
+												  bool skip_replay, end_level_exception* end_level_result)
 {
 	LOG_NG << "in playsingle_controller::play_scenario()...\n";
 
@@ -215,8 +246,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 
 	if(!skip_replay) {
 		for(std::vector<config*>::const_iterator story_i = story.begin(); story_i != story.end(); ++story_i) {
-
-			show_intro(*gui_,**story_i, level_);
+			show_intro(*gui_,vconfig(*story_i, true), level_);
 		}
 	}
 	gui_->labels().read(level_, game_events::get_state_of_game());
@@ -224,13 +254,23 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 	// Find a list of 'items' (i.e. overlays) on the level, and add them
 	const config::child_list& overlays = level_.get_children("item");
 	for(config::child_list::const_iterator overlay = overlays.begin(); overlay != overlays.end(); ++overlay) {
-		gui_->add_overlay(gamemap::location(**overlay,game_events::get_state_of_game()),(**overlay)["image"], (**overlay)["halo"]);
+		gui_->add_overlay(map_location(**overlay, game_events::get_state_of_game()), (**overlay)["image"], (**overlay)["halo"], (**overlay)["team_name"], utils::string_bool((**overlay)["visible_in_fog"], true));
+	}
+
+	// Read sound sources
+	assert(soundsources_manager_ != NULL);
+	const config::child_list& snd_sources = level_.get_children("sound_source");
+	for(config::child_list::const_iterator i = snd_sources.begin(); i != snd_sources.end(); ++i) {
+		assert(*i != NULL);
+		soundsource::sourcespec spec(**i);
+		soundsources_manager_->add(spec);
 	}
 
 	victory_conditions::set_victory_when_enemies_defeated(
-						level_["victory_when_enemies_defeated"] != "no");
+		utils::string_bool(level_["victory_when_enemies_defeated"], true)
+	);
 	victory_conditions::set_carryover_percentage(
-		lexical_cast_default<int>(level_["carryover_percentage"], 
+		lexical_cast_default<int>(level_["carryover_percentage"],
 		game_config::gold_carryover_percentage));
 	victory_conditions::set_carryover_add(utils::string_bool(
 		level_["carryover_add"], game_config::gold_carryover_add));
@@ -259,7 +299,7 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 		std::vector<team>::iterator t;
 		for(t = teams_.begin(); t != teams_.end(); ++t) {
 			std::string countd_enabled = level_["mp_countdown"].c_str();
-			if ( countd_enabled == "yes" && !loading_game_ ){
+			if (utils::string_bool(countd_enabled) && !loading_game_ ){
 				t->set_countdown_time(1000 * lexical_cast_default<int>(level_["mp_countdown_init_time"],0));
 			}
 		}
@@ -281,7 +321,22 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 		log.quit(status_.turn());
 		throw;
 	} catch(end_level_exception& end_level) {
-		bool obs = team_manager_.is_observer();
+		*end_level_result = end_level;
+		if(!end_level.custom_endlevel_music.empty()) {
+			switch(end_level.result) {
+			case DEFEAT:
+				set_defeat_music_list(end_level.custom_endlevel_music);
+				break;
+			default:
+				set_victory_music_list(end_level.custom_endlevel_music);
+			}
+		}
+
+		if(!team_manager_.get_teams().size())
+		{
+			return VICTORY; // this is probably only a story scenario, i.e. has its endlevel in the prestart event
+		}
+		const bool obs = team_manager_.is_observer();
 		if (game_config::exit_at_end) {
 			exit(0);
 		}
@@ -313,24 +368,41 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 			try {
 				game_events::fire("defeat");
 			} catch(end_level_exception&) {
+				ERR_NG << "[endlevel] used in 'defeat' event handler\n";
 			}
 
-			if (!obs)
+			if (!obs) {
+				const std::string& defeat_music = select_defeat_music();
+				if(defeat_music.empty() != true)
+					sound::play_music_once(defeat_music);
+
 				return DEFEAT;
-			else
-				return QUIT;
-		} else if (end_level.result == VICTORY
-		|| end_level.result == LEVEL_CONTINUE 
-		|| end_level.result == LEVEL_CONTINUE_NO_SAVE) {
-			if(end_level.result == LEVEL_CONTINUE_NO_SAVE) {
-				gamestate_.completion = "running";
 			} else {
-				gamestate_.completion = "victory";
+				return QUIT;
 			}
+		} else if (end_level.result == VICTORY)
+		{
+			gamestate_.completion = (!end_level.linger_mode ?
+			                         "running" : "victory");
 			recorder.set_save_info_completion(gamestate_.completion);
 			try {
 				game_events::fire("victory");
 			} catch(end_level_exception&) {
+				ERR_NG << "[endlevel] used in 'victory' event handler\n";
+			}
+			//
+			// Play victory music once all victory events
+			// are finished, if we aren't observers.
+			//
+			// Some scenario authors may use 'continue'
+			// result for something that is not story-wise
+			// a victory, so let them use [music] tags
+			// instead should they want special music.
+			//
+			if(end_level.result == VICTORY && (!obs) && (end_level.linger_mode)) {
+				const std::string& victory_music = select_victory_music();
+				if(victory_music.empty() != true)
+					sound::play_music_once(victory_music);
 			}
 			if (first_human_team_ != -1)
 				log.victory(status_.turn(), teams_[first_human_team_].gold());
@@ -346,30 +418,19 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 					player->name = i->current_player();
 			}
 
-			// Add all the units that survived the scenario.  
+			// Add all the units that survived the scenario.
+			LOG_NG << "Add units that survived the scenario to the recall list.\n";
 			for(unit_map::iterator un = units_.begin(); un != units_.end(); ++un) {
 				player_info *player=gamestate_.get_player(teams_[un->second.side()-1].save_id());
 
 				if(player) {
+					LOG_NG << "Added unit " << un->second.id() << ", " << un->second.name() << "\n";
 					un->second.new_turn();
-					un->second.new_level();
+					un->second.new_scenario();
 					player->available_units.push_back(un->second);
 				}
 			}
 
-			// 'continue' is like a victory, except it doesn't announce victory,
-			// and the player retains 100% of gold.
-			if(end_level.result == LEVEL_CONTINUE || end_level.result == LEVEL_CONTINUE_NO_SAVE) {
-				for(i=teams_.begin(); i!=teams_.end(); ++i) {
-					player_info *player=gamestate_.get_player(i->save_id());
-					if(player) {
-						player->gold = i->gold();
-					}
-				}
-
-				return end_level.result == LEVEL_CONTINUE_NO_SAVE ? LEVEL_CONTINUE_NO_SAVE : VICTORY;
-			}
-
 			std::stringstream report;
 			std::string title;
 
@@ -385,8 +446,8 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 				const int finishing_bonus_per_turn =
 						 map_.villages().size() * game_config::village_income +
 						 game_config::base_income;
-				const int turns_left = maximum<int>(0,status_.number_of_turns() - status_.turn());
-				const int finishing_bonus = end_level.gold_bonus ?
+				const int turns_left = std::max<int>(0,status_.number_of_turns() - status_.turn());
+				const int finishing_bonus = (end_level.gold_bonus && (turns_left > -1)) ?
 						 (finishing_bonus_per_turn * turns_left) : 0;
 
 				for(i=teams_.begin(); i!=teams_.end(); ++i) {
@@ -394,12 +455,12 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 
 					if (player) {
 						// Store the gold for all players.
-						player->gold = ((i->gold() + finishing_bonus) 
+						player->gold = ((i->gold() + finishing_bonus)
 								* end_level.carryover_percentage) / 100;
 						player->gold_add = end_level.carryover_add;
-						
+
 						// Only show the report for ourselves.
-						if (!i->is_persistent())
+						if (!i->is_local())
 							continue;
 
 						if(gamestate_.players.size()>1) {
@@ -415,7 +476,10 @@ LEVEL_RESULT playsingle_controller::play_scenario(const std::vector<config*>& st
 				}
 			}
 
-			gui::message_dialog(*gui_, title, report.str()).show();
+			if(end_level.carryover_report)
+			{
+				gui::message_dialog(*gui_, title, report.str()).show();
+			}
 
 			return VICTORY;
 		} else if (end_level.result == SKIP_TO_LINGER) {
@@ -452,7 +516,7 @@ void playsingle_controller::play_turn(bool save)
 	events::raise_draw_event();
 
 	LOG_NG << "turn: " << status_.turn() << "\n";
-	
+
 	if(non_interactive())
 		std::cout << "Turn " << status_.turn() << ":" << std::endl;
 
@@ -460,13 +524,26 @@ void playsingle_controller::play_turn(bool save)
 	for(player_number_ = first_player_; player_number_ <= teams_.size(); player_number_++) {
 		// If a side is empty skip over it.
 		if (current_team().is_empty()) continue;
-
-		init_side(player_number_ - 1);
+		try {
+			if (skip_next_turn_) {
+				skip_next_turn_ = false;
+				throw end_turn_exception();
+			}
+			init_side(player_number_ - 1);
+		} catch (end_turn_exception) {
+			if (current_team().is_network() == false) {
+				turn_info turn_data(gamestate_, status_, *gui_, map_,
+						teams_, player_number_, units_, replay_sender_, undo_stack_);
+				recorder.end_turn();
+				turn_data.sync_network();
+			}
+			continue;
+		}
 
 		if (replaying_) {
 			LOG_NG << "doing replay " << player_number_ << "\n";
 			try {
-				replaying_ = ::do_replay(*gui_, map_, gameinfo_, units_, teams_,
+				replaying_ = ::do_replay(*gui_, map_, units_, teams_,
 						player_number_, status_, gamestate_);
 			} catch(replay::error&) {
 				gui::message_dialog(*gui_,"",_("The file you have tried to load is corrupt")).show();
@@ -478,7 +555,7 @@ void playsingle_controller::play_turn(bool save)
 			// If a side is dead end the turn.
 			if ((current_team().is_human() && team_units(units_, player_number_) == 0))
 			{
-				turn_info turn_data(gameinfo_, gamestate_, status_, *gui_, map_,
+				turn_info turn_data(gamestate_, status_, *gui_, map_,
 						teams_, player_number_, units_, replay_sender_, undo_stack_);
 				recorder.end_turn();
 				turn_data.sync_network();
@@ -488,24 +565,34 @@ void playsingle_controller::play_turn(bool save)
 		}
 
 		finish_side_turn();
-		
+
 		if(non_interactive()) {
 			std::cout << " Player " << player_number_ << ": " <<
 				current_team().villages().size() << " Villages" <<
 				std::endl;
 		}
-		
+
 		check_victory(units_, teams_, *gui_);
 	}
 
 	// Time has run out
 	check_time_over();
 
-	finish_turn();
+	try {
+		finish_turn();
+	} catch (end_turn_exception) {
+		// If [end_turn] is encountered, skip the first existing side's turn.
+		skip_next_turn_ = true;
+	}
 }
 
 void playsingle_controller::play_side(const unsigned int team_index, bool save)
 {
+	//check for team-specific items in the scenario
+	gui_->parse_team_overlays();
+
+	//flag used when we fallback from ai and give temporarily control to human
+	bool temporary_human = false;
 	do {
 		// Although this flag is used only in this method,
 		// it has to be a class member since derived classes
@@ -513,9 +600,10 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 		player_type_changed_ = false;
 		end_turn_ = false;
 
-		statistics::reset_turn_stats(player_number_);
 
-		if(current_team().is_human()) {
+		statistics::reset_turn_stats(teams_[team_index - 1].save_id());
+
+		if(current_team().is_human() || temporary_human) {
 			LOG_NG << "is human...\n";
 			try{
 				before_human_turn(save);
@@ -527,13 +615,13 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 					// If new controller is not human,
 					// reset gui to prev human one
 					if (!teams_[team_index-1].is_human()) {
+						browse_ = true;
 						int t = find_human_team_before(team_index);
 						if (t > 0) {
 							gui_->set_team(t-1);
 							gui_->recalculate_minimap();
 							gui_->invalidate_all();
-							gui_->draw();
-							gui_->update_display();
+							gui_->draw(true,true);
 						}
 					}
 				}
@@ -544,7 +632,17 @@ void playsingle_controller::play_side(const unsigned int team_index, bool save)
 
 			LOG_NG << "human finished turn...\n";
 		} else if(current_team().is_ai()) {
-			play_ai_turn();
+			try {
+				play_ai_turn();
+			} catch(fallback_ai_to_human_exception&) {
+				//give control to human for the rest of this turn
+				player_type_changed_ = true;
+				temporary_human = true;
+			} catch(end_turn_exception& end_turn) {
+				if (end_turn.redo == team_index) {
+					player_type_changed_ = true;
+				}
+			}
 		}
 	} while (player_type_changed_);
 	// Keep looping if the type of a team (human/ai/networked)
@@ -560,8 +658,7 @@ void playsingle_controller::before_human_turn(bool save)
 	gui_->set_team(player_number_ - 1);
 	gui_->recalculate_minimap();
 	gui_->invalidate_all();
-	gui_->draw();
-	gui_->update_display();
+	gui_->draw(true,true);
 
 	if (save) {
 		menu_handler_.autosave(gamestate_.label, status_.turn(), gamestate_.starting_pos);
@@ -570,7 +667,9 @@ void playsingle_controller::before_human_turn(bool save)
 	if(preferences::turn_bell()) {
 		sound::play_bell(game_config::sounds::turn_bell);
 	}
+}
 
+void playsingle_controller::show_turn_dialog(){
 	if(preferences::turn_dialog()) {
 		std::string message = _("It is now $name|'s turn");
 		utils::string_map symbols;
@@ -578,19 +677,21 @@ void playsingle_controller::before_human_turn(bool save)
 		message = utils::interpolate_variables_into_string(message, &symbols);
 		gui::message_dialog(*gui_, "", message).show();
 	}
+}
 
+void playsingle_controller::execute_gotos(){
 	// Execute goto-movements - first collect gotos in a list
-	std::vector<gamemap::location> gotos;
+	std::vector<map_location> gotos;
 
 	for(unit_map::iterator ui = units_.begin(); ui != units_.end(); ++ui) {
 		if(ui->second.get_goto() == ui->first)
-			ui->second.set_goto(gamemap::location());
+			ui->second.set_goto(map_location());
 
 		if(ui->second.side() == player_number_ && map_.on_board(ui->second.get_goto()))
 			gotos.push_back(ui->first);
 	}
 
-	for(std::vector<gamemap::location>::const_iterator g = gotos.begin(); g != gotos.end(); ++g) {
+	for(std::vector<map_location>::const_iterator g = gotos.begin(); g != gotos.end(); ++g) {
 		unit_map::const_iterator ui = units_.find(*g);
 		menu_handler_.move_unit_to_loc(ui,ui->second.get_goto(),false, player_number_, mouse_handler_);
 	}
@@ -599,13 +700,30 @@ void playsingle_controller::before_human_turn(bool save)
 	gui_->set_route(NULL);
 }
 
-void playsingle_controller::play_human_turn(){
+void playsingle_controller::play_human_turn() {
+	show_turn_dialog();
+	execute_gotos();
+
 	gui_->enable_menu("endturn", true);
 	while(!end_turn_) {
 		play_slice();
 		gui_->draw();
 	}
 }
+struct set_completion
+{
+	set_completion(game_state& state, const std::string& completion) :
+		state_(state), completion_(completion)
+	{
+	}
+	~set_completion()
+	{
+		state_.completion = completion_;
+	}
+	private:
+	game_state& state_;
+	const std::string completion_;
+};
 
 void playsingle_controller::linger(upload_log& log)
 {
@@ -620,7 +738,7 @@ void playsingle_controller::linger(upload_log& log)
 	// this is actually for after linger mode is over -- we don't
 	// want to stay stuck in linger state when the *next* scenario
 	// is over.
-	gamestate_.completion = "running";
+	set_completion setter(gamestate_,"running");
 
 	// change the end-turn button text to its alternate label
 	gui_->get_theme().refresh_title2(std::string("button-endturn"), std::string("title2"));
@@ -671,6 +789,7 @@ void playsingle_controller::end_turn_record_unlock()
 }
 
 void playsingle_controller::after_human_turn(){
+	browse_ = true;
 	end_turn_record();
 	end_turn_record_unlock();
 	menu_handler_.clear_undo_stack(player_number_);
@@ -692,16 +811,23 @@ void playsingle_controller::play_ai_turn(){
 
 	const cursor::setter cursor_setter(cursor::WAIT);
 
-	turn_info turn_data(gameinfo_,gamestate_,status_,*gui_,
+	turn_info turn_data(gamestate_,status_,*gui_,
 			map_, teams_, player_number_, units_, replay_sender_, undo_stack_);
 
-	ai_interface::info ai_info(*gui_,map_,gameinfo_,units_,teams_,player_number_,status_, turn_data, gamestate_);
-	util::scoped_ptr<ai_interface> ai_obj(create_ai(current_team().ai_algorithm(),ai_info));
+	ai_interface::info ai_info(*gui_,map_,units_,teams_,player_number_,status_, turn_data, gamestate_);
+	std::string ai_algorithm = current_team().ai_algorithm();
+
+	boost::intrusive_ptr<ai_interface> ai_obj ;
+	ai_obj = ai_manager::get_ai( ai_algorithm, ai_info ) ;
+
 	ai_obj->user_interact().attach_handler(this);
 	ai_obj->unit_recruited().attach_handler(this);
 	ai_obj->unit_moved().attach_handler(this);
 	ai_obj->enemy_attacked().attach_handler(this);
-	ai_obj->play_turn();
+	try {
+		ai_obj->play_turn();
+	} catch (end_turn_exception) {
+	}
 	recorder.end_turn();
 	turn_data.sync_network();
 
@@ -750,7 +876,8 @@ bool playsingle_controller::can_execute_command(hotkey::HOTKEY_COMMAND command,
 			return (!browse_ || linger_) && !events::commands_disabled;
 
 		case hotkey::HOTKEY_DELAY_SHROUD:
-			return !linger_ && (current_team().uses_fog() || current_team().uses_shroud());
+			return !linger_ && (current_team().uses_fog() || current_team().uses_shroud())
+			&& !events::commands_disabled;
 		case hotkey::HOTKEY_UPDATE_SHROUD:
 			return !linger_ && !events::commands_disabled && current_team().auto_shroud_updates() == false;
 
