@@ -1,6 +1,6 @@
-/* $Id: playturn.cpp 28806 2008-08-20 20:31:15Z suokko $ */
+/* $Id: playturn.cpp 33092 2009-02-25 01:45:48Z soliton $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,25 +14,19 @@
 
 #include "playturn.hpp"
 
-#include "config.hpp"
 #include "construct_dialog.hpp"
-#include "game_display.hpp"
-#include "game_config.hpp"
+#include "game_end_exceptions.hpp"
 #include "game_preferences.hpp"
-#include "gamestatus.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
-#include "menu_events.hpp"
 #include "replay.hpp"
-#include "sound.hpp"
-#include "team.hpp"
-#include "unit.hpp"
+#include "formula_string_utils.hpp"
 
-turn_info::turn_info(const game_data& gameinfo, game_state& state_of_game,
+turn_info::turn_info(game_state& state_of_game,
                      const gamestatus& status, game_display& gui, gamemap& map,
 		     std::vector<team>& teams, unsigned int team_num, unit_map& units,
 			 replay_network_sender& replay_sender, undo_list& undo_stack)
-  : gameinfo_(gameinfo), state_of_game_(state_of_game), status_(status),
+  : state_of_game_(state_of_game), status_(status),
     gui_(gui), map_(map), teams_(teams), team_num_(team_num),
     units_(units), undo_stack_(undo_stack),
 	replay_sender_(replay_sender), replay_error_("network_replay_error"),
@@ -76,6 +70,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	if (cfg.child("message")) {
 		const config& cmessage = *cfg.child("message");
 		const int side = lexical_cast_default<int>(cmessage["side"],0);
+
 		gui_.add_chat_message(time(NULL), cmessage["sender"], side,
 				cmessage["message"], game_display::MESSAGE_PUBLIC,
 				preferences::message_bell());
@@ -115,13 +110,13 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	for(config::child_list::const_iterator t = turns.begin(); t != turns.end(); ++t) {
 
 		if(turn_end == false) {
-			//! @todo FIXME: Check what commands we execute when it's our turn!
+			/** @todo FIXME: Check what commands we execute when it's our turn! */
 			replay replay_obj(**t);
 			replay_obj.set_skip(skip_replay);
 			replay_obj.start_replay();
 
 			try{
-				turn_end = do_replay(gui_, map_, gameinfo_, units_, teams_,
+				turn_end = do_replay(gui_, map_, units_, teams_,
 						team_num_, status_, state_of_game_, &replay_obj);
 			}
 			catch (replay::error& e){
@@ -135,7 +130,6 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 				replay::last_replay_error = e.message; //FIXME: some better way to pass this?
 				replay_error_.notify_observers();
 			}
-
 			recorder.add_config(**t,replay::MARK_AS_SENT);
 		} else {
 
@@ -155,10 +149,11 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		const std::string& player = (*change)["player"];
 
 		if(index < teams_.size()) {
-			teams_[index].set_current_player(player);
+			if (!player.empty())
+				teams_[index].set_current_player(player);
 			const unit_map::iterator leader = find_leader(units_, side);
 			bool restart = gui_.get_playing_team() == index;
-			if(leader != units_.end())
+			if(!player.empty() && leader != units_.end())
 				leader->second.rename(player);
 
 
@@ -168,8 +163,12 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 					gui_.set_team(index);
 				}
 				teams_[index].make_human();
-			} else if ( (controller == "network") && (!teams_[index].is_network()) ){
+			} else if ( (controller == "human_ai" ) && (!teams_[index].is_human_ai() )) {
+				teams_[index].make_human_ai();
+			} else if ( (controller == "network") && (!teams_[index].is_network_human()) ){
 				teams_[index].make_network();
+			} else if ( (controller == "network_ai") && (!teams_[index].is_network_ai()) ){
+				teams_[index].make_network_ai();
 			} else if ( (controller == "ai") && (!teams_[index].is_ai()) ) {
 				teams_[index].make_ai();
 			}
@@ -177,7 +176,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 			{
 				restart = false;
 			}
-			
+
 			return restart ? PROCESS_RESTART_TURN : PROCESS_CONTINUE;
 		}
 	}
@@ -202,8 +201,7 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		if (controller == "ai"){
 			teams_[side_index].make_ai();
 			teams_[side_index].set_current_player("ai"+side_str);
-			if(have_leader)
-				leader->second.rename("ai"+side_str);
+			if(have_leader) leader->second.rename("ai"+side_str);
 
 
 			return restart?PROCESS_RESTART_TURN:PROCESS_CONTINUE;
@@ -254,19 +252,18 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 		//an AI.
 		switch(action) {
 			case 0:
-				teams_[side_index].make_ai();
+				teams_[side_index].make_human_ai();
 				teams_[side_index].set_current_player("ai"+side_str);
-				if(have_leader)
-					leader->second.rename("ai"+side_str);
+				if(have_leader) leader->second.rename("ai"+side_str);
+				change_controller(side_str, "human_ai");
 
 
 				return restart?PROCESS_RESTART_TURN:PROCESS_CONTINUE;
 
-			//we don't have to test have_leader as action > 0 mean have_leader == true
 			case 1:
 				teams_[side_index].make_human();
 				teams_[side_index].set_current_player("human"+side_str);
-				leader->second.rename("human"+side_str);
+				if(have_leader) leader->second.rename("human"+side_str);
 
 
 				return restart?PROCESS_RESTART_TURN:PROCESS_CONTINUE;
@@ -283,9 +280,10 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 						size_t i = index - observers.size();
 						change_side_controller(side_str, allies[i]->save_id(), false /*not our own side*/);
 					} else {
-						teams_[side_index].make_ai();
+						teams_[side_index].make_human_ai();
 						teams_[side_index].set_current_player("ai"+side_str);
-						leader->second.rename("ai"+side_str);
+						if(have_leader) leader->second.rename("ai"+side_str);
+						change_controller(side_str, "human_ai");
 					}
 					return restart?PROCESS_RESTART_TURN:PROCESS_CONTINUE;
 				}
@@ -314,6 +312,17 @@ turn_info::PROCESS_DATA_RESULT turn_info::process_network_data(const config& cfg
 	return turn_end ? PROCESS_END_TURN : PROCESS_CONTINUE;
 }
 
+void turn_info::change_controller(const std::string& side, const std::string& controller)
+{
+	config cfg;
+	config& change = cfg.add_child("change_controller");
+	change["side"] = side;
+	change["controller"] = controller;
+
+	network::send_data(cfg, 0, true);
+}
+
+
 void turn_info::change_side_controller(const std::string& side, const std::string& player, bool own_side)
 {
 	config cfg;
