@@ -1,6 +1,6 @@
-/* $Id: replay.hpp 23970 2008-02-20 12:31:21Z suokko $ */
+/* $Id: replay.hpp 33371 2009-03-06 22:40:59Z alink $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,15 +12,17 @@
    See the COPYING file for more details.
 */
 
-//! @file replay.hpp
-//! Replay control code.
+/**
+ *  @file replay.hpp
+ *  Replay control code.
+ */
 
 #ifndef REPLAY_H_INCLUDED
 #define REPLAY_H_INCLUDED
 
 #include "config.hpp"
 #include "gamestatus.hpp"
-#include "map.hpp"
+#include "map_location.hpp"
 #include "random.hpp"
 
 class config_writer;
@@ -34,13 +36,14 @@ struct verification_manager
 	~verification_manager();
 };
 
-class replay: public rng
+class replay: public rand_rng::rng
 {
 public:
 	replay();
 	explicit replay(const config& cfg);
 
 	void set_save_info(const game_state& save);
+	void set_save_info(const game_state& save, const config::child_list& players);
 	void set_save_info_completion(const std::string &st);
 
 	void set_skip(bool skip);
@@ -50,33 +53,33 @@ public:
 	               const config& starting_pos, bool include_replay = true);
 
 	void add_start();
-	void add_recruit(int unit_index, const gamemap::location& loc);
-	void add_recall(int unit_index, const gamemap::location& loc);
+	void add_recruit(int unit_index, const map_location& loc);
+	void add_recall(int unit_index, const map_location& loc);
 	void add_disband(int unit_index);
 	void add_countdown_update(int value,int team);
-	void add_movement(const gamemap::location& a, const gamemap::location& b);
-	void add_attack(const gamemap::location& a, const gamemap::location& b,
+	void add_movement(const std::vector<map_location>& steps);
+	void add_attack(const map_location& a, const map_location& b,
 	                int att_weapon, int def_weapon);
 	void choose_option(int index);
 	void text_input(std::string input);
 	void set_random_value(const std::string& choice);
 	void add_label(const terrain_label*);
 	void clear_labels(const std::string&);
-	void add_rename(const std::string& name, const gamemap::location& loc);
+	void add_rename(const std::string& name, const map_location& loc);
 	void end_turn();
 	void add_event(const std::string& name,
-		const gamemap::location& loc=gamemap::location::null_location);
-	void add_unit_checksum(const gamemap::location& loc,config* const cfg);
-	void add_checksum_check(const gamemap::location& loc);
+		const map_location& loc=map_location::null_location);
+	void add_unit_checksum(const map_location& loc,config* const cfg);
+	void add_checksum_check(const map_location& loc);
 	/**
 	 * Adds an advancement to the replay, the following option command
 	 * determines which advancement option has been choosen
 	 */
-	void add_advancement(const gamemap::location& loc);
-	
+	void add_advancement(const map_location& loc);
+
 	void add_chat_message_location();
 	void speak(const config& cfg);
-	std::string build_chat_log(const std::string& team);
+	std::string build_chat_log();
 
 	//get data range will get a range of moves from the replay system.
 	//if data_type is 'ALL_DATA' then it will return all data in this range
@@ -118,11 +121,11 @@ public:
 private:
 	//generic for add_movement and add_attack
 	void add_pos(const std::string& type,
-	             const gamemap::location& a, const gamemap::location& b);
+	             const map_location& a, const map_location& b);
 
 	void add_value(const std::string& type, int value);
 
-	void add_chat_log_entry(const config*, std::stringstream&, const std::string&) const;
+	void add_chat_log_entry(const config* speak, std::stringstream& str) const;
 
 	const config::child_list& commands() const;
 	void remove_command(int);
@@ -146,7 +149,7 @@ private:
 	game_state saveInfo_;
 
 	bool skip_;
-	
+
 	std::vector<int> message_locations;
 };
 
@@ -156,13 +159,13 @@ extern replay recorder;
 
 //replays up to one turn from the recorder object
 //returns true if it got to the end of the turn without data running out
-bool do_replay(game_display& disp, const gamemap& map, const game_data& gameinfo,
+bool do_replay(game_display& disp, const gamemap& map,
 	unit_map& units, std::vector<team>& teams, int team_num,
 	const gamestatus& state, game_state& state_of_game, replay* obj=NULL);
 
-bool do_replay_handle(game_display& disp, const gamemap& map, const game_data& gameinfo,
+bool do_replay_handle(game_display& disp, const gamemap& map,
 					  unit_map& units, std::vector<team>& teams, int team_num,
-	   const gamestatus& state, game_state& state_of_game, 
+	   const gamestatus& state, game_state& state_of_game,
 	const std::string& do_untill);
 
 //an object which can be made to undo a recorded move
