@@ -1,6 +1,6 @@
-/* $Id: replay_controller.cpp 24208 2008-03-02 11:40:03Z mordante $ */
+/* $Id: replay_controller.cpp 33833 2009-03-18 15:52:13Z shadowmaster $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -15,41 +15,26 @@
 
 #include "global.hpp"
 
-#include "ai_interface.hpp"
 #include "config_adapter.hpp"
-#include "cursor.hpp"
-#include "filesystem.hpp"
-#include "game_preferences.hpp"
+#include "game_end_exceptions.hpp"
 #include "gettext.hpp"
-#include "game_events.hpp"
-#include "halo.hpp"
-#include "help.hpp"
-#include "intro.hpp"
 #include "log.hpp"
-#include "map_create.hpp"
-#include "preferences_display.hpp"
-#include "replay.hpp"
 #include "replay_controller.hpp"
-#include "sound.hpp"
-#include "tooltips.hpp"
-#include "video.hpp"
 
-#include <iostream>
-#include <iterator>
 
 #define DBG_NG LOG_STREAM(debug, engine)
 #define DBG_REPLAY LOG_STREAM(debug, replay)
 #define LOG_REPLAY LOG_STREAM(info, replay)
 #define ERR_REPLAY LOG_STREAM(err, replay)
 
-LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& game_config,
+LEVEL_RESULT play_replay_level(const config& game_config,
 		const config* level, CVideo& video, game_state& state_of_game)
 {
 	try{
 		const int ticks = SDL_GetTicks();
 		const int num_turns = atoi((*level)["turns"].c_str());
 		DBG_NG << "creating objects... " << (SDL_GetTicks() - ticks) << "\n";
-		replay_controller replaycontroller(*level, gameinfo, state_of_game, ticks, num_turns, game_config, video);
+		replay_controller replaycontroller(*level, state_of_game, ticks, num_turns, game_config, video);
 		DBG_NG << "created objects... " << (SDL_GetTicks() - replaycontroller.get_ticks()) << "\n";
 		const events::command_disabler disable_commands;
 
@@ -63,20 +48,24 @@ LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& game_con
 	} catch (replay::error&) {
 	}
 
-	return LEVEL_CONTINUE;
+	return VICTORY;
 }
 
-replay_controller::replay_controller(const config& level, const game_data& gameinfo, game_state& state_of_game,
-						   const int ticks, const int num_turns, const config& game_config,
-						   CVideo& video)
-	: play_controller(level, gameinfo, state_of_game, ticks, num_turns, game_config, video, false, true),
-	  gamestate_start_(state_of_game), status_start_(level, num_turns, &state_of_game)
+replay_controller::replay_controller(const config& level,
+		game_state& state_of_game, const int ticks, const int num_turns,
+		const config& game_config, CVideo& video) :
+	play_controller(level, state_of_game, ticks, num_turns, game_config, video, false),
+	teams_start_(),
+	gamestate_start_(state_of_game),
+	status_start_(level, num_turns, &state_of_game),
+	units_start_(),
+	current_turn_(1),
+	delay_(0),
+	is_playing_(false),
+	show_everything_(false),
+	show_team_(1),
+	skip_next_turn_(false)
 {
-	current_turn_ = 1;
-	delay_ = 0;
-	is_playing_ = false;
-	show_everything_ = false;
-	show_team_ = 1;
 	init();
 	gamestate_start_ = gamestate_;
 }
@@ -110,7 +99,8 @@ void replay_controller::init(){
 	init_gui();
 	statistics::fresh_stats();
 	victory_conditions::set_victory_when_enemies_defeated(
-						level_["victory_when_enemies_defeated"] != "no");
+		utils::string_bool(level_["victory_when_enemies_defeated"], true)
+	);
 
 	DBG_REPLAY << "first_time..." << (recorder.is_skipping() ? "skipping" : "no skip") << "\n";
 
@@ -176,10 +166,16 @@ void replay_controller::reset_replay(){
 	gamestate_ = gamestate_start_;
 	teams_ = teams_start_;
 	statistics::fresh_stats();
-	if (events_manager_ != NULL){
-		delete events_manager_;
-		events_manager_ = new game_events::manager(level_,*gui_,map_, *soundsources_manager_,
-								units_,teams_, gamestate_,status_,gameinfo_);
+	if (events_manager_ ){
+		// NOTE: this double reset is required so that the new
+		// instance of game_events::manager isn't created before the
+		// old manager is actually destroyed (triggering an assertion
+		// failure)
+		events_manager_.reset();
+		events_manager_.reset(new game_events::manager(level_,map_,
+								units_,teams_, gamestate_,status_));
+		events_manager_->set_gui(*gui_);
+		events_manager_->set_soundsource(*soundsources_manager_);
 	}
 
 	gui_->new_turn();
@@ -216,7 +212,7 @@ void replay_controller::replay_next_turn(){
 	play_turn();
 
  	if (!skip_replay_){
-		gui_->scroll_to_leader(units_, player_number_);
+		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
 	is_playing_ = false;
 	gui::button* b = gui_->find_button("button-nextturn");
@@ -233,7 +229,7 @@ void replay_controller::replay_next_side(){
 	}
 
 	if (!skip_replay_) {
-		gui_->scroll_to_leader(units_, player_number_);
+		gui_->scroll_to_leader(units_, player_number_,game_display::ONSCREEN,false);
 	}
 
 	is_playing_ = false;
@@ -258,7 +254,6 @@ void replay_controller::replay_show_each(){
 void replay_controller::replay_show_team1(){
 	show_everything_ = false;
 	show_team_ = 1;
-	gui_->set_team(show_team_ - 1, show_everything_);
 	update_teams();
 	update_gui();
 }
@@ -324,38 +319,49 @@ void replay_controller::play_side(const unsigned int /*team_index*/, bool){
 	DBG_REPLAY << "Replay_Controller turn number: " << current_turn_ << "\n";
 	DBG_REPLAY << "Player number: " << player_number_ << "\n";
 
-	// If a side is empty skip over it.
-	if (current_team().is_empty()) return;
-
-	statistics::reset_turn_stats(player_number_);
-
 	try{
-		play_controller::init_side(player_number_ - 1, true);
+		// If a side is empty skip over it.
+		if (!current_team().is_empty()) {
+			statistics::reset_turn_stats(current_team().save_id());
 
-		DBG_REPLAY << "doing replay " << player_number_ << "\n";
-		try {
-			::do_replay(*gui_, map_, gameinfo_, units_, teams_,
-					player_number_, status_, gamestate_);
-		} catch(replay::error&) {
-			if(!continue_replay()) {
-				throw;
+			try {
+				if (skip_next_turn_) {
+					skip_next_turn_ = false;
+					throw end_turn_exception();
+				}
+				play_controller::init_side(player_number_ - 1, true);
+
+				DBG_REPLAY << "doing replay " << player_number_ << "\n";
+				try {
+					::do_replay(*gui_, map_, units_, teams_,
+							player_number_, status_, gamestate_);
+				} catch(replay::error&) {
+					if(!continue_replay()) {
+						throw;
+					}
+				}
+			} catch(end_turn_exception) {
 			}
-		}
 
-		finish_side_turn();
+			finish_side_turn();
 
-		// This is necessary for replays in order to show possible movements.
-		for (unit_map::iterator uit = units_.begin(); uit != units_.end(); ++uit) {
-			if (uit->second.side() != static_cast<size_t>(player_number_)) {
-				uit->second.new_turn();
+			// This is necessary for replays in order to show possible movements.
+			for (unit_map::iterator uit = units_.begin(); uit != units_.end(); ++uit) {
+				if (uit->second.side() != static_cast<size_t>(player_number_)) {
+					uit->second.new_turn();
+				}
 			}
 		}
 
 		player_number_++;
 
-		if (static_cast<size_t>(player_number_) > teams_.size()){
+		if (static_cast<size_t>(player_number_) > teams_.size()) {
 			status_.next_turn();
-			finish_turn();
+			try {
+				finish_turn();
+			} catch (end_turn_exception) {
+				skip_next_turn_ = true;
+			}
 			player_number_ = 1;
 			current_turn_++;
 		}
