@@ -1,6 +1,6 @@
-/* $Id: replay_controller.hpp 24208 2008-03-02 11:40:03Z mordante $ */
+/* $Id: replay_controller.hpp 33524 2009-03-10 14:55:11Z silene $ */
 /*
-   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   Copyright (C) 2006 - 2009 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
@@ -16,18 +16,19 @@
 #ifndef REPLAY_CONTROLLER_H_INCLUDED
 #define REPLAY_CONTROLLER_H_INCLUDED
 
-class video;
-
+#include "game_end_exceptions.hpp"
 #include "play_controller.hpp"
 
 #include <vector>
 
+class video;
+
 class replay_controller : public play_controller
 {
 public:
-	replay_controller(const config& level, const game_data& gameinfo, game_state& state_of_game,
+	replay_controller(const config& level, game_state& state_of_game,
 		const int ticks, const int num_turns, const config& game_config, CVideo& video);
-	~replay_controller();
+	virtual ~replay_controller();
 
 	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command, int index=-1) const;
 
@@ -70,10 +71,11 @@ private:
 
 	bool show_everything_;
 	unsigned int show_team_;
+	bool skip_next_turn_;
 };
 
 
-LEVEL_RESULT play_replay_level(const game_data& gameinfo, const config& terrain_config,
+LEVEL_RESULT play_replay_level(const config& terrain_config,
 		const config* level, CVideo& video,
 		game_state& state_of_game);
 
