@@ -1,6 +1,6 @@
-/* $Id: reports.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: reports.cpp 31858 2009-01-01 10:27:41Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -16,19 +16,18 @@
 
 #include "reports.hpp"
 
-#include <cassert>
-#include <map>
 
 namespace {
-	const std::string report_names[] = { "unit_description", "unit_type",
-		"unit_race", "unit_level", "unit_amla", "unit_traits", 
-		"unit_status", "unit_alignment", "unit_abilities", "unit_hp",
-		"unit_xp", "unit_advancement_options", "unit_moves",
+	const std::string report_names[] = {
+		"unit_name", "unit_type",
+		"unit_race", "unit_level", "unit_side", "unit_amla",
+		"unit_traits", "unit_status", "unit_alignment", "unit_abilities",
+		"unit_hp", "unit_xp", "unit_advancement_options", "unit_moves",
 		"unit_weapons", "unit_image", "unit_profile", "time_of_day",
 		"turn", "gold", "villages", "num_units", "upkeep", "expenses",
 		"income", "terrain", "position", "side_playing", "observers",
 		"report_countdown", "report_clock",
-		"selected_terrain", "edit_left_button_function"
+		"selected_terrain", "edit_left_button_function", "editor2_tool_hint"
 	};
 }
 
