@@ -1,6 +1,6 @@
-/* $Id: sdl_utils.cpp 24291 2008-03-04 00:03:37Z soliton $ */
+/* $Id: sdl_utils.cpp 33378 2009-03-07 02:43:01Z alink $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,23 +12,17 @@
    See the COPYING file for more details.
 */
 
-//! @file sdl_utils.cpp
-//! Support-routines for the SDL-graphics-library.
+/**
+ *  @file sdl_utils.cpp
+ *  Support-routines for the SDL-graphics-library.
+ */
 
 #include "global.hpp"
 
 #include "config.hpp"
-#include "log.hpp"
-#include "sdl_utils.hpp"
-#include "util.hpp"
 #include "video.hpp"
 
-#include <algorithm>
-#include <string.h>
-#include <cassert>
-#include <cmath>
-#include <iostream>
-#include <map>
+#include <cstring>
 
 SDL_Color int_to_color(const Uint32 rgb) {
 	SDL_Color to_return = {
@@ -82,13 +76,42 @@ bool rects_overlap(const SDL_Rect& rect1, const SDL_Rect& rect2)
 SDL_Rect intersect_rects(SDL_Rect const &rect1, SDL_Rect const &rect2)
 {
 	SDL_Rect res;
-	res.x = maximum<int>(rect1.x, rect2.x);
-	res.y = maximum<int>(rect1.y, rect2.y);
-	res.w = maximum<int>(minimum<int>(rect1.x + rect1.w, rect2.x + rect2.w) - res.x, 0);
-	res.h = maximum<int>(minimum<int>(rect1.y + rect1.h, rect2.y + rect2.h) - res.y, 0);
+	res.x = std::max<int>(rect1.x, rect2.x);
+	res.y = std::max<int>(rect1.y, rect2.y);
+	res.w = std::max<int>(std::min<int>(rect1.x + rect1.w, rect2.x + rect2.w) - res.x, 0);
+	res.h = std::max<int>(std::min<int>(rect1.y + rect1.h, rect2.y + rect2.h) - res.y, 0);
 	return res;
 }
 
+SDL_Rect get_rect_union(const SDL_Rect& rect1, const SDL_Rect& rect2)
+{
+	const int left_side = std::max(rect1.x, rect2.x);
+	const int right_side = std::min(rect1.x + rect1.w, rect2.x + rect2.w);
+	if(left_side > right_side) {
+		return empty_rect;
+	}
+
+	const int top_side = std::max(rect1.y, rect2.y);
+	const int bottom_side = std::min(rect1.y + rect1.h, rect2.y + rect2.h);
+	if(top_side > bottom_side) {
+		return empty_rect;
+	}
+
+	SDL_Rect result = {
+			left_side,
+			top_side,
+			right_side - left_side,
+			bottom_side - top_side};
+
+	return result;
+}
+
+SDL_Rect create_rect(const int x, const int y, const int w, const int h)
+{
+	SDL_Rect rect = { x, y, w, h };
+	return rect;
+}
+
 bool operator<(const surface& a, const surface& b)
 {
 	return a.get() < b.get();
@@ -124,6 +147,23 @@ surface make_neutral_surface(surface const &surf)
 	return result;
 }
 
+surface create_neutral_surface(int w, int h)
+{
+	if (w < 0 || h < 0) {
+		std::cerr << "error : neutral surface with negative dimensions\n";
+		return NULL;
+	}
+
+	SDL_PixelFormat format = get_neutral_pixel_format();
+	surface result = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h,
+			format.BitsPerPixel,
+			format.Rmask,
+			format.Gmask,
+			format.Bmask,
+			format.Amask);
+
+	return result;
+}
 
 surface create_optimized_surface(surface const &surf)
 {
@@ -142,8 +182,96 @@ surface create_optimized_surface(surface const &surf)
 	return result;
 }
 
+surface stretch_surface_horizontal(
+		const surface& surf, const unsigned w, const bool optimize)
+{
+	// Since SDL version 1.1.5 0 is transparent, before 255 was transparent.
+	assert(SDL_ALPHA_TRANSPARENT==0);
+
+	if(surf == NULL)
+		return NULL;
+
+	if(static_cast<int>(w) == surf->w) {
+		return surf;
+	}
+	assert(w > 0);
+
+	surface dst(create_neutral_surface(w, surf->h));
+
+	surface src(make_neutral_surface(surf));
+	// Now both surfaces are always in the "neutral" pixel format
+
+	if(src == NULL || dst == NULL) {
+		std::cerr << "Could not create surface to scale onto\n";
+		return NULL;
+	}
+
+	{
+		// Extra scoping used for the surface_lock.
+		surface_lock src_lock(src);
+		surface_lock dst_lock(dst);
+
+		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
+		Uint32* dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+
+		for(unsigned y = 0; y < static_cast<unsigned>(src->h); ++y) {
+			const Uint32 pixel = src_pixels [y * src->w];
+			for(unsigned x = 0; x < w; ++x) {
+
+				*dst_pixels++ = pixel;
+
+			}
+		}
+	}
+
+	return optimize ? create_optimized_surface(dst) : dst;
+}
+
+surface stretch_surface_vertical(
+		const surface& surf, const unsigned h, const bool optimize)
+{
+	// Since SDL version 1.1.5 0 is transparent, before 255 was transparent.
+	assert(SDL_ALPHA_TRANSPARENT==0);
+
+	if(surf == NULL)
+		return NULL;
+
+	if(static_cast<int>(h) == surf->h) {
+		return surf;
+	}
+	assert(h > 0);
+
+	surface dst(create_neutral_surface(surf->w, h));
+
+	surface src(make_neutral_surface(surf));
+	// Now both surfaces are always in the "neutral" pixel format
+
+	if(src == NULL || dst == NULL) {
+		std::cerr << "Could not create surface to scale onto\n";
+		return NULL;
+	}
+
+	{
+		// Extra scoping used for the surface_lock.
+		surface_lock src_lock(src);
+		surface_lock dst_lock(dst);
+
+		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
+		Uint32* dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+
+		for(unsigned y = 0; y < static_cast<unsigned>(h); ++y) {
+		  for(unsigned x = 0; x < static_cast<unsigned>(src->w); ++x) {
+
+				*dst_pixels++ = src_pixels[x];
+			}
+		}
+	}
+
+	return optimize ? create_optimized_surface(dst) : dst;
+}
+
 // NOTE: Don't pass this function 0 scaling arguments.
-surface scale_surface(surface const &surf, int w, int h)
+surface scale_surface(surface const &surf, int w, int h, bool optimize)
 {
 	// Since SDL version 1.1.5 0 is transparent, before 255 was transparent.
 	assert(SDL_ALPHA_TRANSPARENT==0);
@@ -157,8 +285,8 @@ surface scale_surface(surface const &surf, int w, int h)
 	assert(w >= 0);
 	assert(h >= 0);
 
-	surface dst(SDL_CreateRGBSurface(SDL_SWSURFACE,w,h,32,0xFF0000,0xFF00,0xFF,0xFF000000));
-		
+	surface dst(create_neutral_surface(w,h));
+
 	if (w == 0 || h ==0) {
 		std::cerr << "Create an empty image\n";
 		return create_optimized_surface(dst);
@@ -300,10 +428,103 @@ surface scale_surface(surface const &surf, int w, int h)
 		}
 	}
 
-	return create_optimized_surface(dst);
+	return optimize ? create_optimized_surface(dst) : dst;
 }
 
-surface scale_surface_blended(surface const &surf, int w, int h)
+
+// This is a copy-paste of the previous function scale_surface
+// We only removed the alpha channel and big comments
+// and change how we optimize the resulting surface
+surface scale_opaque_surface(surface const &surf, int w, int h, bool optimize_format)
+{
+	if(surf == NULL)
+		return NULL;
+
+	if(w == surf->w && h == surf->h) {
+		return surf;
+	}
+	assert(w >= 0);
+	assert(h >= 0);
+
+	surface dst(create_neutral_surface(w,h));
+
+	if (w == 0 || h ==0) {
+		std::cerr << "Create an empty image\n";
+		return create_optimized_surface(dst);
+	}
+
+	surface src(make_neutral_surface(surf));
+	// Now both surfaces are always in the "neutral" pixel format
+
+	if(src == NULL || dst == NULL) {
+		std::cerr << "Could not create surface to scale onto\n";
+		return NULL;
+	}
+
+	const fixed_t xratio = fxpdiv(surf->w,w);
+	const fixed_t yratio = fxpdiv(surf->h,h);
+
+	{
+		surface_lock src_lock(src);
+		surface_lock dst_lock(dst);
+
+		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
+		Uint32* const dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+
+		fixed_t ysrc = ftofxp(0.0);
+		for(int ydst = 0; ydst != h; ++ydst, ysrc += yratio) {
+			fixed_t xsrc = ftofxp(0.0);
+			for(int xdst = 0; xdst != w; ++xdst, xsrc += xratio) {
+				const int xsrcint = fxptoi(xsrc);
+				const int ysrcint = fxptoi(ysrc);
+
+				Uint32* const src_word = src_pixels + ysrcint*src->w + xsrcint;
+				Uint32* const dst_word = dst_pixels +    ydst*dst->w + xdst;
+				const int dx = (xsrcint + 1 < src->w) ? 1 : 0;
+				const int dy = (ysrcint + 1 < src->h) ? src->w : 0;
+
+				Uint8 r,g,b;
+				Uint32 rr,gg,bb;
+				Uint32 pix[4], bilin[4];
+
+				const fixed_t e = 0x000000FF & xsrc;
+				const fixed_t s = 0x000000FF & ysrc;
+				const fixed_t n = 0xFF - s;
+				const fixed_t w = 0xFF - e;
+
+				pix[0] = *src_word;              // northwest
+				pix[1] = *(src_word + dx);       // northeast
+				pix[2] = *(src_word + dy);       // southwest
+				pix[3] = *(src_word + dx + dy);  // southeast
+
+				bilin[0] = n*w;
+				bilin[1] = n*e;
+				bilin[2] = s*w;
+				bilin[3] = s*e;
+
+				int loc;
+				rr = gg = bb = 0;
+				for (loc=0; loc<4; loc++) {
+				  r = pix[loc] >> 16;
+				  g = pix[loc] >> 8;
+				  b = pix[loc] >> 0;
+				  rr += r * bilin[loc];
+				  gg += g * bilin[loc];
+				  bb += b * bilin[loc];
+				}
+				r = rr >> 16;
+				g = gg >> 16;
+				b = bb >> 16;
+				*dst_word = (255 << 24) + (r << 16) + (g << 8) + b;
+			}
+		}
+	}
+
+	return optimize_format ? display_format_alpha(dst) : dst;
+}
+
+
+surface scale_surface_blended(surface const &surf, int w, int h, bool optimize)
 {
 	if(surf== NULL)
 		return NULL;
@@ -314,7 +535,7 @@ surface scale_surface_blended(surface const &surf, int w, int h)
 	assert(w >= 0);
 	assert(h >= 0);
 
-	surface dst(SDL_CreateRGBSurface(SDL_SWSURFACE,w,h,32,0xFF0000,0xFF00,0xFF,0xFF000000));
+	surface dst(create_neutral_surface(w,h));
 
 	if (w == 0 || h ==0) {
 		std::cerr << "Create an empty image\n";
@@ -351,12 +572,12 @@ surface scale_surface_blended(surface const &surf, int w, int h)
 				// We now have a rectangle, (xsrc,ysrc,xratio,yratio)
 				// which we want to derive the pixel from
 				for(double xloc = xsrc; xloc < xsrc+xratio; xloc += 1.0) {
-					const double xsize = minimum<double>(std::floor(xloc+1.0)-xloc,xsrc+xratio-xloc);
+					const double xsize = std::min<double>(std::floor(xloc+1.0)-xloc,xsrc+xratio-xloc);
 					for(double yloc = ysrc; yloc < ysrc+yratio; yloc += 1.0) {
-						const int xsrcint = maximum<int>(0,minimum<int>(src->w-1,static_cast<int>(xsrc)));
-						const int ysrcint = maximum<int>(0,minimum<int>(src->h-1,static_cast<int>(ysrc)));
+						const int xsrcint = std::max<int>(0,std::min<int>(src->w-1,static_cast<int>(xsrc)));
+						const int ysrcint = std::max<int>(0,std::min<int>(src->h-1,static_cast<int>(ysrc)));
 
-						const double ysize = minimum<double>(std::floor(yloc+1.0)-yloc,ysrc+yratio-yloc);
+						const double ysize = std::min<double>(std::floor(yloc+1.0)-yloc,ysrc+yratio-yloc);
 
 						Uint8 r,g,b,a;
 
@@ -384,12 +605,12 @@ surface scale_surface_blended(surface const &surf, int w, int h)
 		}
 	}
 
-	return create_optimized_surface(dst);
+	return optimize ? create_optimized_surface(dst) : dst;
 }
 
-surface adjust_surface_colour(surface const &surf, int r, int g, int b)
+surface adjust_surface_colour(surface const &surf, int red, int green, int blue, bool optimize)
 {
-	if(r == 0 && g == 0 && b == 0 || surf == NULL)
+	if((red == 0 && green == 0 && blue == 0) || surf == NULL)
 		return create_optimized_surface(surf);
 
 	surface nsurf(make_neutral_surface(surf));
@@ -405,27 +626,29 @@ surface adjust_surface_colour(surface const &surf, int r, int g, int b)
 		Uint32* end = beg + nsurf->w*surf->h;
 
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			alpha = (*beg) >> 24;
-			red   = (*beg) >> 16;
-			green = (*beg) >> 8;
-			blue  = (*beg) >> 0;
+			Uint8 alpha = (*beg) >> 24;
 
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg) >> 0;
 
-			red = maximum<int>(8,minimum<int>(255,int(red)+r));
-			green = maximum<int>(0,minimum<int>(255,int(green)+g));
-			blue  = maximum<int>(0,minimum<int>(255,int(blue)+b));
+				r = std::max<int>(0,std::min<int>(255,int(r)+red));
+				g = std::max<int>(0,std::min<int>(255,int(g)+green));
+				b = std::max<int>(0,std::min<int>(255,int(b)+blue));
 
-			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface greyscale_image(surface const &surf)
+surface greyscale_image(surface const &surf, bool optimize)
 {
 	if(surf == NULL)
 		return NULL;
@@ -442,31 +665,35 @@ surface greyscale_image(surface const &surf)
 		Uint32* end = beg + nsurf->w*surf->h;
 
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			Uint8 alpha = (*beg) >> 24;
 
-			//const Uint8 avg = (red+green+blue)/3;
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
+				//const Uint8 avg = (red+green+blue)/3;
 
-			// Use the correct formula for RGB to grayscale conversion.
-			// Ok, this is no big deal :)
-			// The correct formula being:
-			// gray=0.299red+0.587green+0.114blue
-			const Uint8 avg = static_cast<Uint8>((
-				77 * static_cast<Uint16>(red) +
-				150 * static_cast<Uint16>(green) +
-				29 * static_cast<Uint16>(blue)) / 256);
+				// Use the correct formula for RGB to grayscale conversion.
+				// Ok, this is no big deal :)
+				// The correct formula being:
+				// gray=0.299red+0.587green+0.114blue
+				const Uint8 avg = static_cast<Uint8>((
+					77  * static_cast<Uint16>(r) +
+					150 * static_cast<Uint16>(g) +
+					29  * static_cast<Uint16>(b)  ) / 256);
 
-
-			*beg = SDL_MapRGBA(nsurf->format,avg,avg,avg,alpha);
+				*beg = (alpha << 24) | (avg << 16) | (avg << 8) | avg;
+			}
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface darken_image(surface const &surf)
+surface darken_image(surface const &surf, bool optimize)
 {
 	if(surf == NULL)
 		return NULL;
@@ -483,35 +710,78 @@ surface darken_image(surface const &surf)
 		Uint32* end = beg + nsurf->w*surf->h;
 
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			Uint8 alpha = (*beg) >> 24;
 
-			//const Uint8 avg = (red+green+blue)/3;
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
 
-			// Use the correct formula for RGB to grayscale conversion.
-			// Ok, this is no big deal :)
-			// The correct formula being:
-			// gray=0.299red+0.587green+0.114blue
-			const Uint8 avg = static_cast<Uint8>((
-				77 * static_cast<Uint16>(red) +
-				150 * static_cast<Uint16>(green) +
-				29 * static_cast<Uint16>(blue)) / 256);
-			// then tint 77%, 67%, 72%
-			const Uint8 r= ((avg * 196) >> 8);
-			const Uint8 g= ((avg * 171) >> 8);
-			const Uint8 b= ((avg * 184) >> 8);
+				//const Uint8 avg = (red+green+blue)/3;
 
+				// Use the correct formula for RGB to grayscale conversion.
+				// Ok, this is no big deal :)
+				// The correct formula being:
+				// gray=0.299red+0.587green+0.114blue
+				const Uint8 avg = static_cast<Uint8>((
+					77  * static_cast<Uint16>(r) +
+					150 * static_cast<Uint16>(g) +
+					29  * static_cast<Uint16>(b)  ) / 256);
+				// then tint 77%, 67%, 72%
+				r = ((avg * 196) >> 8);
+				g = ((avg * 171) >> 8);
+				b = ((avg * 184) >> 8);
 
-			*beg = SDL_MapRGBA(nsurf->format,r,g,b,alpha);
+				*beg = (alpha << 24) | (r << 16) | (g << 8) | b;
+			}
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb){
+surface shadow_image(surface const &surf, bool optimize)
+{
+	if(surf == NULL)
+		return NULL;
+
+	// we blur it, and reuse the neutral surface created by the blur function (optimized = false)
+	surface nsurf (blur_alpha_surface(surf, 2, false));
+
+	if(nsurf == NULL) {
+		std::cerr << "failed to blur the shadow surface\n";
+		return NULL;
+	}
+
+	{
+		surface_lock lock(nsurf);
+		Uint32* beg = lock.pixels();
+		Uint32* end = beg + nsurf->w*surf->h;
+
+		while(beg != end) {
+			Uint8 alpha = (*beg) >> 24;
+
+			if(alpha) {
+				// increase alpha and color in black (RGB=0)
+				// with some stupid optimization for handling maximum values
+				if (alpha < 255/4)
+					*beg = (alpha*4) << 24;
+				else
+					*beg = 0xFF000000; // we hit the maximum
+			}
+
+			++beg;
+		}
+	}
+
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
+}
+
+
+surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb, bool optimize){
 	if(map_rgb.size()){
 		if(surf == NULL)
 		return NULL;
@@ -526,31 +796,28 @@ surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb){
 		Uint32* beg = lock.pixels();
 		Uint32* end = beg + nsurf->w*surf->h;
 
+		std::map<Uint32, Uint32>::const_iterator map_rgb_end = map_rgb.end();
+
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			Uint8 alpha = (*beg) >> 24;
 
 			if(alpha){	// don't recolor invisible pixels.
-				Uint32 oldrgb = (red<<16) + (green<<8) + (blue);
-				for(std::map<Uint32, Uint32>::const_iterator i=map_rgb.begin(); i!= map_rgb.end(); i++){
-					if(oldrgb==i->first){
-						Uint32 new_rgb = i->second;
-						Uint8 new_r = (new_rgb & 0x00FF0000)>>16;
-						Uint8 new_g = (new_rgb & 0x0000FF00)>>8;
-						Uint8 new_b = (new_rgb & 0x000000FF);
-					*beg = SDL_MapRGBA(nsurf->format,new_r,new_g,new_b,alpha);
-					break; // no double replacements.
-					}
+				// palette use only RGB channels, so remove alpha
+				Uint32 oldrgb = (*beg) & 0x00FFFFFF;
+				std::map<Uint32, Uint32>::const_iterator i = map_rgb.find(oldrgb);
+				if(i != map_rgb.end()){
+					*beg = (alpha << 24) + i->second;
 				}
 			}
 		++beg;
 		}
-		return create_optimized_surface(nsurf);
+
+		return optimize ? create_optimized_surface(nsurf) : nsurf;
 	}
 	return surf;
 }
 
-surface brighten_image(surface const &surf, fixed_t amount)
+surface brighten_image(surface const &surf, fixed_t amount, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -570,23 +837,26 @@ surface brighten_image(surface const &surf, fixed_t amount)
 
 		if (amount < 0) amount = 0;
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			alpha = (*beg) >> 24;
-			red   = (*beg) >> 16;
-			green = (*beg) >> 8;
-			blue  = (*beg) >> 0;
+			Uint8 alpha = (*beg) >> 24;
 
-			red = minimum<unsigned>(unsigned(fxpmult(red,amount)),255);
-			green = minimum<unsigned>(unsigned(fxpmult(green,amount)),255);
-			blue = minimum<unsigned>(unsigned(fxpmult(blue,amount)),255);
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
 
-			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
+				r = std::min<unsigned>(unsigned(fxpmult(r, amount)),255);
+				g = std::min<unsigned>(unsigned(fxpmult(g, amount)),255);
+				b = std::min<unsigned>(unsigned(fxpmult(b, amount)),255);
+
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
 surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize)
@@ -609,25 +879,23 @@ surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize)
 
 		if (amount < 0) amount = 0;
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			alpha = (*beg) >> 24;
-			red   = (*beg) >> 16;
-			green = (*beg) >> 8;
-			blue  = (*beg) >> 0;
+			Uint8 alpha = (*beg) >> 24;
 
-			alpha = minimum<unsigned>(unsigned(fxpmult(alpha,amount)),255);
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
 
-			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
+				alpha = std::min<unsigned>(unsigned(fxpmult(alpha,amount)),255);
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
 
 			++beg;
 		}
 	}
 
-	if (optimize) {
-		return create_optimized_surface(nsurf);
-	} else {
-		return nsurf;
-	}
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
 surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize)
@@ -649,15 +917,17 @@ surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize)
 		Uint32* end = beg + nsurf->w*surf->h;
 
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			alpha = (*beg) >> 24;
-			red   = (*beg) >> 16;
-			green = (*beg) >> 8;
-			blue  = (*beg) >> 0;
+			Uint8 alpha = (*beg) >> 24;
 
-			alpha = Uint8(maximum<int>(0,minimum<int>(255,int(alpha) + amount)));
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
 
-			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
+				alpha = Uint8(std::max<int>(0,std::min<int>(255,int(alpha) + amount)));
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
 
 			++beg;
 		}
@@ -666,7 +936,6 @@ surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize)
 	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-//! Applies a mask on a surface.
 surface mask_surface(surface const &surf, surface const &mask)
 {
 	if(surf == NULL) {
@@ -689,7 +958,7 @@ surface mask_surface(surface const &surf, surface const &mask)
 		std::cerr << "It will not be masked, please use :"<< nmask->w << "x" << nmask->h << "\n";
 		return nsurf;
 	}
-	
+
 	{
 		surface_lock lock(nsurf);
 		surface_lock mlock(nmask);
@@ -700,19 +969,19 @@ surface mask_surface(surface const &surf, surface const &mask)
 		Uint32* mend = mbeg + nmask->w*nmask->h;
 
 		while(beg != end && mbeg != mend) {
-			Uint8 red, green, blue, alpha;
-			Uint8 malpha;
+			Uint8 alpha = (*beg) >> 24;
 
-			alpha = (*beg) >> 24;
-			red   = (*beg) >> 16;
-			green = (*beg) >> 8;
-			blue  = (*beg) >> 0;
+			if(alpha) {
+				Uint8 r, g, b;
+				r = (*beg) >> 16;
+				g = (*beg) >> 8;
+				b = (*beg);
 
-			malpha = (*mbeg) >> 24;
+				Uint8 malpha = (*mbeg) >> 24;
+				if (alpha > malpha) alpha = malpha;
 
-			alpha = Uint8(minimum<int>(malpha, alpha));
-
-			*beg = (alpha << 24) + (red << 16) + (green << 8) + blue;
+				*beg = (alpha << 24) + (r << 16) + (g << 8) + b;
+			}
 
 			++beg;
 			++mbeg;
@@ -723,8 +992,7 @@ surface mask_surface(surface const &surf, surface const &mask)
 	//return create_optimized_surface(nsurf);
 }
 
-//! Cross-fades a surface.
-surface blur_surface(surface const &surf, int depth)
+surface blur_surface(surface const &surf, int depth, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -767,7 +1035,7 @@ surface blur_surface(surface const &surf, int depth)
 
 		p = lock.pixels() + y*res->w;
 		for(x = 0; x < res->w; ++x, ++p) {
-			*p = 0xFF000000 | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			*p = 0xFF000000 | (std::min(red/avg,ff) << 16) | (std::min(green/avg,ff) << 8) | std::min(blue/avg,ff);
 			if(x >= depth) {
 				red -= ((*front) >> 16)&0xFF;
 				green -= ((*front) >> 8)&0xFF;
@@ -811,7 +1079,7 @@ surface blur_surface(surface const &surf, int depth)
 
 		p = lock.pixels() + x;
 		for(y = 0; y < res->h; ++y, p += res->w) {
-			*p = 0xFF000000 | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			*p = 0xFF000000 | (std::min(red/avg,ff) << 16) | (std::min(green/avg,ff) << 8) | std::min(blue/avg,ff);
 			if(y >= depth) {
 				red -= ((*front) >> 16)&0xFF;
 				green -= ((*front) >> 8)&0xFF;
@@ -837,13 +1105,10 @@ surface blur_surface(surface const &surf, int depth)
 		}
 	}
 
-	return create_optimized_surface(res);
+	return optimize ? create_optimized_surface(res) : res;
 }
 
-//! Cross-fades a surface with alpha channel.
-//! @todo FIXME: This is just an adapted copy-paste
-//! of the normal blur but with blur alpha channel too
-surface blur_alpha_surface(surface const &surf, int depth)
+surface blur_alpha_surface(surface const &surf, int depth, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -887,7 +1152,7 @@ surface blur_alpha_surface(surface const &surf, int depth)
 
 		p = lock.pixels() + y*res->w;
 		for(x = 0; x < res->w; ++x, ++p) {
-			*p = (minimum(alpha/avg,ff) << 24) | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			*p = (std::min(alpha/avg,ff) << 24) | (std::min(red/avg,ff) << 16) | (std::min(green/avg,ff) << 8) | std::min(blue/avg,ff);
 			if(x >= depth) {
 				alpha -= ((*front) >> 24)&0xFF;
 				red -= ((*front) >> 16)&0xFF;
@@ -934,7 +1199,7 @@ surface blur_alpha_surface(surface const &surf, int depth)
 
 		p = lock.pixels() + x;
 		for(y = 0; y < res->h; ++y, p += res->w) {
-			*p = (minimum(alpha/avg,ff) << 24) | (minimum(red/avg,ff) << 16) | (minimum(green/avg,ff) << 8) | minimum(blue/avg,ff);
+			*p = (std::min(alpha/avg,ff) << 24) | (std::min(red/avg,ff) << 16) | (std::min(green/avg,ff) << 8) | std::min(blue/avg,ff);
 			if(y >= depth) {
 				alpha -= ((*front) >> 24)&0xFF;
 				red -= ((*front) >> 16)&0xFF;
@@ -962,10 +1227,9 @@ surface blur_alpha_surface(surface const &surf, int depth)
 		}
 	}
 
-	return create_optimized_surface(res);
+	return optimize ? create_optimized_surface(res) : res;
 }
 
-//! Cuts a rectangle from a surface.
 surface cut_surface(surface const &surf, SDL_Rect const &r)
 {
 	surface res = create_compatible_surface(surf, r.w, r.h);
@@ -975,28 +1239,49 @@ surface cut_surface(surface const &surf, SDL_Rect const &r)
 	size_t rbpp = res->format->BytesPerPixel;
 	size_t rpitch = res->pitch;
 
+	// compute the areas to copy
+	SDL_Rect src_rect = r;
+	SDL_Rect dst_rect = { 0, 0, r.w, r.h };
+
+	if (src_rect.x < 0) {
+		if (src_rect.x + src_rect.w <= 0)
+			return res;
+		dst_rect.x -= src_rect.x;
+		dst_rect.w += src_rect.x;
+		src_rect.w += src_rect.x;
+		src_rect.x = 0;
+	}
+	if (src_rect.y < 0) {
+		if (src_rect.y + src_rect.h <= 0)
+			return res;
+		dst_rect.y -= src_rect.y;
+		dst_rect.h += src_rect.y;
+		src_rect.h += src_rect.y;
+		src_rect.y = 0;
+	}
+
+	if(src_rect.x >= surf->w || src_rect.y >= surf->h)
+		return res;
+
 	surface_lock slock(surf);
 	surface_lock rlock(res);
 
 	Uint8* src = reinterpret_cast<Uint8 *>(slock.pixels());
 	Uint8* dest = reinterpret_cast<Uint8 *>(rlock.pixels());
 
-	if(r.x >= surf->w)
-		return res;
+	for(int y = 0; y < src_rect.h && (src_rect.y + y) < surf->h; ++y) {
+		Uint8* line_src  = src  + (src_rect.y + y) * spitch + src_rect.x * sbpp;
+		Uint8* line_dest = dest + (dst_rect.y + y) * rpitch + dst_rect.x * rbpp;
+		size_t size = src_rect.w + src_rect.x <= surf->w ? src_rect.w : surf->w - src_rect.x;
 
-	for(int y = 0; y < r.h && (r.y + y) < surf->h; ++y) {
-		Uint8* line_src = src + (r.y + y) * spitch + r.x * sbpp;
-		Uint8* line_dest = dest + y * rpitch;
-		size_t size = r.w + r.x <= surf->w ? r.w : surf->w - r.x;
-
-		assert(rpitch >= r.w * rbpp);
+		assert(rpitch >= src_rect.w * rbpp);
 		memcpy(line_dest, line_src, size * rbpp);
 	}
 
 	return res;
 }
 
-surface blend_surface(surface const &surf, double amount, Uint32 colour)
+surface blend_surface(surface const &surf, double amount, Uint32 colour, bool optimize)
 {
 	if(surf== NULL) {
 		return NULL;
@@ -1014,33 +1299,36 @@ surface blend_surface(surface const &surf, double amount, Uint32 colour)
 		Uint32* beg = lock.pixels();
 		Uint32* end = beg + nsurf->w*surf->h;
 
-		Uint8 red2, green2, blue2, alpha2;
-		SDL_GetRGBA(colour,nsurf->format,&red2,&green2,&blue2,&alpha2);
+		Uint8 red, green, blue, alpha;
+		SDL_GetRGBA(colour,nsurf->format,&red,&green,&blue,&alpha);
 
-		red2 = Uint8(red2*amount);
-		green2 = Uint8(green2*amount);
-		blue2 = Uint8(blue2*amount);
+		red   = Uint8(red   * amount);
+		green = Uint8(green * amount);
+		blue  = Uint8(blue  * amount);
 
 		amount = 1.0 - amount;
 
 		while(beg != end) {
-			Uint8 red, green, blue, alpha;
-			SDL_GetRGBA(*beg,nsurf->format,&red,&green,&blue,&alpha);
+			Uint8 r, g, b, a;
+			a = (*beg) >> 24;
+			r = (*beg) >> 16;
+			g = (*beg) >> 8;
+			b = (*beg);
 
-			red = Uint8(red*amount) + red2;
-			green = Uint8(green*amount) + green2;
-			blue = Uint8(blue*amount) + blue2;
+			r = Uint8(r * amount) + red;
+			g = Uint8(g * amount) + green;
+			b = Uint8(b * amount) + blue;
 
-			*beg = SDL_MapRGBA(nsurf->format,red,green,blue,alpha);
+			*beg = (a << 24) | (r << 16) | (g << 8) | b;
 
 			++beg;
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface flip_surface(surface const &surf)
+surface flip_surface(surface const &surf, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -1066,10 +1354,10 @@ surface flip_surface(surface const &surf)
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
-surface flop_surface(surface const &surf)
+surface flop_surface(surface const &surf, bool optimize)
 {
 	if(surf == NULL) {
 		return NULL;
@@ -1095,7 +1383,7 @@ surface flop_surface(surface const &surf)
 		}
 	}
 
-	return create_optimized_surface(nsurf);
+	return optimize ? create_optimized_surface(nsurf) : nsurf;
 }
 
 
@@ -1114,6 +1402,161 @@ surface create_compatible_surface(surface const &surf, int width, int height)
 		                        surf->format->Rmask,surf->format->Gmask,surf->format->Bmask,surf->format->Amask);
 }
 
+void blit_surface(const surface& src,
+	const SDL_Rect* srcrect, surface& dst, const SDL_Rect* dstrect)
+{
+	assert(src);
+	assert(dst);
+	assert((src->flags & SDL_RLEACCEL) == 0);
+	assert((dst->flags & SDL_RLEACCEL) == 0);
+
+	// Get the areas to blit
+	SDL_Rect dst_rect = { 0, 0, dst->w, dst->h };
+	if(dstrect) {
+		dst_rect.x = dstrect->x;
+		dst_rect.w -= dstrect->x;
+
+		dst_rect.y = dstrect->y;
+		dst_rect.h -= dstrect->y;
+
+	}
+
+	SDL_Rect src_rect = { 0, 0, src->w, src->h };
+	if(srcrect && srcrect->w && srcrect->h) {
+		src_rect.x = srcrect->x;
+		src_rect.y = srcrect->y;
+
+		src_rect.w = srcrect->w;
+		src_rect.h = srcrect->h;
+
+		if (src_rect.x < 0) {
+            if (src_rect.x + src_rect.w <= 0 || src_rect.x + dst_rect.w <= 0 )
+                return;
+			dst_rect.x -= src_rect.x;
+			dst_rect.w += src_rect.x;
+			src_rect.w += src_rect.x;
+			src_rect.x = 0;
+		}
+		if (src_rect.y < 0) {
+            if (src_rect.y + src_rect.h <= 0 || src_rect.y + dst_rect.h <= 0 )
+                return;
+			dst_rect.y -= src_rect.y;
+			dst_rect.h += src_rect.y;
+			src_rect.h += src_rect.y;
+			src_rect.y = 0;
+		}
+		if (src_rect.x + src_rect.w > src->w) {
+            if (src_rect.x >= src->w)
+                return;
+			src_rect.w = src->w - src_rect.x;
+		}
+		if (src_rect.y + src_rect.h > src->h) {
+            if (src_rect.y >= src->h)
+                return;
+			src_rect.h = src->h - src_rect.y;
+		}
+	}
+
+	assert(dst_rect.x >= 0);
+	assert(dst_rect.y >= 0);
+
+	// Get the blit size limits.
+	const unsigned width = std::min(src_rect.w, dst_rect.w);
+	const unsigned height = std::min(src_rect.h, dst_rect.h);
+// 	std::cout << width << " -- " << height << "\n";
+// 	std::cout << src->w << " -- " << src->h << "\n";
+// 	std::cout << srcrect->x << "," << srcrect->y << " - " << srcrect->w << "x" << srcrect->h << " - " "\n";
+// 	if (dstrect)
+// 		std::cout << dstrect->x << "," << dstrect->y << " - " << dstrect->w << "x" << dstrect->h << " - " "\n";
+// 	std::cout << src_rect.x << "," << src_rect.y << " - " << src_rect.w << "x" << src_rect.h << " - " "\n";
+// 	std::cout << dst_rect.x << "," << dst_rect.y << " - " << dst_rect.w << "x" << dst_rect.h << " - " "\n";
+
+	{
+		// Extra scoping used for the surface_lock.
+		surface_lock src_lock(src);
+		surface_lock dst_lock(dst);
+
+		Uint32* const src_pixels = reinterpret_cast<Uint32*>(src_lock.pixels());
+		Uint32* dst_pixels = reinterpret_cast<Uint32*>(dst_lock.pixels());
+
+		for(unsigned y = 0; y < height; ++y) {
+			for(unsigned x = 0; x < width; ++x) {
+
+				// We need to do the blitting using some optimizations
+				// if the src is fully transparent we can ignore this pixel
+				// if the src is fully opaque we can overwrite the destination with this pixel
+				// if the destination is fully transparent we replace us with the source
+				//
+				// We do these optimizations between the extraction of the variables
+				// to avoid creating variables not used (it might save us some cycles).
+
+				const Uint32 src_pixel = src_pixels[(y + src_rect.y) * src->w + (x + src_rect.x)];
+				const Uint8 src_a = (src_pixel & 0xFF000000) >> 24;
+
+				if(!src_a) {
+					// Fully transparent source, ignore
+					continue;
+				}
+
+				const ptrdiff_t dst_offset = (y + dst_rect.y) * dst->w + (x + dst_rect.x);
+				if(src_a == 255) {
+					// Fully opaque source, copy
+					dst_pixels[dst_offset] = src_pixel;
+					continue;
+				}
+
+				const Uint32 dst_pixel = dst_pixels[dst_offset];
+				Uint8 dst_a = (dst_pixel & 0xFF000000) >> 24;
+
+				if(!dst_a) {
+					// Fully transparent destination, copy
+					dst_pixels[dst_offset] = src_pixel;
+					continue;
+				}
+
+				const Uint8 src_r = (src_pixel & 0x00FF0000) >> 16;
+				const Uint8 src_g = (src_pixel & 0x0000FF00) >> 8;
+				const Uint8 src_b = src_pixel & 0x000000FF;
+
+				Uint8 dst_r = (dst_pixel & 0x00FF0000) >> 16;
+				Uint8 dst_g = (dst_pixel & 0x0000FF00) >> 8;
+				Uint8 dst_b = dst_pixel & 0x000000FF;
+
+				if(dst_a == 255) {
+
+					// Destination fully opaque blend the source.
+					dst_r = (((src_r - dst_r) * src_a) >> 8 ) + dst_r;
+					dst_g = (((src_g - dst_g) * src_a) >> 8 ) + dst_g;
+					dst_b = (((src_b - dst_b) * src_a) >> 8 ) + dst_b;
+
+				} else {
+
+					// Destination and source party transparent.
+
+					// aquired the data now do the blitting
+					const unsigned tmp_a = 255 - src_a;
+
+					const unsigned tmp_r = 1 + (src_r * src_a) + (dst_r * tmp_a);
+					dst_r = (tmp_r + (tmp_r >> 8)) >> 8;
+
+					const unsigned tmp_g = 1 + (src_g * src_a) + (dst_g * tmp_a);
+					dst_g = (tmp_g + (tmp_g >> 8)) >> 8;
+
+					const unsigned tmp_b = 1 + (src_b * src_a) + (dst_b * tmp_a);
+					dst_b = (tmp_b + (tmp_b >> 8)) >> 8;
+
+					dst_a += (((255 - dst_a) * src_a) >> 8);
+				}
+
+				dst_pixels[dst_offset] = (dst_a << 24) | (dst_r << 16) | (dst_g << 8) | (dst_b);
+
+			}
+		}
+	}
+}
+
+
+
 void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &target)
 {
 	if(alpha == SDL_ALPHA_OPAQUE) {
@@ -1134,11 +1577,14 @@ void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &
 	SDL_BlitSurface(tmp,NULL,target,&rect);
 }
 
-surface get_surface_portion(surface const &src, SDL_Rect &area)
+surface get_surface_portion(surface const &src, SDL_Rect &area, bool optimize_format)
 {
+	if (src == NULL) {
+		return NULL;
+	}
+
 	// Check if there is something in the portion
 	if(area.x >= src->w || area.y >= src->h || area.x + area.w < 0 || area.y + area.h < 0) {
-		//std::cerr << "illegal surface portion...\n";
 		return NULL;
 	}
 
@@ -1149,31 +1595,30 @@ surface get_surface_portion(surface const &src, SDL_Rect &area)
 		area.h = src->h - area.y;
 	}
 
-	surface const dst = create_compatible_surface(src,area.w,area.h);
+	// use same format as the source (almost always the screen)
+	surface dst = create_compatible_surface(src, area.w, area.h);
+
 	if(dst == NULL) {
 		std::cerr << "Could not create a new surface in get_surface_portion()\n";
 		return NULL;
 	}
 
-	SDL_BlitSurface(src,&area,dst, NULL);
+	SDL_BlitSurface(src, &area, dst, NULL);
 
-	return dst;
+	return optimize_format ? display_format_alpha(dst) : dst;
 }
 
 namespace {
 
 struct not_alpha
 {
-	not_alpha(SDL_PixelFormat& format) : fmt_(format) {}
+	not_alpha() {}
 
+	// we assume neutral format
 	bool operator()(Uint32 pixel) const {
-		Uint8 r, g, b, a;
-		SDL_GetRGBA(pixel,&fmt_,&r,&g,&b,&a);
-		return a != 0x00;
+		Uint8 alpha = pixel >> 24;
+		return alpha != 0x00;
 	}
-
-private:
-	SDL_PixelFormat& fmt_;
 };
 
 }
@@ -1187,7 +1632,7 @@ SDL_Rect get_non_transparent_portion(surface const &surf)
 		return res;
 	}
 
-	const not_alpha calc(*(nsurf->format));
+	const not_alpha calc;
 
 	surface_lock lock(nsurf);
 	const Uint32* const pixels = lock.pixels();
@@ -1385,3 +1830,10 @@ void draw_centered_on_background(surface surf, const SDL_Rect& rect, const SDL_C
 	}
 	update_rect(rect);
 }
+
+std::ostream& operator<<(std::ostream& s, const SDL_Rect& rect)
+{
+	s << rect.x << ',' << rect.y << " x "  << rect.w << ',' << rect.h;
+	return s;
+}
+
