@@ -1,7 +1,7 @@
-/* $Id: binary_or_text.hpp 26880 2008-05-26 21:36:58Z suokko $ */
+/* $Id: binary_or_text.hpp 33005 2009-02-22 12:19:57Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,12 +13,13 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/binary_or_text.hpp 
-//!
+/** @file serialization/binary_or_text.hpp. */
 
 #ifndef SERIALIZATION_BINARY_OR_TEXT_HPP_INCLUDED
 #define SERIALIZATION_BINARY_OR_TEXT_HPP_INCLUDED
 
+#include "preprocessor.hpp"
+
 #include <iosfwd>
 #include <string>
 
@@ -26,22 +27,27 @@
 
 class config;
 
-//! Reads a file, and detects it is compressed before reading it. 
-//! If it is not a valid file at all, it will throw an error 
-//! as if it was trying to read it as text WML. 
-//! Returns true iff the format is compressed.
+/**
+ * Reads a file, and detects it is compressed before reading it.
+ *
+ * If it is not a valid file at all, it will throw an error as if it was trying
+ * to read it as text WML.
+ *
+ * @returns                       True iff the format is compressed.
+ */
 bool detect_format_and_read(config &cfg, std::istream &in, std::string* error_log=NULL); // Throws config::error
 
-//! Writes a file, compressed or not depending on a flag.
+/** Writes a file, compressed or not depending on a flag. */
 void write_possibly_compressed(std::ostream &out, config &cfg, bool compress);
 
-//! Class for writing a config out to a file in pieces.
+/** Class for writing a config out to a file in pieces. */
 class config_writer
 {
 public:
-	config_writer(std::ostream &out, bool compress, const std::string &textdomain, int level = -1);
+	config_writer(std::ostream &out, bool compress, int level = -1);
 
 	void write(const config &cfg);
+
 	void write_child(const std::string &key, const config &cfg);
 	void write_key_val(const std::string &key, const std::string &value);
 	void open_child(const std::string &key);
