@@ -1,7 +1,7 @@
-/* $Id: binary_wml.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: binary_wml.cpp 33126 2009-02-27 16:32:20Z soliton $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,8 +13,10 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/binary_wml.cpp 
-//! Data compression, designed for network traffic.
+/**
+ * @file serialization/binary_wml.cpp.
+ * Data compression, designed for network traffic.
+ */
 
 #include "global.hpp"
 
@@ -23,23 +25,20 @@
 #include "log.hpp"
 #include "serialization/binary_wml.hpp"
 
-#include <algorithm>
-#include <iostream>
-#include <sstream>
 
 #define ERR_CF LOG_STREAM(err, config)
 
 // Data compression. Compression is designed for network traffic.
 // Assumptions the compression is based on:
 // - most space is taken up by element names and attribute names
-// - there are relatively few element names and attribute names 
+// - there are relatively few element names and attribute names
 //   that are repeated many times
 //
 // How it works: there are some control characters:
 // 'compress_open_element': signals that the next word found is an element.
 // Any words found that are not after this are assumed to be attributes:
 // 'compress_close_element': signals to close the current element.
-// 'compress_schema_item': signals that following is a nul-delimited string, 
+// 'compress_schema_item': signals that following is a nul-delimited string,
 //                         which should be added as a word in the schema.
 // 'compress_literal_word': signals that following is a word stored as a nul-delimited string
 //    (an attribute name, unless it was preceeded by 'compress_open_element').
@@ -48,9 +47,9 @@
 // it is mapped to the first available character. Any attribute found is always followed
 // by a nul-delimited string which is the value for the attribute.
 //
-// The schema objects are designed to be persisted. That is, in a network game, 
-// both peers can store their schema objects, and so rather than sending 
-// schema data each time, the peers use and build their schemas as the 
+// The schema objects are designed to be persisted. That is, in a network game,
+// both peers can store their schema objects, and so rather than sending
+// schema data each time, the peers use and build their schemas as the
 // game progresses, adding a new word to the schema anytime it is required.
 
 static const unsigned int
@@ -160,7 +159,6 @@ void write_compressed(std::ostream &out, config const &cfg, compression_schema &
 
 static void read_compressed_internal(config &cfg, std::istream &in, compression_schema &schema, int level)
 {
-	increment_binary_wml_progress();
 	if (level >= max_recursion_levels)
 		throw config::error("Too many recursion levels in compressed config read");
 
