@@ -1,8 +1,8 @@
-/* $Id: parser.cpp 24794 2008-03-19 04:11:40Z soliton $ */
+/* $Id: parser.cpp 32553 2009-02-02 17:38:43Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,30 +14,27 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/parser.cpp 
-//! Read/Write & analyse WML- and config-files.
+/**
+ * @file serialization/parser.cpp
+ * Read/Write & analyse WML- and config-files.
+ */
+
 
 #include "serialization/parser.hpp"
 
 #include "config.hpp"
-#include "filesystem.hpp"
-#include "gettext.hpp"
-#include "language.hpp"
-#include "loadscreen.hpp"
 #include "log.hpp"
-#include "util.hpp"
+#include "gettext.hpp"
+#include "loadscreen.hpp"
 #include "wesconfig.h"
-#include "serialization/binary_wml.hpp"
-#include "serialization/preprocessor.hpp"
-#include "serialization/string_utils.hpp"
 #include "serialization/tokenizer.hpp"
+#include "foreach.hpp"
 
-#include <cassert>
-#include <sstream>
 #include <stack>
 
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
+#include <boost/algorithm/string/replace.hpp>
 
 #define ERR_CF LOG_STREAM(err, config)
 #define WRN_CF LOG_STREAM(warn, config)
@@ -69,26 +66,28 @@ private:
 	tokenizer *tok_;
 
 	struct element {
-		element(config *cfg, std::string 
-			const &name, std::string const &start_line) :
-				cfg(cfg), 
+		element(config *cfg, std::string
+			const &name, const size_t& start_line = 0, const std::string& file="") :
+				cfg(cfg),
 				name(name),
 				last_element_map(),
-				start_line(start_line) 
+				start_line(start_line),
+			    file(file)
 			{}
 
 		config* cfg;
 		std::string name;
 
 		std::map<std::string, config*> last_element_map;
-		std::string start_line;
+		size_t start_line;
+		std::string file;
 	};
 
 	std::stack<element> elements;
 };
 
 parser::parser(config &cfg, std::istream &in) :
-		cfg_(cfg), 
+		cfg_(cfg),
 		tok_(new tokenizer(in)),
 		elements()
 {
@@ -105,7 +104,7 @@ parser::~parser()
 void parser::operator()(std::string* error_log)
 {
 	cfg_.clear();
-	elements.push(element(&cfg_, "", ""));
+	elements.push(element(&cfg_, ""));
 
 	do {
 		try {
@@ -121,7 +120,14 @@ void parser::operator()(std::string* error_log)
 				parse_variable();
 				break;
 			default:
-				error(_("Unexpected characters at line start"));
+				if ((unsigned char)tok_->current_token().value[0] == 0xEF &&
+				    (unsigned char)tok_->next_token().value[0]    == 0xBB &&
+				    (unsigned char)tok_->next_token().value[0]    == 0xBF)
+				{
+					ERR_CF << "Skipping over a utf8 BOM\n";
+				} else {
+					error(_("Unexpected characters at line start"));
+				}
 				break;
 			case token::END:
 				break;
@@ -147,7 +153,9 @@ void parser::operator()(std::string* error_log)
 	if(elements.size() != 1) {
 		utils::string_map i18n_symbols;
 		i18n_symbols["tag"] = elements.top().name;
-		error(lineno_string(i18n_symbols, elements.top().start_line,
+		std::stringstream ss;
+		ss << elements.top().start_line << " " << elements.top().file;
+		error(lineno_string(i18n_symbols, ss.str(),
 				N_("Missing closing tag for tag $tag at $pos")));
 	}
 }
@@ -168,7 +176,7 @@ void parser::parse_element()
 		// Add the element
 		current_element = &(elements.top().cfg->add_child(elname));
 		elements.top().last_element_map[elname] = current_element;
-		elements.push(element(current_element, elname, tok_->get_line()));
+		elements.push(element(current_element, elname, tok_->get_start_line(), tok_->get_file()));
 		break;
 
 	case '+': // [+element]
@@ -187,7 +195,7 @@ void parser::parse_element()
 			current_element = last_element_itor->second;
 		}
 		elements.top().last_element_map[elname] = current_element;
-		elements.push(element(current_element, elname, tok_->get_line()));
+		elements.push(element(current_element, elname, tok_->get_start_line(), tok_->get_file()));
 		break;
 
 	case '/': // [/element]
@@ -202,7 +210,9 @@ void parser::parse_element()
 			utils::string_map i18n_symbols;
 			i18n_symbols["tag"] = elements.top().name;
 			i18n_symbols["tag2"] = elname;
-			error(lineno_string(i18n_symbols, elements.top().start_line,
+			std::stringstream ss;
+			ss << elements.top().start_line << " " << elements.top().file;
+			error(lineno_string(i18n_symbols, ss.str(),
 					N_("Found invalid closing tag $tag2 for tag $tag (opened at $pos)")));
 		}
 
@@ -327,17 +337,29 @@ std::string parser::lineno_string(utils::string_map &i18n_symbols, std::string c
 	}
 	if (res.empty()) res = "???";
 	i18n_symbols["pos"] = res;
-	return vgettext(error_string.c_str(), i18n_symbols);
+	std::string result = _(error_string.c_str());
+	foreach(utils::string_map::value_type& var, i18n_symbols)
+		boost::algorithm::replace_all(result, std::string("$") + var.first, std::string(var.second));
+	return result;
 }
 
 void parser::error(const std::string& error_type)
 {
 	utils::string_map i18n_symbols;
 	i18n_symbols["error"] = error_type;
-
+	i18n_symbols["value"] = tok_->current_token().value;
+	std::stringstream ss;
+	ss << tok_->get_start_line() << " " << tok_->get_file();
+#ifdef DEBUG
+	i18n_symbols["previous_value"] = tok_->previous_token().value;
 	throw config::error(
-		lineno_string(i18n_symbols, tok_->get_line(),
-		              N_("$error at $pos")));
+		lineno_string(i18n_symbols, ss.str(),
+		              N_("$error, value '$value', previous '$previous_value' at $pos")));
+#else
+	throw config::error(
+		lineno_string(i18n_symbols, ss.str(),
+		              N_("$error, value '$value' at $pos")));
+#endif
 }
 
 } // end anon namespace
@@ -355,6 +377,11 @@ void read(config &cfg, std::string &in, std::string* error_log)
 
 void read_gz(config &cfg, std::istream &file, std::string* error_log)
 {
+	//an empty gzip file seems to confuse boost on msvc
+	//so return early if this is the case
+	if (file.peek() == EOF) {
+		return;
+	}
 	boost::iostreams::filtering_stream<boost::iostreams::input> filter;
 	filter.push(boost::iostreams::gzip_decompressor());
 	filter.push(file);
