@@ -1,7 +1,7 @@
-/* $Id: preprocessor.cpp 26582 2008-05-13 11:11:38Z suokko $ */
+/* $Id: preprocessor.cpp 33310 2009-03-03 17:44:11Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,22 +13,19 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/preprocessor.cpp
-//! WML preprocessor.
+/**
+ * @file serialization/preprocessor.cpp
+ * WML preprocessor.
+ */
 
-#include "../global.hpp"
+#include "global.hpp"
 
-#include "../filesystem.hpp"
-#include "../log.hpp"
-#include "../wesconfig.h"
-#include "preprocessor.hpp"
-#include "string_utils.hpp"
-
-#include <algorithm>
-#include <cassert>
-#include <iostream>
-#include <sstream>
-#include <vector>
+#include "config.hpp"
+#include "filesystem.hpp"
+#include "foreach.hpp"
+#include "log.hpp"
+#include "wesconfig.h"
+#include "serialization/binary_or_text.hpp"
 
 #define ERR_CF LOG_STREAM(err, config)
 #define LOG_CF LOG_STREAM(info, config)
@@ -40,12 +37,87 @@ bool preproc_define::operator==(preproc_define const &v) const {
 	return value == v.value && arguments == v.arguments;
 }
 
+bool preproc_define::operator<(preproc_define const &v) const {
+	if (location < v.location)
+		return true;
+	if (v.location < location)
+		return false;
+	return linenum < v.linenum;
+}
+
+void preproc_define::write_argument(config_writer& writer, const std::string& arg) const
+{
+
+	const std::string key = "argument";
+
+	writer.open_child(key);
+
+	writer.write_key_val("name", arg);
+	writer.close_child(key);
+}
+
+void preproc_define::write(config_writer& writer, const std::string& name) const
+{
+
+	const std::string key = "preproc_define";
+	writer.open_child(key);
+
+	writer.write_key_val("name", name);
+	writer.write_key_val("value", value);
+	writer.write_key_val("textdomain", textdomain);
+	writer.write_key_val("linenum", lexical_cast<std::string>(linenum));
+	writer.write_key_val("location", location);
+
+	foreach (const std::string &arg, arguments)
+		write_argument(writer, arg);
+
+	writer.close_child(key);
+}
+
+void preproc_define::read_argument(const config &cfg)
+{
+	arguments.push_back(cfg["name"]);
+}
+
+void preproc_define::read(const config& cfg)
+{
+	value = cfg["value"];
+	textdomain = cfg["textdomain"];
+	linenum = lexical_cast<int>(cfg["linenum"]);
+	location = cfg["location"];
+
+	foreach (const config *arg, cfg.get_children("argument"))
+		read_argument(*arg);
+}
+
+preproc_map::value_type preproc_define::read_pair(const config* cfg)
+{
+	std::string first = (*cfg)["name"];
+
+	preproc_define second;
+	second.read(*cfg);
+	return std::make_pair(first, second);
+}
+
+std::ostream& operator<<(std::ostream& stream, const preproc_define& def)
+{
+	return stream << std::string("value: ") << def.value << std::string(" arguments: ") << def.location;
+}
+
+std::ostream& operator<<(std::ostream& stream, const preproc_map::value_type& def)
+{
+	return stream << def.second;
+}
+
 class preprocessor;
 class preprocessor_file;
 class preprocessor_data;
 class preprocessor_streambuf;
 struct preprocessor_deleter;
 
+/**
+ * Base class for preprocessing an input.
+ */
 class preprocessor
 {
 	preprocessor *const old_preprocessor_;
@@ -57,16 +129,24 @@ protected:
 	std::vector<std::string> *called_macros_;
 	preprocessor(preprocessor_streambuf &, std::vector<std::string> *);
 public:
+	/**
+	 * Preprocesses and sends some text to the #target_ buffer.
+	 * @return false when the input has no data left.
+	 */
 	virtual bool get_chunk() = 0;
 	virtual ~preprocessor();
 };
 
+/**
+ * Target for sending preprocessed output.
+ * Objects of this class can be plugged into an STL stream.
+ */
 class preprocessor_streambuf: public streambuf
 {
-	std::string out_buffer_;
+	std::string out_buffer_;      /**< Buffer read by the STL stream. */
 	virtual int underflow();
-	std::ostringstream buffer_;
-	preprocessor *current_;
+	std::stringstream buffer_;    /**< Buffer filled by the #current_ preprocessor. */
+	preprocessor *current_;       /**< Input preprocessor. */
 	preproc_map *defines_;
 	preproc_map default_defines_;
 	std::string textdomain_;
@@ -74,7 +154,10 @@ class preprocessor_streambuf: public streambuf
 	std::string *error_log;
 	int linenum_;
 	int depth_;
-	int buffer_size_;
+	/**
+	 * Set to true if one preprocessor for this target started to read a string.
+	 * Deeper-nested preprocessors are then forbidden to.
+	 */
 	bool quoted_;
 	friend class preprocessor;
 	friend class preprocessor_file;
@@ -88,18 +171,17 @@ public:
 };
 
 preprocessor_streambuf::preprocessor_streambuf(preproc_map *def, std::string *err_log) :
-	streambuf(), 
-	out_buffer_(""), 
-	buffer_(), 
-	current_(NULL), 
+	streambuf(),
+	out_buffer_(""),
+	buffer_(),
+	current_(NULL),
 	defines_(def),
 	default_defines_(),
-	textdomain_(PACKAGE), 
-	location_(""), 
+	textdomain_(PACKAGE),
+	location_(""),
 	error_log(err_log),
-	linenum_(0), 
+	linenum_(0),
 	depth_(0),
-	buffer_size_(0),
 	quoted_(false)
 {
 }
@@ -116,13 +198,16 @@ preprocessor_streambuf::preprocessor_streambuf(preprocessor_streambuf const &t)
 	error_log(t.error_log),
 	linenum_(0),
 	depth_(t.depth_),
-	buffer_size_(0), 
 	quoted_(t.quoted_)
 {
 }
 
-// underflow is called when the internal buffer has been consumed 
-// so that more can be prepared
+/**
+ * Called by an STL stream whenever it has reached the end of #out_buffer_.
+ * Fills #buffer_ by calling the #current_ preprocessor, then copies its
+ * content into #out_buffer_.
+ * @return the first character of #out_buffer_ if any, EOF otherwise.
+ */
 int preprocessor_streambuf::underflow()
 {
 	unsigned sz = 0;
@@ -144,12 +229,12 @@ int preprocessor_streambuf::underflow()
 		{
 			buffer_ << out_buffer_;
 		}
-	        buffer_size_ = sz;
 	} else {
 		// The internal get-data pointer is null
 	}
 	const int desired_fill_amount = 2000;
-	while (current_ && buffer_size_ < desired_fill_amount) {
+	while (current_ && buffer_.rdbuf()->in_avail() < desired_fill_amount)
+	{
 		// Process files and data chunks until the desired buffer size is reached
 		if (!current_->get_chunk()) {
 			 // Delete the current preprocessor item to restore its predecessor
@@ -198,13 +283,9 @@ void preprocessor_streambuf::error(const std::string& error_type, const std::str
 
 
 /**
- * preprocessor
- * 
- * This is the base class for all input to be parsed by the preprocessor.
- * When initialized, it will inform the stream (target_) that it is the current scope.
- * When it reaches its end of its scope, the stream will delete it, 
- * and when it is deleted, it will manage the stream 
- * to cause the previous scope to resume.
+ * Sets up a new preprocessor for stream buffer \a t.
+ * Saves the current preprocessing context of #target_. It will be
+ * automatically restored on destruction.
  */
 preprocessor::preprocessor(preprocessor_streambuf &t, std::vector<std::string> *callstack) :
 	old_preprocessor_(t.current_),
@@ -218,15 +299,11 @@ preprocessor::preprocessor(preprocessor_streambuf &t, std::vector<std::string> *
 	target_.current_ = this;
 }
 
-namespace {
-void count_extra_digits(int const& line_number, int& buffer_size) {
-	for(int digit_mark = 10; line_number >= digit_mark; digit_mark *= 10) {
-		// The number is larger than one digit, calculate additional string size
-		++buffer_size;
-	}
-}
-} // end anonymous namespace
-
+/**
+ * Restores the old preprocessing context of #target_.
+ * Appends location and domain directives to the buffer, so that the parser
+ * notices these changes.
+ */
 preprocessor::~preprocessor()
 {
 	assert(target_.current_ == this);
@@ -236,23 +313,17 @@ preprocessor::~preprocessor()
 	target_.textdomain_ = old_textdomain_;
 	if (!old_location_.empty()) {
 		target_.buffer_ << "\376line " << old_linenum_ << ' ' << old_location_ << '\n';
-		const int fixed_char_count = 9;
-		count_extra_digits(old_linenum_, target_.buffer_size_);
-		target_.buffer_size_ += old_location_.size() + fixed_char_count;
     }
 	if (!old_textdomain_.empty()) {
 		target_.buffer_ << "\376textdomain " << old_textdomain_ << '\n';
-		const int fixed_char_count = 13;
-		target_.buffer_size_ += old_textdomain_.size() + fixed_char_count;
     }
 	--target_.depth_;
 }
 
 /**
- *  preprocessor_file
- *
- *  This represents a WML element that resolves to a directory or file inclusion, 
- *  such as '{themes/}'
+ * Specialized preprocessor for handling a file or a set of files.
+ * A preprocessor_file object is created when a preprocessor encounters an
+ * inclusion directive that resolves to a file or directory, e.g. '{themes/}'.
  */
 class preprocessor_file: preprocessor
 {
@@ -263,23 +334,52 @@ public:
 	virtual bool get_chunk();
 };
 
+/**
+ * Specialized preprocessor for handling any kind of input stream.
+ * This is the core of the preprocessor.
+ */
 class preprocessor_data: preprocessor
 {
+	/** Description of a preprocessing chunk. */
 	struct token_desc
 	{
-		//! @todo FIXME: add enum for token type, with explanation of the different types
+		/** @todo FIXME: add enum for token type. */
+		/**
+		 * Preprocessor state.
+		 * - 'i': processing the "if" branch of a ifdef/ifndef (the "else" branch will be skipped)
+		 * - 'j': processing the "else" branch of a ifdef/ifndef
+		 * - 'I': skipping the "if" branch of a ifdef/ifndef (the "else" branch, if any, will be processed)
+		 * - 'J': skipping the "else" branch of a ifdef/ifndef
+		 * - '"': processing a string
+		 * - '{': processing between chunks of a macro call (skip spaces)
+		 * - '[': processing inside a chunk of a macro call (stop on space or '(')
+		 * - '(': processing a parenthesized macro argument
+		 */
 		char type;
+		/** Starting position in #strings_ of the delayed text for this chunk. */
 		int stack_pos;
 		int linenum;
 	};
-	scoped_istream in_;
+	scoped_istream in_; /**< Input stream. */
 	std::string directory_;
+	/** Buffer for delayed input processing. */
 	std::vector< std::string > strings_;
+	/** Stack of nested preprocessing chunks. */
 	std::vector< token_desc > tokens_;
 	bool is_macro;
-	int slowpath_, //! @todo FIXME: add explanation of this variable
-		skipping_, //will get set to true when skipping text(e.g. #ifdef that evaluates to false)
-		linenum_;
+	/**
+	 * Set to true whenever input tokens cannot be directly sent to the target
+	 * buffer. For instance, this happens with macro arguments. In that case,
+	 * the output is redirected toward #strings_ until it can be processed.
+	 */
+	int slowpath_;
+	/**
+	 * Non-zero when the preprocessor has to skip some input text.
+	 * Increased whenever entering a conditional branch that is not useful,
+	 * e.g. a ifdef that evaluates to false.
+	 */
+	int skipping_;
+	int linenum_;
 
 	std::string read_word();
 	std::string read_line();
@@ -288,10 +388,10 @@ class preprocessor_data: preprocessor
 	void push_token(char);
 	void pop_token();
 	void put(char);
-	void put(std::string const & /*, int change_line 
+	void put(std::string const & /*, int change_line
 	= 0 */);
 public:
-	preprocessor_data(preprocessor_streambuf &, 
+	preprocessor_data(preprocessor_streambuf &,
                       std::vector<std::string> *,
                       std::istream *,
 	                  std::string const &history,
@@ -311,8 +411,15 @@ preprocessor_file::preprocessor_file(preprocessor_streambuf &t, std::vector<std:
 {
 	if (is_directory(name))
 		get_files_in_dir(name, &files_, NULL, ENTIRE_FILE_PATH, SKIP_MEDIA_DIR, DO_REORDER);
-	else
-		new preprocessor_data(t, called_macros_, istream_file(name), "", name, 1, directory_name(name), t.textdomain_);
+	else {
+		std::istream * file_stream = istream_file(name);
+		if (!file_stream->good()) {
+			ERR_CF << "Could not open file " << name << "\n";
+			delete file_stream;
+		}
+		else
+			new preprocessor_data(t, called_macros_, file_stream, "", name, 1, directory_name(name), t.textdomain_);
+	}
 	pos_ = files_.begin();
 	end_ = files_.end();
 }
@@ -337,28 +444,23 @@ bool preprocessor_file::get_chunk()
 	return false;
 }
 
-preprocessor_data::preprocessor_data(preprocessor_streambuf &t, std::vector<std::string> *callstack, 
-        std::istream *i, std::string const &history, std::string const &name, int linenum, 
+preprocessor_data::preprocessor_data(preprocessor_streambuf &t, std::vector<std::string> *callstack,
+        std::istream *i, std::string const &history, std::string const &name, int linenum,
 		std::string const &directory, std::string const &domain, std::string *symbol) :
-	preprocessor(t,callstack), 
-	in_(i), 
+	preprocessor(t,callstack),
+	in_(i),
 	directory_(directory),
 	strings_(),
 	tokens_(),
-	slowpath_(0), 
-	skipping_(0), 
+	is_macro(symbol != NULL),
+	slowpath_(0),
+	skipping_(0),
 	linenum_(linenum)
 {
-    if(symbol!=NULL)
-    {
-        is_macro=true;
+    if(is_macro) {
         called_macros_->push_back(*symbol);
     }
-    else
-    {
-        is_macro=false;
-    }
-    
+
 	std::ostringstream s;
 
 	s << history;
@@ -376,9 +478,6 @@ preprocessor_data::preprocessor_data(preprocessor_streambuf &t, std::vector<std:
 
 	t.buffer_ << "\376line " << linenum
 		<< ' ' << t.location_ << "\n\376textdomain " << domain << '\n';
-	const int fixed_char_count = 22;
-	count_extra_digits(linenum, t.buffer_size_);
-	t.buffer_size_ += t.location_.size() + domain.size() + fixed_char_count;
 
 	push_token('*');
 }
@@ -470,22 +569,18 @@ void preprocessor_data::put(char c)
 		strings_.back() += c;
 		return;
 	}
-	if (linenum_ != target_.linenum_ && c != '\n' ||
-	    linenum_ != target_.linenum_ + 1 && c == '\n') {
+	if ((linenum_ != target_.linenum_ && c != '\n') ||
+	    (linenum_ != target_.linenum_ + 1 && c == '\n')) {
 		target_.linenum_ = linenum_;
 		if (c == '\n')
 			--target_.linenum_;
 
 		target_.buffer_ << "\376line " << target_.linenum_
 			<< ' ' << target_.location_ << '\n';
-		const int fixed_char_count = 9;
-		count_extra_digits(target_.linenum_, target_.buffer_size_);
-		target_.buffer_size_ += target_.location_.size() + fixed_char_count;
 	}
 	if (c == '\n')
 		++target_.linenum_;
 	target_.buffer_ << c;
-    target_.buffer_size_ += 1;
 }
 
 void preprocessor_data::put(std::string const &s /*, int line_change*/)
@@ -498,7 +593,6 @@ void preprocessor_data::put(std::string const &s /*, int line_change*/)
 	}
 	target_.buffer_ << s;
 //	target_.linenum_ += line_change;
-    target_.buffer_size_ += s.size();
 }
 
 bool preprocessor_data::get_chunk()
@@ -506,7 +600,7 @@ bool preprocessor_data::get_chunk()
 	char c = in_->get();
 	token_desc &token = tokens_.back();
 	if (!in_->good()) {
-		// The end of file was reached. 
+		// The end of file was reached.
 		// Make sure we don't have any incomplete tokens.
 		char const *s;
 		switch (token.type) {
@@ -522,7 +616,7 @@ bool preprocessor_data::get_chunk()
 		default: s = "???";
 		}
 		std::ostringstream error;
-		error<<s<<" not terminated";		
+		error<<s<<" not terminated";
         std::ostringstream location;
 		location<<linenum_<<' '<<target_.location_;
 		pop_token();
@@ -579,7 +673,7 @@ bool preprocessor_data::get_chunk()
 			int linenum = linenum_;
 			std::vector< std::string > items = utils::split(read_line(), ' ');
 			if (items.empty()) {
-				std::string error="No macro name found after #define directive";		
+				std::string error="No macro name found after #define directive";
 				std::ostringstream location;
 				location<<linenum_<<' '<<target_.location_;
 				target_.error(error, location.str());
@@ -598,14 +692,15 @@ bool preprocessor_data::get_chunk()
 				if (d == '#')
 					found_enddef = 1;
 				else if (found_enddef > 0)
-					if (++found_enddef == 7)
+					if (++found_enddef == 7) {
 						if (std::equal(buffer.end() - 6, buffer.end(), "enddef"))
 							break;
 						else
 							found_enddef = 0;
+					}
 			}
 			if (found_enddef != 7) {
-				std::string error="Unterminated preprocessor definition at";		
+				std::string error="Unterminated preprocessor definition at";
 		        std::ostringstream location;
 				location<<linenum_<<' '<<target_.location_;
 				target_.error(error, location.str());
@@ -718,19 +813,26 @@ bool preprocessor_data::get_chunk()
 				std::string::iterator b = symbol.begin(); // invalidated at each iteration
 				symbol.erase(b + pos, b + symbol.find('\n', pos + 1) + 1);
 			}
-			// If this is a known pre-processing symbol, then we insert it, 
+			// If this is a known pre-processing symbol, then we insert it,
 			// otherwise we assume it's a file name to load.
 			preproc_map::const_iterator macro = target_.defines_->find(symbol);
-			if(macro != target_.defines_->end()) {			
+			if(macro != target_.defines_->end()) {
 
-//! @todo it seems some addons use other names instead of include so disable the 
-//! code for now. Once 1.4 has been released we can try to fix it again or
-//! make it mandatory to use INCLUDE for this purpose.
-#if 0			
+/**
+ * @todo it seems some addons use other names instead of include so disable the
+ * code for now. Once 1.4 has been released we can try to fix it again or make
+ * it mandatory to use INCLUDE for this purpose.
+ *
+ * ESR says: No, it was a fundamentally mistaken idea to do this here; wmllint
+ * could do the same direct-recursion check without imposing runtime overhead
+ * or causing the problems this code did.  Leaving this code in for
+ * documentation purposes only, don't re-enable it.
+ */
+#if 0
 				// INCLUDE is special and is allowed to be used recusively.
 				if(symbol != "INCLUDE") {
-				    for(std::vector<std::string>::iterator 
-							iter=called_macros_->begin(); 
+				    for(std::vector<std::string>::iterator
+							iter=called_macros_->begin();
 							iter!=called_macros_->end(); ++iter) {
 						if(*iter==symbol) {
 							std::ostringstream error;
@@ -741,7 +843,7 @@ bool preprocessor_data::get_chunk()
 						}
 					}
 				}
-#endif					
+#endif // 0
 				preproc_define const &val = macro->second;
 				size_t nb_arg = strings_.size() - token.stack_pos - 1;
 				if (nb_arg != val.arguments.size()) {
@@ -790,7 +892,7 @@ bool preprocessor_data::get_chunk()
 								c = '\n';
 							} else if((i_end - i_beg < 6 || (!std::equal(i_beg, i_beg + 6, "define")
 							&& !std::equal(i_beg, i_beg + 6, "ifndef")))
-							&& (i_end - i_beg < 5 || (!std::equal(i_beg, i_beg + 5, "ifdef") 
+							&& (i_end - i_beg < 5 || (!std::equal(i_beg, i_beg + 5, "ifdef")
 							&& !std::equal(i_beg, i_beg + 5, "endif") && !std::equal(i_beg, i_beg + 5, "undef")))
 							&& (i_end - i_beg < 4 || !std::equal(i_beg, i_beg + 4, "else"))) {
 								// Check for define, ifdef, ifndef, endif, undef, else.
@@ -842,44 +944,11 @@ bool preprocessor_data::get_chunk()
 					strings_.back() += res.str();
 				}
 			} else if (target_.depth_ < 40) {
+				LOG_CF << "Macro definition not found for " << symbol << " , attempting to open as file.\n";
 				pop_token();
 				std::string prefix;
-				std::string nfname;
-				std::string const &newfilename = symbol;
-				// If the filename begins with a '~', 
-				//  then look in the user's data directory. 
-				// If the filename begins with a '@', 
-				//  then we look in the user's data directory,
-				// but default to the standard data directory if it's not found there.
-				if(newfilename != "" && (newfilename[0] == '~' || newfilename[0] == '@')) {
-					nfname = newfilename;
-					nfname.erase(nfname.begin(),nfname.begin()+1);
-					nfname = get_user_data_dir() + "/data/" + nfname;
-
-					LOG_CF << "got relative name '" << newfilename
-						<< "' -> '" << nfname << "'\n";
-
-					if(newfilename[0] == '@' && file_exists(nfname) == false
-					   && is_directory(nfname) == false)
-					{
-						nfname = "data/" + newfilename.substr(1);
-					}
-				} else if(newfilename.size() >= 2 && newfilename[0] == '.'
-					&& newfilename[1] == '/' )
-				{
-					// If the filename begins with a "./", 
-					// then look in the same directory as the file 
-					// currrently being preprocessed.
-					nfname = newfilename;
-					nfname.erase(nfname.begin(),nfname.begin()+2);
-					nfname = directory_ + nfname;
-				} else {
-					nfname = "data/" + newfilename;
-				}
-
-				// Ignore filenames that start with '../' or contain '/../'.
-				if (newfilename.rfind("../", 0) == std::string::npos
-					&& newfilename.find("/../") == std::string::npos)
+				std::string nfname = get_wml_location(symbol, directory_);
+				if (!nfname.empty())
 				{
 					if (!slowpath_)
 						new preprocessor_file(target_, called_macros_, nfname);
@@ -893,10 +962,9 @@ bool preprocessor_data::get_chunk()
 						delete buf;
 						strings_.back() += res.str();
 					}
-				} else {
-					ERR_CF << "Illegal path '" << newfilename
-						<< "' found (../ not allowed).\n";
 				}
+				else
+					ERR_CF << "File not found '" << symbol << "'\n";
 			} else {
 				ERR_CF << "Too much nested preprocessing inclusions at "
 				       << linenum_ << ' ' << target_.location_
@@ -922,8 +990,8 @@ struct preprocessor_deleter: std::basic_istream<char>
 	~preprocessor_deleter();
 };
 
-preprocessor_deleter::preprocessor_deleter(preprocessor_streambuf *buf, 
-		preproc_map *defines, 
+preprocessor_deleter::preprocessor_deleter(preprocessor_streambuf *buf,
+		preproc_map *defines,
 		std::vector<std::string> *callstack)
 	: std::basic_istream<char>(buf), buf_(buf), defines_(defines), callstack_(callstack), error_log(buf->error_log)
 {
@@ -944,8 +1012,8 @@ std::istream *preprocess_file(std::string const &fname,
 	log_scope("preprocessing file...");
 	preproc_map *owned_defines = NULL;
 	if (!defines) {
-		// If no preproc_map has been given, create a new one, 
-		// and ensure it is destroyed when the stream is 
+		// If no preproc_map has been given, create a new one,
+		// and ensure it is destroyed when the stream is
 // ??
 		// by giving it to the deleter.
 		owned_defines = new preproc_map;
