@@ -1,7 +1,7 @@
-/* $Id: preprocessor.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: preprocessor.hpp 33309 2009-03-03 17:44:06Z silene $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
-   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2009 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/preprocessor.hpp
+/** @file serialization/preprocessor.hpp. */
 
 #ifndef SERIALIZATION_PREPROCESSOR_HPP_INCLUDED
 #define SERIALIZATION_PREPROCESSOR_HPP_INCLUDED
@@ -23,10 +23,18 @@
 #include <string>
 #include <vector>
 
+#include "game_errors.hpp"
+
+class config_writer;
+class config;
+
+struct preproc_define;
+typedef std::map< std::string, preproc_define > preproc_map;
+
 struct preproc_define
 {
-	preproc_define() : value(""), arguments(), textdomain(""), linenum(0), location("") {}
-	explicit preproc_define(std::string const &val) : value(val), arguments(), textdomain(""), linenum(0), location("") {}
+	preproc_define() : value(), arguments(), textdomain(), linenum(0), location() {}
+	explicit preproc_define(std::string const &val) : value(val), arguments(), textdomain(), linenum(0), location() {}
 	preproc_define(std::string const &val, std::vector< std::string > const &args,
 	               std::string const &domain, int line, std::string const &loc)
 		: value(val), arguments(args), textdomain(domain), linenum(line), location(loc) {}
@@ -35,22 +43,34 @@ struct preproc_define
 	std::string textdomain;
 	int linenum;
 	std::string location;
+	void write(config_writer&, const std::string&) const;
+	void write_argument(config_writer&, const std::string&) const;
+	void read(const config&);
+	void read_argument(const config &);
+	static preproc_map::value_type read_pair(const config*);
 	bool operator==(preproc_define const &) const;
+	bool operator<(preproc_define const &) const;
 	bool operator!=(preproc_define const &v) const { return !operator==(v); }
 };
 
+std::ostream& operator<<(std::ostream& stream, const preproc_define& def);
+
 struct preproc_config {
-	struct error {
-		error(const std::string& msg) : message(msg) {}
-		std::string message;
+	struct error : public game::error {
+		error(const std::string& message) : game::error(message) {}
 	};
 };
 
-typedef std::map< std::string, preproc_define > preproc_map;
 
-//! Function to use the WML preprocessor on a file, 
-//! and returns the resulting preprocessed file data. 
-//! defines is a map of symbols defined.
+std::ostream& operator<<(std::ostream& stream, const preproc_map::value_type& def);
+
+/**
+ * Function to use the WML preprocessor on a file.
+ *
+ * @param defines                 A map of symbols defined.
+ *
+ * @returns                       The resulting preprocessed file data.
+ */
 std::istream *preprocess_file(std::string const &fname,
                               preproc_map *defines = NULL, std::string *error_log=NULL);
 
