@@ -1,8 +1,8 @@
-/* $Id: string_utils.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: string_utils.cpp 32608 2009-02-04 21:48:16Z grzywacz $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 - 2008 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2009 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -14,117 +14,24 @@
    See the COPYING file for more details.
 */
 
-//! @file serialization/string_utils.cpp
-//! Various string-routines.
+/**
+ * @file serialization/string_utils.cpp
+ * Various string-routines.
+ */
 
 #include "global.hpp"
 
-#include <cctype>
-#include <sstream>
 #include "gettext.hpp"
-#include "util.hpp"
 #include "log.hpp"
 #include "serialization/string_utils.hpp"
-#include "wctype.h"
 
 #define ERR_GENERAL LOG_STREAM(err, general)
+#define ERR_NG LOG_STREAM(err, engine)
 
 variable_set::~variable_set()
 {
 }
 
-static bool two_dots(char a, char b) { return a == '.' && b == '.'; }
-
-static std::string do_interpolation(const std::string &str, const variable_set& set)
-{
-	std::string res = str;
-	// This needs to be able to store negative numbers to check for the while's condition
-	// (which is only false when the previous '$' was at index 0)
-	int rfind_dollars_sign_from = res.size();
-	while(rfind_dollars_sign_from >= 0) {
-		// Going in a backwards order allows nested variable-retrieval, e.g. in arrays.
-		// For example, "I am $creatures[$i].user_description!"
-		const std::string::size_type var_begin_loc = res.rfind('$', rfind_dollars_sign_from);
-
-		// If there are no '$' left then we're done.
-		if(var_begin_loc == std::string::npos) {
-			break;
-		}
-
-		// For the next iteration of the loop, search for more '$'
-		// (not from the same place because sometimes the '$' is not replaced)
-		rfind_dollars_sign_from = int(var_begin_loc) - 1;
-
-
-		const std::string::iterator var_begin = res.begin() + var_begin_loc;
-
-		// The '$' is not part of the variable name.
-		const std::string::iterator var_name_begin = var_begin + 1;
-
-		// Find the maximum extent of the variable name (it may be shortened later).
-		std::string::iterator var_end = var_name_begin;
-		for(int bracket_nesting_level = 0; var_end != res.end(); ++var_end) {
-			const char c = *var_end;
-			if(c == '[') {
-				++bracket_nesting_level;
-			}
-			else if(c == ']') {
-				if(--bracket_nesting_level < 0) {
-					break;
-				}
-			}
-			else if(!isdigit(c) && !isalpha(c) && c != '.' && c != '_') {
-				break;
-			}
-		}
-
-		// Two dots in a row cannot be part of a valid variable name.
-		// That matters for random=, e.g. $x..$y
-		var_end = std::adjacent_find(var_name_begin, var_end, two_dots);
-
-		// If the last character is '.', then it can't be a sub-variable.
-		// It's probably meant to be a period instead. Don't include it.
-		// Would need to do it repetitively if there are multiple '.'s at the end,
-		// but don't actually need to do so because the previous check for adjacent '.'s would catch that.
-		// For example, "My score is $score." or "My score is $score..."
-		if(*(var_end-1) == '.'
-		// However, "$array[$i]" by itself does not name a variable,
-		// so if "$array[$i]." is encountered, then best to include the '.',
-		// so that it more closely follows the syntax of a variable (if only to get rid of all of it).
-		// (If it's the script writer's error, they'll have to fix it in either case.)
-		// For example in "$array[$i].$field_name", if field_name does not exist as a variable,
-		// then the result of the expansion should be "", not "." (which it would be if this exception did not exist).
-		&& *(var_end-2) != ']') {
-			--var_end;
-		}
-
-		const std::string var_name(var_name_begin, var_end);
-
-		if(*var_end == '|') {
-			// It's been used to end this variable name; now it has no more effect.
-			// This can allow use of things like "$$composite_var_name|.x"
-			// (Yes, that's a WML 'pointer' of sorts. They are sometimes useful.)
-			// If there should still be a '|' there afterwards to affect other variable names (unlikely),
-			// just put another '|' there, one matching each '$', e.g. "$$var_containing_var_name||blah"
-			var_end++;
-		}
-
-
-		if (var_name == "") {
-			// Allow for a way to have $s in a string.
-			// $| will be replaced by $.
-			res.replace(var_begin, var_end, "$");
-		}
-		else {
-			// The variable is replaced with its value.
-			res.replace(var_begin, var_end,
-			    set.get_variable_const(var_name));
-		}
-	}
-
-	return res;
-}
-
 namespace utils {
 
 bool isnewline(const char c)
@@ -149,7 +56,6 @@ bool notspace(const char c)
 	return !portable_isspace(c);
 }
 
-//! Remove whitespace from the front and back of the string 'str'.
 std::string &strip(std::string &str)
 {
 	// If all the string contains is whitespace,
@@ -164,7 +70,6 @@ std::string &strip(std::string &str)
 	return str;
 }
 
-//! Removes character 'c' from the first and last position of the string 'str'.
 std::string& strip_char(std::string &str, const char c) {
 	if (*str.begin() == c)
 		str.erase(str.begin(), str.begin() + 1);
@@ -208,25 +113,9 @@ std::vector< std::string > split(std::string const &val, char c, int flags)
 	return res;
 }
 
-//! Splits a string based either on a separator where text within paranthesis
-//! is protected from splitting (Note that one can use the same character for
-//! both the left and right paranthesis. In this mode it usually makes only
-//! sense to have one character for the left and right paranthesis.)
-//! or if the separator == 0 it splits a string into an odd number of parts:
-//! - The part before the first '(',
-//! - the part between the first '('
-//! - and the matching right ')', etc ...
-//! and the remainder of the string.
-//! Note that this will find the first matching char in the left string
-//! and match against the corresponding char in the right string.
-//! In this mode, a correctly processed string should return with
-//! an odd number of elements to the vector and
-//! an empty elements are never removed as they are placeholders.
-//! hence REMOVE EMPTY only works for the separator split.
-//!
-//! parenthetical_split("a(b)c{d}e(f{g})h",0,"({",")}") should return
-//! a vector of <"a","b","c","d","e","f{g}","h">
-std::vector< std::string > paranthetical_split(std::string const &val, const char separator, std::string const &left, std::string const &right,int flags)
+std::vector< std::string > paranthetical_split(std::string const &val,
+		const char separator, std::string const &left,
+		std::string const &right,int flags)
 {
 	std::vector< std::string > res;
 	std::vector<char> part;
@@ -311,38 +200,6 @@ std::vector< std::string > paranthetical_split(std::string const &val, const cha
 	return res;
 }
 
-class string_map_variable_set : public variable_set
-{
-public:
-	string_map_variable_set(const string_map& map) : map_(map) {};
-
-	virtual const t_string& get_variable_const (const std::string& key) const
-	{
-		static const t_string empty_string = "";
-
-		const string_map::const_iterator itor = map_.find(key);
-		if(itor == map_.end()) {
-			return empty_string;
-		} else {
-			return itor->second;
-		}
-	};
-private:
-	const string_map& map_;
-
-};
-
-std::string interpolate_variables_into_string(const std::string &str, const string_map * const symbols)
-{
-	string_map_variable_set set(*symbols);
-	return do_interpolation(str, set);
-}
-
-std::string interpolate_variables_into_string(const std::string &str, const variable_set& variables)
-{
-	return do_interpolation(str, variables);
-}
-
 // Modify a number by string representing integer difference, or optionally %
 int apply_modifier( const int number, const std::string &amount, const int minimum ) {
 	// wassert( amount.empty() == false );
@@ -356,7 +213,6 @@ int apply_modifier( const int number, const std::string &amount, const int minim
 	return value;
 }
 
-//! Prepends a configurable set of characters with a backslash
 std::string &escape(std::string &str, const std::string& special_chars)
 {
 	std::string::size_type pos = 0;
@@ -370,16 +226,12 @@ std::string &escape(std::string &str, const std::string& special_chars)
 	return str;
 }
 
-//! Prepend all special characters with a backslash.
-// Special characters are:
-// #@{}+-,\*=
 std::string& escape(std::string& str)
 {
 	static const std::string special_chars("#@{}+-,\\*=");
 	return escape(str, special_chars);
 }
 
-//! Remove all escape characters (backslash)
 std::string &unescape(std::string &str)
 {
 	std::string::size_type pos = 0;
@@ -395,28 +247,29 @@ std::string &unescape(std::string &str)
 
 bool string_bool(const std::string& str, bool def) {
 	if (str.empty()) return def;
-	if (str == "yes" || str == "on" || str == "true"
-	|| lexical_cast_default<int>(str, 0)) {
-		return true;
-	}
-	if (str == "no" || str == "off" || str == "false"
-	|| !lexical_cast_default<int>(str, 1)) {
+
+	// yes/no is the standard, test it first
+	if (str == "yes") return true;
+	if (str == "no"|| str == "false" || str == "off" || str == "0" || str == "0.0")
 		return false;
-	}
-	return def;
+
+	// all other non-empty string are considered as true
+	return true;
 }
 
-bool is_username_char(char c) {
+static bool is_username_char(char c) {
 	return ((c == '_') || (c == '-'));
 }
 
-//! Check if the username is valid.
-//! (all alpha-numeric characters plus underscore and hyphen)
+static bool is_wildcard_char(char c) {
+    return ((c == '?') || (c == '*'));
+}
+
 bool isvalid_username(const std::string& username) {
 	const size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
 	const size_t valid_char =
 			std::count_if(username.begin(), username.end(), is_username_char);
-	if ((alnum + valid_char != username.size()) 
+	if ((alnum + valid_char != username.size())
 			|| valid_char == username.size() || username.empty() )
 	{
 		return false;
@@ -424,12 +277,21 @@ bool isvalid_username(const std::string& username) {
 	return true;
 }
 
-//! Try to complete the last word of 'text' with the 'wordlist'.
-//! @param[in]  'text'     Text where we try to complete the last word of.
-//! @param[out] 'text'     Text with completed last word.
-//! @param[in]  'wordlist' A vector of strings to complete against.
-//! @param[out] 'wordlist' A vector of strings that matched 'text'.
-//! @return 'true' iff text is just one word (no spaces)
+bool isvalid_wildcard(const std::string& username) {
+    const size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
+	const size_t valid_char =
+			std::count_if(username.begin(), username.end(), is_username_char);
+    const size_t wild_char =
+            std::count_if(username.begin(), username.end(), is_wildcard_char);
+	if ((alnum + valid_char + wild_char != username.size())
+			|| valid_char == username.size() || username.empty() )
+	{
+		return false;
+	}
+	return true;
+}
+
+
 bool word_completion(std::string& text, std::vector<std::string>& wordlist) {
 	std::vector<std::string> matches;
 	const size_t last_space = text.rfind(" ");
@@ -477,11 +339,10 @@ bool word_completion(std::string& text, std::vector<std::string>& wordlist) {
 	return text_start;
 }
 
-bool is_word_boundary(char c) {
+static bool is_word_boundary(char c) {
 	return (c == ' ' || c == ',' || c == ':' || c == '\'' || c == '"' || c == '-');
 }
 
-//! Check if a string contains a word.
 bool word_match(const std::string& message, const std::string& word) {
 	size_t first = message.find(word);
 	if (first == std::string::npos) return false;
@@ -494,8 +355,6 @@ bool word_match(const std::string& message, const std::string& word) {
 	return false;
 }
 
-//! Match using '*' as any number of characters (including none), 
-//! and '?' as any one character.
 bool wildcard_string_match(const std::string& str, const std::string& match) {
 	const bool wild_matching = (!match.empty() && match[0] == '*');
 	const std::string::size_type solid_begin = match.find_first_not_of('*');
@@ -531,25 +390,6 @@ bool wildcard_string_match(const std::string& str, const std::string& match) {
 	return matches;
 }
 
-std::string join(std::vector< std::string > const &v, char c)
-{
-	std::stringstream str;
-	for(std::vector< std::string >::const_iterator i = v.begin(); i != v.end(); ++i) {
-		str << *i;
-		if (i + 1 != v.end())
-			str << c;
-	}
-
-	return str.str();
-}
-
-// This function is identical to split(), except it does not split
-// when it otherwise would if the previous character was identical to the parameter 'quote'.
-// i.e. it does not split quoted commas.
-// This method was added to make it possible to quote user input,
-// particularly so commas in user input will not cause visual problems in menus.
-//
-//! @todo Why not change split()? That would change the methods post condition.
 std::vector< std::string > quoted_split(std::string const &val, char c, int flags, char quote)
 {
 	std::vector<std::string> res;
@@ -612,7 +452,7 @@ std::vector< std::pair< int, int > > parse_ranges(std::string const &str)
 	return to_return;
 }
 
-int byte_size_from_utf8_first(unsigned char ch)
+static int byte_size_from_utf8_first(unsigned char ch)
 {
 	int count;
 
@@ -634,7 +474,7 @@ int byte_size_from_utf8_first(unsigned char ch)
 	return count;
 }
 
-utf8_iterator::utf8_iterator(const std::string& str) : 
+utf8_iterator::utf8_iterator(const std::string& str) :
 	current_char(0),
 	string_end(str.end()),
 	current_substr(std::make_pair(str.begin(), str.begin()))
@@ -642,7 +482,7 @@ utf8_iterator::utf8_iterator(const std::string& str) :
 	update();
 }
 
-utf8_iterator::utf8_iterator(std::string::const_iterator const &beg, 
+utf8_iterator::utf8_iterator(std::string::const_iterator const &beg,
 		std::string::const_iterator const &end) :
 	current_char(0),
 	string_end(end),
@@ -753,7 +593,7 @@ std::string wstring_to_string(const wide_string &src)
 			}
 
 			if(count == 1) {
-				push_back(ret,static_cast<char>(ch));
+				ret.push_back(static_cast<char>(ch));
 			} else {
 				for(int j = static_cast<int>(count) - 1; j >= 0; --j) {
 					unsigned char c = (ch >> (6 * j)) & 0x3f;
@@ -761,7 +601,7 @@ std::string wstring_to_string(const wide_string &src)
 					if(j == static_cast<int>(count) - 1) {
 						c |= 0xff << (8 - count);
 					}
-					push_back(ret, c);
+					ret.push_back(c);
 				}
 			}
 
@@ -792,7 +632,7 @@ wide_string string_to_wstring(const std::string &src)
 
 		// Equivalent to res.insert(res.end(),i1,i2) which doesn't work on VC++6.
 		while(i1 != i2) {
-			push_back(res,*i1);
+			res.push_back(*i1);
 			++i1;
 		}
 	}
@@ -809,7 +649,7 @@ utf8_string capitalize(const utf8_string& s)
 	if(s.size() > 0) {
 		utf8_iterator itor(s);
 #if defined(__APPLE__) || defined(__AMIGAOS4__)
-		//! @todo FIXME: Should we support towupper on recent OSX platforms?
+		/** @todo FIXME: Should we support towupper on recent OSX platforms? */
 		wchar_t uchar = *itor;
 		if(uchar >= 0 && uchar < 0x100)
 			uchar = toupper(uchar);
@@ -831,7 +671,7 @@ utf8_string uppercase(const utf8_string& s)
 
 		for(;itor != utf8_iterator::end(s); ++itor) {
 #if defined(__APPLE__) || defined(__AMIGAOS4__)
-			//! @todo FIXME: Should we support towupper on recent OSX platforms?
+			/** @todo FIXME: Should we support towupper on recent OSX platforms? */
 			wchar_t uchar = *itor;
 			if(uchar >= 0 && uchar < 0x100)
 				uchar = toupper(uchar);
@@ -854,7 +694,7 @@ utf8_string lowercase(const utf8_string& s)
 
 		for(;itor != utf8_iterator::end(s); ++itor) {
 #if defined(__APPLE__) || defined(__OpenBSD__) || defined(__AMIGAOS4__)
-			//! @todo FIXME: Should we support towupper on recent OSX platforms?
+			/** @todo FIXME: Should we support towupper on recent OSX platforms? */
 			wchar_t uchar = *itor;
 			if(uchar >= 0 && uchar < 0x100)
 				uchar = tolower(uchar);
@@ -870,15 +710,6 @@ utf8_string lowercase(const utf8_string& s)
 	return s;
 }
 
-//! Truncates a string.
-//!
-//! If the string send has more than size utf-8 characters it will be truncated
-//! to this size. 
-//! No assumptions can be made about the actual size of the string.
-//! 
-//! @param[in]  str     String which can be converted to utf-8.
-//! @param[out] str     String which contains maximal size utf-8 characters.
-//! @param size         The size to truncate at.
 void truncate_as_wstring(std::string& str, const size_t size)
 {
 	wide_string utf8_str = utils::string_to_wstring(str);
@@ -889,17 +720,3 @@ void truncate_as_wstring(std::string& str, const size_t size)
 }
 
 } // end namespace utils
-
-std::string vgettext(const char *msgid, const utils::string_map& symbols)
-{
-	const std::string orig(_(msgid));
-	const std::string msg = utils::interpolate_variables_into_string(orig, &symbols);
-	return msg;
-}
-
-std::string vngettext(const char* sing, const char* plur, int n, const utils::string_map& symbols)
-{
-	const std::string orig(_n(sing, plur, n));
-	const std::string msg = utils::interpolate_variables_into_string(orig, &symbols);
-	return msg;
-}
