@@ -1,6 +1,6 @@
-/* $Id: metrics.cpp 26582 2008-05-13 11:11:38Z suokko $ */
+/* $Id: metrics.cpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,26 +12,24 @@
    See the COPYING file for more details.
 */
 
-//! @file server/metrics.cpp
-//! Various server-statistics.
+/**
+ * @file server/metrics.cpp
+ * Various server-statistics.
+ */
 
 #include "../global.hpp"
 
 #include "metrics.hpp"
 
 #include <algorithm>
-#include <time.h>
 #include <iostream>
 
-bool operator<(const metrics::sample& s, const simple_wml::string_span& name)
-{
-	return s.name < name;
-}
-
-bool operator<(const simple_wml::string_span& name, const metrics::sample& s)
-{
-	return name < s.name;
-}
+struct compare_samples_to_stringspan {
+	bool operator()(const simple_wml::string_span& a, const simple_wml::string_span& b)
+	{
+		return a < b;
+	}
+};
 
 struct compare_samples_by_time {
 	bool operator()(const metrics::sample& a, const metrics::sample& b) const {
@@ -40,9 +38,14 @@ struct compare_samples_by_time {
 };
 
 
-metrics::metrics() : most_consecutive_requests_(0),
-                     current_requests_(0), nrequests_(0),
-                     nrequests_waited_(0), started_at_(time(NULL))
+metrics::metrics() :
+	samples_(),
+	most_consecutive_requests_(0),
+	current_requests_(0),
+	nrequests_(0),
+	nrequests_waited_(0),
+	started_at_(time(NULL)),
+	terminations_()
 {}
 
 metrics::~metrics()
@@ -77,28 +80,27 @@ void metrics::no_requests()
 void metrics::record_sample(const simple_wml::string_span& name,
                             clock_t parsing_time, clock_t processing_time)
 {
-	std::pair<std::vector<sample>::iterator,std::vector<sample>::iterator>
-	    range = std::equal_range(samples_.begin(), samples_.end(), name);
-	if(range.first == range.second) {
+	std::vector<sample>::iterator isample = std::lower_bound(samples_.begin(), samples_.end(), name,compare_samples_to_stringspan());
+	if(isample == samples_.end()
+		|| isample->name != name) {
 		//protect against DoS with memory exhaustion
 		if(samples_.size() > 30) {
 			return;
 		}
-		int index = range.first - samples_.begin();
+		int index = isample - samples_.begin();
 		simple_wml::string_span dup_name(name.duplicate());
 		sample new_sample;
 		new_sample.name = dup_name;
-		new_sample.nsamples = 0;
-		new_sample.parsing_time = 0;
-		new_sample.processing_time = 0;
-		samples_.insert(range.first, new_sample);
+		samples_.insert(isample, new_sample);
 
-		range.first = samples_.begin() + index;
+		isample = samples_.begin() + index;
 	}
 
-	range.first->nsamples++;
-	range.first->parsing_time += parsing_time;
-	range.first->processing_time += processing_time;
+	isample->nsamples++;
+	isample->parsing_time += parsing_time;
+	isample->processing_time += processing_time;
+	isample->max_parsing_time = std::max(parsing_time,isample->max_parsing_time);
+	isample->max_processing_time = std::max(processing_time,isample->max_processing_time);
 }
 
 void metrics::game_terminated(const std::string& reason)
@@ -106,6 +108,21 @@ void metrics::game_terminated(const std::string& reason)
 	terminations_[reason]++;
 }
 
+std::ostream& metrics::games(std::ostream& out)
+{
+	if (terminations_.empty()) return out << "No game ended so far.";
+
+	size_t n = 0;
+	out << "Games have been terminated in the following ways:\n";
+	for(std::map<std::string,int>::const_iterator i = terminations_.begin(); i != terminations_.end(); ++i) {
+		out << i->first << ": " << i->second << "\n";
+		n += i->second;
+	}
+	out << "Total number of games = " << n;
+
+	return out;
+}
+
 std::ostream& operator<<(std::ostream& out, metrics& met)
 {
 	const time_t time_up = time(NULL) - met.started_at_;
@@ -119,24 +136,30 @@ std::ostream& operator<<(std::ostream& out, metrics& met)
 	    << minutes << " minutes, " << seconds << " seconds\n"
 	    << met.nrequests_ << " requests serviced. " << requests_immediate
 	    << " (" << percent_immediate << "%) "
-	    << " requests were serviced immediately\n"
-	    << "longest burst of requests was " << met.most_consecutive_requests_ << "\n";
+	    << "requests were serviced immediately.\n"
+	    << "longest burst of requests was: " << met.most_consecutive_requests_;
 
-	if(met.terminations_.empty() == false) {
-		out << "Games have been terminated in the following ways: \n";
-		for(std::map<std::string,int>::const_iterator i = met.terminations_.begin(); i != met.terminations_.end(); ++i) {
-			out << i->first << ": " << i->second << "\n";
-		}
-	}
+	if (met.samples_.empty()) return out;
 
 	std::vector<metrics::sample> ordered_samples = met.samples_;
 	std::sort(ordered_samples.begin(), ordered_samples.end(), compare_samples_by_time());
 
-	out << "\n\nRequest types:\n";
+	out << "\nSampled request types:\n";
 
+	size_t n = 0;
+	size_t pa = 0;
+	size_t pr = 0;
 	for(std::vector<metrics::sample>::const_iterator s = ordered_samples.begin(); s != ordered_samples.end(); ++s) {
-		out << "'" << s->name << "' called " << s->nsamples << " times " << s->parsing_time << " parsing time, " << s->processing_time << " processing time\n";
+		out << "'" << s->name << "' called " << s->nsamples << " times "
+			<< s->parsing_time << "("<< s->max_parsing_time <<") parsing time, "
+			<< s->processing_time << "("<<s->max_processing_time<<") processing time\n";
+		n += s->nsamples;
+		pa += s->parsing_time;
+		pr += s->processing_time;
 	}
+	out << "Total number of request samples = " << n << "\n"
+		<< "Total parsing time = " << pa << "\n"
+		<< "Total processing time = " << pr;
 
 	return out;
 }
