@@ -1,6 +1,6 @@
-/* $Id: metrics.hpp 26582 2008-05-13 11:11:38Z suokko $ */
+/* $Id: metrics.hpp 31859 2009-01-01 10:28:26Z mordante $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,7 @@
    See the COPYING file for more details.
 */
 
-//! @file server/metrics.hpp
-//!
+/** @file server/metrics.hpp */
 
 #ifndef METRICS_HPP_INCLUDED
 #define METRICS_HPP_INCLUDED
@@ -22,6 +21,9 @@
 
 #include <map>
 #include <string>
+#ifdef _WIN32
+#include <time.h>
+#endif
 
 #include "simple_wml.hpp"
 
@@ -39,12 +41,30 @@ public:
 
 	void game_terminated(const std::string& reason);
 
+	std::ostream& games(std::ostream& out);
 	friend std::ostream& operator<<(std::ostream& out, metrics& met);
 
 	struct sample {
+
+		sample() :
+			name(),
+			nsamples(0),
+			parsing_time(0),
+			processing_time(0),
+			max_parsing_time(0),
+			max_processing_time(0)
+		{
+		}
+
 		simple_wml::string_span name;
 		int nsamples;
 		clock_t parsing_time, processing_time;
+		clock_t max_parsing_time, max_processing_time;
+
+		operator const simple_wml::string_span&()
+		{
+			return name;
+		}
 	};
 
 private:
