@@ -1,20 +1,21 @@
-#include <assert.h>
-#include <algorithm>
 #include <iostream>
 #include <sstream>
-#include <string>
-#include <zlib.h>
 
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
 
-#include "../serialization/binary_wml.hpp"
 #include "simple_wml.hpp"
 
 namespace simple_wml {
 
 namespace {
 
+void debug_delete(node* n) {
+	if(n) {
+		delete n;
+	}
+}
+
 char* uncompress_buffer(const string_span& input, string_span* span)
 {
 	std::istringstream stream(std::string(input.begin(), input.end()));
@@ -27,6 +28,10 @@ char* uncompress_buffer(const string_span& input, string_span* span)
 	int len = 0;
 	int pos = 0;
 	while(filter.good() && (len = filter.read(&buf[pos], chunk_size).gcount()) == chunk_size) {
+		if(pos + chunk_size > 40000000) {
+			throw error("WML document exceeds 40MB limit");
+		}
+
 		pos += len;
 		buf.resize(pos + chunk_size);
 		len = 0;
@@ -57,7 +62,7 @@ char* compress_buffer(const char* input, string_span* span)
 
 	std::vector<char> buf(in.size()*2 + 80);
 	const int len = filter.read(&buf[0], buf.size()).gcount();
-	if(!filter.eof() && !filter.good() || len == buf.size()) {
+	if((!filter.eof() && !filter.good()) || len == static_cast<int>(buf.size())) {
 		throw error("failed to compress");
 	}
 
@@ -108,6 +113,7 @@ char* string_span::duplicate() const
 }
 
 error::error(const char* msg)
+  : message(msg)
 {
 	std::cerr << "ERROR: '" << msg << "'\n";
 }
@@ -118,15 +124,25 @@ std::ostream& operator<<(std::ostream& o, const string_span& s)
 	return o;
 }
 
-node::node(document& doc, node* parent)
-  : doc_(&doc), parent_(parent)
+node::node(document& doc, node* parent) :
+	doc_(&doc),
+	attr_(),
+	parent_(parent),
+	children_(),
+	ordered_children_(),
+	output_cache_()
 {
 }
 
-node::node(document& doc, node* parent, const char** str, int depth)
-  : doc_(&doc), parent_(parent)
+node::node(document& doc, node* parent, const char** str, int depth) :
+	doc_(&doc),
+	attr_(),
+	parent_(parent),
+	children_(),
+	ordered_children_(),
+	output_cache_()
 {
-	if(depth >= 30) {
+	if(depth >= 1000) {
 		throw error("elements nested too deep");
 	}
 
@@ -153,11 +169,14 @@ node::node(document& doc, node* parent, const char** str, int depth)
 				throw error("unterminated element");
 			}
 
-			child_list& list = get_children(string_span(s, end - s));
+			const int list_index = get_children(string_span(s, end - s));
+			check_ordered_children();
 
 			s = end + 1;
 
-			list.push_back(new node(doc, this, str, depth+1));
+			children_[list_index].second.push_back(new node(doc, this, str, depth+1));
+			ordered_children_.push_back(node_pos(list_index, children_[list_index].second.size() - 1));
+			check_ordered_children();
 
 			break;
 		}
@@ -236,14 +255,15 @@ node::node(document& doc, node* parent, const char** str, int depth)
 	}
 
 	output_cache_ = string_span(begin, s - begin);
+	check_ordered_children();
 }
 
 node::~node()
 {
 	for(child_map::iterator i = children_.begin(); i != children_.end(); ++i) {
 		for(child_list::iterator j = i->second.begin(); j != i->second.end(); ++j) {
-			delete *j;
-		}	
+			debug_delete(*j);
+		}
 	}
 }
 
@@ -330,12 +350,17 @@ node& node::add_child_at(const char* name, size_t index)
 {
 	set_dirty();
 
-	child_list& list = get_children(name);
+	const int list_index = get_children(name);
+	child_list& list = children_[list_index].second;
 	if(index > list.size()) {
 		index = list.size();
 	}
 
+	check_ordered_children();
 	list.insert(list.begin() + index, new node(*doc_, this));
+	insert_ordered_child(list_index, index);
+
+	check_ordered_children();
 	return *list[index];
 }
 
@@ -344,8 +369,12 @@ node& node::add_child(const char* name)
 {
 	set_dirty();
 
-	child_list& list = get_children(name);
+	const int list_index = get_children(name);
+	check_ordered_children();
+	child_list& list = children_[list_index].second;
 	list.push_back(new node(*doc_, this));
+	ordered_children_.push_back(node_pos(list_index, list.size() - 1));
+	check_ordered_children();
 	return *list.back();
 }
 
@@ -364,14 +393,115 @@ void node::remove_child(const string_span& name, size_t index)
 		return;
 	}
 
-	delete list[index];
+	remove_ordered_child(itor - children_.begin(), index);
+
+	debug_delete(list[index]);
 	list.erase(list.begin() + index);
 
 	if(list.empty()) {
+		remove_ordered_child_list(itor - children_.begin());
 		children_.erase(itor);
 	}
 }
 
+void node::insert_ordered_child(int child_map_index, int child_list_index)
+{
+	bool inserted = false;
+	std::vector<node_pos>::iterator i = ordered_children_.begin();
+	while(i != ordered_children_.end()) {
+		if(i->child_map_index == child_map_index && i->child_list_index > child_list_index) {
+			i->child_list_index++;
+		} else if(i->child_map_index == child_map_index && i->child_list_index == child_list_index) {
+			inserted = true;
+			i->child_list_index++;
+			i = ordered_children_.insert(i, node_pos(child_map_index, child_list_index));
+			++i;
+		}
+
+		++i;
+	}
+
+	if(!inserted) {
+		ordered_children_.push_back(node_pos(child_map_index, child_list_index));
+	}
+}
+
+void node::remove_ordered_child(int child_map_index, int child_list_index)
+{
+	int erase_count = 0;
+	std::vector<node_pos>::iterator i = ordered_children_.begin();
+	while(i != ordered_children_.end()) {
+		if(i->child_map_index == child_map_index && i->child_list_index == child_list_index) {
+			i = ordered_children_.erase(i);
+			++erase_count;
+		} else {
+			if(i->child_map_index == child_map_index && i->child_list_index > child_list_index) {
+				i->child_list_index--;
+			}
+			++i;
+		}
+	}
+
+	assert(erase_count == 1);
+}
+
+void node::insert_ordered_child_list(int child_map_index)
+{
+	std::vector<node_pos>::iterator i = ordered_children_.begin();
+	while(i != ordered_children_.end()) {
+		if(i->child_map_index >= child_map_index) {
+			i->child_map_index++;
+		}
+	}
+}
+
+void node::remove_ordered_child_list(int child_map_index)
+{
+	std::vector<node_pos>::iterator i = ordered_children_.begin();
+	while(i != ordered_children_.end()) {
+		if(i->child_map_index == child_map_index) {
+			assert(false);
+			i = ordered_children_.erase(i);
+		} else {
+			if(i->child_map_index > child_map_index) {
+				i->child_map_index--;
+			}
+
+			++i;
+		}
+	}
+}
+
+void node::check_ordered_children() const
+{
+// only define this symbol in debug mode to work out child ordering.
+#ifdef CHECK_ORDERED_CHILDREN
+	std::vector<node_pos>::const_iterator i = ordered_children_.begin();
+	while(i != ordered_children_.end()) {
+		assert(i->child_map_index < children_.size());
+		assert(i->child_list_index < children_[i->child_map_index].second.size());
+		++i;
+	}
+
+	for(child_map::const_iterator j = children_.begin(); j != children_.end(); ++j) {
+		const unsigned short child_map_index = j - children_.begin();
+		for(child_list::const_iterator k = j->second.begin(); k != j->second.end(); ++k) {
+			const unsigned short child_list_index = k - j->second.begin();
+			bool found = false;
+			for(int n = 0; n != ordered_children_.size(); ++n) {
+				if(ordered_children_[n].child_map_index == child_map_index &&
+				   ordered_children_[n].child_list_index == child_list_index) {
+					found = true;
+					break;
+				}
+			}
+
+			assert(found);
+		}
+	}
+#endif // CHECK_ORDERED_CHILDREN
+}
+
 void node::remove_child(const char* name, size_t index)
 {
 	remove_child(string_span(name), index);
@@ -416,21 +546,21 @@ const node::child_list& node::children(const char* name) const
 	return empty;
 }
 
-node::child_list& node::get_children(const char* name)
+int node::get_children(const char* name)
 {
 	return get_children(string_span(name));
 }
 
-node::child_list& node::get_children(const string_span& name)
+int node::get_children(const string_span& name)
 {
 	for(child_map::iterator i = children_.begin(); i != children_.end(); ++i) {
 		if(i->first == name) {
-			return i->second;
+			return i - children_.begin();
 		}
 	}
 
 	children_.push_back(child_pair(string_span(name), child_list()));
-	return children_.back().second;
+	return children_.size() - 1;
 }
 
 node::child_map::const_iterator node::find_in_map(const child_map& m, const string_span& attr)
@@ -469,6 +599,7 @@ const string_span& node::first_child() const
 
 int node::output_size() const
 {
+	check_ordered_children();
 	if(output_cache_.empty() == false) {
 		return output_cache_.size();
 	}
@@ -478,13 +609,17 @@ int node::output_size() const
 		res += i->first.size() + i->second.size() + 4;
 	}
 
+	size_t count_children = 0;
 	for(child_map::const_iterator i = children_.begin(); i != children_.end(); ++i) {
 		for(child_list::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
 			res += i->first.size()*2 + 7;
 			res += (*j)->output_size();
+			++count_children;
 		}
 	}
 
+	assert(count_children == ordered_children_.size());
+
 	return res;
 }
 
@@ -532,27 +667,41 @@ void node::output(char*& buf)
 		*buf++ = '\n';
 	}
 
-	for(child_map::iterator i = children_.begin(); i != children_.end(); ++i) {
-		assert(i->second.empty() == false);
-		for(child_list::iterator j = i->second.begin(); j != i->second.end(); ++j) {
-			*buf++ = '[';
-			memcpy(buf, i->first.begin(), i->first.size());
-			i->first = string_span(buf, i->first.size());
-			buf += i->first.size();
-			*buf++ = ']';
-			*buf++ = '\n';
-			(*j)->output(buf);
-			*buf++ = '[';
-			*buf++ = '/';
-			memcpy(buf, i->first.begin(), i->first.size());
-			buf += i->first.size();
-			*buf++ = ']';
-			*buf++ = '\n';
-		}
+	for(std::vector<node_pos>::const_iterator i = ordered_children_.begin();
+	    i != ordered_children_.end(); ++i) {
+		assert(i->child_map_index < children_.size());
+		assert(i->child_list_index < children_[i->child_map_index].second.size());
+		string_span& attr = children_[i->child_map_index].first;
+		*buf++ = '[';
+		memcpy(buf, attr.begin(), attr.size());
+		attr = string_span(buf, attr.size());
+		buf += attr.size();
+		*buf++ = ']';
+		*buf++ = '\n';
+		children_[i->child_map_index].second[i->child_list_index]->output(buf);
+		*buf++ = '[';
+		*buf++ = '/';
+		memcpy(buf, attr.begin(), attr.size());
+		buf += attr.size();
+		*buf++ = ']';
+		*buf++ = '\n';
 	}
 
 	output_cache_ = string_span(begin, buf - begin);
 }
+const char* node::output() {
+	const char* out;
+
+	const int buf_size = output_size() + 1;
+	char* buf = new char[buf_size];
+	out = buf;
+
+	output(buf);
+	*buf++ = 0;
+	assert(buf == out + buf_size);
+
+	return out;
+}
 
 void node::copy_into(node& n) const
 {
@@ -565,17 +714,13 @@ void node::copy_into(node& n) const
 		n.set_attr(key, value);
 	}
 
-	for(child_map::const_iterator i = children_.begin(); i != children_.end(); ++i) {
-		if(i->second.empty()) {
-			continue;
-		}
-
-		char* buf = i->first.duplicate();
+	for(std::vector<node_pos>::const_iterator i = ordered_children_.begin();
+	    i != ordered_children_.end(); ++i) {
+		assert(i->child_map_index < children_.size());
+		assert(i->child_list_index < children_[i->child_map_index].second.size());
+		char* buf = children_[i->child_map_index].first.duplicate();
 		n.doc_->take_ownership_of_buffer(buf);
-
-		for(child_list::const_iterator j = i->second.begin(); j != i->second.end(); ++j) {
-			(*j)->copy_into(n.add_child(buf));
-		}
+		children_[i->child_map_index].second[i->child_list_index]->copy_into(n.add_child(buf));
 	}
 }
 
@@ -690,13 +835,24 @@ void node::set_dirty()
 	}
 }
 
-document::document() : output_(NULL),
-					   root_(new node(*this, NULL))
+document::document() :
+	compressed_buf_(),
+	output_(NULL),
+	buffers_(),
+	root_(new node(*this, NULL)),
+	prev_(NULL),
+	next_(NULL)
 {
 	attach_list();
 }
 
-document::document(char* buf, INIT_BUFFER_CONTROL control) : output_(buf), root_(NULL)
+document::document(char* buf, INIT_BUFFER_CONTROL control) :
+	compressed_buf_(),
+	output_(buf),
+	buffers_(),
+	root_(NULL),
+	prev_(NULL),
+	next_(NULL)
 {
 	if(control == INIT_TAKE_OWNERSHIP) {
 		buffers_.push_back(buf);
@@ -707,10 +863,14 @@ document::document(char* buf, INIT_BUFFER_CONTROL control) : output_(buf), root_
 	attach_list();
 }
 
-document::document(const char* buf, INIT_STATE state) : output_(NULL),
-                                                        root_(NULL)
+document::document(const char* buf, INIT_STATE state) :
+	compressed_buf_(),
+	output_(buf),
+	buffers_(),
+	root_(NULL),
+	prev_(NULL),
+	next_(NULL)
 {
-	output_ = buf;
 	if(state == INIT_COMPRESSED) {
 		output_compressed();
 		output_ = NULL;
@@ -721,10 +881,13 @@ document::document(const char* buf, INIT_STATE state) : output_(NULL),
 	attach_list();
 }
 
-document::document(string_span compressed_buf)
-  : compressed_buf_(compressed_buf),
-    output_(NULL),
-	root_(NULL)
+document::document(string_span compressed_buf) :
+	compressed_buf_(compressed_buf),
+	output_(NULL),
+	buffers_(),
+	root_(NULL),
+	prev_(NULL),
+	next_(NULL)
 {
 	string_span uncompressed_buf;
 	buffers_.push_back(uncompress_buffer(compressed_buf, &uncompressed_buf));
@@ -748,7 +911,7 @@ document::~document()
 	}
 
 	buffers_.clear();
-	delete root_;
+	debug_delete(root_);
 
 	detach_list();
 }
@@ -816,7 +979,7 @@ string_span document::output_compressed()
 void document::compress()
 {
 	output_compressed();
-	delete root_;
+	debug_delete(root_);
 	root_ = NULL;
 	output_ = NULL;
 	std::vector<char*> new_buffers;
@@ -841,6 +1004,7 @@ void document::generate_root()
 		output_ = uncompressed_buf.begin();
 	}
 
+	assert(root_ == NULL);
 	const char* cbuf = output_;
 	root_ = new node(*this, NULL, &cbuf);
 }
@@ -867,7 +1031,7 @@ void document::clear()
 {
 	compressed_buf_ = string_span();
 	output_ = NULL;
-	delete root_;
+	debug_delete(root_);
 	root_ = new node(*this, NULL);
 	for(std::vector<char*>::iterator i = buffers_.begin(); i != buffers_.end(); ++i) {
 		delete [] *i;
