@@ -1,6 +1,6 @@
-/* $Id: sound.cpp 24196 2008-03-02 07:16:28Z sapient $ */
+/* $Id: sound.cpp 33429 2009-03-08 14:50:25Z shadowmaster $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -13,21 +13,14 @@
 */
 
 #include "global.hpp"
-#include "events.hpp"
 #include "filesystem.hpp"
-#include "game_config.hpp"
+#include "foreach.hpp"
 #include "game_preferences.hpp"
 #include "log.hpp"
-#include "random.hpp"
 #include "sound.hpp"
-#include "wesconfig.h"
 
-#include "SDL.h"
 #include "SDL_mixer.h"
 
-#include <cassert>
-#include <iostream>
-#include <map>
 #include <list>
 
 #define LOG_AUDIO LOG_STREAM(info, audio)
@@ -41,8 +34,8 @@ std::vector<Mix_Chunk*> channel_chunks;
 // is playing on a channel for fading/panning)
 std::vector<int> channel_ids;
 
-static bool play_sound_internal(const std::string& files, channel_group group, bool sound_on, unsigned int repeats=0,
-						 unsigned int distance=0, int id=-1, int loop_ticks=0, int fadein_ticks=0);
+static void play_sound_internal(const std::string& files, channel_group group, unsigned int repeats=0,
+				 unsigned int distance=0, int id=-1, int loop_ticks=0, int fadein_ticks=0);
 }
 
 namespace {
@@ -55,11 +48,11 @@ bool want_new_music = false;
 int fadingout_time=5000;
 bool no_fading = false;
 
-// number of allocated channels, 
+// number of allocated channels,
 const size_t n_of_channels = 16;
 
 // we need 2 channels, because we it for timer as well
-const size_t bell_channel = 0; 
+const size_t bell_channel = 0;
 const size_t timer_channel = 1;
 
 // number of channels reserved for sound sources
@@ -168,17 +161,11 @@ music_track::music_track(const std::string &tname)
 music_track::music_track(const std::string &tname,
 						 const std::string &ms_before_str,
 						 const std::string &ms_after_str)
-	: name(tname), once(false)
+	: name(tname),
+	ms_before(lexical_cast_default<int>(ms_before_str)),
+	ms_after(lexical_cast_default<int>(ms_after_str)),
+	once(false)
 {
-	if (ms_before_str.empty())
-		ms_before = 0;
-	else
-		ms_before = lexical_cast<int,std::string>(ms_before_str);
-
-	if (ms_after_str.empty())
-		ms_after = 0;
-	else
-		ms_after = lexical_cast<int,std::string>(ms_after_str);
 }
 
 void music_track::write(config &snapshot, bool append)
@@ -407,8 +394,9 @@ void reset_sound() {
 
 	if (music || sound || bell || UI_sound) {
 		sound::close_sound();
-		if (!sound::init_sound())
+		if (!sound::init_sound()) {
 			ERR_AUDIO << "Error initializing audio device: " << Mix_GetError() << "\n";
+		}
 		if (!music)
 			sound::stop_music();
 		if (!sound)
@@ -433,10 +421,8 @@ void stop_music() {
 
 void stop_sound() {
 	if(mix_ok) {
-		{
-			Mix_HaltGroup(SOUND_SOURCES);
-			Mix_HaltGroup(SOUND_FX);
-		}
+		Mix_HaltGroup(SOUND_SOURCES);
+		Mix_HaltGroup(SOUND_FX);
 		sound_cache_iterator itor = sound_cache.begin();
 		while(itor != sound_cache.end())
 		{
@@ -501,6 +487,11 @@ void play_no_music()
 	}
 }
 
+void empty_playlist()
+{
+    current_track_list = std::vector<music_track>();
+}
+
 void play_music()
 {
 	music_start_time = 1; //immediate (same as effect as SDL_GetTicks())
@@ -509,10 +500,11 @@ void play_music()
 	fadingout_time=current_track.ms_after;
 }
 
-void play_new_music()
+static void play_new_music()
 {
 	music_start_time = 0; //reset status: no start time
-	want_new_music=true;
+	want_new_music = true;
+
 	if(!preferences::music_on() || !mix_ok || current_track.name.empty())
 		return;
 
@@ -521,7 +513,15 @@ void play_new_music()
 		const std::string& filename = get_binary_file_location("music", current_track.name);
 
 		if(filename.empty()) {
-			ERR_AUDIO << "Could not open track '" << current_track.name << "'\n";
+			const std::string bad_track = current_track.name;
+			ERR_AUDIO << "Could not open track '" << current_track.name << "', disabling for this playlist\n";
+
+			current_track.name = "";
+			foreach(music_track& trk, current_track_list) {
+				if(trk.name == bad_track) {
+					trk.name = "";
+				}
+			}
 			return;
 		}
 
@@ -534,17 +534,20 @@ void play_new_music()
 		itor = music_cache.insert(std::pair<std::string,Mix_Music*>(current_track.name,music)).first;
 		last_track=current_track;
 	}
+
 	LOG_AUDIO << "Playing track '" << current_track.name << "'\n";
 	int fading_time=current_track.ms_before;
 	if(no_fading)
 	{
 		fading_time=0;
 	}
+
 	const int res = Mix_FadeInMusic(itor->second, 1, fading_time);
 	if(res < 0)
 	{
 		ERR_AUDIO << "Could not play music: " << Mix_GetError() << " " << current_track.name <<" \n";
 	}
+
 	want_new_music=false;
 }
 
@@ -565,9 +568,7 @@ void play_music_repeatedly(const std::string &name)
 
 void play_music_config(const config &music)
 {
-	struct music_track track(music["name"],
-							 music["ms_before"],
-							 music["ms_after"]);
+	struct music_track track(music["name"], music["ms_before"], music["ms_after"]);
 
 	// If they say play once, we don't alter playlist.
 	if (utils::string_bool(music["play_once"])) {
@@ -583,14 +584,14 @@ void play_music_config(const config &music)
 	}
 
 	// Avoid 2 tracks with the same name, since that can cause an infinite loop
-	// in choose_track(), 2 tracks with the same name will always return the 
+	// in choose_track(), 2 tracks with the same name will always return the
 	// current track and track_ok() doesn't allow that.
 	std::vector<music_track>::const_iterator itor = current_track_list.begin();
 	while(itor != current_track_list.end()) {
 		if(itor->name == track.name) break;
 		++itor;
 	}
-	
+
 	if(itor == current_track_list.end()) {
 		current_track_list.push_back(track);
 	} else {
@@ -613,9 +614,11 @@ void music_thinker::process(events::pump_info &info) {
 			no_fading=true;
 			fadingout_time=0;
 		}
+
 		if(music_start_time && info.ticks(&music_refresh, music_refresh_rate) >= music_start_time - fadingout_time) {
 			want_new_music=true;
 		}
+
 		if(want_new_music) {
 			if(Mix_PlayingMusic()) {
 				Mix_FadeOutMusic(fadingout_time);
@@ -665,13 +668,12 @@ void write_music_play_list(config& snapshot)
 
 void reposition_sound(int id, unsigned int distance)
 {
-	audio_lock lock();
+	audio_lock lock;
 	for(unsigned int ch = 0; ch < channel_ids.size(); ++ch) {
 		int& ch_id = channel_ids[ch];
 		if(ch_id == id) {
 			if(distance >= DISTANCE_SILENT) {
 				Mix_FadeOutChannel(ch, 100);
-				ch_id = -1;
 			}
 			else {
 				Mix_SetDistance(ch, distance);
@@ -682,7 +684,7 @@ void reposition_sound(int id, unsigned int distance)
 
 bool is_sound_playing(int id)
 {
-	audio_lock lock();
+	audio_lock lock;
 	return std::find(channel_ids.begin(), channel_ids.end(), id) != channel_ids.end();
 }
 
@@ -691,36 +693,19 @@ void stop_sound(int id)
 	reposition_sound(id, DISTANCE_SILENT);
 }
 
-void play_sound_positioned(const std::string &files, int id, unsigned int distance)
+void play_sound_positioned(const std::string &files, int id, int repeats, unsigned int distance)
 {
-	play_sound_internal(files, SOUND_SOURCES, preferences::sound_on(), 0, distance, id);
+	if(preferences::sound_on()) {
+		play_sound_internal(files, SOUND_SOURCES, repeats, distance, id);
+	}
 }
 
-bool play_sound_internal(const std::string& files, channel_group group, bool sound_on, unsigned int repeats, 
-			unsigned int distance, int id, int loop_ticks, int fadein_ticks)
-{
-	if(files.empty() || distance >= DISTANCE_SILENT || !sound_on || !mix_ok) {
-		return false;
-	}
+struct chunk_load_exception { };
 
-	std::string file = pick_one(files);
+static Mix_Chunk* load_chunk(const std::string& file, channel_group group)
+{
 	sound_cache_iterator it_bgn, it_end;
 	sound_cache_iterator it;
-	int channel;
-
-	audio_lock lock();
-
-	// find a free channel in the desired group
-	channel = Mix_GroupAvailable(group);
-	if(channel == -1) {
-		LOG_AUDIO << "All channels dedicated to sound group(" << group << ") are busy, skipping.\n";
-		return false;
-	}
-	channel_ids[channel] = id;
-	if(group != SOUND_UI) {
-		//FIXME: why is this (group != SOUND_UI) check necessary?
-		Mix_SetDistance(channel, distance);
-	}
 
 	sound_cache_chunk temp_chunk(file); // search the sound cache on this key
 	it_bgn = sound_cache.begin(), it_end = sound_cache.end();
@@ -731,6 +716,7 @@ bool play_sound_internal(const std::string& files, channel_group group, bool sou
 			// cached item has been used in multiple sound groups
 			it->group = NULL_CHANNEL;
 		}
+
 		//splice the most recently used chunk to the front of the cache
 		sound_cache.splice(it_bgn, sound_cache, it);
 	} else {
@@ -746,28 +732,70 @@ bool play_sound_internal(const std::string& files, channel_group group, bool sou
 		}
 		if(cache_full) {
 			LOG_AUDIO << "Maximum sound cache size reached and all are busy, skipping.\n";
-			return false;
+			throw chunk_load_exception();
 		}
 		temp_chunk.group = group;
 		std::string const &filename = get_binary_file_location("sounds", file);
+
 		if (!filename.empty()) {
 			temp_chunk.set_data(Mix_LoadWAV(filename.c_str()));
+		} else {
+			ERR_AUDIO << "Could not load sound with empty filename\n";
+			throw chunk_load_exception();
 		}
+
 		if (temp_chunk.get_data() == NULL) {
 			ERR_AUDIO << "Could not load sound file '" << filename << "': "
 				<< Mix_GetError() << "\n";
-			return false;
+			throw chunk_load_exception();
 		}
+
 		sound_cache.push_front(temp_chunk);
-		it = sound_cache.begin();
+	}
+
+	return sound_cache.begin()->get_data();
+}
+
+void play_sound_internal(const std::string& files, channel_group group, unsigned int repeats,
+			unsigned int distance, int id, int loop_ticks, int fadein_ticks)
+{
+	if(files.empty() || distance >= DISTANCE_SILENT || !mix_ok) {
+		return;
+	}
+
+	audio_lock lock;
+
+	// find a free channel in the desired group
+	int channel = Mix_GroupAvailable(group);
+	if(channel == -1) {
+		LOG_AUDIO << "All channels dedicated to sound group(" << group << ") are busy, skipping.\n";
+		return;
+	}
+
+	Mix_Chunk *chunk;
+	std::string file = pick_one(files);
+
+	try {
+		chunk = load_chunk(file, group);
+		assert(chunk);
+	} catch(const chunk_load_exception&) {
+		return;
+	}
+
+	/*
+	 * This check prevents SDL_Mixer from blowing up on Windows when UI sound is played
+	 * in response to toggling the checkbox which disables sound.
+	 */
+	if(group != SOUND_UI) {
+		Mix_SetDistance(channel, distance);
 	}
 
 	int res;
 	if(loop_ticks > 0) {
 		if(fadein_ticks > 0) {
-			res = Mix_FadeInChannelTimed(channel, it->get_data(), -1, fadein_ticks, loop_ticks);
+			res = Mix_FadeInChannelTimed(channel, chunk, -1, fadein_ticks, loop_ticks);
 		} else {
-			res = Mix_PlayChannel(channel, it->get_data(), -1);
+			res = Mix_PlayChannel(channel, chunk, -1);
 		}
 
 		if(res >= 0) {
@@ -775,43 +803,53 @@ bool play_sound_internal(const std::string& files, channel_group group, bool sou
 		}
 	} else {
 		if(fadein_ticks > 0) {
-			res = Mix_FadeInChannel(channel, it->get_data(), repeats, fadein_ticks);
+			res = Mix_FadeInChannel(channel, chunk, repeats, fadein_ticks);
 		} else {
-			res = Mix_PlayChannel(channel, it->get_data(), repeats);
+			res = Mix_PlayChannel(channel, chunk, repeats);
 		}
 	}
+
 	if(res < 0) {
 		ERR_AUDIO << "error playing sound effect: " << Mix_GetError() << "\n";
 		//still keep it in the sound cache, in case we want to try again later
-		return false;
+		return;
 	}
 
+	channel_ids[channel] = id;
+
 	//reserve the channel's chunk from being freed, since it is playing
-	channel_chunks[res] = it->get_data();
-	return true;
+	channel_chunks[res] = chunk;
 }
 
 void play_sound(const std::string& files, channel_group group, unsigned int repeats)
 {
-	play_sound_internal(files, group, preferences::sound_on(), repeats);
+	if(preferences::sound_on()) {
+		play_sound_internal(files, group, repeats);
+	}
 }
 
 // Play bell with separate volume setting
 void play_bell(const std::string& files)
 {
-	play_sound_internal(files, SOUND_BELL, preferences::turn_bell());
+	if(preferences::sound_on() && preferences::turn_bell()) {
+		play_sound_internal(files, SOUND_BELL);
+	}
 }
 
 // Play timer with separate volume setting
 void play_timer(const std::string& files, int loop_ticks, int fadein_ticks)
 {
-	play_sound_internal(files, SOUND_TIMER, true, 0, 0, -1, loop_ticks, fadein_ticks);
+	if(preferences::sound_on()) {
+		play_sound_internal(files, SOUND_TIMER, 0, 0, -1, loop_ticks, fadein_ticks);
+	}
 }
 
 // Play UI sounds on separate volume than soundfx
 void play_UI_sound(const std::string& files)
 {
-	play_sound_internal(files, SOUND_UI, preferences::UI_sound_on());
+	if(preferences::UI_sound_on()) {
+		play_sound_internal(files, SOUND_UI);
+	}
 }
 
 
@@ -820,6 +858,7 @@ void set_music_volume(int vol)
 	if(mix_ok && vol >= 0) {
 		if(vol > MIX_MAX_VOLUME)
 			vol = MIX_MAX_VOLUME;
+
 		Mix_VolumeMusic(vol);
 	}
 }
