@@ -1,6 +1,6 @@
-/* $Id: sound.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: sound.hpp 32112 2009-01-13 10:26:54Z ilor $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,7 @@
 #include <string>
 #include <vector>
 #include "config.hpp"
+#include "events.hpp"
 
 namespace sound {
 
@@ -47,8 +48,10 @@ void commit_music_changes();
 void play_music_repeatedly(const std::string &name);
 // Play this particular music file once, then silence.
 void play_music_once(const std::string &name);
-// Empty the playlist.
+// Empty the playlist (and stop playing)
 void play_no_music();
+// Empty the playlist
+void empty_playlist();
 // Start playing current music.
 void play_music();
 
@@ -67,7 +70,7 @@ void play_sound(const std::string& files, channel_group group = SOUND_FX, unsign
 
 // Play sound, or random one of comma-separated sounds. Use specified
 // distance and associate it with specified id (of a sound source).
-void play_sound_positioned(const std::string &files, int id, unsigned int distance);
+void play_sound_positioned(const std::string &files, int id, int repeats, unsigned int distance);
 
 // Play sound, or random one of comma-separated sounds in bell channel
 void play_bell(const std::string& files);
