@@ -1,6 +1,6 @@
-/* $Id: team.cpp 29170 2008-09-01 17:41:59Z suokko $ */
+/* $Id: team.cpp 33693 2009-03-15 17:46:14Z jhinrichs $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,26 +12,39 @@
    See the COPYING file for more details.
 */
 
-//! @file team.cpp
-//! Team-management, allies, setup at start of scenario.
+/**
+ *  @file team.cpp
+ *  Team-management, allies, setup at start of scenario.
+ */
 
 #include "global.hpp"
 
+#include "foreach.hpp"
 #include "game_events.hpp"
-#include "game_errors.hpp"
 #include "gamestatus.hpp"
 #include "log.hpp"
-#include "network.hpp"
-#include "team.hpp"
-#include "util.hpp"
-#include "serialization/string_utils.hpp"
+#include "map.hpp"
 
-#include <algorithm>
-#include <cassert>
-#include <cstdlib>
-#include <sstream>
 
 #define LOG_NG LOG_STREAM(info, engine)
+#define WRN_NG LOG_STREAM(warn, engine)
+
+//some default values for the AI parameters following the default values listed
+//in the wiki at http://www.wesnoth.org/wiki/AiWML
+
+//A number 0 or higher which determines how many scouts the AI recruits. If 0,
+//the AI doesn't recruit scouts to capture villages.
+const int ai_default_villages_per_scout = 4;
+//A number 0 or higher which determines how much the AI targets enemy leaders.
+const double ai_default_leader_value = 3.0;
+//A number 0 or higher which determines how much the AI tries to capture villages.
+const double ai_default_village_value = 1.0;
+//details see in the [AI] tag explaination in the wiki:
+//http://www.wesnoth.org/wiki/AiWML#the_.5Bai.5D_tag
+const double ai_default_aggression = 0.5;
+//details see in the [AI] tag explaination in the wiki:
+//http://www.wesnoth.org/wiki/AiWML#the_.5Bai.5D_tag
+const double ai_default_caution = 0.25;
 
 namespace {
 	std::vector<team>* teams = NULL;
@@ -42,6 +55,7 @@ const int team::default_team_gold = 100;
 
 teams_manager::teams_manager(std::vector<team>& teams_list)
 {
+	assert(!teams);
 	teams = &teams_list;
 }
 
@@ -50,10 +64,16 @@ teams_manager::~teams_manager()
 	teams = NULL;
 }
 
+const std::vector<team>& teams_manager::get_teams()
+{
+	assert(teams);
+	return *teams;
+}
+
 bool teams_manager::is_observer()
 {
 	for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
-		if(i->is_human() || i->is_persistent()) {
+		if(i->is_local()) {
 			return false;
 		}
 	}
@@ -77,6 +97,8 @@ team::team_info::team_info(const config& cfg) :
 		start_gold(),
 		income(cfg["income"]),
 		income_per_village(),
+		average_price(0),
+		number_of_possible_recruits_to_force_recruit(lexical_cast_default<float>(cfg["number_of_possible_recruits_to_force_recruit"],0.0)),
 		can_recruit(),
 		global_recruitment_pattern(utils::split(cfg["global_recruitment_pattern"])),
 		recruitment_pattern(utils::split(cfg["recruitment_pattern"])),
@@ -89,35 +111,37 @@ team::team_info::team_info(const config& cfg) :
 		action_bonus_count(lexical_cast_default<int>(cfg["action_bonus_count"])),
 		flag(cfg["flag"]),
 		flag_icon(cfg["flag_icon"]),
-		description(cfg["description"]),
+		description(cfg["id"]),
 		objectives(cfg["objectives"]),
 		objectives_changed(utils::string_bool(cfg["objectives_changed"])),
 		controller(),
-		persistent(),
 		ai_algorithm(cfg["ai_algorithm"]),
 		ai_params(),
 		ai_memory_(),
-		villages_per_scout(),
-		leader_value(0.0),
-		village_value(0.0),
-		aggression_(0.5), 
-		caution_(0.25),
+		villages_per_scout(ai_default_villages_per_scout),
+		leader_value(ai_default_leader_value),
+		village_value(ai_default_village_value),
+		aggression_(ai_default_aggression),
+		caution_(ai_default_caution),
 		targets(),
 		share_maps(false),
 		share_view(false),
 		disallow_observers(utils::string_bool(cfg["disallow_observers"])),
 		allow_player(utils::string_bool(cfg["allow_player"], true)),
 		no_leader(utils::string_bool(cfg["no_leader"])),
+		hidden(utils::string_bool(cfg["hidden"])),
 		music(cfg["music"]),
 		colour(cfg["colour"].size() ? cfg["colour"] : cfg["side"])
 {
+	// If are starting new scenario overide settings from [ai] tags
+	if (!user_team_name.translatable())
+		user_team_name = user_team_name.from_serialized(user_team_name);
 	config global_ai_params;
 	const config::child_list& ai_parameters = cfg.get_children("ai");
-	for(config::child_list::const_reverse_iterator aiparam = ai_parameters.rbegin(); aiparam != ai_parameters.rend(); ++aiparam) {
+	for(config::child_list::const_iterator aiparam = ai_parameters.begin(); aiparam != ai_parameters.end(); ++aiparam) {
 		ai_params.push_back(**aiparam);
 
 		if((**aiparam)["turns"].empty() && (**aiparam)["time_of_day"].empty()) {
-//			LOG_NG << "Global ai entry: " << **aiparam << "\n";
 			global_ai_params.append(**aiparam);
 		}
 	}
@@ -163,7 +187,7 @@ team::team_info::team_info(const config& cfg) :
 	if(village_income.empty())
 		income_per_village = game_config::village_income;
 	else
-		income_per_village = atoi(village_income.c_str());
+		income_per_village = lexical_cast_default<int>(village_income, game_config::village_income);
 
 	const std::string& enemies_list = cfg["enemy"];
 	if(!enemies_list.empty()) {
@@ -176,82 +200,91 @@ team::team_info::team_info(const config& cfg) :
 	std::string control = cfg["controller"];
 	if (control == "human")
 		controller = HUMAN;
+	else if (control == "human_ai")
+		controller = HUMAN_AI;
 	else if (control == "network")
 		controller = NETWORK;
+	else if (control == "network_ai")
+		controller = NETWORK_AI;
 	else if (control == "null")
+	{
+		disallow_observers = utils::string_bool(cfg["disallow_observers"],true);
 		controller = EMPTY;
+	}
 	else
 		controller = AI;
 
-	std::string persist = cfg["persistent"];
-	if (persist.empty()) persistent = controller == HUMAN;
-	else persistent = persist == "1";
-
-	if(ai_algorithm.empty()) {
+	if(ai_algorithm.empty())
+	{
 		ai_algorithm = global_ai_params["ai_algorithm"];
 	}
 
+	if (!cfg.has_attribute("number_of_possible_recruits_to_force_recruit"))
+	{
+		number_of_possible_recruits_to_force_recruit = lexical_cast_default<float>(global_ai_params["number_of_possible_recruits_to_force_recruit"],3.1);
+	}
+
 	std::string scouts_val = cfg["villages_per_scout"];
 
-	if(scouts_val.empty()
-) {
+	if(scouts_val.empty())
+	{
 		scouts_val = global_ai_params["villages_per_scout"];
 	}
 
 	if(scouts_val.empty()) {
-		villages_per_scout = 4;
+		villages_per_scout = ai_default_villages_per_scout;
 	} else {
 		villages_per_scout = atoi(scouts_val.c_str());
 	}
 
 	std::string leader_val = cfg["leader_value"];
 
-	if(leader_val.empty()
-) {
+	if(leader_val.empty())
+	{
 		leader_val = global_ai_params["leader_value"];
 	}
 
 	if(leader_val.empty()) {
-		leader_value = 3.0;
+		leader_value = ai_default_leader_value;
 	} else {
 		leader_value = atof(leader_val.c_str());
 	}
 
 	std::string village_val = cfg["village_value"];
 
-	if(village_val.empty()
-) {
+	if(village_val.empty())
+	{
 		village_val = global_ai_params["village_value"];
 	}
 
 	if(village_val.empty()) {
-		village_value = 1.0;
+		village_value = ai_default_village_value;
 	} else {
 		village_value = atof(village_val.c_str());
 	}
 
 	std::string aggression_val = cfg["aggression"];
 
-	if(aggression_val.empty()
-) {
+	if(aggression_val.empty())
+	{
 		aggression_val = global_ai_params["aggression"];
 	}
 
 	if(aggression_val.empty()) {
-		aggression_ = 0.5;
+		aggression_ = ai_default_aggression;
 	} else {
 		aggression_ = atof(aggression_val.c_str());
 	}
 
 	std::string caution_val = cfg["caution"];
 
-	if(caution_val.empty()
-) {
+	if(caution_val.empty())
+	{
 		caution_val = global_ai_params["caution"];
 	}
 
 	if(caution_val.empty()) {
-		caution_ = 1.0;
+		caution_ = ai_default_caution;
 	} else {
 		caution_ = atof(caution_val.c_str());
 	}
@@ -261,10 +294,10 @@ team::team_info::team_info(const config& cfg) :
 	for(std::vector<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
 		can_recruit.insert(*i);
 	}
-	
-	if(recruitment_pattern.empty()
-) {
-		recruitment_pattern = 
+
+	if(recruitment_pattern.empty())
+	{
+		recruitment_pattern =
 			utils::split(global_ai_params["recruitment_pattern"]);
 		LOG_NG << "Recruitment pattern: " << global_ai_params["recruitment_pattern"] << "\n";
 	} else {
@@ -321,7 +354,7 @@ void team::team_info::write(config& cfg) const
 	cfg["current_player"] = current_player;
 	cfg["flag"] = flag;
 	cfg["flag_icon"] = flag_icon;
-	cfg["description"] = description;
+	cfg["id"] = description;
 	cfg["objectives"] = objectives;
 	cfg["objectives_changed"] = objectives_changed ? "yes" : "no";
 	cfg["countdown_time"]= countdown_time;
@@ -330,6 +363,8 @@ void team::team_info::write(config& cfg) const
 	cfg["disallow_observers"] = disallow_observers ? "yes" : "no";
 	cfg["allow_player"] = allow_player ? "yes" : "no";
 	cfg["no_leader"] = no_leader ? "yes" : "no";
+	cfg["hidden"] = hidden ? "yes" : "no";
+	cfg["number_of_possible_recruits_to_force_recruit"] = lexical_cast<std::string>(number_of_possible_recruits_to_force_recruit);
 
 	std::stringstream enemies_str;
 	for(std::vector<int>::const_iterator en = enemies.begin(); en != enemies.end(); ++en) {
@@ -343,12 +378,13 @@ void team::team_info::write(config& cfg) const
 	switch(controller) {
 	case AI: cfg["controller"] = "ai"; break;
 	case HUMAN: cfg["controller"] = "human"; break;
+	case HUMAN_AI: cfg["controller"] = "human_ai"; break;
 	case NETWORK: cfg["controller"] = "network"; break;
+	case NETWORK_AI: cfg["controller"] = "network_ai"; break;
 	case EMPTY: cfg["controller"] = "null"; break;
 	default: assert(false); return;
 	}
 
-	cfg["persistent"] = persistent ? "1" : "0";
 	cfg["villages_per_scout"] = str_cast(villages_per_scout);
 	cfg["leader_value"] = str_cast(leader_value);
 	cfg["village_value"] = str_cast(village_value);
@@ -400,13 +436,18 @@ void team::team_info::write(config& cfg) const
 	cfg["colour"] = lexical_cast_default<std::string>(colour);
 }
 
-team::team(const config& cfg, int gold) : 
+void team::merge_shroud_map_data(const std::string& shroud_data)
+{
+	shroud_.merge(shroud_data);
+}
+
+team::team(const config& cfg, const gamemap& map, int gold) :
 		gold_(gold),
 		villages_(),
 		shroud_(),
 		fog_(),
-		auto_shroud_updates_(true), 
-		info_(cfg), 
+		auto_shroud_updates_(true),
+		info_(cfg),
 		countdown_time_(0),
 		action_bonus_count_(0),
 		aiparams_(),
@@ -415,8 +456,8 @@ team::team(const config& cfg, int gold) :
 		ally_shroud_(),
 		ally_fog_()
 {
-	fog_.set_enabled(cfg["fog"] == "yes");
-	shroud_.set_enabled(cfg["shroud"] == "yes");
+	fog_.set_enabled( utils::string_bool(cfg["fog"]) );
+	shroud_.set_enabled( utils::string_bool(cfg["shroud"]) );
 	shroud_.read(cfg["shroud_data"]);
 
 	LOG_NG << "team::team(...): team_name: " << info_.team_name
@@ -426,7 +467,7 @@ team::team(const config& cfg, int gold) :
 	// gold is the maximum of 'gold' and what is given in the config file
 	if(info_.gold.empty() == false)
 	{
-		gold_ = maximum(gold,::atoi(info_.gold.c_str()));
+		gold_ = std::max(gold,::atoi(info_.gold.c_str()));
 		if (gold_ != ::atoi(info_.gold.c_str()))
 			info_.start_gold = str_cast(gold) + " (" + info_.start_gold + ")";
 	}
@@ -434,7 +475,12 @@ team::team(const config& cfg, int gold) :
 	// Load in the villages the side controls at the start
 	const config::child_list& villages = cfg.get_children("village");
 	for(config::child_list::const_iterator v = villages.begin(); v != villages.end(); ++v) {
-		villages_.insert(gamemap::location(**v,game_events::get_state_of_game()));
+		map_location loc(**v,game_events::get_state_of_game());
+		if (map.is_village(loc)) {
+			villages_.insert(loc);
+		} else {
+			WRN_NG << "[side] " << name() << " [village] points to a non-village location " << loc << "\n";
+		}
 	}
 
 	countdown_time_=lexical_cast_default<int>(cfg["countdown_time"],0);
@@ -449,7 +495,7 @@ void team::write(config& cfg) const
 	cfg["gold"] = str_cast(gold_);
 
 	// Write village locations
-	for(std::set<gamemap::location>::const_iterator t = villages_.begin(); t != villages_.end(); ++t) {
+	for(std::set<map_location>::const_iterator t = villages_.begin(); t != villages_.end(); ++t) {
 		t->write(cfg.add_child("village"));
 	}
 
@@ -459,19 +505,79 @@ void team::write(config& cfg) const
 	cfg["action_bonus_count"] = str_cast(action_bonus_count_);
 }
 
-bool team::get_village(const gamemap::location& loc)
+bool team::get_village(const map_location& loc)
 {
 	villages_.insert(loc);
 	return game_events::fire("capture",loc);
 }
 
-void team::lose_village(const gamemap::location& loc)
+void team::lose_village(const map_location& loc)
 {
 	if(owns_village(loc)) {
 		villages_.erase(villages_.find(loc));
 	}
 }
 
+void team::remove_recruit(const std::string& recruit)
+{
+	info_.can_recruit.erase(recruit);
+	info_.average_price = 0;
+}
+
+void team::set_recruits(const std::set<std::string>& recruits)
+{
+	info_.can_recruit = recruits;
+	info_.average_price = 0;
+}
+
+void team::add_recruits(const std::set<std::string>& recruits)
+{
+	std::copy(recruits.begin(),recruits.end(),
+			std::inserter(info_.can_recruit, info_.can_recruit.end()));
+	info_.average_price = 0;
+}
+
+
+namespace {
+	struct count_average {
+		count_average(size_t& a) : a_(a), sum_(0), count_(0)
+		{}
+		~count_average() {
+			// If no recruits disable leader from moving to keep
+			if (count_ == 0)
+				a_ = 424242;
+			else
+				a_ = sum_/count_;
+		}
+		void operator()(const std::string& recruit)
+		{
+			unit_type_data::unit_type_map::const_iterator i = unit_type_data::types().find_unit_type(recruit);
+			if (i == unit_type_data::types().end())
+				return;
+			++count_;
+			sum_ += i->second.cost();
+		}
+		private:
+		size_t& a_;
+		size_t sum_;
+		size_t count_;
+	};
+}
+
+size_t team::average_recruit_price()
+{
+	if (info_.average_price)
+		return info_.average_price;
+
+	{
+		count_average avg(info_.average_price);
+		BOOST_FOREACH(std::string type, info_.can_recruit){
+			avg(type);
+		}
+	}
+	return info_.average_price;
+}
+
 void team::set_time_of_day(int turn, const time_of_day& tod)
 {
 	aiparams_.clear();
@@ -510,8 +616,9 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 	info_.recruitment_pattern = utils::split(aiparams_["recruitment_pattern"]);
 	if (info_.recruitment_pattern.empty())
 		info_.recruitment_pattern = info_.global_recruitment_pattern;
-	info_.aggression_ = lexical_cast_default<double>(aiparams_["aggression"],0.5);
-	info_.caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
+	info_.aggression_ = lexical_cast_default<double>(aiparams_["aggression"],info_.aggression_);
+	info_.caution_ = lexical_cast_default<double>(aiparams_["caution"],info_.caution_);
+	info_.number_of_possible_recruits_to_force_recruit = lexical_cast_default<float>(aiparams_["number_of_possible_recruits_to_force_recruit"],info_.number_of_possible_recruits_to_force_recruit);
 }
 
 bool team::calculate_enemies(size_t index) const
@@ -552,8 +659,12 @@ void team::change_controller(const std::string& controller)
 	team::team_info::CONTROLLER cid;
 	if (controller == "human")
 		cid = team::team_info::HUMAN;
+	else if (controller == "human_ai")
+		cid = team::team_info::HUMAN_AI;
 	else if (controller == "network")
 		cid = team::team_info::NETWORK;
+	else if (controller == "network_ai")
+		cid = team::team_info::NETWORK_AI;
 	else if (controller == "null")
 		cid = team::team_info::EMPTY;
 	else
@@ -578,6 +689,8 @@ void team::change_team(const std::string& name, const std::string& user_name)
 	if(teams != NULL) {
 		for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
 			i->enemies_.clear();
+			i->ally_shroud_.clear();
+			i->ally_fog_.clear();
 		}
 	}
 }
@@ -596,7 +709,16 @@ void team::set_ai_memory(const config& ai_mem){
   info_.ai_memory_=ai_mem;
 }
 
-bool team::shrouded(const gamemap::location& loc) const
+void team::set_ai_parameters(const config::child_list& ai_parameters)
+{
+	info_.ai_params.clear();		// override
+	config::child_list::const_iterator i;
+	for (i = ai_parameters.begin(); i != ai_parameters.end(); ++i) {
+		info_.ai_params.push_back(**i);
+	}
+}
+
+bool team::shrouded(const map_location& loc) const
 {
 	if(!teams)
 		return shroud_.value(loc.x+1,loc.y+1);
@@ -604,7 +726,7 @@ bool team::shrouded(const gamemap::location& loc) const
 	return shroud_.shared_value(ally_shroud(*teams),loc.x+1,loc.y+1);
 }
 
-bool team::fogged(const gamemap::location& loc) const
+bool team::fogged(const map_location& loc) const
 {
 	if(shrouded(loc)) return true;
 
@@ -640,7 +762,7 @@ const std::vector<const team::shroud_map*>& team::ally_fog(const std::vector<tea
 	return ally_fog_;
 }
 
-bool team::knows_about_team(size_t index) const
+bool team::knows_about_team(size_t index, bool is_multiplayer) const
 {
 	const team& t = (*teams)[index];
 
@@ -654,7 +776,7 @@ bool team::knows_about_team(size_t index) const
 	if(is_enemy(index+1)) return false;
 
 	// We know our allies in multiplayer
-	if(network::nconnections() > 0) return true;
+	if (is_multiplayer) return true;
 
 	// We know about allies we're sharing maps with
 	if(share_maps() && t.uses_shroud()) return true;
@@ -689,7 +811,7 @@ bool is_observer()
 	}
 
 	for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
-		if(i->is_human()) {
+		if(i->is_local()) {
 			return false;
 		}
 	}
@@ -794,6 +916,7 @@ std::string team::shroud_map::write() const
 
 void team::shroud_map::read(const std::string& str)
 {
+	data_.clear();
 	for(std::string::const_iterator sh = str.begin(); sh != str.end(); ++sh) {
 		if(*sh == '|')
 			data_.resize(data_.size()+1);
@@ -807,6 +930,22 @@ void team::shroud_map::read(const std::string& str)
 	}
 }
 
+void team::shroud_map::merge(const std::string& str)
+{
+	int x=0, y=0;
+	for(std::string::const_iterator sh = str.begin(); sh != str.end(); ++sh) {
+		if(*sh == '|' && sh != str.begin()) {
+			y=0;
+			x++;
+		} else if(*sh == '1') {
+			clear(x,y);
+			y++;
+		} else if(*sh == '0') {
+			y++;
+		}
+	}
+}
+
 bool team::shroud_map::copy_from(const std::vector<const shroud_map*>& maps)
 {
 	if(enabled_ == false)
@@ -879,7 +1018,7 @@ void team::log_recruitable(){
 }
 
 namespace player_teams {
-int village_owner(const gamemap::location& loc)
+int village_owner(const map_location& loc)
 {
 	if(! teams) {
 		return -1;
@@ -891,4 +1030,3 @@ int village_owner(const gamemap::location& loc)
 	return -1;
 }
 }
-
