@@ -1,6 +1,6 @@
-/* $Id: terrain_translation.cpp 23963 2008-02-20 00:16:11Z esr $ */
+/* $Id: terrain_translation.cpp 33719 2009-03-15 22:57:51Z ilor $ */
 /*
-   Copyright (C) 2006 - 2008 by Mark de Wever <koraq@xs4all.nl>
+   Copyright (C) 2006 - 2009 by Mark de Wever <koraq@xs4all.nl>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,8 +12,12 @@
    See the COPYING file for more details.
 */
 
-//! @file terrain_translation.cpp
-//! Routines for terrain-conversion.
+/**
+ *  @file terrain_translation.cpp
+ *  Routines for terrain-conversion.
+ */
+
+#define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "global.hpp"
 #include "gettext.hpp"
@@ -22,13 +26,16 @@
 #include "serialization/string_utils.hpp"
 #include "wml_exception.hpp"
 
-#include <iostream>
 
 #define ERR_G  LOG_STREAM(err, general)
 #define WRN_G  LOG_STREAM(warn, general)
 
 namespace t_translation {
 
+	size_t max_map_size() {
+		return 1000; //TODO make this overridable by the user without having to rebuild
+	}
+
 /***************************************************************************************/
 // forward declaration of internal functions
 
@@ -39,40 +46,42 @@ namespace t_translation {
 	/**
 	 * Get the mask for a single layer.
 	 *
-	 * @param terrain	1 layer of a terrain, might have a wildcard
+	 * @param terrain   1 layer of a terrain, might have a wildcard.
 	 *
-	 * @return			mask for that layer
+	 * @return          Mask for that layer.
 	 */
 	static t_layer get_layer_mask_(t_layer terrain); //inlined
 
 	/**
 	 * Gets a mask for a terrain, this mask is used for wildcard matching.
 	 *
-	 * @param terrain	the terrain which might have a wildcard
+	 * @param terrain   The terrain which might have a wildcard.
 	 *
-	 * @return			the mask for this terrain
+	 * @return          The mask for this terrain.
 	 */
 	static t_terrain get_mask_(const t_terrain& terrain);
 
 	/**
 	 * Converts a string to a layer.
 	 *
-	 * @param str		the terrain string to convert, but needs to be sanitized
-	 *					so no spaces and only the terrain to convert
+	 * @param str       The terrain string to convert, but needs to be
+	 *                  sanitized so no spaces and only the terrain to convert.
 	 *
-	 * @return			the converted layer
+	 * @return          The converted layer.
 	 */
 	static t_layer string_to_layer_(const std::string& str);
 
 	/**
 	 * Converts a terrain string to a number.
-	 * @param str				the terrain string with an optional number
-	 * @param start_position	returns the start_position, the caller should set it on -1
-	 *						and it's only changed it there is a starting position found
-	 * @param filler			if the terrain has only 1 layer then the filler will be used
-	 *							as the second layer.
+	 * @param str               The terrain string with an optional number.
+	 * @param start_position    Returns the start_position, the caller should
+	 *                          set it on -1 and it's only changed it there is
+	 *                          a starting position found.
+	 * @param filler            If the terrain has only 1 layer then the filler
+	 *                          will be used as the second layer.
 	 *
-	 * @return					the terrain code found in the string
+	 * @return                  The terrain code found in the string if no
+	 *                          valid terrain is found VOID will be returned.
 	 */
 	static t_terrain string_to_number_(std::string str, int& start_position, const t_layer filler);
 	static t_terrain string_to_number_(const std::string& str, const t_layer filler = NO_LAYER);
@@ -80,13 +89,15 @@ namespace t_translation {
 	/**
 	 * Converts a terrain number to a string
 	 *
-	 * @param terrain				the terrain number to convert
-	 * @param start_position		the starting position, if smaller than 0 it's
-	 *								ignored else it's written
-	 * @param min_size				padds the results with spaces if required,
-	 *								until the result has a length of min_size.
-	 * @return						the converted string, if no starting position
-	 *								given it's padded to 4 chars else padded to 7 chars
+	 * @param terrain               The terrain number to convert.
+	 * @param start_position        The starting position, if smaller than 0
+	 *                              it's ignored else it's written.
+	 * @param min_size              Padds the results with spaces if required,
+	 *                              until the result has a length of min_size.
+	 *
+	 * @return                      The converted string, if no starting
+	 *                              position given it's padded to 4 chars else
+	 *                              padded to 7 chars.
 	 */
 	static std::string number_to_string_(t_terrain terrain, const int start_position, const size_t min_size);
 	static std::string number_to_string_(t_terrain terrain, const int start_position = -1);
@@ -95,9 +106,9 @@ namespace t_translation {
 	 * Converts a terrain string to a number for the builder.
 	 * The translation rules differ from the normal conversion rules
 	 *
-	 * @param str	the terrain string
+	 * @param str   The terrain string.
 	 *
-	 * @return		number for the builder map
+	 * @return      Number for the builder map.
 	 */
 	static t_terrain string_to_builder_number_(std::string str);
 
@@ -127,6 +138,7 @@ const t_terrain PLUS = string_to_number_("+");
 const t_terrain MINUS = string_to_number_("-");
 const t_terrain NOT = string_to_number_("!");
 const t_terrain STAR = string_to_number_("*");
+const t_terrain BASE = string_to_number_("_bas");
 
 /***************************************************************************************/
 
@@ -312,6 +324,10 @@ t_map read_game_map(const std::string& str,	std::map<int, coordinate>& starting_
 					ERR_G << "Map not a rectangle error occured at line offset " << y << " position offset " << x << "\n";
 					throw error("Map not a rectangle.");
 				}
+				if (y > max_map_size()) {
+					ERR_G << "Map size exceeds limit (y > " << max_map_size() << ")\n";
+					throw error("Map height limit exceeded.");
+				}
 			}
 
 			// Prepare next iteration
@@ -334,6 +350,10 @@ t_map read_game_map(const std::string& str,	std::map<int, coordinate>& starting_
 		} else {
 			++x;
 			offset = pos_separator + 1;
+			if (x > max_map_size()) {
+				ERR_G << "Map size exceeds limit (x > " << max_map_size() << ")\n";
+				throw error("Map width limit exceeded.");
+			}
 		}
 
 	}
@@ -727,16 +747,16 @@ static t_terrain string_to_number_(std::string str, int& start_position, const t
 		try {
 			start_position = lexical_cast<int>(str.substr(0, offset));
 		} catch(bad_lexical_cast&) {
-			goto terrain_error;
+			return VOID_TERRAIN;
 		}
 		str.erase(0, offset + 1);
 	}
 
-    offset = str.find('^', 0);
-    if(offset !=  std::string::npos) {
+	offset = str.find('^', 0);
+	if(offset !=  std::string::npos) {
 		// If either string is longer than 4 characters bail out
 		if(offset > 4 || (str.size() - offset) > 5) {
-			goto terrain_error;
+			return VOID_TERRAIN;
 		}
 		const std::string base_str(str, 0, offset);
 		const std::string overlay_str(str, offset + 1, str.size());
@@ -744,7 +764,7 @@ static t_terrain string_to_number_(std::string str, int& start_position, const t
 	} else {
 		// If the string is longer than 4 characters bail out
 		if(str.size() > 4) {
-			goto terrain_error;
+			return VOID_TERRAIN;
 		}
 		result = t_terrain(str, filler);
 
@@ -757,14 +777,6 @@ static t_terrain string_to_number_(std::string str, int& start_position, const t
 	}
 
 	return result;
-
-terrain_error:
-	// When this string is removed, also test
-	// whether the gettext include is still required
-	//! @todo 1.5 remove this test and let the wml_exception to the test
-	lg::wml_error << _("Invalid terrain found probably an 1.2 terrain format, "
-		"terrain = ") << input << '\n';
-	return VOID_TERRAIN;
 }
 
 static std::string number_to_string_(t_terrain terrain, const int start_position)
