@@ -1,6 +1,6 @@
-/* $Id: titlescreen.cpp 26222 2008-04-28 21:59:25Z shadowmaster $ */
+/* $Id: titlescreen.cpp 33383 2009-03-07 14:53:43Z ilor $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,11 +12,14 @@
    See the COPYING file for more details.
 */
 
-//! @file titlescreen.cpp
-//! Shows the titlescreen, with main-menu and tip-of-the-day.
-//!
-//! The menu consists of buttons, such als Start-Tutorial, Start-Campaign, Load-Game, etc. \n
-//! As decoration, the wesnoth-logo and a landmap in the background are shown.
+/**
+ *  @file titlescreen.cpp
+ *  Shows the titlescreen, with main-menu and tip-of-the-day.
+ *
+ *  The menu consists of buttons, such als Start-Tutorial, Start-Campaign,
+ *  Load-Game, etc.  As decoration, the wesnoth-logo and a landmap in the
+ *  background are shown.
+ */
 
 #include "global.hpp"
 
@@ -24,6 +27,7 @@
 #include "construct_dialog.hpp"
 #include "cursor.hpp"
 #include "game_display.hpp"
+#include "game_preferences.hpp"
 #include "events.hpp"
 #include "filesystem.hpp"
 #include "game_config.hpp"
@@ -36,35 +40,35 @@
 #include "sdl_utils.hpp"
 #include "show_dialog.hpp"
 #include "titlescreen.hpp"
-#include "util.hpp"
 #include "video.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "wesconfig.h"
 #include <algorithm>
+#include <vector>
 
-#include "sdl_ttf/SDL_ttf.h"
+#include "SDL_ttf.h"
 
-//! Log info-messages to stdout during the game, mainly for debugging
+/** Log info-messages to stdout during the game, mainly for debugging */
 #define LOG_DP LOG_STREAM(info, display)
-//! Log error-messages to stdout during the game, mainly for debugging
+/** Log error-messages to stdout during the game, mainly for debugging */
 #define ERR_DP LOG_STREAM(err, display)
 #define LOG_CONFIG LOG_STREAM(info, config)
 #define ERR_CONFIG LOG_STREAM(err, config)
 
-//! Fade-in the wesnoth-logo.
-//!
-//! Animation-effect: scroll-in from right. \n
-//! Used only once, after the game is started.
-//!
-//! @param	screen	surface to operate on
-//! @param	xpos	x-position of logo
-//! @param	ypos	y-position of logo
-//!
-//! @return		Result of running the routine
-//! @retval true	operation finished (successful or not)
-//! @retval false	operation failed (because modeChanged), need to retry
-//!
+/**
+ *  Fade-in the wesnoth-logo.
+ *
+ *  Animation-effect: scroll-in from right. \n
+ *  Used only once, after the game is started.
+ *
+ *  @param	screen	surface to operate on
+ *  @param	xpos	x-position of logo
+ *  @param	ypos	y-position of logo
+ *
+ *  @return		Result of running the routine
+ *  @retval true	operation finished (successful or not)
+ *  @retval false	operation failed (because modeChanged), need to retry
+ */
 static bool fade_logo(game_display& screen, int xpos, int ypos)
 {
 	const surface logo(image::get_image(game_config::game_logo));
@@ -79,8 +83,9 @@ static bool fade_logo(game_display& screen, int xpos, int ypos)
 		return true;
 	}
 
-	// Only once, when the game is first started, the logo fades in
-	static bool faded_in = false;
+	// Only once, when the game is first started, the logo fades in unless
+	// it was disabled in adv. preferences
+	static bool faded_in = !preferences::startup_effect();
 //	static bool faded_in = true;	// for faster startup: mark logo as 'has already faded in'
 
 	CKey key;
@@ -125,30 +130,40 @@ static bool fade_logo(game_display& screen, int xpos, int ypos)
 }
 
 
-//! Read the file with the tips-of-the-day.
-static const void read_tips_of_day(config& tips_of_day)
+/** Read the file with the tips-of-the-day. */
+static void read_tips_of_day(config& tips_of_day)
 {
 	tips_of_day.clear();
 	LOG_CONFIG << "Loading tips of day\n";
 	try {
-		scoped_istream stream = preprocess_file("data/hardwired/tips.cfg");
+		scoped_istream stream = preprocess_file(get_wml_location("hardwired/tips.cfg"));
 		read(tips_of_day, *stream);
 	} catch(config::error&) {
 		ERR_CONFIG << "Could not read data/hardwired/tips.cfg\n";
 	}
 
-	//we shuffle the tips after each initial loading
+	//we shuffle the tips after each initial loading. We only shuffle if
+	//the upload_log preference has been set. If it hasn't been set, it's the
+	//user's first time playing since this feature has been added, so we'll
+	//leave the tips in their default order, which will always contain a tip
+	//regarding the upload log first, so the user sees it.
 	config::child_itors tips = tips_of_day.child_range("tip");
-	if (tips.first != tips.second) {
+	if (tips.first != tips.second && preferences::has_upload_log()) {
 		std::random_shuffle(tips.first, tips.second);
 	}
+
+	//Make sure that the upload log preference is set, if it's not already, so
+	//that we know next time we've already seen the message about uploads.
+	if(!preferences::has_upload_log()) {
+		preferences::set_upload_log(preferences::upload_log());
+	}
 }
 
-//! Go to the next tips-of-the-day
-static const void next_tip_of_day(config& tips_of_day, bool reverse = false)
+/** Go to the next tips-of-the-day */
+static void next_tip_of_day(config& tips_of_day, bool reverse = false)
 {
 	// we just rotate the tip list, to avoid the need to keep track
-	// of the current one, and keep it valid, cycle it, etc... 
+	// of the current one, and keep it valid, cycle it, etc...
 	config::child_itors tips = tips_of_day.child_range("tip");
 	if (tips.first != tips.second) {
 		config::child_iterator direction = reverse ? tips.first+1 : tips.second-1;
@@ -156,7 +171,7 @@ static const void next_tip_of_day(config& tips_of_day, bool reverse = false)
 	}
 }
 
-//! Return the text for one of the tips-of-the-day.
+/** Return the text for one of the tips-of-the-day. */
 static const config* get_tip_of_day(config& tips_of_day)
 {
 	if (tips_of_day.empty()) {
@@ -164,7 +179,7 @@ static const config* get_tip_of_day(config& tips_of_day)
 	}
 
 	const config::child_list& tips = tips_of_day.get_children("tip");
-	
+
 	// next_tip_of_day rotate tips, so better stay iterator-safe
 	for (size_t t=0; t < tips.size(); t++, next_tip_of_day(tips_of_day)) {
 		const config* tip = tips.front();
@@ -194,8 +209,10 @@ static const config* get_tip_of_day(config& tips_of_day)
 	return NULL;
 }
 
-//! Show one tip-of-the-day in a frame on the titlescreen.
-//! This frame has 2 buttons: Next-Tip, and Show-Help.
+/**
+ *  Show one tip-of-the-day in a frame on the titlescreen.
+ *  This frame has 2 buttons: Next-Tip, and Show-Help.
+ */
 static void draw_tip_of_day(game_display& screen,
 							config& tips_of_day,
 							const gui::dialog_frame::style& style,
@@ -218,9 +235,9 @@ static void draw_tip_of_day(game_display& screen,
     	int tip_width = game_config::title_tip_width * screen.w() / 1024;
 
 		try {
-	        const std::string& text = 
+	        const std::string& text =
 				font::word_wrap_text((*tip)["text"], font::SIZE_NORMAL, tip_width);
-			const std::string& source = 
+			const std::string& source =
 				font::word_wrap_text((*tip)["source"], font::SIZE_NORMAL, tip_width);
 
 			const int pad = game_config::title_tip_padding;
@@ -228,7 +245,7 @@ static void draw_tip_of_day(game_display& screen,
 			SDL_Rect area = font::text_area(text,font::SIZE_NORMAL);
 			area.w = tip_width;
 			SDL_Rect source_area = font::text_area(source, font::SIZE_NORMAL, TTF_STYLE_ITALIC);
-			area.w = maximum<size_t>(area.w, source_area.w) + 2*pad;
+			area.w = std::max<size_t>(area.w, source_area.w) + 2*pad;
 			area.h += source_area.h + next_tip_button->location().h + 3*pad;
 
 			area.x = main_dialog_area->x - (game_config::title_tip_x * screen.w() / 1024) - area.w;
@@ -251,7 +268,7 @@ static void draw_tip_of_day(game_display& screen,
 
 			button_x = area.x + pad;
 			help_tip_button->set_location(button_x, button_y);
-			help_tip_button->set_dirty(); 
+			help_tip_button->set_dirty();
 
 			gui::dialog_frame f(screen.video(), "", style, false);
 			tip_of_day_restorer = surface_restorer(&screen.video(), f.layout(area).exterior);
@@ -274,60 +291,45 @@ static void draw_tip_of_day(game_display& screen,
 	}
 }
 
-namespace gui {
-
-//! Show titlepage with logo and background, menu-buttons and tip-of-the-day.
-//!
-//! After the page is shown, this routine waits
-//! for the user to click one of the menu-buttons,
-//! or a keypress.
-//!
-//! @param	screen			surface to write on
-//! @param	tips_of_day		list of tips
-//! @param	ntip			number of the tip to show
-//!
-//! @return	the value of the menu-item the user has choosen.
-//! @retval	see @ref TITLE_RESULT for possible values
-//!
-TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
+/**
+ *  Draw the map image background, revision number
+ *  and fade the log the first time
+ */
+static void draw_background(game_display& screen)
 {
-	cursor::set(cursor::NORMAL);
-
-	const preferences::display_manager disp_manager(&screen);
-	const hotkey::basic_handler key_handler(&screen);
-
-	const font::floating_label_context label_manager;
-
-	screen.video().modeChanged(); // resets modeChanged value
-
 	bool fade_failed = false;
 	do {
 		int logo_x = game_config::title_logo_x * screen.w() / 1024,
-		    logo_y = game_config::title_logo_y * screen.h() / 768;
+			logo_y = game_config::title_logo_y * screen.h() / 768;
 
-		surface const title_surface(scale_surface(
-			image::get_image(game_config::game_title),
-			screen.w(), screen.h()));
+		/*Select a random game_title*/
+		std::vector<std::string> game_title_list =
+			utils::split(game_config::game_title, ',', utils::STRIP_SPACES | utils::REMOVE_EMPTY);
 
-		
-		if (title_surface.null()) {
-			ERR_DP << "Could not find title image\n";
+		if(game_title_list.empty()) {
+			ERR_CONFIG << "No title image defined\n";
 		} else {
-			screen.video().blit_surface(0, 0, title_surface);
-			update_rect(screen_area());
-			LOG_DP << "displayed title image\n";
+			surface const title_surface(scale_opaque_surface(
+				image::get_image(game_title_list[rand()%game_title_list.size()]),
+				screen.w(), screen.h()));
+
+
+			if (title_surface.null()) {
+				ERR_DP << "Could not find title image\n";
+			} else {
+				screen.video().blit_surface(0, 0, title_surface);
+				update_rect(screen_area());
+				LOG_DP << "displayed title image\n";
+			}
 		}
 
 		fade_failed = !fade_logo(screen, logo_x, logo_y);
 	} while (fade_failed);
 	LOG_DP << "faded logo\n";
 
-	// Display Wesnoth version and (if configured with --enable-display-revision) the svn-revision
-	const std::string rev =  game_config::svnrev.empty() ? "" :
-		" (" + game_config::svnrev + ")";
-
+	// Display Wesnoth version and (if possible) revision
 	const std::string& version_str = _("Version") +
-		std::string(" ") + game_config::version + rev;
+		std::string(" ") + game_config::revision;
 
 	const SDL_Rect version_area = font::draw_text(NULL, screen_area(),
 								  font::SIZE_TINY, font::NORMAL_COLOUR,
@@ -342,6 +344,66 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 	}
 
 	LOG_DP << "drew version number\n";
+}
+
+namespace {
+
+/**
+ *  Handler for forcing a discrete ESC keypress to quit the game (bug #12747)
+ *  This hack is here because the GUI code used here doesn't handle this yet.
+ *  Once it does, revert this part of r31758.
+ */
+class titlescreen_handler : public events::handler
+{
+public:
+	titlescreen_handler(bool ignore_esc = false)
+		: handler(), ignore_esc_(ignore_esc)
+	{
+		if(ignore_esc) {
+			LOG_DP << "ignoring held ESCAPE key\n";
+		}
+	}
+
+	bool get_esc_ignore() { return ignore_esc_; }
+	void set_esc_ignore(bool ignore) { ignore_esc_ = ignore; }
+
+	virtual void handle_event(const SDL_Event& event)
+	{
+		if(event.type == SDL_KEYUP && event.key.keysym.sym == SDLK_ESCAPE) {
+			ignore_esc_ = false;
+			LOG_DP << "ESCAPE key no longer ignored\n";
+		}
+	}
+
+private:
+	bool ignore_esc_;
+};
+
+}	// end anon namespace
+
+namespace gui {
+
+
+static bool background_is_dirty_ = true;
+void set_background_dirty() {
+	background_is_dirty_ = true;
+}
+
+
+TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
+{
+	cursor::set(cursor::NORMAL);
+
+	const preferences::display_manager disp_manager(&screen);
+	const hotkey::basic_handler key_handler(&screen);
+
+	const font::floating_label_context label_manager;
+
+	screen.video().modeChanged(); // resets modeChanged value
+
+	if (background_is_dirty_) {
+		draw_background(screen);
+	}
 
 	//- Texts for the menu-buttons.
 	//- Members of this array must correspond to the enumeration TITLE_RESULT
@@ -351,6 +413,9 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 					       N_("TitleScreen button^Multiplayer"),
 					       N_("TitleScreen button^Load"),
 					       N_("TitleScreen button^Add-ons"),
+#ifndef DISABLE_EDITOR2
+					       N_("TitleScreen button^Map Editor"),
+#endif
 					       N_("TitleScreen button^Language"),
 					       N_("TitleScreen button^Preferences"),
 					       N_("TitleScreen button^Credits"),
@@ -368,6 +433,9 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 						    N_("Play multiplayer (hotseat, LAN, or Internet), or a single scenario against the AI"),
 						    N_("Load a saved game"),
 						    N_("Download usermade campaigns, eras, or map packs"),
+#ifndef DISABLE_EDITOR2
+						    N_("Start the map editor"),
+#endif
 						    N_("Change the language"),
 						    N_("Configure the game's settings"),
 						    N_("View the credits"),
@@ -396,10 +464,10 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 	for(b = 0; b != nbuttons; ++b) {
 		buttons.push_back(button(screen.video(),sgettext(button_labels[b])));
 		buttons.back().set_help_string(sgettext(help_button_labels[b]));
-		max_width = maximum<size_t>(max_width,buttons.back().width());
+		max_width = std::max<size_t>(max_width,buttons.back().width());
 
 		n_menubuttons = b;
-		if(b == QUIT_GAME) break;	// Menu-frame ends at the quit-button
+		if(b + TUTORIAL == QUIT_GAME) break;	// Menu-frame ends at the quit-button
 	}
 
 	SDL_Rect main_dialog_area = {menu_xbase-padding, menu_ybase-padding, max_width+padding*2,
@@ -407,29 +475,33 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 
 	gui::dialog_frame main_frame(screen.video(), "", gui::dialog_frame::titlescreen_style, false);
 	main_frame.layout(main_dialog_area);
-	main_frame.draw_background();
-	main_frame.draw_border();
+
+	// we only redraw transparent parts when asked,
+	// to prevent alpha growing
+	if (background_is_dirty_) {
+		main_frame.draw_background();
+		main_frame.draw_border();
+	}
 
 	for(b = 0; b != nbuttons; ++b) {
 		buttons[b].set_width(max_width);
 		buttons[b].set_location(menu_xbase + b*menu_xincr, menu_ybase + b*menu_yincr);
-		if(b == QUIT_GAME) break;
+		if(b + TUTORIAL == QUIT_GAME) break;
 	}
 
-	b = TIP_PREVIOUS;
+	b = TIP_PREVIOUS - TUTORIAL;
 	gui::button previous_tip_button(screen.video(),sgettext(button_labels[b]),button::TYPE_PRESS,"lite_small");
 	previous_tip_button.set_help_string( sgettext(button_labels[b] ));
 
-	b = TIP_NEXT;
+	b = TIP_NEXT - TUTORIAL;
 	gui::button next_tip_button(screen.video(),sgettext(button_labels[b]),button::TYPE_PRESS,"lite_small");
 	next_tip_button.set_help_string( sgettext(button_labels[b] ));
 
-	b = SHOW_HELP;
+	b = SHOW_HELP - TUTORIAL;
 	gui::button help_tip_button(screen.video(),sgettext(button_labels[b]),button::TYPE_PRESS,"lite_small");
 	help_tip_button.set_help_string( sgettext(button_labels[b] ));
 
-	//! @todo FIXME: Translatable string is here because we WILL put text in before 1.2!
-	gui::button beg_button(screen.video(),("Help Wesnoth"),button::TYPE_IMAGE,"menu-button",button::MINIMUM_SPACE);
+	gui::button beg_button(screen.video(),_("Help Wesnoth"),button::TYPE_IMAGE,"menu-button",button::MINIMUM_SPACE);
 	beg_button.set_help_string(_("Help Wesnoth by sending us information"));
 
 	next_tip_of_day(tips_of_day);
@@ -448,16 +520,21 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 
 	CKey key;
 
-	bool last_escape = key[SDLK_ESCAPE] != 0;
+	size_t keyboard_button = nbuttons;
+	bool key_processed = false;
 
 	update_whole_screen();
+	background_is_dirty_ = false;
+
+	titlescreen_handler ts_handler(key[SDLK_ESCAPE] != 0); //!= 0 to avoid a MSVC warning C4800
 
 	LOG_DP << "entering interactive loop...\n";
 
 	for(;;) {
+		events::pump();
 		for(size_t b = 0; b != buttons.size(); ++b) {
 			if(buttons[b].pressed()) {
-				return TITLE_RESULT(b);
+				return static_cast<TITLE_RESULT>(b + TUTORIAL);
 			}
 		}
 
@@ -478,23 +555,52 @@ TITLE_RESULT show_title(game_display& screen, config& tips_of_day)
 		if(beg_button.pressed()) {
 			return BEG_FOR_UPLOAD;
 		}
+		if (key[SDLK_UP]) {
+			if (!key_processed) {
+				buttons[keyboard_button].set_active(false);
+				if (keyboard_button == 0) {
+					keyboard_button = nbuttons - 1;
+				} else {
+					keyboard_button--;
+				}
+				key_processed = true;
+				buttons[keyboard_button].set_active(true);
+			}
+		} else if (key[SDLK_DOWN]) {
+			if (!key_processed) {
+				buttons[keyboard_button].set_active(false);
+				if (keyboard_button > nbuttons - 1) {
+					keyboard_button = 0;
+				} else {
+					keyboard_button++;
+				}
+				key_processed = true;
+				buttons[keyboard_button].set_active(true);
+			}
+		} else {
+			key_processed = false;
+		}
 
 		events::raise_process_event();
 		events::raise_draw_event();
 
 		screen.flip();
 
-		if(!last_escape && key[SDLK_ESCAPE])
+		if (key[SDLK_ESCAPE] && !ts_handler.get_esc_ignore())
 			return QUIT_GAME;
+		if (key[SDLK_F5])
+			return RELOAD_GAME_DATA;
+
+		if (key[SDLK_RETURN] && keyboard_button < nbuttons) {
+			return static_cast<TITLE_RESULT>(keyboard_button + TUTORIAL);
+		}
 
-		last_escape = key[SDLK_ESCAPE] != 0;
 
-		events::pump();
 
 		// If the resolution has changed due to the user resizing the screen,
 		// or from changing between windowed and fullscreen:
 		if(screen.video().modeChanged()) {
-			return TIP_NEXT;
+			return REDRAW_BACKGROUND;
 		}
 
 		screen.delay(20);
