@@ -1,6 +1,6 @@
-/* $Id: unit_abilities.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: unit_abilities.cpp 33798 2009-03-18 03:48:06Z sapient $ */
 /*
-   Copyright (C) 2006 - 2008 by Dominic Bolin <dominic.bolin@exong.net>
+   Copyright (C) 2006 - 2009 by Dominic Bolin <dominic.bolin@exong.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,18 +12,16 @@
    See the COPYING file for more details.
 */
 
-//! @file unit_abilities.cpp
-//! Manage unit-abilities, like heal, cure, and weapon_specials.
+/**
+ *  @file unit_abilities.cpp
+ *  Manage unit-abilities, like heal, cure, and weapon_specials.
+ */
 
 #include "unit.hpp"
 #include "unit_abilities.hpp"
 
-#include "log.hpp"
-#include "pathutils.hpp"
 #include "terrain_filter.hpp"
-#include "variable.hpp"
 
-#include <cassert>
 
 #define LOG_NG LOG_STREAM(info, engine)
 
@@ -102,7 +100,7 @@ A poisoned unit cannot be cured of its poison by a healer, and must seek the car
 
 namespace unit_abilities {
 
-bool affects_side(const config& cfg, const std::vector<team>& teams, size_t side, size_t other_side)
+static bool affects_side(const config& cfg, const std::vector<team>& teams, size_t side, size_t other_side)
 {
 	if (side == other_side)
 		return utils::string_bool(cfg["affect_allies"], true);
@@ -115,7 +113,7 @@ bool affects_side(const config& cfg, const std::vector<team>& teams, size_t side
 }
 
 
-bool unit::get_ability_bool(const std::string& ability, const gamemap::location& loc) const
+bool unit::get_ability_bool(const std::string& ability, const map_location& loc) const
 {
 	const config* abilities = cfg_.child("abilities");
 	if(abilities) {
@@ -128,8 +126,8 @@ bool unit::get_ability_bool(const std::string& ability, const gamemap::location&
 		}
 	}
 
-	assert(units_ && teams_);
-	gamemap::location adjacent[6];
+	assert(units_);
+	map_location adjacent[6];
 	get_adjacent_tiles(loc,adjacent);
 	for(int i = 0; i != 6; ++i) {
 		const unit_map::const_iterator it = units_->find(adjacent[i]);
@@ -141,7 +139,7 @@ bool unit::get_ability_bool(const std::string& ability, const gamemap::location&
 		const config::child_list& list = adj_abilities->get_children(ability);
 		for (config::child_list::const_iterator j = list.begin(),
 		     j_end = list.end(); j != j_end; ++j) {
-			if (unit_abilities::affects_side(**j, *teams_, side(), it->second.side()) &&
+			if (unit_abilities::affects_side(**j, teams_manager::get_teams(), side(), it->second.side()) &&
 			    it->second.ability_active(ability, **j, adjacent[i]) &&
 			    ability_affects_adjacent(ability,  **j, i, loc))
 				return true;
@@ -151,7 +149,7 @@ bool unit::get_ability_bool(const std::string& ability, const gamemap::location&
 
 	return false;
 }
-unit_ability_list unit::get_abilities(const std::string& ability, const gamemap::location& loc) const
+unit_ability_list unit::get_abilities(const std::string& ability, const map_location& loc) const
 {
 	unit_ability_list res;
 
@@ -162,13 +160,13 @@ unit_ability_list unit::get_abilities(const std::string& ability, const gamemap:
 		     i_end = list.end(); i != i_end; ++i) {
 			if (ability_active(ability, **i, loc) &&
 			    ability_affects_self(ability, **i, loc))
-				res.cfgs.push_back(std::pair<config*, gamemap::location>
+				res.cfgs.push_back(std::pair<config*, map_location>
 					(*i, loc));
 		}
 	}
 
 	assert(units_ != NULL);
-	gamemap::location adjacent[6];
+	map_location adjacent[6];
 	get_adjacent_tiles(loc,adjacent);
 	for(int i = 0; i != 6; ++i) {
 		const unit_map::const_iterator it = units_->find(adjacent[i]);
@@ -180,10 +178,10 @@ unit_ability_list unit::get_abilities(const std::string& ability, const gamemap:
 		const config::child_list& list = adj_abilities->get_children(ability);
 		for (config::child_list::const_iterator j = list.begin(),
 		     j_end = list.end(); j != j_end; ++j) {
-			if (unit_abilities::affects_side(**j, *teams_, side(), it->second.side()) &&
+			if (unit_abilities::affects_side(**j, teams_manager::get_teams(), side(), it->second.side()) &&
 			    it->second.ability_active(ability, **j, adjacent[i]) &&
 			    ability_affects_adjacent(ability, **j, i, loc))
-				res.cfgs.push_back(std::pair<config*, gamemap::location>
+				res.cfgs.push_back(std::pair<config*, map_location>
 					(*j, adjacent[i]));
 		}
 	}
@@ -210,8 +208,13 @@ std::vector<std::string> unit::unit_ability_tooltips() const
 	for (config::child_map::const_iterator i = list_map.begin(),
 	     i_end = list_map.end(); i != i_end; ++i) {
 		for (config::child_list::const_iterator j = i->second.begin(),
-		     j_end = i->second.end(); j != j_end; ++j) {
-			std::string const &name = (**j)["name"];
+		     j_end = i->second.end(); j != j_end; ++j)
+		{
+			std::string const &name = (
+				this->gender_ == unit_race::MALE || (**j)["female_name"].empty() ?
+				(**j)["name"] :
+				(**j)["female_name"]
+			);
 			if (!name.empty()) {
 				res.push_back(name);
 				res.push_back((**j)["description"]);
@@ -221,7 +224,25 @@ std::vector<std::string> unit::unit_ability_tooltips() const
 	return res;
 }
 
-std::vector<std::string> unit::ability_tooltips(const gamemap::location& loc) const
+std::vector<std::string> unit::get_ability_list() const
+{
+	std::vector<std::string> res;
+
+	const config* abilities = cfg_.child("abilities");
+	if (!abilities) return res;
+	const config::child_map& list_map = abilities->all_children();
+	for (config::child_map::const_iterator i = list_map.begin(), i_end = list_map.end(); i != i_end; ++i) {
+		for (config::child_list::const_iterator j = i->second.begin(), j_end = i->second.end(); j != j_end; ++j) {
+			std::string const &id = (**j)["id"];
+
+			if (!id.empty())
+				res.push_back(id);
+		}
+	}
+	return res;
+}
+
+std::vector<std::string> unit::ability_tooltips(const map_location& loc) const
 {
 	std::vector<std::string> res;
 
@@ -233,13 +254,23 @@ std::vector<std::string> unit::ability_tooltips(const gamemap::location& loc) co
 			for (config::child_list::const_iterator j = i->second.begin(),
 			     j_end = i->second.end(); j != j_end; ++j) {
 				if (ability_active(i->first, **j, loc)) {
-					std::string const &name = (**j)["name"];
+					std::string const &name = (
+						gender_ == unit_race::MALE || (**j)["female_name"].empty() ?
+						(**j)["name"] :
+						(**j)["female_name"]
+					);
+
 					if (!name.empty()) {
 						res.push_back(name);
 						res.push_back((**j)["description"]);
 					}
 				} else {
-					std::string const &name = (**j)["name_inactive"];
+					std::string const &name = (
+						gender_ == unit_race::MALE || (**j)["female_name_inactive"].empty() ?
+						(**j)["name_inactive"] :
+						(**j)["female_name_inactive"]
+					);
+
 					if (!name.empty()) {
 						res.push_back(name);
 						res.push_back((**j)["description_inactive"]);
@@ -250,7 +281,7 @@ std::vector<std::string> unit::ability_tooltips(const gamemap::location& loc) co
 	}
 	/*
 	assert(units_ != NULL);
-	gamemap::location adjacent[6];
+	map_location adjacent[6];
 	get_adjacent_tiles(loc,adjacent);
 	for(int i = 0; i != 6; ++i) {
 		const unit_map::const_iterator it = units_->find(adjacent[i]);
@@ -313,16 +344,16 @@ static bool cache_illuminates(int &cache, std::string const &ability)
 	return (cache != 0);
 }
 
-bool unit::ability_active(const std::string& ability,const config& cfg,const gamemap::location& loc) const
+bool unit::ability_active(const std::string& ability,const config& cfg,const map_location& loc) const
 {
 	int illuminates = -1;
 	assert(units_ && map_ && gamestatus_);
-	
+
 	if (const config* afilter = cfg.child("filter"))
 		if (!matches_filter(afilter,loc, cache_illuminates(illuminates, ability)))
 			return false;
-	
-	gamemap::location adjacent[6];
+
+	map_location adjacent[6];
 	get_adjacent_tiles(loc,adjacent);
 	config::child_list::const_iterator i, i_end;
 	const config::child_list& adj_filt = cfg.get_children("filter_adjacent");
@@ -330,9 +361,9 @@ bool unit::ability_active(const std::string& ability,const config& cfg,const gam
 		std::vector<std::string> dirs = utils::split((**i)["adjacent"]);
 		for (std::vector<std::string>::const_iterator j = dirs.begin(),
 		     j_end = dirs.end(); j != j_end; ++j) {
-			gamemap::location::DIRECTION index =
-				gamemap::location::parse_direction(*j);
-			if (index == gamemap::location::NDIRECTIONS)
+			map_location::DIRECTION index =
+				map_location::parse_direction(*j);
+			if (index == map_location::NDIRECTIONS)
 				continue;
 			unit_map::const_iterator unit = units_->find(adjacent[index]);
 			if (unit == units_->end())
@@ -347,8 +378,8 @@ bool unit::ability_active(const std::string& ability,const config& cfg,const gam
 		std::vector<std::string> dirs = utils::split((**i)["adjacent"]);
 		for (std::vector<std::string>::const_iterator j = dirs.begin(),
 		     j_end = dirs.end(); j != j_end; ++j) {
-			gamemap::location::DIRECTION index = gamemap::location::parse_direction(*j);
-			if (index == gamemap::location::NDIRECTIONS) {
+			map_location::DIRECTION index = map_location::parse_direction(*j);
+			if (index == map_location::NDIRECTIONS) {
 				continue;
 			}
 			/* GCC-3.3 doesn't accept vconfig(*i) in adj_filter */
@@ -367,10 +398,10 @@ bool unit::ability_active(const std::string& ability,const config& cfg,const gam
  * cfg: an ability WML structure
  *
  */
-bool unit::ability_affects_adjacent(const std::string& ability, const config& cfg,int dir,const gamemap::location& loc) const
+bool unit::ability_affects_adjacent(const std::string& ability, const config& cfg,int dir,const map_location& loc) const
 {
 	int illuminates = -1;
-	
+
 	assert(dir >=0 && dir <= 5);
 	static const std::string adjacent_names[6] = {"n","ne","se","s","sw","nw"};
 	const config::child_list& affect_adj = cfg.get_children("affect_adjacent");
@@ -393,7 +424,7 @@ bool unit::ability_affects_adjacent(const std::string& ability, const config& cf
  * cfg: an ability WML structure
  *
  */
-bool unit::ability_affects_self(const std::string& ability,const config& cfg,const gamemap::location& loc) const
+bool unit::ability_affects_self(const std::string& ability,const config& cfg,const map_location& loc) const
 {
 	int illuminates = -1;
 	config const *filter = cfg.child("filter_self");
@@ -417,16 +448,16 @@ bool unit_ability_list::empty() const
 	return cfgs.empty();
 }
 
-std::pair<int,gamemap::location> unit_ability_list::highest(const std::string& key, int def) const
+std::pair<int,map_location> unit_ability_list::highest(const std::string& key, int def) const
 {
 	if(cfgs.empty()) {
-		return std::make_pair(def,gamemap::location::null_location);
+		return std::make_pair(def,map_location::null_location);
 	}
-	gamemap::location best_loc = gamemap::location::null_location;
+	map_location best_loc = map_location::null_location;
 	int abs_max = -10000;
 	int flat = -10000;
 	int stack = 0;
-	for (std::vector< std::pair<config*, gamemap::location> >::const_iterator i = cfgs.begin(),
+	for (std::vector< std::pair<config*, map_location> >::const_iterator i = cfgs.begin(),
 	     i_end = cfgs.end(); i != i_end; ++i) {
 		std::string const &text = (*i->first)[key];
 		int value = lexical_cast_default<int>(text);
@@ -438,7 +469,7 @@ std::pair<int,gamemap::location> unit_ability_list::highest(const std::string& k
 			}
 		} else {
 			int val = text.empty() ? def : value;
-			flat = maximum<int>(flat,val);
+			flat = std::max<int>(flat,val);
 			if (value > abs_max) {
 				abs_max = value;
 				best_loc = i->second;
@@ -447,16 +478,16 @@ std::pair<int,gamemap::location> unit_ability_list::highest(const std::string& k
 	}
 	return std::make_pair(flat + stack, best_loc);
 }
-std::pair<int,gamemap::location> unit_ability_list::lowest(const std::string& key, int def) const
+std::pair<int,map_location> unit_ability_list::lowest(const std::string& key, int def) const
 {
 	if(cfgs.empty()) {
-		return std::make_pair(def,gamemap::location::null_location);
+		return std::make_pair(def,map_location::null_location);
 	}
-	gamemap::location best_loc = gamemap::location::null_location;
+	map_location best_loc = map_location::null_location;
 	int abs_max = 10000;
 	int flat = 10000;
 	int stack = 0;
-	for (std::vector< std::pair<config*, gamemap::location> >::const_iterator i = cfgs.begin(),
+	for (std::vector< std::pair<config*, map_location> >::const_iterator i = cfgs.begin(),
 	     i_end = cfgs.end(); i != i_end; ++i) {
 		std::string const &text = (*i->first)[key];
 		int value = lexical_cast_default<int>(text);
@@ -468,7 +499,7 @@ std::pair<int,gamemap::location> unit_ability_list::lowest(const std::string& ke
 			}
 		} else {
 			int val = text.empty() ? def : value;
-			flat = minimum<int>(flat,val);
+			flat = std::min<int>(flat,val);
 			if (value < abs_max) {
 				abs_max = value;
 				best_loc = i->second;
@@ -508,15 +539,31 @@ std::pair<int,gamemap::location> unit_ability_list::lowest(const std::string& ke
  *
  */
 
+namespace {
+	bool get_special_children(std::vector<const config*>& result, const config& parent,
+	                           const std::string& id, bool just_peeking=false) {
+		config::all_children_iterator it, it_end = parent.ordered_end();
+		for(it = parent.ordered_begin(); it != it_end; ++it) {
+			if(it.get_key() == id || it.get_child()["id"] == id) {
+				if(just_peeking) {
+					return true; // peek succeeded, abort
+				} else {
+					result.push_back(it->second);
+				}
+			}
+		}
+		return false;
+	}
+}
 
 bool attack_type::get_special_bool(const std::string& special,bool force) const
 {
 //	log_scope("get_special_bool");
 	const config* specials = cfg_.child("specials");
 	if(specials) {
-		const config::child_list& list = specials->get_children(special);
-		if (!list.empty() && force) return true;
-		for (config::child_list::const_iterator i = list.begin(),
+		std::vector<const config*> list;
+		if (get_special_children(list, *specials, special, force)) return true;
+		for (std::vector<const config*>::iterator i = list.begin(),
 		     i_end = list.end(); i != i_end; ++i) {
 			if (special_active(**i, true))
 				return true;
@@ -525,8 +572,9 @@ bool attack_type::get_special_bool(const std::string& special,bool force) const
 	if (force || !other_attack_) return false;
 	specials = other_attack_->cfg_.child("specials");
 	if (specials) {
-		const config::child_list& list = specials->get_children(special);
-		for (config::child_list::const_iterator i = list.begin(),
+		std::vector<const config*> list;
+		get_special_children(list, *specials, special);
+		for (std::vector<const config*>::iterator i = list.begin(),
 		     i_end = list.end(); i != i_end; ++i) {
 			if (other_attack_->special_active(**i, false))
 				return true;
@@ -544,7 +592,7 @@ unit_ability_list attack_type::get_specials(const std::string& special) const
 		for (config::child_list::const_iterator i = list.begin(),
 		     i_end = list.end(); i != i_end; ++i) {
 			if (special_active(**i, true))
-				res.cfgs.push_back(std::pair<config*, gamemap::location>
+				res.cfgs.push_back(std::pair<config*, map_location>
 					(*i, attacker_ ? aloc_ : dloc_));
 		}
 	}
@@ -555,16 +603,16 @@ unit_ability_list attack_type::get_specials(const std::string& special) const
 		for (config::child_list::const_iterator i = list.begin(),
 		     i_end = list.end(); i != i_end; ++i) {
 			if (other_attack_->special_active(**i, false))
-				res.cfgs.push_back(std::pair<config*, gamemap::location>
+				res.cfgs.push_back(std::pair<config*, map_location>
 					(*i, attacker_ ? dloc_ : aloc_));
 		}
 	}
 	return res;
 }
-std::vector<std::string> attack_type::special_tooltips(bool force) const
+std::vector<t_string> attack_type::special_tooltips(bool force) const
 {
 //	log_scope("special_tooltips");
-	std::vector<std::string> res;
+	std::vector<t_string> res;
 	const config* specials = cfg_.child("specials");
 	if (!specials) return res;
 
@@ -574,13 +622,13 @@ std::vector<std::string> attack_type::special_tooltips(bool force) const
 		for (config::child_list::const_iterator j = i->second.begin(),
 		     j_end = i->second.end(); j != j_end; ++j) {
 			if (force || special_active(**j, true)) {
-				std::string const &name = (**j)["name"];
+				t_string const &name = (**j)["name"];
 				if (!name.empty()) {
 					res.push_back(name);
 					res.push_back((**j)["description"]);
 				}
 			} else {
-				std::string const &name = (**j)["name_inactive"];
+				t_string const &name = (**j)["name_inactive"];
 				if (!name.empty()) {
 					res.push_back(name);
 					res.push_back((**j)["description_inactive"]);
@@ -721,7 +769,7 @@ bool attack_type::special_active(const config& cfg,bool self,bool report) const
 			}
 		}
 	}
-	gamemap::location adjacent[6];
+	map_location adjacent[6];
 	if(attacker_) {
 		get_adjacent_tiles(aloc_,adjacent);
 	} else {
@@ -733,9 +781,9 @@ bool attack_type::special_active(const config& cfg,bool self,bool report) const
 		std::vector<std::string> dirs = utils::split((**i)["adjacent"]);
 		for (std::vector<std::string>::const_iterator j = dirs.begin(),
 		     j_end = dirs.end(); j != j_end; ++j) {
-			gamemap::location::DIRECTION index =
-				gamemap::location::parse_direction(*j);
-			if (index == gamemap::location::NDIRECTIONS)
+			map_location::DIRECTION index =
+				map_location::parse_direction(*j);
+			if (index == map_location::NDIRECTIONS)
 				continue;
 			unit_map::const_iterator unit = unitmap_->find(adjacent[index]);
 			if (unit == unitmap_->end() ||
@@ -749,9 +797,9 @@ bool attack_type::special_active(const config& cfg,bool self,bool report) const
 		std::vector<std::string> dirs = utils::split((**i)["adjacent"]);
 		for (std::vector<std::string>::const_iterator j = dirs.begin(),
 		     j_end = dirs.end(); j != j_end; ++j) {
-			gamemap::location::DIRECTION index =
-				gamemap::location::parse_direction(*j);
-			if (index == gamemap::location::NDIRECTIONS)
+			map_location::DIRECTION index =
+				map_location::parse_direction(*j);
+			if (index == map_location::NDIRECTIONS)
 				continue;
 			/* GCC-3.3 doesn't accept vconfig(*i) in adj_filter */
 			const vconfig& v = vconfig(*i);
@@ -805,31 +853,27 @@ bool attack_type::special_affects_self(const config& cfg) const
 		return true;
 	return false;
 }
-void attack_type::set_specials_context(const gamemap::location& aloc,const gamemap::location& dloc,
-                              const game_data* gamedata, const unit_map* unitmap,
+void attack_type::set_specials_context(const map_location& aloc,const map_location& dloc,
+                              const unit_map* unitmap,
 							  const gamemap* map, const gamestatus* game_status,
-							  const std::vector<team>* teams, bool attacker,const attack_type* other_attack) const
+							  const std::vector<team>* /*teams*/, bool attacker,const attack_type* other_attack) const
 {
 	aloc_ = aloc;
 	dloc_ = dloc;
-	gamedata_ = gamedata;
 	unitmap_ = unitmap;
 	map_ = map;
 	game_status_ = game_status;
-	teams_ = teams;
 	attacker_ = attacker;
 	other_attack_ = other_attack;
 }
 
-void attack_type::set_specials_context(const gamemap::location& loc, const gamemap::location& dloc, const unit& un, bool attacker) const
+void attack_type::set_specials_context(const map_location& loc, const map_location& dloc, const unit& un, bool attacker) const
 {
 	aloc_ = loc;
 	dloc_ = dloc;
-	gamedata_ = un.gamedata_;
 	unitmap_ = un.units_;
 	map_ = un.map_;
 	game_status_ = un.gamestatus_;
-	teams_ = un.teams_;
 	attacker_ = attacker;
 	other_attack_ = NULL;
 }
@@ -841,11 +885,16 @@ namespace unit_abilities
 {
 
 
-individual_effect::individual_effect(value_modifier t,int val,config* abil,const gamemap::location& l)
+individual_effect::individual_effect(value_modifier t,int val,config* abil,const map_location& l) :
+	type(),
+	value(0),
+	ability(NULL),
+	loc()
 {
 	set(t,val,abil,l);
 }
-void individual_effect::set(value_modifier t,int val,config* abil,const gamemap::location& l)
+
+void individual_effect::set(value_modifier t,int val,config* abil,const map_location& l)
 {
 	type=t;
 	value=val;
@@ -853,18 +902,19 @@ void individual_effect::set(value_modifier t,int val,config* abil,const gamemap:
 	loc=l;
 }
 
-
-
-effect::effect(const unit_ability_list& list, int def, bool backstab)
+effect::effect(const unit_ability_list& list, int def, bool backstab) :
+	effect_list_(),
+	composite_value_(0)
 {
 
-	int value_set = def; bool value_is_set = false;
+	int value_set = def;
+	bool value_is_set = false;
 	std::map<std::string,individual_effect> values_add;
 	std::map<std::string,individual_effect> values_mul;
 
 	individual_effect set_effect;
 
-	for (std::vector< std::pair<config*, gamemap::location> >::const_iterator
+	for (std::vector< std::pair<config*, map_location> >::const_iterator
 	     i = list.cfgs.begin(), i_end = list.cfgs.end(); i != i_end; ++i) {
 		const config& cfg = (*i->first);
 		std::string const &effect_id = cfg[cfg["id"].empty() ? "name" : "id"];
@@ -900,7 +950,7 @@ effect::effect(const unit_ability_list& list, int def, bool backstab)
 				value_set = value;
 				set_effect.set(SET, value, i->first, i->second);
 			} else {
-				if (cumulative) value_set = maximum<int>(value_set, def);
+				if (cumulative) value_set = std::max<int>(value_set, def);
 				if (value > value_set) {
 					value_set = value;
 					set_effect.set(SET, value, i->first, i->second);
