@@ -1,6 +1,6 @@
-/* $Id: menu.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: menu.cpp 32635 2009-02-07 13:00:40Z baufo $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -12,24 +12,20 @@
    See the COPYING file for more details.
 */
 
+#define GETTEXT_DOMAIN "wesnoth-lib"
+
 #include "global.hpp"
 
 #include "widgets/menu.hpp"
 
 #include "game_config.hpp"
 #include "font.hpp"
-#include "image.hpp"
 #include "language.hpp"
 #include "marked-up_text.hpp"
 #include "sound.hpp"
-#include "sdl_utils.hpp"
-#include "util.hpp"
 #include "video.hpp"
 #include "wml_separators.hpp"
-#include "serialization/string_utils.hpp"
 
-#include <algorithm>
-#include <cassert>
 #include <numeric>
 
 namespace gui {
@@ -299,10 +295,10 @@ void menu::update_size()
 {
 	unsigned int h = heading_height();
 	for(size_t i = get_position(),
-	    i_end = minimum(items_.size(), i + max_items_onscreen());
+	    i_end = std::min(items_.size(), i + max_items_onscreen());
 	    i < i_end; ++i)
 		h += get_item_rect(i).h;
-	h = maximum(h, height());
+	h = std::max(h, height());
 	if (max_height_ > 0 && h > static_cast<unsigned>(max_height_)) {
 		h = max_height_;
 	}
@@ -312,7 +308,7 @@ void menu::update_size()
 	unsigned w = std::accumulate(widths.begin(), widths.end(), 0);
 	if (items_.size() > max_items_onscreen())
 		w += scrollbar_width();
-	w = maximum(w, width());
+	w = std::max(w, width());
 	if (max_width_ > 0 && w > static_cast<unsigned>(max_width_)) {
 		use_ellipsis_ = true;
 		w = max_width_;
@@ -521,6 +517,14 @@ void menu::move_selection(size_t id)
 	}
 }
 
+// public function
+void menu::move_selection_keeping_viewport(size_t id)
+{
+	if(id < item_pos_.size()) {
+		set_selection_pos(item_pos_[id], true, NO_MOVE_VIEWPORT);
+	}
+}
+
 void menu::reset_selection()
 {
 	set_selection_pos(0);
@@ -603,9 +607,9 @@ void menu::handle_event(const SDL_Event& event)
 		// Only pass key events if we have the focus
 		if (focus(&event))
 			key_press(event.key.keysym.sym);
-	} else if(event.type == SDL_MOUSEBUTTONDOWN &&
-	         (event.button.button == SDL_BUTTON_LEFT || event.button.button == SDL_BUTTON_RIGHT) ||
-	         event.type == DOUBLE_CLICK_EVENT) {
+	} else if(!mouse_locked() && ((event.type == SDL_MOUSEBUTTONDOWN &&
+	         (event.button.button == SDL_BUTTON_LEFT || event.button.button == SDL_BUTTON_RIGHT)) ||
+	         event.type == DOUBLE_CLICK_EVENT)) {
 
 		int x = 0;
 		int y = 0;
@@ -657,7 +661,7 @@ void menu::handle_event(const SDL_Event& event)
 				sort_by(heading);
 			}
 		}
-	} else if(event.type == SDL_MOUSEMOTION) {
+	} else if(!mouse_locked() && event.type == SDL_MOUSEMOTION) {
 		if(click_selects_) {
 			const int item = hit(event.motion.x,event.motion.y);
 			const bool out = (item == -1);
@@ -758,14 +762,14 @@ SDL_Rect menu::style::item_size(const std::string& item) const {
 			surface const img = get_item_image(image_name);
 			if(img != NULL) {
 				res.w += img->w;
-				res.h = maximum<int>(img->h, res.h);
+				res.h = std::max<int>(img->h, res.h);
 			}
 		} else {
 			const SDL_Rect area = {0,0,10000,10000};
 			const SDL_Rect font_size =
 				font::draw_text(NULL,area,get_font_size(),font::NORMAL_COLOUR,str,0,0);
 			res.w += font_size.w;
-			res.h = maximum<int>(font_size.h, res.h);
+			res.h = std::max<int>(font_size.h, res.h);
 		}
 	}
 	return res;
@@ -891,7 +895,7 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 				const std::string image_name(str.begin()+1,str.end());
 				const surface img = style_->get_item_image(image_name);
 				const int remaining_width = max_width_ < 0 ? area.w :
-				minimum<int>(max_width_, ((lang_rtl)? xpos - rect.x : rect.x + rect.w - xpos));
+				std::min<int>(max_width_, ((lang_rtl)? xpos - rect.x : rect.x + rect.w - xpos));
 				if(img != NULL && img->w <= remaining_width
 				&& rect.y + img->h < area.h) {
 					const size_t y = rect.y + (rect.h - img->h)/2;
@@ -909,7 +913,7 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 				const std::string to_show =
 					(use_ellipsis_ && !has_wrap) ?
 						font::make_text_ellipsis(str, style_->get_font_size(),
-							loc.w - (xpos - rect.x) - 2*style_->get_thickness(), false)
+							loc.w - (xpos - rect.x) - 2*style_->get_thickness(), false, true)
 						: str;
 				const SDL_Rect& text_size = font::text_area(str,style_->get_font_size());
 				const size_t y = rect.y + (rect.h - text_size.h)/2;
@@ -1126,7 +1130,7 @@ size_t menu::get_item_height_internal(const std::vector<std::string>& item) cons
 	size_t res = 0;
 	for(std::vector<std::string>::const_iterator i = item.begin(); i != item.end(); ++i) {
 		SDL_Rect rect = style_->item_size(*i);
-		res = maximum<int>(rect.h,res);
+		res = std::max<int>(rect.h,res);
 	}
 
 	return res;
@@ -1138,7 +1142,7 @@ size_t menu::heading_height() const
 		heading_height_ = int(get_item_height_internal(heading_));
 	}
 
-	return minimum<unsigned int>(heading_height_,max_height_);
+	return std::min<unsigned int>(heading_height_,max_height_);
 }
 
 size_t menu::get_item_height(int) const
@@ -1148,7 +1152,7 @@ size_t menu::get_item_height(int) const
 
 	size_t max_height = 0;
 	for(size_t n = 0; n != items_.size(); ++n) {
-		max_height = maximum<int>(max_height,get_item_height_internal(items_[n].fields));
+		max_height = std::max<int>(max_height,get_item_height_internal(items_[n].fields));
 	}
 
 	return item_height_ = max_height;
